angular.module('configAppliances', ['ngDialog', 'angulartics', 'angulartics.google.analytics'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])
    .controller('AppliancesCtrl', ['$scope', '$rootScope', '$http', 'ngDialog', 'wizardService', '$analytics',
        function($scope, $rootScope, $http, ngDialog, wizardService, $analytics) {

        $scope.initCtrl = function () {
            processAppliances(PHD, window, $analytics);
        };

        $scope.listViewFlag =false;
        $scope.tableView = true;
        $scope.getView = function(tabular) {
            var view = tabular || $scope.tableView;
            return (view  ===true ? "Table" : "List");
        };


        $scope.STORAGE_TAB = 0;
        $scope.TARGETS_TAB = 1;
        $scope.NETWORK_TAB = 2;
        $scope.ASSETS_TAB = 3;

        $scope.toggleView = function() {
            $scope.tableView = !$scope.tableView;
            
            if($scope.currentDetail == 0){
                $scope.setApplianceStorageTab();
            }else if($scope.currentDetail == 1){
                $scope.setApplianceTargetsTab();
            }else if($scope.currentDetail == 2){
                $scope.setApplianceNetworkTab();
            }

            if(!$scope.tableView){
                $scope.selectApplianceFromList($scope.selectedID);
                $("#" + ($scope.selectedID)).css("background-color","#f5f5f5");
            }
        };

        function processAppliances(PHD, window, $analytics) {
            var     REFRESH_INTERVAL = 60000;
            var     STORAGE_TAB = $scope.STORAGE_TAB,
                    TARGETS_TAB = $scope.TARGETS_TAB,
                    NETWORK_TAB = $scope.NETWORK_TAB,
                    ASSETS_TAB = $scope.ASSETS_TAB;


            var storageTypeLookup = {
                "backup": gettext("Backup"),
                "archive": gettext("Archive"),
                "sync": gettext("Sync")
            };

            var storageDialogTitle = gettext("Add %(storage_type)s %(usage)s Storage");

            var backupCopyDialogTitle = gettext("Add Backup Copy %(storage_type)s Target");

            function usageLookup(usage) {
                var icon = '<i class="icon-uui-backup' + ' icon-large text-standard" title="' + 'Backup' + '"></i> ';
                if (usage == 'archive') {
                    icon = '<i class="icon-uui-backup-copy icon-large text-standard" title="' + 'Backup Copy' + '"></i> ';
                }
            }

            var storageLookup = {};
            storageLookup[""] = "attached";
            storageLookup["internal"] = "attached";
            storageLookup["stateless"] = "attached";
            storageLookup["disk"] = "attached";
            storageLookup["iscsi"] = "attached";
            storageLookup["added_internal"] = "attached";

            function getStorageType(row) {
                var type = 'attached';
                if (row.type == 'nas') {
                    if (typeof row.properties !== 'undefined' && row.properties != null) {
                        type = row.properties.protocol;
                    }
                } else {
                    type = storageLookup[row.type];
                }
                return type;
            }

            var cellFormatter = {
                role: function(value, row) {
                    var value = row.role;
                    var html = title = "";
                    if (typeof value !== 'undefined') {
                        if (value[2]) {
                            title = gettext("Presentation Appliance");
                            html += '<i class="icon-phd-layer-presentation icon-large vba-pres" title="' + title + '"></i>';
                        }
                        if (value[1]) {
                            title = gettext("Management Appliance");
                            html += '<i class="icon-phd-layer-management icon-large vba-mgmt" title="' + title + '"></i>';
                        }
                        if (value[0]) {
                            title = gettext("Engine Appliance");
                            html += '<i class="icon-phd-layer-engine icon-large vba-eng" title="' + title + '"></i>';
                        }
                    }
                    return html;
                },
                total_mb_size: function(value, row) {
                    var size = row.total_mb_size;
                    var free = row.total_mb_free;
                    var used = size / free;
                    var used_allocated = "";
                    var percent_used = 0;
                    var state = "storage-";
                    var warn_threshold = 0.75;
                    var stop_threshold = 0.95;
                    if(used < warn_threshold) state += "ok";
                    if(used >= warn_threshold && used < stop_threshold) state += "warn";
                    if(used >= stop_threshold) state += "critical";
                    if (row.total_mb_size !== undefined) {
                        var sizeVal = row.total_mb_size;
                        var freeVal = row.total_mb_free;
                        if (sizeVal === 0) {
                            used_allocated = "Not Available";
                            color = "color:gray";
                            percent_used = 100;
                        } else {
                            var size = PHD.util.formatMB(sizeVal);
                            var used = PHD.util.formatMB(sizeVal - freeVal);
                            used_allocated = "" + used + " / " + size;
                            percent_used = ((sizeVal - freeVal) * 100.0) / sizeVal;
                        }
                    }
                    return '<div class="progress-bar" title="' + used_allocated + '">' +
                        '<div class="progress-meter ' + state + '" style="width:' + percent_used + '%;"></div></div>';
                },
                type: function(value, row) {
                    var usage = usageLookup(row.usage);
                    icon = '<i class="icon-phd-'+ usage +' icon-large" title="' + usage + '"></i> ';
                    return icon + ' ' + value;
                },
                type: function (value, row) {
                    var usage = usageLookup(row.usage),
                        stoType = getStorageType(row),
                        usageIcon = '<i class="icon-phd-' + usage + ' icon-large text-standard" title="' + usage + '"></i> ',
                        stoIcon = '<i class="icon-phd-bds-' + stoType + ' icon-large text-standard" title="' + stoType + '"></i>';
                    return usageIcon + stoIcon;
                },
                status: function(value, row) {
                    var statustype = row.online;
                    var title = "";
                    var iconClass = "";
                    var lastupdated = (typeof(row.version_status) != "undefined") ? ('Last Updated: ' + row.version_status) : "";
                    var html = "";
                    var statusname = "";
                    
                    if (statustype == true) {
                        iconClass = "okcircle";
                        statusname = "Online";
                    }else{
                        iconClass = "exclamationsign";
                        statusname = "Offline";
                    }
                    
                    /*else if (value === "offline") {
                        iconClass = "exclamationsign";
                    } else if (value === 'deleting') {
                        iconClass = "forbidden";
                    } else if (value === "online") {
                        iconClass = "okcircle";
                    } */
                    
                    if(statustype == true){
                        title += 'Pending backup copy request.\n' + lastupdated + '\n\n' +
                            'Click to allow or deny backup copies to be \n received from this appliance.';
                        
                        html= '<i class="icon-'+iconClass+' icon-large" title="' + title + '"></i> ' + statusname;
                        
                    }else{
                        title += lastupdated;
                        
                        html = '<i class="icon-'+iconClass+' icon-large" title="' + title + '"></i> ' + statusname;
                    }
                    
                    return html;
                },
                appliance_type: function(value, row) {
                    var applianceType = row.local;
                    var iconClass = "";
                    var displayStatus = "";
                    if (applianceType === true) {
                        iconClass = "hv-compute_resource";
                        displayStatus = "Source Appliance";
                    } else if (applianceType === false) {
                        iconClass = "uui-backup-copy";
                        displayStatus = "Target Appliance";
                    } else{
                        iconClass = "";
                        displayStatus = "";
                    }
                    return '<i class="icon-'+iconClass+' icon-large" title="' + displayStatus + '"></i> ';
                },
                version: function(value, row) {
                    var icon = title = "";
                    //if(row.state.version_info === VERSION_NOT_UP_TO_DATE) {
                    if(row.version_status === "update_available") {
                        title = gettext("An update is available. Use the Global tab to apply updates.");
                        icon = '<i class="icon-phd-update-needed" title="'+title+'"></i>';
                        //} else if (row.state.version_info === VERSION_UPDATE_IN_PROGRESS) {
                    } else if (row.version_info === VERSION_UPDATE_IN_PROGRESS) {
                        title = gettext("Update in progress");
                        icon = '<i class="icon-phd-update-in-progress icon-pulse" title="'+title+'"></i>';
                    }
                    return icon + " " + value;
                },
                mb_size: function(value, row) {
                    return PHD.util.formatMB(value);
                },
                mb_free: function(value, row) {
                    return PHD.util.formatMB(value);
                },
                percent_used: function (value, row) {
                    var percent_used = row.percent_used,
                        state = "storage-";
                    var used_allocated = "";
                    var color = "";
                    if (row.mb_size !== undefined) {
                        var sizeVal = row.mb_size;
                        var freeVal = row.mb_free;
                        if (sizeVal === 0) {
                            used_allocated = "Not Available";
                            percent_used = "0%";
                        } else {
                            var size = PHD.util.formatMB(sizeVal);
                            var free = PHD.util.formatMB(freeVal);
                            used_allocated = "" + free + " / " + size;
                        }
                    }
                    var will_purge = row.mb_to_purge;
                    var to_display = used_allocated;
                    return '<div class="progress-bar" title="' + to_display + '">' +
                        '<div class="progress-meter ' + state + 'ok" style="width:' + percent_used + ';' + color + '"></div></div>';
                }
            };

            var networkFormatter  = {
                dns: function(value) {
                    var html = "";
                    if(value[0]) {
                        html += '<span class="truncates item-name" title="' + value[0] + '">' + value[0] + '</span>';
                    }
                    if(value[1]) {
                        html += '<span class="truncates item-name" title="' + value[1] + '">' + value[1] + '</span>';
                    }
                    return html;
                },
                search: function(value) {
                    var html = "";
                    if(value[0]) {
                        html += '<span class="truncates item-name" title="' + value[0] + '">' + value[0] + '</span>';
                    }
                    if(value[1]) {
                        html += '<span class="truncates item-name" title="' + value[1] + '">' + value[1] + '</span>';
                    }
                    return html;
                }
            };
            
            var replicationFormatter = {
                status: function(value, row) {
                    var targetStatus = row.status;
                    var displayStatus = "";
                    var title = (typeof(row.message) != "undefined") ? row.message : "";
                    var iconClass = "";
                    var iconColor = "black";
                    var html = "";
                    
                    if (targetStatus === 'pending') {
                        iconClass = "warningsign";
                        displayStatus = "Pending";
                        iconColor = "#FF9933";
                    } else if (targetStatus === "online") {
                        iconClass = "okcircle";
                        displayStatus = "Online";
                        iconColor = "green";
                    } else if(targetStatus === "rejected"){
                        iconClass = "forbidden";
                        displayStatus = "Rejected";
                        iconColor = "#e00034";
                    } else if(targetStatus === "failed"){
                        iconClass = "erroralt";
                        displayStatus = "Failed";
                        iconColor = "#e00034";
                    } else if(targetStatus === "offline"){
                        iconClass = "exclamationsign";
                        displayStatus = "Offline";
                        iconColor = "#e00034";
                    }
                    
                    if(typeof(row.created) != "undefined" ){
                        title += '\n\nCreated: ' + row.created;
                    }
                    
                    if((typeof(row.created) != "undefined" ) && (typeof(row.updated) != "undefined" )){
                        title += '\n' + 'Last Updated: ' + row.updated;
                    }else if ((typeof(row.updated) != "undefined" )){
                         title += '\n\nLast Updated: ' + row.updated;
                    }
                    
                    html = '<i class="icon-'+iconClass + 
                    ' icon-large" title="' + title + '" style="color:' + iconColor + ';margin-right:2px;"></i>    ' + displayStatus;

                    return html;
                },
                version: function(value, row) {
                    var icon = title = "";
                    //if(row.state.version_info === VERSION_NOT_UP_TO_DATE) {
                    if(row.version_status === "update_available") {
                        title = gettext("An update is available. Use the Global tab to apply updates.");
                        icon = '<i class="icon-phd-update-needed" title="'+title+'"></i>';
                        //} else if (row.state.version_info === VERSION_UPDATE_IN_PROGRESS) {
                    } else if (row.version_info === VERSION_UPDATE_IN_PROGRESS) {
                        title = gettext("Update in progress");
                        icon = '<i class="icon-phd-update-in-progress icon-pulse" title="'+title+'"></i>';
                    }
                    return icon + " " + value;
                },
                capacity: function (value, row) {
                    var size = row.gb_size;
                    var free = row.gb_free;
                    var used = size - free;
                    var used_allocated = "";
                    var percent_used = 0;
                    var state = "storage-";
                    var max_threshold = row.max_size;
                    var html = "";
                    
                    if(used < max_threshold) state += "ok";
                    if(used == max_threshold) state += "warn";
                    if(used > max_threshold) state += "critical";
                    if (row.gb_size !== undefined) {
                        var sizeVal = row.gb_size;
                        var freeVal = row.gb_free;
                        if (sizeVal === 0) {
                            used_allocated = "Not Available";
                            color = "color:gray";
                            percent_used = 100;
                        } else {
                            var size = PHD.util.formatMB(sizeVal);
                            var used = PHD.util.formatMB(sizeVal - freeVal);
                            used_allocated = "" + used + " / " + size;
                            percent_used = ((sizeVal - freeVal) * 100.0) / sizeVal;
                        }
                    }
                    
                    if(typeof(row.gb_size) == "undefined"){
                        html += '<div>--</div>';
                    }else{
                        html += '<div class="progress-bar" title="' + used_allocated + '">' +
                        '<div class="progress-meter ' + state + '" style="width:' + percent_used + '%;"></div></div>';
                    }
                    return html;
                },
                type: function(value, row) {
                    var targetType = row.type;
                    var iconClass = "";
                    if (targetType == "cloud_storage") {
                        iconClass = "phd-storage";
                    } else if (targetType == "appliance") {
                        iconClass = "hv-compute_resource";
                    }else if(targetType == "managed_cloud"){
                        iconClass = "uui-cloud";
                    }
                    return '<i class="icon-'+iconClass+' icon-large" title="' + targetType + '" style="margin-right:2px;"></i> ' + targetType;
                },
            };

            var Appliance = function($el) {
                $scope.el = $el;
                this.$el = $el;
                this.dbid = this.$el.data("dbid");
                this.$header = $el.children(".environment-header");
                this.$appliances = $el.children(".environment-appliances");

                this.$buttons = $el.find(".button-bar a");
                $scope.buttons = this.$buttons;
                this.$options = this.$buttons.filter(".btn-environment-options");

                this.appliancesInit = false;

                var applianceOptions = {
                    cellClasses: ["long-text truncate", null, null, null, null, "numerical", null, null]
                };

                this.$applianceTable = this.$appliances.find(".appliance-table"); //this.$appliances.find(".appliance-table");
                this.ApplianceTable = PHD.DataTable(this.$applianceTable, applianceOptions).registerFormatters(cellFormatter);
                $scope.ApplianceTable = this.ApplianceTable;

                this.appliances = [];
                this.$selectedAppliance = $();
                
                this.selectedID = null;
                $scope.selectedID = null;
                PHD.appliance_sid = this.selectedID;


                this.$storageTable = this.$el.find(".bds-table");
                this.StorageTable = PHD.DataTable(this.$storageTable, {}).registerFormatters(cellFormatter);
                $scope.StorageTalbeObj = this.StorageTable;
                
                this.$storageTableList = this.$el.find(".bds-table-list");
                this.StorageTableList = PHD.DataTable(this.$storageTableList, {}).registerFormatters(cellFormatter);
                $scope.StorageTableListObj = this.StorageTableList;
                
                // lmc
                this.storageData = [];

                this.$networkTable = this.$el.find(".network-table");
                this.NetworkTable = PHD.DataTable(this.$networkTable, {integerIDs: false}).registerFormatters(networkFormatter);
                $scope.NetworkTableObj = this.NetworkTable;
                
                this.$networkTableList = this.$el.find(".network-table-list");
                this.NetworkTableList = PHD.DataTable(this.$networkTableList, {integerIDs: false}).registerFormatters(networkFormatter);
                $scope.NetworkTableListObj = this.NetworkTableList;

                this.currentDetail = STORAGE_TAB;
                $scope.currentDetail = this.currentDetail;

                this.$selectedStorage = $();
                this.selectedStorageID = null;

                this.$selectedNic = $();
                this.selectedNicID = null;
                
                 this.$replicationTable = this.$el.find(".targets-table");
                this.ReplicationTable = PHD.DataTable(this.$replicationTable, {}).registerFormatters(replicationFormatter);
                $scope.ReplicationTableObj = this.ReplicationTable;
                
                this.$replicationTableList = this.$el.find(".targets-table-list");
                this.ReplicationTableList = PHD.DataTable(this.$replicationTableList, {}).registerFormatters(replicationFormatter);
                $scope.ReplicationTableListObj = this.ReplicationTableList;
                
                this.replicationData = [];
                this.$selectedReplicationTarget = $();
                this.selectedReplicationTargetID = null;
                $scope.selectedReplicationTarget = null;

                this.applianceProcess = null;
                this.detailProcess = null;

                this.timestamp = 0;
                this.refreshInterval = null;
                this.isActive = false;
            };
           
            Appliance.prototype.init = function() {
                var self = this;

                this.initButtons();

                this.$applianceTable
                    .on("rowselect", function(event, dbid, $el, DataTable) {
                        self.$selectedAppliance = $el;
                        $scope.el = $el;
                        self.selectedID = dbid;
                        PHD.appliance_sid = self.selectedID;
                        $scope.selectedID = self.selectedID;
                        self.$selectedStorage = $();
                        self.selectedStorageID = null;
                        self.$selectedNic = $();
                        self.selectedNicID = null;
                        self.$selectedReplicationTarget = $();
                        self.selectedReplicationTargetID = null;
                        $scope.selectedReplicationTarget = null;
                        self.updateButtons();
                    
                        var appliancedatatable = DataTable.data;
                        $rootScope.replicationApplianceId=self.selectedID; //:TODO - throttleChange
                        if(typeof(appliancedatatable) != "undefined"){
                            for (var i=0 ;i < appliancedatatable.length ; i++ ){
                                if (appliancedatatable[i].id ==$rootScope.replicationApplianceId){
                                    $rootScope.replicationApplianceName=appliancedatatable[i].name;
                                }
                            }
                        }
                    
                        if($(".storagesubtab").closest("li").hasClass("ui-tabs-active ui-state-active")){
                            this.currentDetail = 0;
                        }else if($(".targetssubtab").closest("li").hasClass("ui-tabs-active ui-state-active")){
                            this.currentDetail = 1;
                        }else if($(".networksubtab").closest("li").hasClass("ui-tabs-active ui-state-active")){
                            this.currentDetail = 2;
                        }else{
                            this.currentDetail = $scope.currentDetail;
                        }
                    
                        self.refresh(this.updateAppliances, false);
                    })
                    .on("loadfinished updatefinished", function(event, newRows, DataTable) {
                        self.$applianceTable.scrollTable({"height": ".environment-appliances"});
                        if(event.type === "loadfinished" && self.appliances.length) {
                            //self.ApplianceTable.selectByID(self.appliances[0].id);
                        }
                    })
                    .on("click", "td i.icon-okcircle.icon-large", function(event,dbid, $el, DataTable) {
                        event.preventDefault();
                    
                        ngDialog.open({
                            template: 'app/configure/appliances/replicationStatusOptionTemplate.html',
                            scope: $scope,
                            data: {appliancename: $rootScope.replicationApplianceName, selectedReplicationTarget: $scope.selectedReplicationTarget},
                            overlay:true,
                            modelDialogId:'replication-approval-dailog',
                            ngDialogStyle:'width:400px;'
                        });
                    });


                this.$storageTable
                    .on("click", ".help-trigger i", function(event) {
                        var $anchor = $(this).parent();

                        $("<div>", {
                            html: $anchor.data("value")
                        }).popover({
                            autoOpen: true,
                            anchor: $anchor,
                            popoverClass: "nowrap",
                            position: {
                                my: "center bottom",
                                at: "center top-5"
                            }
                        });
                    })
                    .on("rowselect", ".clickable", function(event, dbid, $el, DataTable) {
                        self.$selectedStorage = $el;
                        self.selectedStorageID = dbid;
                        self.updateButtons();
                    })
                    .on("loadfinished updatefinished", function(event, newRows, DataTable) {
                        self.updateButtons();
                    });

                this.$storageTableList
                    .on("click", ".help-trigger i", function(event) {
                        var $anchor = $(this).parent();

                        $("<div>", {
                            html: $anchor.data("value")
                        }).popover({
                            autoOpen: true,
                            anchor: $anchor,
                            popoverClass: "nowrap",
                            position: {
                                my: "center bottom",
                                at: "center top-5"
                            }
                        });
                    })
                    .on("rowselect", ".clickable", function(event, dbid, $el, DataTable) {
                        self.$selectedStorage = $el;
                        self.selectedStorageID = dbid;
                        self.updateButtons();
                    })
                    .on("loadfinished updatefinished", function(event, newRows, DataTable) {
                        self.updateButtons();
                    });
                

                this.$networkTable
                    .on("rowselect", ".clickable", function(event, dbid, $el, DataTable) {
                        self.$selectedNic = $el;
                        self.selectedNicID = dbid;
                        self.updateButtons();
                    });
                
                this.$networkTableList
                    .on("rowselect", ".clickable", function(event, dbid, $el, DataTable) {
                        self.$selectedNic = $el;
                        self.selectedNicID = dbid;
                        self.updateButtons();
                    });
                
                this.$replicationTable
                    .on("rowselect", ".clickable", function(event, dbid, $el, DataTable) {
                        self.$selectedReplicationTarget = $el;
                        self.selectedReplicationTargetID = dbid;
                    
                        if((typeof(DataTable) != "undefined") && (typeof(DataTable.data) != "undefined")){
                            for (var i=0 ;i < (DataTable.data).length ; i++ ){
                                if ((DataTable.data)[i].id ==self.selectedReplicationTargetID){
                                    $scope.selectedReplicationTarget=(DataTable.data)[i];
                                }
                            }
                        }

                        self.updateButtons();
                    })


                this.$replicationTableList
                    .on("rowselect", ".clickable", function(event, dbid, $el, DataTable) {
                        self.$selectedReplicationTarget = $el;
                        self.selectedReplicationTargetID = dbid;
                        
                        if((typeof(DataTable) != "undefined") && (typeof(DataTable.data) != "undefined")){
                            for (var i=0 ;i < (DataTable.data).length ; i++ ){
                                if ((DataTable.data)[i].id ==self.selectedReplicationTargetID){
                                    $scope.selectedReplicationTarget=(DataTable.data)[i];
                                }
                            }
                        }
                    
                        self.updateButtons();
                    });

                $(document)
                    .on("bdssave nicsave", function(event, applianceID, data) {
                        if(self.selectedID === applianceID)
                            self.fetchDetails(true);
                    })
                    .on("vbaadd", function(event) {
                        self.addVBA();
                    });

                this.$appliances.find(".sub-tabs-wrapper").tabs({
                    activate: function(event, ui) {
                        self.currentDetail = ui.newTab.index();
                        $scope.currentDetail = self.currentDetail;
                        console.log('currentDetail is ' + self.currentDetail + ' and isactive = ' + self.isActive);
                        
                        if(self.selectedID && self.isActive) {
                            self.fetchDetails(true);
                        }
                    }
                });

                return this;
            };
            
            Appliance.prototype.initButtons = function() {
                var self = this;

                this.$el.find(".environment-options-menu").flyout();
                this.$el.find(".vba-storage-menu").flyout();

                this.$el
                    .on("click", ".btn-add-attached", function(event) {
                        event.preventDefault();
                        if(!$(this).data("disabled") && self.selectedID) {
                            var bds = $(this).closest(".menu-group").data("bds"),
                                title = interpolate(storageDialogTitle, {
                                    storage_type: gettext("Attached Disk"),
                                    usage: storageTypeLookup[bds]
                                }, true);

                            selectedStorage = null;
                            $analytics && $analytics.eventTrack('Add Disk Storage', {  category: 'Configure', label: 'Storage' });

                            PHD.wizard('app/configure/storage/attached/attached.html', "add-bds", {
                                helpArticle: PHD.App.getHelpLink("addbds", true),
                                title: title,
                                width: 1024,
                                height: 700,
                                open: function(event, ui) {
                                    $("#id_usage").val(bds);
                                    $("#id_type").val(5);
                                }
                            });
                        }
                    })
                    .on("click", ".btn-add-cifs", function(event) {
                        event.preventDefault();
                        if(!$(this).data("disabled") && self.selectedID) {
                            var bds = $(this).closest(".menu-group").data("bds"),
                                title = interpolate(storageDialogTitle, {
                                    storage_type: gettext("CIFS"),
                                    usage: storageTypeLookup[bds]
                                }, true);

                            selectedStorage = null;

                            $analytics && $analytics.eventTrack('Add CIFS Storage', {  category: 'Configure', label: 'Storage' });

                            // lmc - build page by loading template and variable value. differentiate usage?
                            PHD.wizard('app/configure/storage/cifs/cifs.html', "add-bds", {
                                helpArticle: PHD.App.getHelpLink("addbds", true),
                                title: title,
                                width: 500,
                                height: 480,
                                open: function(event, ui) {
                                    $("#id_usage").val(bds);
                                }
                            });
                        }
                    })
                    .on("click", ".btn-add-nfs", function(event) {
                        event.preventDefault();
                        if(!$(this).data("disabled") && self.selectedID) {
                            var bds = $(this).closest(".menu-group").data("bds"),
                                title = interpolate(storageDialogTitle, {
                                    storage_type: gettext("NFS"),
                                    usage: storageTypeLookup[bds]
                                }, true);

                            $analytics && $analytics.eventTrack('Add NFS Storage', {  category: 'Configure', label: 'Storage' });

                            //PHD.wizard('/config/bds/'+bds.toLowerCase()+'/nfs/' + self.selectedID + "/", "add-bds", {
                            // lmc - build page by loading template and variable value. differentiate types?
                            PHD.wizard('app/configure/storage/nfs/nfs.html', "add-bds", {
                                helpArticle: PHD.App.getHelpLink("addbds", true),
                                title: title,
                                width: 500,
                                height: 480,
                                open: function(event, ui) {
                                    $("#id_usage").val(bds);
                                }
                            });
                        }
                    })
                    .on("click", ".btn-add-ircache", function(event) {
                        event.preventDefault();
                        if(!$(this).data("disabled") && self.selectedID) {
                            //PHD.wizard('/config/bds/ir_cache/attached/' + self.selectedID + "/", "add-bds", {
                            // lmc - build page by loading template and variable value. differentiate types?
                            PHD.wizard('app/configure/storage/attached/ir.html', "add-bds", {
                                helpArticle: PHD.App.getHelpLink("addbds", true),
                                title: gettext("Add Instant Recovery Write Space"),
                                helpArticle: PHD.App.getHelpLink("addircache", true),
                                width: 1024,
                                height: 520,
                                open: function(event, ui) {
                                    $("#id_usage").val(6); // TODO - usage for "IR"
                                    $("#id_type").val(5); // added internal disk only.
                                }
                            });
                        }
                    })
                    .on("click", ".btn-add-cloud", function(event) {
                        event.preventDefault();
                        if(!$(this).data("disabled") && self.selectedID) {
                            var bds = $(this).closest(".menu-group").data("bds"),
                                title = interpolate(backupCopyDialogTitle, {
                                    storage_type: gettext("Cloud")
                                }, true);

                            selectedStorage = null;

                            $analytics && $analytics.eventTrack('Add Cloud Target', {  category: 'Configure', label: 'Target' });

                            // lmc - build page by loading template and variable value. differentiate usage?
                            PHD.wizard('app/configure/storage/cloud/cloud.html', "add-bds", {
                                helpArticle: PHD.App.getHelpLink("addcloud", true),
                                title: title,
                                width: 900,
                                height: 500,
                                open: function(event, ui) {
                                    // lmc - no menu group until "optimized" backup copy returns.
                                    //$("#id_usage").val(bds);
                                    $("#id_usage").val("archive");
                                }
                            });
                        }
                    })
                    .on("click", ".btn-edit-storage", function(event) {
                        event.preventDefault();
                        if(!$(this).data("disabled") && self.selectedStorageID) {
                            console.log('selected id = ' + self.selectedStorageID);
                            var storage = _.find(self.storageData, function(obj) { return obj.id == self.selectedStorageID });
                            if (storage !== undefined) {
                                var type = storage.type;
                                var properties = storage.properties;
                                var protocol = properties !== undefined ? properties.protocol : null;
                                console.log('type = ' + type);
                                var spec = getHTMLSpec(type, protocol);
                                var message;

                                $analytics && $analytics.eventTrack('Edit Storage', {  category: 'Configure', label: 'Storage' });

                                var url = 'app/configure/storage/' + spec.htmlFile;

                                selectedStorage = storage;
                                if (selectedStorage.name == 'Internal' || selectedStorage.usage == 'stateless') {
                                    var resp = PHD.Ajax.get("/api/storage/ir");
                                    resp.done(function(data) {
                                        var ir_GB = data["ir storage"].allowed;
                                        var d2d_GB;
                                        PHD.wizard(url, "edit-storage", {
                                            helpArticle: PHD.App.getHelpLink("addbds", true),
                                            title: gettext("Edit Storage"),
                                            width: spec.width,
                                            height: spec.height,
                                            open: function(event, ui) {
                                                $("body")
                                                    .addClass("dialog-open")
                                                    .css("overflow", "auto");
                                                $("#id_name").val(storage.name);
                                                $("#id_name").disable();
                                                $("#edit_object_id").text(storage.id);
                                                $("#id_type").disable(); // type is not changeable on edit.
                                                $("#id_usage").disable(); // usage is not changeable on edit
                                                if (properties !== undefined) {
                                                    $("#id_host").val(properties.hostname);
                                                    $("#id_share").val(properties.share_name);
                                                    $("#id_username").val(properties.username);
                                                    $("#id_port").val(properties.port);
                                                }
                                                var storage_GB = Math.round(selectedStorage.mb_size / 1024);
                                                ir_GB = parseInt(ir_GB, 10);
                                                d2d_GB = storage_GB - ir_GB;
                                                var ir_percent = Math.round(ir_GB / storage_GB * 100);
                                                var d2d_percent = 100 - ir_percent;
                                                $("#id_ir_gb").text("(" + ir_GB + " GB)");
                                                $("#id_d2d_gb").text("(" + d2d_GB + " GB)");
                                                $("#id_ir_allocation").val(ir_percent);
                                                $("#id_d2d_allocation").val(d2d_percent);
                                            }
                                        });
                                    });
                                } else {
                                    PHD.wizard(url, "edit-storage", {
                                        helpArticle: PHD.App.getHelpLink("addbds", true),
                                        title: gettext("Edit Storage"),
                                        width: spec.width,
                                        height: spec.height,
                                        open: function(event, ui) {
                                            $("body")
                                                .addClass("dialog-open")
                                                .css("overflow", "auto");
                                            $("#id_name").val(storage.name);
                                            $("#id_name").disable();
                                            $("#edit_object_id").text(storage.id);
                                            $("#id_type").disable(); // type is not changeable on edit.
                                            $("#id_usage").disable(); // usage is not changeable on edit
                                            if (properties !== undefined) {
                                                $("#id_host").val(properties.hostname);
                                                $("#id_share").val(properties.share_name);
                                                $("#id_username").val(properties.username);
                                                $("#id_port").val(properties.port);
                                            }
                                        }
                                    });
                                }
                            } else {
                                message = "Error finding this storage, please level and re-enter the configuration subsystem and retry.";
                                PHD.throwError({"result": [{"message": gettext(message)}]});
                            }
                        }
                    })
                    .on("click", ".btn-edit-target", function(event) {
                        event.preventDefault();
                        if(!$(this).data("disabled") && self.selectedTargetID) {
                            console.log('selected id = ' + self.selectedTargetID);
                            var target = _.find(self.targetsData, function(obj) { return obj.id == self.selectedTargetID });
                            if (target !== undefined) {
                                var type = target.type;
                                var properties = target.properties;
                                var protocol = properties !== undefined ? properties.protocol : null;
                                console.log('type = ' + type);
                                var spec = getHTMLSpec(type, protocol);
                                var message;

                                $analytics && $analytics.eventTrack('Add Copy Target', {  category: 'Configure', label: 'Target' });

                                var url = 'app/configure/storage/' + spec.htmlFile;

                                selectedStorage = target;
                                PHD.wizard(url, "edit-target", {
                                    helpArticle: PHD.App.getHelpLink("addbds", true),
                                    title: gettext("Edit Backup Copy Target"),
                                    width: spec.width,
                                    height: spec.height,
                                    open: function(event, ui) {
                                        $("body")
                                            .addClass("dialog-open")
                                            .css("overflow", "auto");
                                        $("#id_name").val(target.name);
                                        $("#id_name").disable();
                                        $("#edit_object_id").text(target.id);
                                        //$("#id_type").disable(); // type is not changeable on edit.
                                        $("#id_usage").disable(); // usage is not changeable on edit
                                        if (properties !== undefined) {
                                            $("#id_host").val(properties.hostname);
                                            $("#id_share").val(properties.share_name);
                                            $("#id_username").val(properties.username);
                                            $("#id_port").val(properties.port);
                                            var max_size = properties.max_size;
                                            if (max_size !== undefined) {
                                                $("#id_max_size").val(max_size == 0 ? 2048 : max_size);
                                            }
                                        }
                                    }
                                });
                            } else {
                                message = "Error finding this target, please level and re-enter the configuration subsystem and retry.";
                                PHD.throwError({"result": [{"message": gettext(message)}]});
                            }
                        }
                    })
                    .on("click", ".btn-onoff-storage", function(event) {
                        if(self.selectedStorageID) {
                            var storage = _.find(self.storageData, function(obj) { return obj.id == self.selectedStorageID });
                            if (storage !== undefined) {
                                if (storage.name === "Internal" && storage.status === 'online') {
                                    PHD.throwError({"result": [{"message": gettext("Internal storage cannot be disabled.")}]});
                                } else if (storage.status === 'deleting') {
                                    PHD.throwError({"result": [{"message": gettext("Storage being deleted cannot be enabled.")}]});
                                } else {
                                    var dbid = self.selectedStorageID;
                                    var storage = _.findWhere(self.storageData, {"id": dbid});
                                    var url = '/api/storage/';
                                    if (storage) {
                                        if (storage.status === 'online') {
                                            url += 'offline' + '/' + dbid + '/?sid=' + self.selectedID;
                                        } else if (storage.status === 'offline') {
                                            url += 'online' + '/' + dbid + '/?sid=' + self.selectedID;
                                        }
                                    }
                                    var appliance = self;
                                    var resp = PHD.Ajax.put(url);
                                    resp.done(function (data) {
                                        appliance.fetchDetails(true);
                                    });
                                }
                            }
                        }
                    })
                    .on("click", ".btn-remove-storage", function(event) {
                        event.preventDefault();
                        if(!$(this).data("disabled") && self.selectedStorageID) {
                            var storage = _.find(self.storageData, function(obj) { return obj.id == self.selectedStorageID });
                            if (storage !== undefined) {
                                if (storage.name === "Internal") {
                                    PHD.throwError({"result": [{"message": gettext("Internal storage cannot be deleted.")}]});
                                } else if (storage.status === 'deleting') {
                                    PHD.throwError({"result": [{"message": gettext("This storage is already being deleted.")}]});
                                } else {
                                    var dbid = self.selectedStorageID,
                                        confirm = PHD.confirmDialog(
                                            "<p>"+gettext("Are you sure you want to remove this storage?")+"</p>" +
                                            "<p>"+gettext("Removing unmounts the storage volume from the appliance. Attached disks remain connected to the appliance virtual machine and can be reused or deleted manually.")+"</p>",
                                            { title: gettext("Confirm Storage Removal") }
                                        );

                                    confirm.done(function() {
                                        $analytics && $analytics.eventTrack('Delete Storage', {  category: 'Configure', label: 'Storage' });
                                        var url = '/api/storage/' + self.selectedStorageID + '/?sid=' + self.selectedID;
                                        var resp = PHD.Ajax.delete(url);
                                        resp.done(function(data) {
                                            self.removeBDS(data, dbid);
                                        });
                                    });
                                }
                            } else {
                                message = "Error finding this storage, please level and re-enter the configuration subsystem and retry.";
                                PHD.throwError({"result": [{"message": gettext(message)}]});
                            }
                        }
                    })
                    .on("click", ".btn-remove-target", function(event) {
                        event.preventDefault();
                        if(!$(this).data("disabled") && self.selectedTargetID) {
                            var target = _.find(self.targetsData, function(obj) { return obj.id == self.selectedTargetID });
                            if (target !== undefined) {
                                if (target.status === 'deleting') {
                                    PHD.throwError({"result": [{"message": gettext("This target is already being deleted.")}]});
                                } else {
                                    var dbid = self.selectedTargetID,
                                        confirm = PHD.confirmDialog(
                                            "<p>"+gettext("Are you sure you want to remove this target?")+"</p>" +
                                            "<p>"+gettext("Removing a target does not delete backups that have been copied to it.  ") +
                                            gettext("If you wish to remove the backups, you should use the \"Erase\" button.") + "</p>",
                                            { title: gettext("Confirm Target Removal") }
                                        );

                                    confirm.done(function() {
                                        $analytics && $analytics.eventTrack('Delete Cloud Target', {  category: 'Configure', label: 'Target' });
                                        var url = '/api/storage/' + self.selectedTargetID + '/?sid=' + self.selectedID;
                                        var resp = PHD.Ajax.delete(url);
                                        resp.done(function(data) {
                                            self.removeTarget(data, dbid);
                                        });
                                    });
                                }
                            } else {
                                message = "Error finding this target, please level and re-enter the configuration subsystem and retry.";
                                PHD.throwError({"result": [{"message": gettext(message)}]});
                            }
                        }
                    })
                    .on("click", ".btn-prepare-target", function(event) {
                        if(self.selectedTargetID) {
                            var dbid = self.selectedTargetID;
                            var target = _.findWhere(self.targetsData, {"id": dbid});
                            if (target !== undefined) {
                                if (target.properties != undefined && target.properties.protocol == 'cloud') {

                                    confirm = PHD.confirmDialog(
                                        "<p>"+gettext("Are you sure you want to erase the target ") + target.name + "?" + "</p>" +
                                        "<p>"+gettext("Erasing a target will delete all backups that have been copied to it.")+"</p>",
                                        { title: gettext("Confirm Erase") }
                                    );

                                    confirm.done(function() {

                                        var sid = "/?sid=" + self.selectedID;
                                        var appliance = self;
                                        var check_url = '/api/archive/media/connected' + sid;
                                        var load = PHD.showLoadingIndicator(appliance.$targetsTable, "Scanning Targets...");
                                        var resp = PHD.Ajax.get(check_url, load);
                                        resp.done(function (data) {
                                            if (data.data !== undefined) {

                                                $analytics && $analytics.eventTrack('Prepare Target', {  category: 'Configure', label: 'Target' });
                                                var url = '/api/archive/media/prepare/';
                                                if (target) {
                                                    url += target.name + '/?sid=' + self.selectedID;
                                                }
                                                var obj = {};
                                                obj.label = "Cloud_Target";
                                                obj = JSON.stringify(obj);
                                                var load = PHD.showLoadingIndicator(appliance.$storageTable, "Preparing Target...");
                                                var resp = PHD.Ajax.put(url, obj, load);
                                                resp.done(function (data) {
                                                    appliance.fetchDetails(true);
                                                });
                                            }
                                        });
                                    });
                                } else {
                                    PHD.throwError({"result": [{"message": gettext("Only cloud targets can be completely erased.")}]});
                                }
                            }
                        }
                    })
                    .on("click", ".btn-edit-nic", function(event) {
                        event.preventDefault();
                        if(!$(this).data("disabled") && self.selectedNicID) {
                            $analytics && $analytics.eventTrack('Edit Nic', {  category: 'Configure', label: 'Network' });
                            PHD.wizard("app/configure/networks/edit.html", "edit-nic", {
                                title: gettext("Edit Network Adapter ") + self.selectedNicID,
                                width: 500,
                                height: 500,
                                open: function(event, ui) {
                                    $("body")
                                        .addClass("dialog-open")
                                        .css("overflow", "auto");
                                    var mac = self.$selectedNic.find("[data-source=\"mac\"]").html();
                                    var ip = self.$selectedNic.find("[data-source=\"ip\"]").html();
                                    var gateway = self.$selectedNic.find("[data-source=\"gateway\"]").html();
                                    var netmask = self.$selectedNic.find("[data-source=\"netmask\"]").html();
                                    var dns1 = self.$selectedNic.find("[data-source=\"dns1\"]").html();
                                    var dns2 = self.$selectedNic.find("[data-source=\"dns2\"]").html();
                                    var network_type = self.$selectedNic.find("[data-source=\"network_type\"]").html();
                                    $("h4").text("MAC Address: " + mac);
                                    $("#id_ip").val(ip);
                                    $("#id_gateway").val(gateway);
                                    $("#id_subnet").val(netmask);
                                    $("#id_dns1").val(dns1);
                                    $("#id_dns2").val(dns2);
                                    $("#edit_object_id").text(self.selectedNicID);
                                }
                            });
                        }
                    })
                    .on("click", ".btn-refresh-nics", function(event) {
                        event.preventDefault();
                        if(self.selectedID) {
                            self.$selectedNic = $();
                            self.selectedNicID = null;
                            //var resp = PHD.Ajax.get('/config/appliances/' + self.selectedID + '/networks/');
                            // lmc
                            var resp = PHD.Ajax.get('/api/networks/?sid=' + self.selectedID);
                            resp.done(function(data){
                                self.buildNetwork(data);
                            });
                        }
                    })
                    .on("click", ".btn-add-appliance", function(event) {
                        event.preventDefault();
                        $analytics && $analytics.eventTrack('Add Appliance', {  category: 'Configure', label: 'Appliance' });
                        PHD.wizard("app/configure/appliances/add.html", "add-vba", {
                            title: gettext("Add Appliance"),
                            width: 600,
                            height: "auto",
                            helpArticle: PHD.App.getHelpLink("addvba", true)
                        });
                    })
                    .on("click", ".btn-edit-appliance", function(event) {
                        event.preventDefault();
                        $analytics && $analytics.eventTrack('Edit Appliance', {  category: 'Configure', label: 'Appliance' });
                        selectedAppliance = _.find(self.appliances, function(obj) { return obj.id == self.selectedID });
                        PHD.wizard("app/configure/appliances/edit.html", "edit-vba", {
                            title: gettext("Edit Appliance"),
                            width: 600,
                            height: "auto",
                            helpArticle: PHD.App.getHelpLink("addvba", true),
                            open: function(event, ui) {
                                $("body")
                                    .addClass("dialog-open")
                                    .css("overflow", "auto");
                            },
                            beforeClose: function (event, ui) {
                               self.refresh(self.fetchAppliances);
                            },
                            close: function(event,ui){
                                $("body")
                                    .removeClass("dialog-open")
                                    .css("overflow", "hidden");
                                $(this).wizard("destroy").remove();
                            }
                        });
                    })
                    .on("click", ".btn-remove-appliance", function() {
                        var confirm = PHD.confirmDialog(
                            "<p>"+gettext("Are you sure you want to remove the appliance?")+"</p>",
                            { title: gettext("Confirm Appliance Removal") }
                        );

                        confirm.done($.proxy(self.kill, self));
                    });

                return this;
            };

            Appliance.prototype.setRefreshInterval = function(interval) {
                interval = interval || REFRESH_INTERVAL;
                clearInterval(this.refreshInterval);
                
                if($(".storagesubtab").closest("li").hasClass("ui-tabs-active ui-state-active")){
                    this.currentDetail = 0;
                }else if($(".targetssubtab").closest("li").hasClass("ui-tabs-active ui-state-active")){
                    this.currentDetail = 1;
                }else if($(".networksubtab").closest("li").hasClass("ui-tabs-active ui-state-active")){
                    this.currentDetail = 2;
                }else{
                    this.currentDetail = $scope.currentDetail;
                }
                
                this.refreshInterval = setTimeout($.proxy(this.refresh, this), interval);
                return this;
            };

            Appliance.prototype.refresh = function(fn, updateDetails) {
                var self = this;

                if (!$rootScope.maintenance_mode) {
                    if (this.isActive || $scope.listViewFlag) {
                        fn = fn || this.updateAppliances;
                        updateDetails = !(updateDetails === false);

                        this.applianceProcess = this.fetchAppliances(fn);
                        this.detailProcess = this.fetchDetails(updateDetails);

                        $.when(this.applianceProcess, this.detailProcess).done(function () {
                            self.applianceProcess = null;
                            self.detailProcess = null;
                            if (!self.selectedID && self.appliances.length) self.ApplianceTable.selectByID(self.appliances[0].id);
                            self.setRefreshInterval();
                        });
                    }
                } else {
                    console.log("In maintenance mode: Skipping appliance refresh.....")
                }
            };

            Appliance.prototype.fetchAppliances = function(fn) {
                if(!_.isFunction(fn)) {
                    fn = this.buildAppliances;
                }
                //this.ApplianceTable.showLoadingIndicator();

                return PHD.Ajax
                    //lmc
                    .get("/api/systems/?timestamp=" + this.timestamp)
                    .done($.proxy(fn, this));
            };

            Appliance.prototype.fetchDetails = function(update) {
                var fn;
                this.selectedID = $scope.selectedID;
                
                update = (update === true);

                if (!this.selectedID) {
                    return $.Deferred().resolve();
                }

                if(this.currentDetail === STORAGE_TAB) {
                    if(update === true) {
                        fn = this.updateBDS;
                    } else {
                        fn = this.buildBDS;
                    }
                    console.log('in detail storage, id is ' + this.selectedID);
                    return PHD.Ajax
                        .get("/api/storage/?sid=" + this.selectedID)
                        .done($.proxy(fn, this))
                        .always($.proxy(this.hideLoaders, this));
                } else if(this.currentDetail === NETWORK_TAB) {
                    console.log('in detail network, id is ' + this.selectedID);
                    //this.NetworkTable.showLoadingIndicator();
                    if(update === true) {
                        //fn = this.updateNetwork; TODO - API does not return sid.
                        fn = this.buildNetwork;
                    } else {
                        fn = this.buildNetwork;
                    }

                    return PHD.Ajax
                        .get("/api/networks/?sid=" + this.selectedID)
                        .done($.proxy(fn, this))
                        .always($.proxy(this.hideLoaders, this));
                    
                } else if(this.currentDetail === TARGETS_TAB || $scope.currentDetail  === TARGETS_TAB){
                    if(update === true) {
                        fn = this.buildReplicationTarget;  // this.updateReplicationTarget;
                    } else {
                        fn = this.buildReplicationTarget;
                    }

                    /*return PHD.Ajax
                        .get("/api/replication/targets/?sid=" + this.selectedID)
                        .done($.proxy(fn, this))
                        .always($.proxy(this.hideLoaders, this));*/
                    
                    $.ajax({
                    url: "app/configure/appliances/replicationDataTemp.json",
                    success: function(data) {
                        if( data && ((data.targets[0].id) == $scope.selectedID)){
                            this.replicationData = data.targets[0].targets;
                        }else{
                            this.replicationData = null;
                        }

                        if(this.ReplicationTable == undefined){
                            this.ReplicationTable = $scope.ReplicationTableObj;
                        }

                        if(this.ReplicationTableList == undefined){
                             this.ReplicationTableList = $scope.ReplicationTableListObj;
                        }

                        this.ReplicationTable.load(this.replicationData);
                        this.ReplicationTableList.load(this.replicationData);
                        //this.updateButtons();

                        if(this.detailProcess) this.detailProcess.resolve();
                        
                    }})
                }
            };
              
            $scope.applianceDataArr = [];
            $scope.selectedAppliance; 
            Appliance.prototype.buildAppliances = function(data) {
                if(data) {
                    this.appliances = data.appliance;
                    this.timestamp = data.timestamp;
                    this.ApplianceTable.load(this.appliances);
                    
                    $scope.applianceDataArr = data.appliance;
                    $scope.selectedAppliance = $scope.applianceDataArr[0];
                    $("#" + ($scope.applianceDataArr[0].id)).css("background-color","#f5f5f5");
                }
                if(this.applianceProcess) this.applianceProcess.resolve();
            };

            Appliance.prototype.updateAppliances = function(data) {
                if(data) {
                    this.appliances = data.appliance;
                    this.timestamp = data.timestamp;
                    //this.ApplianceTable.update(this.appliances);
                }
                if(this.applianceProcess) this.applianceProcess.resolve();
            };

            Appliance.prototype.buildBDS = function(data) {
                //if(data && data.id === this.selectedID) {
                //lmc
                //if(data && data.storage && data.storage.sid === this.selectedID) {
                if (data && data.storage) {
                    this.storageData = data.storage;
                    
                    if(this.StorageTable == undefined){
                        this.StorageTable = $scope.StorageTalbeObj;
                    }

                    if(this.StorageTableList == undefined){
                        this.StorageTableList = $scope.StorageTableListObj;
                    }
                   
                    this.StorageTable.load(this.storageData);
                    this.StorageTableList.load(this.storageData);
                    this.updateButtons();
                }
                if(this.detailProcess) this.detailProcess.resolve();
            };

            Appliance.prototype.updateBDS = function(data) {
                if(data && data.storage != undefined && data.storage[0].sid  === this.selectedID) {
                    this.storageData = data.storage;

                    if(this.StorageTable == undefined){
                        this.StorageTable = $scope.StorageTalbeObj;
                    }

                    if(this.StorageTableList == undefined){
                         this.StorageTableList = $scope.StorageTableListObj;
                    }

                     this.StorageTable.update(this.storageData, true);
                    this.StorageTableList.update(this.storageData, true);
                    this.updateButtons();
                }
                if(this.detailProcess) this.detailProcess.resolve();
            };

            Appliance.prototype.removeBDS = function(data, dbid) {
                this.StorageTableList.deselect();
                this.StorageTable.deselect();
                this.$selectedStorage = $();
                this.selectedStorageID = null;
                this.fetchDetails(true);
                this.updateButtons();
            };

            Appliance.prototype.buildNetwork = function(data) {
                //if(data && data.id === this.selectedID) {
                // lmc - know result is of specified system
                if (data) {
                    this.networkdata = data.networks;

                    if(this.NetworkTable == undefined){
                        this.NetworkTable = $scope.NetworkTableObj;
                    }

                    if(this.NetworkTableList == undefined){
                         this.NetworkTableList = $scope.NetworkTableListObj;
                    }
                    
                    this.NetworkTable.load(this.networkdata);
                    this.NetworkTableList.load(this.networkdata);
                    this.updateButtons();
                }
                if(this.detailProcess) this.detailProcess.resolve();
            };

            Appliance.prototype.updateNetwork = function(data) {
                if(data && data.id === this.selectedID) {
                    this.networkdata = data.networks;

                    if(this.NetworkTable == undefined){
                        this.NetworkTable = $scope.NetworkTableObj;
                    }

                    if(this.NetworkTableList == undefined){
                         this.NetworkTableList = $scope.NetworkTableListObj;
                    }

                    this.NetworkTable.update(this.networkdata, true);
                    this.NetworkTableList.update(this.networkdata, true);
                    this.updateButtons();
                }
                if(this.detailProcess) this.detailProcess.resolve();
            };
            
            Appliance.prototype.buildReplicationTarget = function(data) {
                if (data && data.targets) {
                    this.replicationData = data.targets.targets;

                    if(this.ReplicationTable == undefined){
                        this.ReplicationTable = $scope.ReplicationTableObj;
                    }

                    if(this.ReplicationTableList == undefined){
                         this.ReplicationTableList = $scope.ReplicationTableListObj;
                    }
                    
                    this.ReplicationTable.load(this.replicationData);
                    this.ReplicationTableList.load(this.replicationData);
                    this.updateButtons();
                }
                if(this.detailProcess) this.detailProcess.resolve();
            };
            
            Appliance.prototype.updateReplicationTarget = function(data) {
                if(data && data.id === this.selectedID) {
                     this.replicationData = data.targets.targets;

                    if(this.ReplicationTable == undefined){
                        this.ReplicationTable = $scope.ReplicationTableObj;
                    }

                    if(this.ReplicationTableList == undefined){
                         this.ReplicationTableList = $scope.ReplicationTableListObj;
                    }

                    this.ReplicationTable.update(this.replicationData, true);
                    this.ReplicationTableList.update(this.replicationData, true);
                    this.updateButtons();
                }
                if(this.detailProcess) this.detailProcess.resolve();
            };

            Appliance.prototype.isReady = function() {
                return this.isActive && (this.applianceProcess === null || this.applianceProcess.state() === 'resolved') &&
                    (this.detailProcess === null || this.detailProcess.state() === 'resolved');
            };

            Appliance.prototype.hasStorage = function(type) {
                if(type) {
                    if($scope.tableView){
                        return (this.$storageTable.find(".bds-type-" + type).not(".bds-type-replication").length > 0);    
                    }else{
                        return (this.$storageTableList.find(".bds-type-" + type).not(".bds-type-replication").length > 0);    
                    }
                } else {
                    if($scope.tableView){
                        return (this.$storageTable.find(".clickable").not(".bds-type-replication, .bds-type-ir-cache").length > 0);
                    }else{
                        return (this.$storageTableList.find(".clickable").not(".bds-type-replication, .bds-type-ir-cache").length > 0);
                    }
                }
            };

            Appliance.prototype.updateButtons = function() {
                if(this.$buttons == undefined){
                    this.$buttons = $scope.buttons;
                }
                
                this.$buttons.filter(".btn-edit-storage")
                    .toggleButton(this.selectedID && this.selectedStorageID && !this.$selectedStorage.is(".sync-status-pending"));
                this.$buttons.filter(".btn-remove-storage")
                    .toggleButton(this.selectedID && this.selectedStorageID && !this.$selectedStorage.is(".sync-status-pending"));
                this.$buttons.filter(".btn-edit-nic").toggleButton(this.selectedID && this.selectedNicID);

                if((typeof($scope.selectedReplicationTarget) != "undefined") && (($scope.selectedReplicationTarget) != null)){
                    if($scope.selectedReplicationTarget.type == "managed_cloud"){
                        this.$buttons.filter(".btn-edit-target").disableButton();
                        this.$buttons.filter(".btn-remove-target").toggleButton(this.selectedID && this.selectedReplicationTargetID);
                        this.$buttons.filter(".btn-onoff-target").disableButton();
                        this.$buttons.filter(".btn-prepare-target").disableButton();
                        
                    }else{
                        this.$buttons.filter(".btn-edit-target").toggleButton(this.selectedID && this.selectedReplicationTargetID);
                        this.$buttons.filter(".btn-remove-target").toggleButton(this.selectedID && this.selectedReplicationTargetID);
                        this.$buttons.filter(".btn-onoff-target").toggleButton(this.selectedID && this.selectedReplicationTargetID);
                        this.$buttons.filter(".btn-prepare-target").toggleButton(this.selectedID && this.selectedReplicationTargetID);
                    }
                }else{
                    this.$buttons.filter(".btn-edit-target").toggleButton(this.selectedID && this.selectedReplicationTargetID);
                    this.$buttons.filter(".btn-remove-target").toggleButton(this.selectedID && this.selectedReplicationTargetID);
                    this.$buttons.filter(".btn-onoff-target").toggleButton(this.selectedID && this.selectedReplicationTargetID);
                    this.$buttons.filter(".btn-prepare-target").toggleButton(this.selectedID && this.selectedReplicationTargetID);
                }

                // check for storage status.
                $enableDisable = this.$buttons.filter(".btn-onoff-storage");
                if (this.selectedStorageID) {
                    var dbid = this.selectedStorageID;
                    var storage = _.findWhere(this.storageData, {"id": dbid});
                    if (storage != null && storage.status === 'online') {
                        $enableDisable.html('<i class="icon-off icon-large"></i> Disable');
                    } else {
                        $enableDisable.html('<i class="icon-restart icon-large"></i> Enable');
                    }
                }
                return this;
            };

            Appliance.prototype.setButtonLock = function(locked) {
                this.$buttons.each(function() {
                    $(this).toggleButton(!locked);
                });
            };

            Appliance.prototype.initAppliances = function() {
                this.refresh(this.buildAppliances);
                this.appliancesInit = true;
                return this;
            };

            Appliance.prototype.addVBA = function() {
                this.refresh(this.updateAppliances);
            };

            Appliance.prototype.hideLoaders = function() {
                if(this.currentDetail === STORAGE_TAB) {
                    //this.StorageTable.hideLoadingIndicator();
                } else if(this.currentDetail === NETWORK_TAB) {
                    //this.NetworkTable.hideLoadingIndicator();
                }
            };

            Appliance.prototype.kill = function() {
                var self = this,
                    resp = PHD.Ajax.delete("/api/systems/" + this.selectedID);

                resp.done(function(data) {
                    console.log("in done with DELETE!");
                    //$(document).trigger("systemremove", [self.selectedID]);
                    self.selectedID = null;
                    PHD.appliance_sid = self.selectedID;
                    self.refresh(self.fetchAppliances);
                    /*if(data.result[0].code == AJAX_RESULT_SUCCESS) {
                        $(document).trigger("systemremove", [self.dbid]);
                    } else {
                        PHD.throwError(data);
                    }*/
                });
            };

            Appliance.prototype.activate = function() {
                this.isActive = true;
                if(!this.appliancesInit) {
                    this.initAppliances();
                } else {
                    this.refresh(this.updateAppliances);
                }
                return this;
            };

            Appliance.prototype.deactivate = function() {
                this.isActive = false;
                clearInterval(this.refreshInterval);
                return this;
            };
            
            var ApplianceTab = function(index, $panel) {
                this.index = index;
                this.$panel = $panel;
                this.$applianceArea = this.$panel.children("#appliances-list");
                this.environmentList = {};
                this.environments = {};
                this.activeEnvironment = null;
                this.isActive = false;
                this.isBootstrapped = false;
            };

            ApplianceTab.prototype.fetchEnvironments = function() {
                this.bootstrap();
            };

            ApplianceTab.prototype.bootstrap = function(data) {

                this.$environments = this.$applianceArea.find(".environment");

                //this.environmentList = this.$environments.map(function() {
               //     return new Appliance($(this)).init();
                //}).get();

                this.environmentList = new Appliance(this.$environments).init();
                this.activeEnvironment = this.environmentList.activate();
                this.isBootstrapped = true;
            };

            ApplianceTab.prototype.init = function() {
                var self = this;
                $(document)
                    .on("systemremove", function(event, dbid) {

                        //self.removeEnvironment(dbid);
                    });

                return this;
            };

            ApplianceTab.prototype.activate = function() {
                this.isActive = true;
                if(!this.isBootstrapped) {
                    this.fetchEnvironments();
                }
                if(this.activeEnvironment) {
                    this.activeEnvironment.activate();
                }
                return this;
            };

            ApplianceTab.prototype.deactivate = function() {
                this.isActive = false;
                if(this.activeEnvironment) {
                    this.activeEnvironment.deactivate();
                }
                return this;
            };

          
            
            function getHTMLSpec(type, protocol) {
                var spec = {};
                spec.width = 1024;
                spec.height = 700;
                spec.htmlFile = 'attached/attached.html';

                if (type == 'added_disk') {
                    ;
                } else if (type == 'nas') {
                    if (protocol == 'cifs') {
                        spec.htmlFile = 'cifs/cifs.html';
                        spec.width = 500;
                        spec.height = 480;
                    } else if (protocol == 'nfs') {
                        spec.htmlFile = 'nfs/nfs.html';
                        spec.width = 500;
                        spec.height = 480;
                    } else {
                        // TODO - cloudhook will go here.
                    }
                } else if (type == 'ir') {
                    ; // TODO - need to specify IR storage path here.
                    spec.height = 520;
                }

                return spec;
            }
            
             Appliance.prototype.refresh_replicationTarget = function(){
				var self = this;
				self.fetchDetails(true);
             }
            Appliance.prototype.setSelectedAppliance = function(selectedId){
                var self = this;
                $scope.listViewFlag = true;
                
                self.ApplianceTable = $scope.ApplianceTable;
                
                self.currentDetail = $scope.currentDetail;
                self.$selectedAppliance = $scope.el;
                self.$selectedStorage = $();
                self.selectedStorageID = null;
                self.$selectedNic = $();
                self.selectedNicID = null;
                self.$selectedReplicationTarget = $();
                self.selectedReplicationTargetID = null;
                
                self.selectedID = selectedId;
                PHD.appliance_sid = selectedId;
                
                self.refresh(this.updateAppliances, false);
               // self.updateButtons();

                var x = this.ApplianceTable.$table[0].rows;
                for (var i = 1; i < x.length; i++) {
                    
                    var rowDetails = x[i];
                    
                    if((parseInt(x[i].dataset.dbid)) == selectedId){
                        if(!($(x[i]).hasClass("current-row"))){
                            $(x[i]).addClass("current-row");
                            $scope.selectedID = selectedId;
                            this.$selectedAppliance = $(x[i]);
                            this.selectedID = selectedId;
                        }
                        
                    }else{
                        if($(x[i]).hasClass("clickable current-row")){
                            $(x[i]).removeClass("current-row");
                        }
                    }
                }
                
                this.ApplianceTable.$table[0].rows = x;
                self.updateButtons();
            }
            
            
            $scope.selectApplianceFromList = function(selectedId){
                var arrLength=($scope.applianceDataArr).length;

                for (var i=0; i < arrLength; i++){
                    if ($scope.applianceDataArr[i].id===selectedId){
                       $scope.selectedAppliance = $scope.applianceDataArr[i];
                       $("#" + selectedId).css("background-color","#f5f5f5");
                       $scope.selectedID = selectedId;
                    }else{
                        $("#" + $scope.applianceDataArr[i].id).css("background-color","#fff");
                    }
                }
                
                Appliance.prototype.setSelectedAppliance(selectedId);
            };
            
            $(".storagesubtab").closest("li").click(function(){
                this.currentDetail = 0;
                $scope.setApplianceStorageTab();
                Appliance.prototype.setSelectedAppliance($scope.selectedID);
            });
            
            $(".networksubtab").closest("li").click(function(){
                this.currentDetail = 2;
                $scope.setApplianceNetworkTab();
                Appliance.prototype.setSelectedAppliance($scope.selectedID);
            });
            
             $(".targetssubtab").closest("li").click(function(){
                this.currentDetail = 1;
                $scope.setApplianceTargetsTab();
                Appliance.prototype.setSelectedAppliance($scope.selectedID);
            });
            
            $scope.setApplianceStorageTab = function(){
                $scope.currentDetail = STORAGE_TAB;
                $(".storagesubtab").closest("li").addClass("ui-tabs-active ui-state-active");
                $(".targetssubtab").closest("li").removeClass("ui-tabs-active ui-state-active");
                $(".networksubtab").closest("li").removeClass("ui-tabs-active ui-state-active");

                $("#tableStorageTab").closest("li").attr("aria-selected", true);
                $("#listStorageTab").closest("li").attr("aria-selected", true);
                $("#tableStorageTab").closest("li").attr("aria-expanded", true);
                $("#listStorageTab").closest("li").attr("aria-expanded", true);

                $("#tableTargetsTab").closest("li").attr("aria-selected", false);
                $("#listTargetsTab").closest("li").attr("aria-selected", false);
                $("#tableTargetsTab").closest("li").attr("aria-expanded", false);
                $("#listTargetsTab").closest("li").attr("aria-expanded", false);

                $("#tableNetworkTab").closest("li").attr("aria-selected", false);
                $("#listNetworkTab").closest("li").attr("aria-selected", false);
                $("#tableNetworkTab").closest("li").attr("aria-expanded", false);
                $("#listNetworkTab").closest("li").attr("aria-expanded", false);

                $(".config-storage").attr("aria-hidden", false);
                $(".config-targets").attr("aria-hidden", true);
                $(".config-network").attr("aria-hidden", true);

                $(".config-storage").css("display","block");
                $(".config-targets").css("display","none");
                $(".config-network").css("display","none");
            };

            $scope.setApplianceTargetsTab = function(){
                $scope.currentDetail = TARGETS_TAB;
                $(".storagesubtab").closest("li").removeClass("ui-tabs-active ui-state-active");
                $(".targetssubtab").closest("li").addClass("ui-tabs-active ui-state-active");
                $(".networksubtab").closest("li").removeClass("ui-tabs-active ui-state-active");

                $("#tableStorageTab").closest("li").attr("aria-selected", false);
                $("#listStorageTab").closest("li").attr("aria-selected", false);
                $("#tableStorageTab").closest("li").attr("aria-expanded", false);
                $("#listStorageTab").closest("li").attr("aria-expanded", false);

                $("#tableTargetsTab").closest("li").attr("aria-selected", true);
                $("#listTargetsTab").closest("li").attr("aria-selected", true);
                $("#tableTargetsTab").closest("li").attr("aria-expanded", true);
                $("#listTargetsTab").closest("li").attr("aria-expanded", true);

                $("#tableNetworkTab").closest("li").attr("aria-selected", false);
                $("#listNetworkTab").closest("li").attr("aria-selected", false);
                $("#tableNetworkTab").closest("li").attr("aria-expanded", false);
                $("#listNetworkTab").closest("li").attr("aria-expanded", false);

                $(".config-storage").attr("aria-hidden", true);
                $(".config-targets").attr("aria-hidden", false);
                $(".config-network").attr("aria-hidden", true);

                $(".config-storage").css("display","none");
                $(".config-targets").css("display","block");
                $(".config-network").css("display","none");
            };

            $scope.setApplianceNetworkTab = function(){
                $scope.currentDetail = NETWORK_TAB;
                $(".storagesubtab").closest("li").removeClass("ui-tabs-active ui-state-active");
                $(".targetssubtab").closest("li").removeClass("ui-tabs-active ui-state-active");
                $(".networksubtab").closest("li").addClass("ui-tabs-active ui-state-active");

                $("#tableStorageTab").closest("li").attr("aria-selected", false);
                $("#listStorageTab").closest("li").attr("aria-selected", false);
                $("#tableStorageTab").closest("li").attr("aria-expanded", false);
                $("#listStorageTab").closest("li").attr("aria-expanded", false);

                $("#tableTargetsTab").closest("li").attr("aria-selected", false);
                $("#listTargetsTab").closest("li").attr("aria-selected", false);
                $("#tableTargetsTab").closest("li").attr("aria-expanded", false);
                $("#listTargetsTab").closest("li").attr("aria-expanded", false);

                $("#tableNetworkTab").closest("li").attr("aria-selected", true);
                $("#listNetworkTab").closest("li").attr("aria-selected", true);
                $("#tableNetworkTab").closest("li").attr("aria-expanded", true);
                $("#listNetworkTab").closest("li").attr("aria-expanded", true);

                $(".config-storage").attr("aria-hidden", true);
                $(".config-targets").attr("aria-hidden", true);
                $(".config-network").attr("aria-hidden", false);

                $(".config-storage").css("display","none");
                $(".config-targets").css("display","none");
                $(".config-network").css("display","block");
            };
            
            $scope.replicationPendingApproval = function(){
                ngDialog.open({
                    dialogType:'Information',
                    scope:this,
                    dialogMessage:"Replication pending approval sucessfully"
                })
            };

            $scope.setApplianceNetworkTab = function() {
                $scope.currentDetail = NETWORK_TAB;
                $(".storagesubtab").closest("li").removeClass("ui-tabs-active ui-state-active");
                $(".targetssubtab").closest("li").removeClass("ui-tabs-active ui-state-active");
                $(".networksubtab").closest("li").addClass("ui-tabs-active ui-state-active");
            }
			$scope.refreshReplicationTarget = function(){
				Appliance.prototype.refresh_replicationTarget();
			}
            PHD.ApplianceTab = ApplianceTab;
        }
		/*
		*openReplicationBandwithDailog : opens dailog-box for throttle schedule*
		*/
		$scope.openReplicationBandwithDailog = function(){
            
                ngDialog.open({
                    template: 'app/configure/appliances/replicationBandwidthScheduleTemplate.html',
                    scope: $scope,
                    overlay:false,
                    ngDialogStyle:'width:650px;',
                    modelDialogId:'replication-bandwidth-dailog',
                    preCloseCallback: function (value) {
                        
                    }
                })
        }
        /**************************************** Configure replication Target**********************************************/
	
		$scope.addressFormData = {};
		$scope.replicationTargetData = {};
		var dataObject = {};		
		$scope.formName = {}
	    $scope.replicationTargetData.radioBtnSelection = "2";
		$scope.expandText = false;
		
		$(".btn-replication-unitrends-cloud").click(function(){
			 ngDialog.open({
                            template: 'app/configure/appliances/addUnitrendsCloudTarget.html',
                            scope: $scope,
                            overlay:true,
                            modelDialogId:'replication-configureCloudTarget-dailog',
							ngDialogStyle:'width:555px;',
                            preCloseCallback: function (value) {
							 $scope.setDefaultValOfUnitrendsCloudDialog();
						  }
            });
			
		});
		
		$( ".btn-replication-unitrends-appliance" ).click(function(){
		    ngDialog.open({
                            template: 'app/configure/appliances/configure-target-wizard/replication-wizard.html',
                            scope: $scope,
                            overlay:true,
                            modelDialogId:'replication-configureTarget-dailog',
							ngDialogStyle:'width:590px; height:420px;',
                            preCloseCallback: function (value) {
							  $scope.addressFormData.replicationTarget_cloudAdress = "";
							  $scope.addressFormData.replicationTarget_hostname = "";
							  $scope.replicationTargetData.radioBtnSelection="2";
							  if(!angular.isUndefined($scope.formName.addressForm.replicationTarget_cloudAdress)){
                                  $scope.formName.addressForm.replicationTarget_cloudAdress.$dirty = false;
                              }
							  if(!angular.isUndefined($scope.formName.addressForm.replicationTarget_hostname)){
                                  $scope.formName.addressForm.replicationTarget_hostname.$dirty = false;
                              }
								//  $scope.refreshReplicationTarget(); // TODO:for refresh back appliance data table.
						    }
            });
			
		});
		
		 /**********function to set default value of unitrends cloud dialog componets ************/
		$scope.setDefaultValOfUnitrendsCloudDialog = function(){
			
							  $scope.addressFormData.replicationTarget_cloudAdress = "";
							  $scope.addressFormData.replicationTarget_IPAddress = "";
							  $scope.addressFormData.replicationTarget_port = "";
							  $scope.addressFormData.replicationTarget_activationCode = "";
							  $scope.expandText = false;
							
							  if(!angular.isUndefined($scope.formName.addressForm.replicationTarget_cloudAdress)){
                                  $scope.formName.addressForm.replicationTarget_cloudAdress.$dirty = false;
                              }
							  if(!angular.isUndefined($scope.formName.addressForm.replicationTarget_IPAddress)){
                                  $scope.formName.addressForm.replicationTarget_IPAddress.$dirty = false;
                              }
							  if(!angular.isUndefined($scope.formName.addressForm.replicationTarget_port)){
                                  $scope.formName.addressForm.replicationTarget_port.$dirty = false;
                              }
							  if(!angular.isUndefined($scope.formName.addressForm.replicationTarget_activationCode)){
                                  $scope.formName.addressForm.replicationTarget_activationCode.$dirty = false;
                              } 		
			
		}
		
		/**********function to show popover on clcik of unitrends cloud dialog help icon************/
		$scope.showPopover = function(){
		  var form1 = angular.element("#unitrendsCloudForm");
			console.log(form1)
	   	   form1.find(".definition").popover({
				appendTo: form1
			});

		}

		/***********function for toggle unitrends cloud account text**************/
		$scope.expandAcctText = function(){
			$scope.expandText = !$scope.expandText;
			if($scope.expandText){
				$("#acctTextId").removeClass("reportExpand");
				$("#acctTextId").addClass("reportExpanded");
			}else{
			    $("#acctTextId").removeClass("reportExpanded");
				$("#acctTextId").addClass("reportExpand");
			}
			
		}
		
		/***********function to post unitrends cloud target**************/
		
		 $scope.addUnitrendsCloundTarget = function(){
			     var targetName ={};
			     var params = {};
			 
				if (
					 $scope.formName.addressForm.replicationTarget_IPAddress.$invalid ||
					 $scope.formName.addressForm.replicationTarget_port.$invalid ||
					 $scope.formName.addressForm.replicationTarget_activationCode.$invalid 
					){
                    return;

				 }
			 
			    targetName.target = $scope.addressFormData.replicationTarget_cloudAdress;
			    params.sid = $rootScope.replicationApplianceId;
			  
			  
	           var url = "/api/replication/target/";
			    $http({
					method: 'post',
					params:params,
					url :url ,
					data: JSON.stringify( targetName)
				}).success(function(data, status, headers){
				    $scope.refreshReplicationTarget();	// TODO:for refresh back appliance data table.
				    $scope.addReplicationTargetNotification();
					//ngDialog.close('replication-configureCloudTarget-dailog');
        		  
				}).error(function(response) { 
					 ngDialog.open({
                     dialogType:'retry',
					 modelDialogId:'replication-dailog',
                     dialogMessage:response.result[0].message
                    })
				});
            
			 }
		 
		 /***********function to post unitrends appliance target**************/
		 $scope.nextWizard = function($event){
                   var url = "";
			       $scope.applianceGridData = [ ];
			  	   var params = {};
			 
                   if(wizardTitle == "Configure Target Type"){
					   if($scope.replicationTargetData.radioBtnSelection  == 2){
							$http({
								  method:'get',
								  url:"/api/systems/",
							}).success(function(data, status, headers){

								for(var i=0; i<data.appliance.length;i++){
									if(data.appliance[i].id != $rootScope.replicationApplianceId){
										$scope.applianceGridData.push(data.appliance[i]);
									}
								}
								if($scope.applianceGridData.length == 0) $('#saveWizardBtn').attr('disabled','disabled');
								 $rootScope.$broadcast("nextBtnClick"); 
							}).error(function() { 
								 window.PHD.throwError({"result": [{"message": gettext(response.result[0].message)}]});
							});
                       	   
					     }else{
							  $('#saveWizardBtn').removeAttr('disabled');
							  $rootScope.$broadcast("nextBtnClick"); 	
						 }
                  }
			      if(wizardTitle == "Configure Target"){
					  
                       if ($scope.formName.addressForm.replicationTarget_hostname.$invalid && isApplianceNotSelected) {
					              return;
                        }

                        var applianceAdressDataObj ={};

                        if($scope.replicationTargetData.radioBtnSelection  == 3){
                             applianceAdressDataObj.target = $scope.addressFormData.replicationTarget_hostname;
                        }else{
							
                             applianceAdressDataObj.target = $scope.selectedIDs.toString();
                        }		
					  	console.log(applianceAdressDataObj.target);
					    url =  "/api/replication/target/";
					    params.sid = $rootScope.replicationApplianceId;
					  
                        dataObject = {
                                        method:'put',
                                        url:url,
                                        wizardFormData:applianceAdressDataObj,
                                        params:params
                                    };
                        wizardService.nextWizard(dataObject);
					    ngDialog.close('replication-configureTarget-dailog'); // to close the dialog on save button click.
					    isApplianceNotSelected = true;
			     }
		}
   
	/*** for appliance Grid***/
		$scope.mySelections = [];
		var isApplianceNotSelected = true;
		$scope.applianceGridData =[];
		$scope.replicationGridOptions = {   data: 'applianceGridData',
											selectedItems: $scope.mySelections,
											columnDefs:[{field: '',width: '10%',cellTemplate: '<div class="ngSelectionCell"><input tabindex="-1" class="ngSelectionCheckbox"                                                            													 type="checkbox" ng-checked="row.selected" /> </div>'},
													   {field:'name',displayName:'Name',width:'30%'},
													   {field:'host',displayName:'IP Adress',width:'30%'},
													   {field:'total_mb_free',displayName:'Avilable Storage',width:'30%'}],
											   afterSelectionChange:function(){
												$scope.selectedIDs = [];
												angular.forEach($scope.mySelections, function ( item ) {
													$scope.selectedIDs.push( item.name )
												});
												console.log("selected::::::"+$scope.selectedIDs);
												if($scope.selectedIDs.length>0){
												  isApplianceNotSelected = false;
												}
											}			   
										};
	
	
		/*** for radio button***/
		$scope.selectionChange = function(radioBtnSelection){
			 $scope.replicationTargetData.radioBtnSelection = radioBtnSelection;
		}
		/*** nested function on success of wizard backend save call***/
		$rootScope.$on('nestedFunction', function (event){
			 $scope.refreshReplicationTarget();	// TODO:for refresh back appliance data table.
		});
		
		$scope.addReplicationTargetNotification = function(){
                ngDialog.open({
                    dialogType:'Information',
                    dialogMessage:"Task to add Unitrends Cloud started and its status will get updated in Backup Cloud Targets Tab."
                })
         };
    }]);