(function($, PHD, window, undefined) {
    
    var $el = $("#credentials-dialog");
    var $credentialsTable;
    var applianceSelect;
    var CredentialsTable;
    var allCredentials;
    var appliances;
    var credentialRowData = null;
    var base_url = "/api/credentials/";
    var credentialEdit = false;

    function initForm() {

        function refreshCredentialList() {
            var systems = PHD.Ajax.get('/api/systems');
            systems.done(function(data) {
                allCredentials = [];
                appliances = data.appliance;
                appliances.forEach(function(system, index) {
                    var resp = PHD.Ajax.get(base_url + "?sid=" + system.id);
                    resp.done(function(data){
                        var credentialList = data.data;
                        credentialList.forEach(function(item, index) {
                            item.id = item.credential_id;
                            item.system_id = system.id;
                            item.system_name = system.name;
                        });
                        allCredentials = $.merge(allCredentials, credentialList);
                        CredentialsTable.load(allCredentials);
                    });
                });
            });
        }

        // Initialize table, appliance list
        $credentialsTable = $el.find(".credentials-table");
        CredentialsTable = PHD.DataTable($credentialsTable, {});
        applianceSelect = $("#id_appliance_list")[0];
        refreshCredentialList();

        $credentialsTable.on("rowselect", "tr", function(event, dbid, $el, DataTable, DataRow) {
            credentialRowData = DataRow.data;
        });


        $(".credentials-add").click(function() {
            credentialEdit = false;
            console.log("add clicked");
            $add = $(".modal-form-no-box");
            $add.toggleClass("hidden", false);
            $("#id_appliance_list").empty();
            applianceSelect.options.length = 0;
            if (appliances.length > 0) {
                applianceSelect.options[0] = new Option('Please select... ');
                appliances.forEach(function(system, index) {
                    applianceSelect.options[index + 1] = new Option(system.name, system.id);
                })
            }
            $("#id_display_name").val("");
            $("#id_username").val("");
            $("#id_password").val("");
            $("#id_domain").val("");
            $("#id_default").prop('checked', false);
        });

        $(".credentials-edit").click(function() {
            credentialEdit = true;
            console.log("edit clicked, id selected is " + CredentialsTable.selectedID);
            if (CredentialsTable.selectedID != null) {
                $edit = $(".modal-form-no-box");
                $edit.toggleClass("hidden", false);
                $("#id_appliance_list").empty();
                applianceSelect.options.length = 0;
                appliances.forEach(function(system, index) {
                    if (credentialRowData.system_id == system.id) {
                        applianceSelect.options[0] = new Option(system.name, system.id);
                    }
                });
                $("#id_display_name").val(credentialRowData.display_name);
                $("#id_username").val(credentialRowData.username);
                $("#id_domain").val(credentialRowData.domain);
                $("#id_default").prop('checked', credentialRowData.is_default);
            }
        });

        $(".credentials-delete").click(function() {
            credentialEdit = false;
            if (CredentialsTable.selectedID != null) {
                var id_to_delete = CredentialsTable.selectedID;
                PHD.confirmDialog("<p>" + gettext("Are you sure you want to delete the selected credential?") + "</p>", {
                    title: gettext("Confirm Credential Deletion")
                })
                    .done(function() {
                        var url = base_url + id_to_delete + "?sid=" + credentialRowData.system_id;
                        PHD.Ajax
                            .delete(url)
                            .done(function (data) {
                                CredentialsTable.deleteRow(id_to_delete);
                            });
                    }
                )
            }
        });

        $(".btn-pref-save").click(function() {
            var obj = {};
            var display_name = $("#id_display_name").val();
            if (display_name != "") {
                obj.display_name = display_name;
            }
            obj.username = $("#id_username").val();
            var password = $("#id_password").val();
            if (password != "") {
                obj.password = password;
            }
            var domain = $("#id_domain").val();
            obj.domain = domain;

            obj.is_default = $("#id_default").prop("checked");
            var resp;
            var url = base_url;
            if (credentialEdit) {
                url += CredentialsTable.selectedID + "/?sid=" + credentialRowData.system_id;
                obj = JSON.stringify(obj);
                resp = PHD.Ajax.put(url, obj);
            } else {
                $selected = $("#id_appliance_list option:selected");
                console.log("selected html " + $selected.html());
                console.log("selected val " + $selected.val());
                var sid = $("#id_appliance_list option:selected").val();
                url += "?sid=" + sid;
                obj = JSON.stringify(obj);
                resp = PHD.Ajax.post(url, obj);
            }
            resp.done(function(data) {
                $add = $(".modal-form-no-box");
                $add.toggleClass("hidden", true);
                refreshCredentialList();
            });

        });

        $(".btn-pref-cancel").click(function() {
            credentialEdit = false;
            $add = $(".modal-form-no-box");
            $add.toggleClass("hidden", true);
        });

    }

    /*
    *   DOM Ready Function
    */
    $(function() {
        PHD.currentMiniWizard.one("dialogfocus", function() {
            initForm();
        });

    });

})(jQuery, window.PHD || (window.PHD = {}), window);
