(function($, PHD, window, undefined) {
    
    var $form = $("#form-hypervisor"),
        HypervisorForm,
        FQDN_REGEXP = /^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)*([A-Za-z0-9]|[A-Za-z0-9][A-Za-z0-9\-]*[A-Za-z0-9])(\:[0-9]{1,5})?$/i;

    var applianceSelect = $("#id_system")[0];

    function initForm() {

        if(typeof EditDisabled !== 'undefined'){
            if(EditDisabled === true){
                $form.find("#id_server").disable();
                $form.find("#common_dialog_label").hide();
                $("#host_name_tooltip").text("The name of VMware Host cannot be changed.");
                $("#ip_tooltip").text("The IP address of VMware Host.");
                EditDisabled = false;
            }
        }

        $form.find(".definition").popover({
            appendTo: $form,
            position: {
                my: "left center",
                at: "right+10 center",
                collision: "flip fit",
                within: $form
            }
        });

        function refreshSystemList() {
            PHD.appliance_sid = 1; // default to local
            console.log('set appliance id to ' + PHD.appliance_sid);
            var systems = PHD.Ajax.get('/api/systems');
            systems.done(function(data) {
                var appliances = data.appliance.sort(sortIDs);
                appliances.forEach(function(system, index) {
                    console.log("adding system " + system.name);
                    applianceSelect.options[index] = new Option(system.name, system.id);
                });
            });
        }

        function sortIDs(a, b) {
            return (a.id < b.id) ? -1 : ((a.id > b.id) ? 1 : 0);
        }

        HypervisorForm = PHD.FormController($form, {})
            .validate({
                rules: {
                    name: {
                        required: true
                    },
                    "credentials.username": {
                        required: function(element) {
                            return $("#id_os_type").val() == "VMware";
                        }
                    },
                    "credentials.password": {
                        required: function(element) {
                            return $("#id_os_type").val() == "VMware";
                        }
                    },
                    "host_info.ip": {
                        required: false,
                        pattern: FQDN_REGEXP
                    }
                },
                messages: {
                    name: {
                        required: gettext("Server name is required")
                    },
                    "credentials.username": {
                        required: gettext("Username is required")
                    },
                    "credentials.password": {
                        required: gettext("Password is required")
                    },
                    "host_info.ip": {
                        pattern: gettext("IP address is not valid")
                    }
                }
            })
            .on("formcancel", function(event) {
                PHD.currentDialog.wizard("close");
            })
            .on("formsubmit", function(event, data) {
                $(document).trigger("assetChange");
                PHD.currentDialog.wizard("close");

                var scope = angular.element(document).scope();
                if(scope.user.showTour) {
                    if(scope.tourBackupDisabled) {
                        scope.tourBackupDisabled = false;
                    }
                    window.location.href = "#";
                }
            });

        $("#id_system").on("change", function (event) {
            var sid = $("#id_system option:selected").val();
            console.log("set sid to " + sid);
            PHD.appliance_sid = sid;
        });

        refreshSystemList();
        if ($("#edit_object_id").text() !== "none") {
            // on edit cannot modify appliance.
            $("#id_system").prop("disabled", true);
            $("#id_os_type").prop("disabled", true);
        }
    }

    /*
    *   DOM Ready Function
    */
    $(function() {
        PHD.currentDialog.one("wizardfocus", function() {
            initForm();
        });
    });

})(jQuery, window.PHD || (window.PHD = {}), window);
