(function($, PHD, window, undefined) {
    
    var $form = $("#form-server"),
        ServerForm,
        FQDN_REGEXP = /^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)*([A-Za-z0-9]|[A-Za-z0-9][A-Za-z0-9\-]*[A-Za-z0-9])(\:[0-9]{1,5})?$/i;

    var credentials = [];
    var credentialsSelect;
    var $credentialsSelect = $("#id_credentials_list");
    var $credentialsCreate = $form.find(".modal-form-credential");
    var $editCredential = $form.find(".btn-edit-credential");
    var selectedCredentialID = 0;
    var $credentialsInput = $("#id_credentials_username, #id_credentials_password");

    function initForm() {

        $form.find(".definition").popover({
            appendTo: $form,
            position: {
                my: "left center",
                at: "right+10 center",
                collision: "flip fit",
                within: $form
            }
        });

        var resp = PHD.Ajax.get('/api/credentials');
        resp.done(function(data) {
            allCredentials = [];
            credentials = data.data;
            credentials.forEach(function(credential, index) {
                var display =  credential.display_name != undefined ? credential.display_name : "N/A";
                if (credential.is_default) {
                    display += " (Default)";
                }
                credential.display = display;
            });
            credentialsSelect = $credentialsSelect[0];
            credentialsSelect.options.length = 0;
            if (credentials.length > 0) {
                credentialsSelect.options[0] = new Option('None', -1);
                credentials.forEach(function(credential, index) {
                    credentialsSelect.options[index + 1] = new Option(credential.display, credential.credential_id);
                })
            }
            if (selectedCredential > 0) {
                $credentialsSelect.val(selectedCredential);
            }
            $credentialsSelect.on("change", function(event) {
                selectedCredentialID = $credentialsSelect.val();
                console.log("selected = " + selectedCredentialID);
                if (selectedCredentialID == 0) {
                    $credentialsCreate.toggleClass("hidden", true);
                    $editCredential.toggleClass("hidden", true);
                    $("#id_credentials_username").val("");
                } else {
                    $credentialsCreate.toggleClass("hidden", false);
                    $credentialsInput.prop("disabled", true);
                    $editCredential.toggleClass("hidden", false);
                    if (selectedCredentialID != 0) {
                        for (var i = 0; i < credentials.length; i++) {
                            if (selectedCredentialID == credentials[i].credential_id) {
                                $("#id_credentials_username").val(credentials[i].username);
                                break;
                            }
                        }
                    }
                }
            })
        });

        $form.on("click", ".btn-add-credential", function(event) {
                event.preventDefault();
                //if (!$(this).data("disabled")) {
                //$add = $form.find(".modal-form-credential");
                //console.log("add = " + $add.html());
                $credentialsCreate.toggleClass("hidden", false);
                $credentialsInput.prop("disabled", false);
                $("#id_credentials_username").val("");
                $credentialsSelect.val(0);
                //}
            }
        );

        $form.on("click", ".btn-edit-credential", function(event) {
                event.preventDefault();
                $credentialsCreate.toggleClass("hidden", false);
                $credentialsInput.prop("disabled", false);
            }
        );


        ServerForm = PHD.FormController($form, {})
            .validate({
                rules: {
                    name: {
                        required: true
                    },
                    "host_info.ip": {
                        required: false,
                        pattern: FQDN_REGEXP
                    }
                },
                messages: {
                    name: {
                        required: gettext("Server name is required")
                    },
                    "host_info.ip": {
                        pattern: gettext("IP address is not valid")
                    }
                }
            })
            .on("formcancel", function(event) {
                PHD.currentDialog.wizard("close");
            })
            .on("formsubmit", function(event, data) {
                $(document).trigger("assetChange");
                PHD.currentDialog.wizard("close");

                var scope = angular.element(document).scope();
                if(scope.user.showTour) {
                    if(scope.tourBackupDisabled) {
                        scope.tourBackupDisabled = false;
                    }
                    window.location.href = "#";
                }
            });
    }

    /*
    *   DOM Ready Function
    */
    $(function() {
        PHD.currentDialog.one("wizardfocus", function() {
            initForm();
        });
    });

})(jQuery, window.PHD || (window.PHD = {}), window);
