angular.module('configLicense', [])

    .controller('LicenseCtrl', ['$scope', '$rootScope', '$http', function($scope, $rootScope, $http) {

        $scope.initCtrl = function () {
            processLicense(PHD, window);
        }

        function processLicense(PHD, window) {
            var $form = $("#form-license"),
                $btnLaunch = $(".btn-launch-license-submit"),
                $btnSubmit = $(".btn-submit-license"),
                $btnCancel = $(".btn-cancel-submit-license"),
                $btnApply = $(".btn-apply-sockets"),
                detailsTmpl = _.template($("#tmpl-license-details").html()),
                $details = $("#license-details"),
                $hostSocketForm = $("#form-sockets"),
                $hostTable = $("#license-host-table"),
                HostTable,
                $features = $("#license-features"),
                $featureTable = $("#license-feature-table"),
                FeatureTable,
                hypervisors = {},
                multiManager = {"xen": false, "vmware": false, "hyperv": false},
                appliedEnvs = {"xen": [], "vmware": [], "hyperv": []},
                hypervisorTranslate = {"1": "xen", "3": "vmware", "4": "hyperv"}; // TODO: put in constants

            function canAddSocket(hv, env, count) {
                var hypervisor = hypervisorTranslate[hv];
                if (hypervisors[hv] !== undefined) {
                    if (hypervisors[hv].count == LICENSING_UNLIMITED_SOCKETS || hypervisors[hv].count >= hypervisors[hv].applied + count) {
                        if (multiManager[hypervisorTranslate[hv]] || appliedEnvs[hypervisor].length == 0 || appliedEnvs[hypervisor].indexOf(env) != -1) {
                            if (appliedEnvs[hypervisor].indexOf(env) == -1) {
                                appliedEnvs[hypervisor].push(env);
                            }
                            return true;
                        } else {
                            return gettext("You do not have multi-manager capability for this hypervisor.");
                        }
                    } else {
                        return gettext("Maximum number of sockets has been reached. De-select existing hosts or purchase additional licenses.");
                    }
                } else {
                    return gettext("You are not licensed for this hypervisor.");
                }
            }

            function updateEnvArray(hv, env) {
                if ($hostTable.find("[data-env=" + env + "]:checked").length == 0) {
                    var arr = appliedEnvs[hypervisorTranslate[hv]];
                    arr.splice(arr.indexOf(env), 1);
                }
            }

            function setLicenseInfo(data) {
                //var multiManagerFeature = data.license.features[4].data;
                // lmc
                var multiManagerFeature = data.license.features[4];
                var hosts = data.hosts;
                var env, hypervisor;
                multiManager.xen = multiManagerFeature.xen;
                multiManager.vmware = multiManagerFeature.vmware;
                multiManager.hyperv = multiManagerFeature.hyperv;
                for (var i = 0; i < hosts.length; ++i) {
                    /* lmc - remove extra data, state.
                     if (hosts[i].data.licensed_sockets == hosts[i].data.sockets) {
                     env = hosts[i].state.environment;
                     hypervisor = hypervisorTranslate[hosts[i].state.hypervisor];
                     if (appliedEnvs[hypervisor].indexOf(env) == -1) {
                     appliedEnvs[hypervisor].push(env);
                     }
                     }
                     */
                    if (hosts[i].licensed_sockets == hosts[i].sockets) {
                        env = hosts[i].environment;
                        hypervisor = hypervisorTranslate[hosts[i].hypervisor];
                        if (appliedEnvs[hypervisor].indexOf(env) == -1) {
                            appliedEnvs[hypervisor].push(env);
                        }
                    }
                }
            }

            var LicenseTab = function(index, $panel) {
                this.index = index;
                this.$panel = $panel;
                this.$table = $hostTable.filterTable().scrollTable();
            };

            LicenseTab.prototype.init = function() {
                var self = this;

                // User submits a license key
                $btnSubmit.on("click", function() {
                    PHD.Ajax
                        .post($form.attr("action"), $form.serialize())
                        .done(function(data) {
                            if(data.result[0].code === AJAX_RESULT_SUCCESS) {
                                //self.updateLicenseDetails(data.result[0].data.license);
                                // lmc
                                self.updateLicenseDetails(data.result[0].license);
                                self.resetSocketCounts();
                                //setLicenseInfo(data.result[0].data);
                                setLicenseInfo(data.result[0]);
                                PHD.throwNotice(data);
                                $form.dialog("close");
                            } else {
                                PHD.throwError(data);
                            }
                        });
                });

                // User submits a license key
                $btnCancel.on("click", function() {
                    $form.dialog("close");
                });

                // User applies sockets to hosts
                $btnApply.on("click", function() {
                    PHD.Ajax
                        .post($hostSocketForm.attr("action"), $hostSocketForm.serialize())
                        .done(function(data) {
                            if(data.result[0].code === AJAX_RESULT_SUCCESS) {
                                PHD.throwNotice(data);
                            } else {
                                PHD.throwError(data);
                            }
                        });
                });

                HostTable = PHD.DataTable($hostTable, {
                    select: false,
                    placeholder: gettext("There are no hosts in your environment")
                }).registerFormatters(socketFormatter);

                HostTable.on("loadfinished updatefinished", function(event, newRows) {
                    self.updateSocketCount();
                });

                $hostTable.on("change", ":checkbox", function(event, newRows) {
                    var retVal;
                    if($(this).prop("checked")) {
                        if((retVal = canAddSocket($(this).data("hv"), $(this).data("env"), $(this).data("count"))) === true) {
                            self.updateSocketCount();
                        } else {
                            $(this).prop("checked", false);
                            PHD.alert(retVal);
                            return false;
                        }
                    } else {
                        updateEnvArray($(this).data("hv"), $(this).data("env"))
                        self.updateSocketCount();
                    }

                });

                FeatureTable = PHD.DataTable($featureTable, {
                    select: false
                }).registerFormatters(featureFormatter);

                $details.on("click", ".btn-license-features", $.proxy(this.showLicenseFeatures, this));

                $btnLaunch.on("click", $.proxy(this.launchLicenseSubmission, this));

                return this;
            };

            LicenseTab.prototype.fetch = function() {
                var self = this;
                PHD.Ajax
                    // lmc - use our apis, also call /api/license to get feature info.
                    .get("/api/license/resources/?sid=1")
                    .done(function(data) {
                        var license = {};
                        license.system_name = data.license[0].license;
                        license.resources = data.license[0].resources;
                        license.fields = {};
                        PHD.Ajax
                            .get("/api/license/?sid=1")
                            .done(function(data) {
                                license.fields = data;
                                self.update(license);
                                // we cannot call this as it will throw an exception, assuming UVB license formats.
                                //self.update(data.license[0]);
                                //setLicenseInfo(data);
                            });
                    });

                return this;
            };

            LicenseTab.prototype.update = function(data) {
                this.updateLicenseDetails(data);
                // TODO: Hide the host table if license is pay-per-use
                if(data.isPayPerUse) {
                    $hostSocketForm.addClass("hidden");
                } else {
                    $hostSocketForm.removeClass("hidden");
                    this.updateHostTable(data.hosts);
                }
            };

            // Updates the top information area and saves hypervisor-specific information to `var hypervisors`
            LicenseTab.prototype.updateLicenseDetails = function(licenseData) {
                $details.removeClass("hidden").find(".license-specifications").html(detailsTmpl(licenseData));
                hypervisors = _.indexBy(licenseData.limits.sockets, "hypervisor");
                FeatureTable.load(licenseData.features);
            };

            // Updates host table & column filters
            LicenseTab.prototype.updateHostTable = function(hostData) {
                HostTable.update(hostData).$table.scrollTable("form");
                this.updateFilter("tbody td[data-source=hypervisor]", $("tfoot th[data-source=hypervisor] select"));
                this.updateFilter("tbody td[data-source=environment]", $("tfoot th[data-source=environment] select"));
            };

            // Updates socket count text in UI and `var hypervisors`'s data
            LicenseTab.prototype.updateSocketCount = function() {
                _.each(hypervisors, function(data, hv) {
                    var count = PHD.util.sumArray(
                        $hostTable.find("[data-hv="+hv+"]:checked").map(function() {
                            return $(this).data("count");
                        }).get());

                    $(".socket-counts [data-hv=" + hv + "] .sockets-applied").text(count);
                    hypervisors[hv].applied = count;
                });
            };

            LicenseTab.prototype.resetSocketCounts = function() {
                $(".sockets-applied").text("0");
                $hostTable.find(":checkbox").prop("checked", false);
                this.updateSocketCount();
            };

            LicenseTab.prototype.showLicenseFeatures = function() {
                $features.dialog({
                    title: gettext("Licensed Features"),
                    dialogClass: "standalone",
                    width: 600,
                    height: "auto",
                    buttons: [
                        {
                            text: gettext("Close"),
                            click: function() {
                                $(this).dialog("close");
                            }
                        }
                    ]
                });
                return this;
            };

            LicenseTab.prototype.updateFilter = function(selector, $control) {
                var options = [];
                this.$table.find(selector).each(function() {
                    if($.inArray($(this).text(), options) === -1) {
                        options.push($(this).text());
                    }
                });
                $control.find("option:not(.option-all)").remove();
                options.sort().forEach(function(item, index) {
                    $control.append('<option value="' + item + '">' + item + '</option>');
                });
            };

            LicenseTab.prototype.launchLicenseSubmission = function() {
                $form.dialog({
                    title: gettext("Submit License"),
                    dialogClass: "standalone",
                    width: 640,
                    height: 300
                });
                return this;
            };

            LicenseTab.prototype.activate = function() {
                return this.fetch();
            };

            LicenseTab.prototype.deactivate = function() {
                return this;
            };

            PHD.LicenseTab = LicenseTab;

            var e = $.Event('LicenseTab');
            $(document).trigger(e);

            var socketFormatter = {
                apply: function(value, row) {
                    // lmc - remove data
                    //var checked = (row.data.licensed_sockets > 0) ? " checked" : "";
                    var checked = (row.licensed_sockets > 0) ? " checked" : "";
                    /* lmc - remove state
                     return '<input type="checkbox"'+checked+' name="host_id" value="'+row.state.id+'"' +
                     ' data-count="'+row.data.sockets+'" data-hv="'+row.state.hypervisor+'"' +
                     ' data-env="'+row.state.environment+'">';
                     */
                    return '<input type="checkbox"'+checked+' name="host_id" value="'+row.id+'"' +
                        ' data-count="'+row.sockets+'" data-hv="'+row.hypervisor+'"' +
                        ' data-env="'+row.environment+'">';
                }
            };

            var featureFormatter = {
                xen: function(value, row) {
                    var checked = value ? "icon-checkmark text-success" : "icon-multiply text-error";
                    return '<i class="'+checked+'""></i>';
                }
            };

            featureFormatter.vmware = featureFormatter.hyperv = featureFormatter.xen;
        }

    }]);
