(function($, PHD, window, undefined) {
    
    var $form = $("#nic-form"),
        NICForm,
        $netType = $("[name=network_type]"),
        $dnsType = $("[name=dns_network_type]"),
        netType, dnsType,
        $address = $("[name=ip], [name=gateway], [name=subnet]"),
        $dns = $("[name=dns1], [name=dns2]");


    function handleDHCP(event) {
        netType = $netType.filter(":checked").val();
        $address.prop("disabled", (netType === NETWORK_TYPE_DHCP));
        $dns.prop("disabled", (netType === NETWORK_TYPE_DHCP && dnsType === NETWORK_TYPE_DHCP));

        $dnsType.prop("disabled", (netType === NETWORK_TYPE_STATIC));
        if(netType === NETWORK_TYPE_STATIC)
            $dnsType.filter("[value=" + NETWORK_TYPE_STATIC + "]").prop("checked", true).trigger("change");
    }

    function handleDNS(event) {
        dnsType = $dnsType.filter(":checked").val();
        $dns.prop("disabled", (netType === NETWORK_TYPE_DHCP && dnsType === NETWORK_TYPE_DHCP));
    }

    function initForm() {
        $.validator.addMethod("ipv4", function(value, element, param) {
            return this.optional(element) || /^(25[0-5]|2[0-4]\d|[01]?\d\d?)\.(25[0-5]|2[0-4]\d|[01]?\d\d?)\.(25[0-5]|2[0-4]\d|[01]?\d\d?)\.(25[0-5]|2[0-4]\d|[01]?\d\d?)$/i.test(value);
        }, "Please enter a valid IP v4 address.");

        NICForm = PHD.FormController($form, {})
            .validate({
                rules: {
                    ip: {
                        required: true,
                        ipv4: true
                    },
                    gateway: {
                        required: true,
                        ipv4: true
                    },
                    subnet: {
                        required: true,
                        ipv4: true
                    },
                    dns1: {
                        required: true,
                        ipv4: true
                    }
                },
                messages: {
                    ip: {
                        required: gettext("IP Address is required")
                    },
                    gateway: {
                        required: gettext("Gateway is required")
                    },
                    subnet: {
                        required: gettext("Subnet is required")
                    },
                    dns1: {
                        required: gettext("Preferred DNS is required")
                    }
                }
            })
            .on("formcancel", function(event) {
                PHD.currentDialog.wizard("close");
            })
            .on("formsubmit", function(event, data) {
                PHD.currentDialog.wizard("close");
                $(document).trigger("nicsave", [PHD.appliance_sid, data]);
            });

        $netType.on("change", handleDHCP).trigger("change");
        $dnsType.on("change", handleDNS).trigger("change");
    }

    /*
    *   DOM Ready Function
    */
    $(function() {
        PHD.currentDialog.one("wizardfocus", function() {
            initForm();
        });
    });

})(jQuery, window.PHD || (window.PHD = {}), window);
