(function($, PHD, window, undefined) {
    var $form = $("#retention-form"),
        RetentionForm,
        $btnCreate = $form.find(".btn-create-policy"),
        $btnAdd = $('<button>', {
            "type": "button",
            "class": "btn btn-right btn-add-policy clearleft",
            "html": gettext("Add Policy")
        }),
        $btnCancel = $('<button>', {
            "type": "button",
            "class": "btn btn-right btn-cancel-add-policy clearleft",
            "html": gettext("Cancel")
        }),
        $btnDelete = $('<button>', {
            "type": "button",
            "class": "btn btn-right btn-delete-policy clearfull",
            "html": gettext("Delete")
        }),
        $policySelect = $form.find("#policies"),
        $policies = $form.find(".retention-forms"),
        $policyEditArea = $form.find(".manage-policy"),
        $policyTemplate = $("#policy-template"),
        $formCounter = $form.find("#id_form-TOTAL_FORMS"),
        formCount = parseInt($form.find("#id_form-INITIAL_FORMS").val(), 10),
        spinnerOptions = {
            "min": 0,
            "max": 999,
            "step": 1
        },
        dirty = false,
        staged = false,
        confirmCancelTitle = gettext("Confirm Cancellation of Changes"),
        confirmCancelMessage = gettext("Any policies you have added will lost and changes made to existing policies will not take effect."),
        regex = /__prefix__/gi;


    function clearPolicyForm() {
        $policyEditArea
            .addClass("hidden")
        .find(".active-form")
            .detach()
            .removeClass("active-form")
            .appendTo($policies);

        $policyEditArea.html("");
    }

    function validatePolicy(event) {
        if($form.valid()) {
            return true;
        } else {
            event.preventDefault();
            return false;
        }
    }

    function savePolicyName(id, name) {
        if($(".active-form").length) {
            var id = $(".active-form").data("dbid"),
                name = $(".active-form").find("[name$=name]").val();
            $policySelect.find("[value=" + id + "]").text(name);   
        }
    }

    function editPolicy(event) {
        if($form.valid()) {
            if(!$policySelect.children(":selected").hasClass("deleted")) {
                savePolicyName();
                clearPolicyForm();
                var val = $policySelect.val();
                
                if(val !== "") {
                    var $f = $policies.find("[data-dbid=" + val + "]").detach();
                    $policyEditArea.append($f.addClass("active-form")).append($btnDelete.clone().attr("value", val)).removeClass("hidden");
                }
            } else {
                $policySelect.val("");
            }
        } else {
            event.preventDefault();
            return false;
        }
    }

    function showPolicyCreateForm(event) {
        savePolicyName();
        clearPolicyForm();
        $policySelect.val("");
        var name = "My Retention Policy",
            html = $policyTemplate.html();

        $policyEditArea
            .html(html)
            //.append($btnAdd.clone())
            //.append($btnCancel.clone())
            .removeClass("hidden")
        .find("[name$=name]")
            .val(name)
            .focus()
            .select();

        $policyEditArea.find(".spinner-group input")
            .spinner(spinnerOptions);

        $policySelect.disable();
        staged = true;
        //$("#retention-form-footer").hide();
    }

    function addNewPolicy(event) {
        if($form.valid()) {
            var name = $policyEditArea.find("[name$=name]").val(),
                val = "new" + formCount;

            $policyEditArea
                .addClass("hidden")
            .find(".empty-form")
                .attr("data-dbid", val)
                .removeClass("empty-form")
                .detach()
                .appendTo($policies)
            .find("input, label")
                .each(function() {
                    var $el = $(this), newID, newName;
                    if($el.is("input")) {
                        newID = $el.attr("id").replace(regex, formCount);
                        newName = $el.attr("name").replace(regex, formCount);
                        $el.attr("name", newName).attr("id", newID);
                    } else {
                        newID = $el.attr("for").replace(regex, formCount);
                        $el.attr("for", newID);
                    }
                });

            $policySelect.append('<option value="'+val+'">' + name + '</option>');
            $formCounter.val(++formCount);
            $policyEditArea.html("");

            $policySelect.enable();
            staged = false;
            dirty = true;
            $("#retention-form-footer").show();
        }
    }

    function cancelAddPolicy(event) {
        $policyEditArea
            .addClass("hidden")
            .html("");

        $policySelect.enable();
        staged = false;
        $("#retention-form-footer").show();
    }

    function confirmDeletePolicy(event) {
        var confirm = gettext("Are you sure you want to delete '%s'? VMs using this policy will be set to 'Typical'."),
            message = interpolate(confirm, [getPolicyName($(this).val())]);

        $policySelect.disable();
        PHD.confirmDialog(
            "<p>" + message + "</p>", {
            title: gettext("Confirm Retention Policy Deletion")
        })
        .done($.proxy(deletePolicy, this))
        .fail(function() {$policySelect.enable();});
    }

    function deletePolicy() {
        var trans = gettext("%s will be deleted when changes are saved."),
            message = interpolate(trans, [getPolicyName($(this).val()) ]);
        PHD.throwNotice({
            result: [{
                message: message
            }]
        });
        $policyEditArea.find("[name$=DELETE]").prop("checked", true);
        $policySelect
            .val("")
        .find("option[value=" + $(this).val() + "]")
            .addClass("deleted")
            .append(" [deleted]");
        clearPolicyForm();
        $policySelect.enable();
        dirty = true;
    }

    function getPolicyName(id) {
        return $policySelect.find("[value=" + id + "]").text();
    }

    function getNumRetentionBackups(recent, days, weeks, months, years) {
        if (isNaN(recent = parseInt(recent)))
            return isNaN;
        if (isNaN(days = parseInt(days)))
            return isNaN;
        if (isNaN(weeks = parseInt(weeks)))
            return isNaN;
        if (isNaN(months = parseInt(months)))
            return isNaN;
        if (isNaN(years = parseInt(years)))
            return isNaN;

        var numBackups = days >= recent ? days : recent;
        if (weeks > 0) {
            numBackups += weeks - 1;
        }
        if (months > 0) {
            numBackups += months - 1;
        }
        return numBackups;
    }

    function displayRetentionPolicyDetails(event) {
        var $this = $(this),
            parentGroup = $this.closest(".virtual-full-spinners"),
            textFormat,
            backups,
            chains;
        if (parentGroup.length) {
            backups = getNumRetentionBackups(
                parentGroup.find("[type=text]:eq(0)").val(),
                parentGroup.find("[type=text]:eq(1)").val(),
                parentGroup.find("[type=text]:eq(2)").val(),
                parentGroup.find("[type=text]:eq(3)").val(),
                parentGroup.find("[type=text]:eq(4)").val());
            if (!isNaN(backups)) {
                textFormat = ngettext(RETENTION_VIRTUAL_FILL_SINGULAR, RETENTION_VIRTUAL_FILL_PLURAL, backups);
                parentGroup.find(".virtual-full-details").text(interpolate(textFormat, [backups]));
            } else {
                parentGroup.find(".virtual-full-details").text("");
            }
        } else {
            parentGroup = $this.closest(".full-inc-spinners");
            chains = parseInt($this.val());
            if (!isNaN(chains)) {
                textFormat = gettext(RETENTION_FULL_INC);
                parentGroup.find(".full-inc-details").text(interpolate(textFormat, [chains, chains * 6]));
            } else {
                parentGroup.find(".full-inc-details").text("");
            }
        }
    }

    $(".btn-add-policy").click(function() {

        console.log("Add clicked");
        var obj={};
        var policy_name = $("#policy-name").val();

        if (policy_name != "") {
            obj.name = policy_name;
        }

        if($("#default").prop("checked")){
            obj.is_default = true;
        }
        else{
            obj.is_default = false;
        }

        obj.days = Number($("#days").val());
        obj.weeks = Number($("#weeks").val());
        obj.months = Number($("#months").val());
        obj.years = Number($("#years").val());

        if($("#days-forever").prop("checked")){
            obj.days = -1;
            obj.weeks = 0;
            obj.months = 0;
            obj.years = 0;
        }else if($("#weeks-forever").prop("checked")){
            obj.days = 0;
            obj.weeks = -1;
            obj.months = 0;
            obj.years = 0;
        }else if($("#months-forever").prop("checked")){
            obj.days = 0;
            obj.weeks = 0;
            obj.months = -1;
            obj.years = 0;
        }else if($("#years-forever").prop("checked")){
            obj.days = 0;
            obj.weeks = 0;
            obj.months = 0;
            obj.years = -1;
        }

        var base_url = "/api/retention_policy/";

        obj = JSON.stringify(obj);
        if(policyEdit){
            console.log("PUT called");
            base_url +=id + "/";
            resp = PHD.Ajax.put(base_url + "?sid=" +  PHD.appliance_sid, obj);
        }
        else{
            resp = PHD.Ajax.post(base_url + "?sid=" +  PHD.appliance_sid, obj);
        }

        resp.done(function(data) {
            var result = data.result[0];
            if(result.code === AJAX_RESULT_SUCCESS) {
                PHD.throwNotice(data, {});
                PHD.currentDialog.wizard("close");
                $(document).trigger("retentionupdate", [result.data]);
                PHD.currentDialog.wizard("close");
            } else if(result.code === AJAX_RESULT_ERROR) {
                PHD.throwError(data);
            }
            PHD.currentDialog.wizard("close");
        });
    });

    $("#days-forever").change(function () {
        if($("#days-forever").prop("checked")){
            console.log("Days forever enabled");
            days =  $("#days").val();
            $("#days").spinner("disable");
            $("#days").val("");
            $("#weeks-forever").disable();
            $("#weeks").spinner("disable");
            weeks =  $("#weeks").val();
            $("#weeks").val("");
            $("#months-forever").disable();
            $("#months").spinner("disable");
            months = $("#months").val();
            $("#months").val("");
            $("#years-forever").disable();
            $("#years").spinner("disable");
            years =  $("#years").val();
            $("#years").val("");
        }
        else{
            console.log("Days forever disabled");
            $("#days").spinner("enable");
            $("#days").val(days);
            $("#weeks-forever").enable();
            $("#weeks").enable();
            $("#weeks").spinner("enable");
            $("#weeks").val(weeks);
            $("#months-forever").enable();
            $("#months").enable();
            $("#months").spinner("enable");
            $("#months").val(months);
            $("#years-forever").enable();
            $("#years").enable();
            $("#years").spinner("enable");
            $("#years").val(years);
        }
    });

    $("#weeks-forever").change(function () {
        if($("#weeks-forever").prop("checked")){
            console.log("Weeks forever enabled");
            weeks =  $("#weeks").val();
            $("#weeks").val("");
            $("#weeks").spinner("disable");
            $("#months-forever").disable();
            $("#months").spinner("disable");
            months = $("#months").val();
            $("#months").val("");
            $("#years-forever").disable();
            $("#years").spinner("disable");
            years =  $("#years").val();
            $("#years").val("");
        }else{
            console.log("Weeks forever disabled");
            $("#weeks").spinner("enable");
            $("#weeks").val(weeks);
            $("#months-forever").enable();
            $("#months").enable();
            $("#months").spinner("enable");
            $("#months").val(months);
            $("#years-forever").enable();
            $("#years").enable();
            $("#years").spinner("enable");
            $("#years").val(years);
        }
    });

    $("#months-forever").change(function () {
        if($("#months-forever").prop("checked")){
            console.log("Months forever enabled");
            months = $("#months").val();
            $("#months").val("");
            $("#months").spinner("disable");
            $("#years-forever").disable();
            $("#years").spinner("disable");
            years =  $("#years").val();
            $("#years").val("");
        }else{
            console.log("Months forever disabled");
            $("#months").val(months);
            $("#months").spinner("enable");
            $("#years-forever").enable();
            $("#years").enable();
            $("#years").spinner("enable");
            $("#years").val(years);
        }
    });

    $("#years-forever").change(function () {
        if($("#years-forever").prop("checked")){
            console.log("Years forever enabled");
            years =  $("#years").val();
            $("#years").spinner("disable");
            $("#years").val("");
        }else{
            console.log("Years forever disabled");
            $("#years").val(years);
            $("#years").spinner("enable");
        }
    });

    function initForm() {

        $(".Calendar").click(function() {
            PHD.miniWizard("app/configure/retention/calendarview.html", "calendar",{
                title: "Retention Calendar",
                helpArticle: PHD.App.getHelpLink("editasset", true),
                width: 500,
                height: 500,
                modal: true,
                zIndex: 1001,
                buttons: [
                    {
                        text: "Close",
                        click: function() {
                            $(this).dialog("close");
                        }
                    }
                ]
            });
            return true;
        });

        if(typeof policyEdit!== 'undefined'){
            if(policyEdit){
                $("#save_policy").text("Save");
            }
        }

        $.validator.addClassRules("retention-int", {
            required: true,
            digits: true,
            min: 0,
            max: 999
        });

        $.validator.addClassRules("retention-name", {
            required: true
        });

        RetentionForm = PHD.FormController($form, {
            submitOnEnter: false,
            debug: false
        });

        RetentionForm
            .validate({
                rules: {
                    policy_name: {
                        required: true
                    }
                },
                messages: {
                    policy_name: {
                        required: gettext("Policy name is required")
                    }
                }
            })
            .on("formcancel", function(event) {
                if(dirty || staged)
                    PHD.confirmDialog("<p>" + confirmCancelMessage + "</p>", { title: confirmCancelTitle })
                        .done(function() { PHD.currentDialog.wizard("close"); });
                else
                    PHD.currentDialog.wizard("close");
            })

        $policySelect.on("mousedown", validatePolicy);
        $policySelect.on("change", editPolicy);

        $btnCreate.on("click", showPolicyCreateForm);

        $form.on("change spin spinchange", "[type=text]", displayRetentionPolicyDetails);

        $form
            .on("click", ".btn-add-policy", addNewPolicy)
            .on("click", ".btn-cancel-add-policy", cancelAddPolicy)
            .on("click", ".btn-delete-policy", confirmDeletePolicy)
            .one("change spin spinchange", "[type=text]", setDirty)
        .find(".spinner-group input")
            .spinner(spinnerOptions);

        $form.valid();
    }

    function setDirty() {
        //console.log("dirty");
        dirty = true;
    }

    /*
    *   DOM Ready Function
    */
    $(function() {
        // Initialize form validation and event handlers
        initForm();
    });

})(jQuery, window.PHD || (window.PHD = {}), window);
