(function($, PHD, window, undefined) {

    var $form = $("#bds-form"),
        BDSForm,
        NAME_REGEX = /^[a-zA-Z0-9\-\_]+$/,
        $bdsmount = $form.find("[name=bdsmount_id]"),
        $override = $('<input type="hidden" name="override_warning" value="1">'),
        //$credentialToggle = $("#id_change_credentials"),
        $credentialToggle = $("#checkbox_change_credentials"),
        $backendLogin = $("#id_username"),
        $backendPassword = $("#id_backend_password"),
        $backendPasswordConfirm = $("#backend_password_confirm"),
        $cloudStorage = $("#id_host"),
        $cloudBucket = $("#id_share"),
        $retention = $("#id_retention"),
        $enableThreshold = $("#id_set_threshhold"),
        $retentionSpinners = $("#retention-inputs").find(".spinner-group input"),
        retentionSpinnerOptions = {"min": 0, "max": 163840, "step": 1},
        editingExisting = false,
        system_id = PHD.appliance_sid;


    function initTables() {
        //$includedTable.scrollTable({"height": 121, "resize": false});
        //$availableTable.scrollTable({"height": 121, "resize": false});
    }

    function handleCredentials(event) {
        if($credentialToggle.length) {
            var checked = $credentialToggle.prop("checked");
            //$backendLogin.prop("disabled", !checked);
            $backendPassword.prop("disabled", !checked);
            $backendPasswordConfirm.prop("disabled", !checked);
        }
    }

    function handleRetentionChange(event) {
        var value = parseInt($retention.val(), 10),
            enabled = (value === ARCHIVE_RETENTION_CUSTOM),
            hidden = (value === ARCHIVE_RETENTION_KEEP_ALL),
            action = enabled ? "enable" : "disable";

        $retentionSpinners.prop("disabled", !enabled).spinner(action);
        $("#retention-inputs").toggleClass("hidden", hidden);
    }

    function handleStorageType(event) {
        // 2 is Amazon S3, so those will be the default labels on an add before any cloud storage is selected.
        var value = $cloudStorage.val() || "s3",
            //spec = cloudTypes[value],
            $helpTrigger;

        console.log("handleStorageType: value = " + value);

        switch (value) {
            case "":
            case "s3":
                if (!editingExisting) {
                    $cloudBucket.val('');
                }
                $("#id_example_text").text('example: <bucket name>(/<folder>/)');
                $("#id_backend_login_label").text("Access Key ID");
                $("#id_backend_password_label").text("Secret Access Key");
                $("#id_backend_password_label_confirm").text("Confirm Access Key");
                break;
            case "swift":
                if (!editingExisting) {
                    $cloudBucket.val('auth.api.rackspacecloud.com/');
                }
                $("#id_example_text").text("example: auth.api.rackspacecloud.com/<container name>");
                $("#id_backend_login_label").text("Username");
                $("#id_backend_password_label").text("Password");
                $("#id_backend_password_label_confirm").text("Confirm Password");
                break;
            case "gs":
                if (!editingExisting) {
                    $cloudBucket.val('');
                }
                $("#id_example_text").text('example: <bucket name>(/<folder>/)');
                $("#id_backend_login_label").text("Access Key");
                $("#id_backend_password_label").text("Secret");
                $("#id_backend_password_label_confirm").text("Confirm Secret");
                break;
        }
    }

    function handleThresholdChange(event){

        var checked = $enableThreshold.prop("checked");
        var action = checked ? "enable" : "disable";
        $retentionSpinners.prop("disabled", !checked).spinner(action);
    }

    function initForm() {
        editingExisting = selectedStorage !== null;
        if (editingExisting) {
            $form.find("[name$=cloud_storage]").readonly();
            $cloudStorage.disable();
            $cloudBucket.disable();
            $enableThreshold.prop("checked", selectedStorage.max_size !== undefined && selectedStorage.max_size > 0);
        } else {
            $credentialToggle.prop("checked", true);
            $("#div_credentials").hide();
        }

        $form.find(".definition").popover({
            appendTo: $form,
            position: {
                my: "left bottom",
                at: "right top-15"
            }
        });

        $retentionSpinners.spinner(retentionSpinnerOptions);

        $cloudStorage.on("change", handleStorageType).trigger("change");
        $credentialToggle.on("change", handleCredentials).trigger("change");
        $retention.on("change", handleRetentionChange).trigger("change");
        $enableThreshold.on("change", handleThresholdChange).trigger("change");

        $.validator.addClassRules("retention-int", {
            required: true,
            digits: true,
            min: 0,
            max: 163840
        });

        $.validator.addMethod("noslashes", function(value, element) {
                return !/[\/\\]/.test(value);
            },
            gettext("File Prefix must not contain slashes")
        );

        BDSForm = PHD.FormController($form, {})
            .validate({
                rules: {
                    name: {
                        required: true,
                        pattern: NAME_REGEX
                    },
                    "properties.hostname": {
                        required: true
                    },
                    "properties.share_name": {
                        required: true
                    },
                    "properties.username": {
                        required: true
                    },
                    "properties.password": {
                        required: true
                    },
                    confirm_password: {
                        equalTo: "#id_backend_password"
                    }
                },
                messages: {
                    name: {
                        required: gettext("Please name your Cloud Target")
                    },
                    "properties.hostname": {
                        required: gettext("You must select a cloud storage type")
                    },
                    "properties.share_name": {
                        required: gettext("Storage path is required")
                    },
                    "properties.username": {
                        required: gettext("Access key is required")
                    },
                    "properties.password": {
                        required: gettext("Secret is required")
                    }
                }
            })
            .on("formcancel", function(event) {
                PHD.currentDialog.wizard("close");
            })
            .on("formsubmit", function(event, data) {
                $(document).trigger("bdssave", [system_id, data]);
                PHD.currentDialog.wizard("close");
            });
            /*.on("formsubmit", function(event, data) {
                if(data.result[0].code === AJAX_RESULT_SUCCESS) {
                    PHD.throwNotice(data);
                    PHD.currentDialog.wizard("close");
                    $(document).trigger("bdssave", [vbaID, data]);
                } else if(data.result[0].code === AJAX_RESULT_WARNING) {
                    // Jobs are running and changing the BDS would be disastrous.
                    PHD.confirmDialog('<p>' + data.result[0].message + '</p>', {'title': gettext('Confirm Storage Edit')})
                        .done(function() {
                            $override.appendTo($form);
                            BDSForm.submit();
                        });
                } else if(data.result[0].code === AJAX_RESULT_ERROR) {
                    PHD.throwError(data);
                }
            }); */

        BDSForm.submit = function() {
            var self = this,
                load;

            this.$buttons.blur().disable();

            var submitData = form2js($form.attr("id"), '.', false);
            var storageObj = submitData;
            console.log('=== Submit form: object = ' + JSON.stringify(submitData));

            var $editing_id = $form.find("#edit_object_id");
            console.log('edit id text = ' + $editing_id.text() + " and appliance id is " + PHD.appliance_sid);
            var sid = (PHD.appliance_sid === null) ? "" : "?sid=" + PHD.appliance_sid;
            if ($editing_id.text() != "none") {
                load = PHD.showLoadingIndicator("body", true, "Saving...");
                var threshold_checked = $("#id_set_threshhold").prop("checked");
                if (storageObj.properties !== undefined) {
                    if (storageObj.properties.password == "") {
                        console.log("delete 'em");
                        delete storageObj.properties.password;
                        delete storageObj.confirm_password;
                    }
                    storageObj.properties.max_size = threshold_checked ? submitData.properties.max_size : 0;
                    storageObj.usage = selectedStorage.usage;
                }
                submitData = JSON.stringify(storageObj);
                console.log('=== Submit form: object = ' + submitData);
                var myID = $editing_id.text();
                var url = this.url + myID + "/" + sid;
                console.log("IS Editing, url is " + url);

                //data = [];
                //self.$form.trigger("formsubmit", [data, self.$form, self]);
                //PHD.hideLoadingIndicator(load);
                PHD.Ajax
                    .put(url, submitData)
                    .done(function (data) {
                        self.$form.trigger("formsubmit", [data, self.$form, self]);
                    })
                    .always(function () {
                        self.$buttons.enable();
                        PHD.hideLoadingIndicator(load);
                    });
            } else {
                load = PHD.showLoadingIndicator("body", true, "Adding and checking for existing Backup Copies.  Please wait...");
                submitData = JSON.stringify(submitData);
                var url = this.url + sid;
                console.log("Is Adding, not Editing, url is " + url);
                PHD.Ajax
                    .post(url, submitData)
                    .done(function (data) {
                        self.$form.trigger("formsubmit", [data, self.$form, self]);
                    })

                    .always(function () {
                        self.$buttons.enable();
                        PHD.hideLoadingIndicator(load);
                    });
            }
            return this;
        };
    }

    /*
     *   DOM Ready Function
     */
    $(function() {
        //cloudTypes = _.indexBy(cloudTypes, "id");

        PHD.currentDialog.one("wizardfocus", function() {
            initForm();
            initTables();
        });

    });

})(jQuery, window.PHD || (window.PHD = {}), window);
