angular.module('dashBackup', [])

    .controller('BackupDashCtrl', ['$scope', '$rootScope', '$http', function($scope, $rootScope, $http) {

        $scope.initCtrl = function() {
            processBackupWidget(PHD, window, $scope, $rootScope);
        }

        //(function($, PHD, window, undefined) {
        function processBackupWidget(PHD, window, $scope, $rootScope) {

            var $backupWidget = $("#widget-backup"),
                $backupTrigger = $backupWidget.find(".preference-trigger"),
                backupRefreshRate = refreshRates.dash_backup_refresh,
                backupNamespace = "backupwidget",
                BackupUserPreference,
                BackupTimer;

            var pluralizer = {
                "backup_errors": function(count) {
                    return ngettext("Error", "Errors", count);
                }
            };

            function updateWidget(init) {

                var subsystem = "/backup/";
                if (!$rootScope.user.loggedIn) {
                    BackupTimer.rerun();
                    return;
                }

                var url = '/api/summary/counts' + subsystem;
                $.ajax(url, {
                    dataType: 'json',
                    success: function(data) {
                        $(".status-number").each(function(index) {
                            var $el = $(this),
                                key = $el.attr("id"),
                                $plural = $el.next(".plural");

                            if (data.hasOwnProperty(key)) {
                                $el.text(data[key]);
                                if($plural.length)
                                    $plural.text(pluralizer[key](data[key]));
                            }
                        });

                        refreshBackup(init);
                    },
                    error: function () {
                        // TODO:handle error
                        //alert("Status Error");
                    }
                });
            }

            function refreshBackup(init) {
                if(init === true) {
                    BackupTimer.start();
                } else {
                    BackupTimer.resume();
                }
            }

            BackupUserPreference = PHD.UserPreference($backupTrigger, {
                name: gettext("Backup Widget"),
                namespace: backupNamespace,
                key: "dash_backup_refresh",
                refresh_name: "Backup refresh",
                defaultValue: backupRefreshRate
            });

            $(document)
                .on("login", function(event) {
                    console.log("=====>Status, caught LOGIN, name = ", event.name);
                    updateWidget(true);
                    stopAll();
                })
                .on("logout", function(event) {
                    console.log("=====>Status, LOGOUT");
                    stopAll();
                })
                .on("uinav.dashboard", function(event) {
                    console.log("=====>Selected Dashboard");
                })
                .on("uinav.protect", function(event) {
                    console.log("=====>Selected Protect");
                    stopAll();
                })
                .on("uinav.recover", function(event) {
                    console.log("=====>Selected Recover");
                    stopAll();
                })
                .on("uinav.jobs", function(event) {
                    console.log("=====>Selected Jobs");
                    stopAll();
                })
                .on("uinav.configure", function(event) {
                    console.log("=====>Selected Configure");
                    stopAll();
                })
                .on("uinav.reports", function(event) {
                    console.log("=====>Selected Reports");
                    stopAll();
                })
                .on("userpreferencesave.backupwidget", function(event, key, value) {
                    backupRefreshRate = value;
                    BackupTimer.setInterval(backupRefreshRate).reset();
                })
                .on("timertick." + backupNamespace, function() {
                    updateWidget();
                })
                .on("timersetmanual." + backupNamespace, function(event, $control, PageTimer) {
                    $control.show();
                })
                .on("timerunsetmanual." + backupNamespace, function(event, $control, PageTimer) {
                    $control.hide();
                });


            function stopAll() {
                BackupTimer.stop();
                $(document).off("login");
                $(document).off("logout");
                $(document).off("uinav");
                $(document).off("userpreferencesave." + backupNamespace);
                $(document).off("timertick." + backupNamespace);
                $(document).off("timersetmanual." + backupNamespace);
                $(document).off("timerunsetmanual." + backupNamespace);
                delete $(document);
            }

            /*
             *   DOM Ready Function
             */
            $(function() {
                BackupTimer = PHD.Timer({
                    namespace: backupNamespace,
                    interval: backupRefreshRate,
                    refreshControl: $("#backup-widget-preferences .timer-refresh")
                });

                updateWidget(true);
            });

        }
        //})(jQuery, window.PHD || (window.PHD = {}), window);
    }]);
