angular.module('dashReplication', [])

    .controller('ReplicationDashCtrl', ['$scope', '$rootScope', '$http', function($scope, $rootScope, $http) {

        $scope.initCtrl = function () {
            processReplicationWidget(PHD, window, $scope, $rootScope);
        }

        function processReplicationWidget(PHD, window, $scope, $rootScope) {

            var $replicationWidget = $("#widget-replication"),
                $replicationTrigger = $replicationWidget.find(".preference-trigger"),
                replicationRefreshRate = refreshRates.dash_replication_refresh,
                replicationNamespace = "replicationwidget",
                ReplicationUserPreference,
                ReplicationTimer;

            var pluralizer = {
                "replication_errors": function (count) {
                    return ngettext("Error", "Errors", count);
                },
                "replication_replicas": function (count) {
                    return ngettext("Replica", "Replicas", count);
                }
            };

            function updateWidget(init) {

                var subsystem = "/replication/";
                if (!$rootScope.user.loggedIn) {
                    ReplicationTimer.rerun();
                    return;
                }

                var url = '/api/summary/counts' + subsystem;
                $.ajax(url, {
                    dataType: 'json',
                    success: function (data) {
                        $(".status-number").each(function (index) {
                            var $el = $(this),
                                key = $el.attr("id"),
                                $plural = $el.next(".plural");

                            if (data.hasOwnProperty(key)) {
                                $el.text(data[key]);
                                if ($plural.length)
                                    $plural.text(pluralizer[key](data[key]));
                            }
                        });

                        refreshReplication(init);
                    },
                    error: function () {
                        // TODO:handle error
                        //alert("Status Error");
                    }
                });
            }

            function refreshReplication(init) {
                if (init === true) {
                    ReplicationTimer.start();
                } else {
                    ReplicationTimer.resume();
                }
            }

            ReplicationUserPreference = PHD.UserPreference($replicationTrigger, {
                name: gettext("Replication Widget"),
                namespace: "replicationwidget",
                key: "dash_replication_refresh",
                refresh_name: "Replication refresh",
                defaultValue: replicationRefreshRate
            });

            $(document)
                .on("login", function (event) {
                    console.log("=====>Status, caught LOGIN, name = ", event.name);
                    updateWidget(true);
                    stopAll();
                })
                .on("logout", function (event) {
                    console.log("=====>Status, LOGOUT");
                    stopAll();
                })
                .on("uinav.dashboard", function (event) {
                    console.log("=====>Selected Dashboard");
                })
                .on("uinav.protect", function (event) {
                    console.log("=====>Selected Protect");
                    stopAll();
                })
                .on("uinav.recover", function (event) {
                    console.log("=====>Selected Recover");
                    stopAll();
                })
                .on("uinav.jobs", function (event) {
                    console.log("=====>Selected Jobs");
                    stopAll();
                })
                .on("uinav.configure", function (event) {
                    console.log("=====>Selected Configure");
                    stopAll();
                })
                .on("uinav.reports", function (event) {
                    console.log("=====>Selected Reports");
                    stopAll();
                })
                .on("userpreferencesave.replicationwidget", function (event, key, value) {
                    replicationRefreshRate = value;
                    ReplicationTimer.setInterval(replicationRefreshRate).reset();
                })
                .on("timertick." + replicationNamespace, function () {
                    updateWidget();
                })
                .on("timersetmanual." + replicationNamespace, function (event, $control, PageTimer) {
                    $control.show();
                })
                .on("timerunsetmanual." + replicationNamespace, function (event, $control, PageTimer) {
                    $control.hide();
                });


            function stopAll() {
                ReplicationTimer.stop();
                $(document).off("login");
                $(document).off("logout");
                $(document).off("uinav");
                $(document).off("userpreferencesave." + replicationNamespace);
                $(document).off("timertick." + replicationNamespace);
                $(document).off("timersetmanual." + replicationNamespace);
                $(document).off("timerunsetmanual." + replicationNamespace);
                delete $(document);
            }

            /*
             *   DOM Ready Function
             */
            $(function () {

                ReplicationTimer = PHD.Timer({
                    namespace: replicationNamespace,
                    interval: replicationRefreshRate,
                    refreshControl: $("#replicate-widget-preferences .timer-refresh")
                });

                /*$(".job-explorer-pop-out").on("click", function (event) {
                    event.preventDefault();
                    PHD.popOut($(this).attr("href"), gettext("Job Explorer"), "view_job_explorer", 1024, 768);
                }); */

                updateWidget(true);
            });

        }
    }]);
