﻿angular.module('dashStorage', [])

    .controller('StorageDashCtrl', ['$scope', '$rootScope', '$http', function($scope, $rootScope, $http) {

        $scope.initCtrl = function() {
            processStorageWidget(PHD, window, $scope, $rootScope);
        }

        //(function($, PHD, window, undefined) {
        function processStorageWidget(PHD, window, $scope, $rootScope) {

            var $widget = $("#widget-storage"),
                $prefTrigger = $widget.find(".preference-trigger"),
                $storageTable = $("#dashboard-storage"),
                $storageTotalProgress = $("#storage-widget-total-progress"),
                refreshRate = refreshRates.dash_storage_refresh,
                namespace = "storagewidget",
                StorageTable, UserPreference, Timer,
                threshold_lookup = [];

            var dtOptions = {
                select: false,
                multiselect: false,
                className: "storage",
                cellClasses: [null, "long-text truncate", "long-text truncate", null, "table-icon-small", null, null]
            };


            function usageLookup(usage) {
                if (usage == '') {
                    return 'backup';
                } else {
                    return usage;
                }
            }

            var storageLookup = {};
            storageLookup[""] = "attached";
            storageLookup["internal"] = "attached";
            storageLookup["stateless"] = "attached";
            storageLookup["disk"] = "attached";
            storageLookup["iscsi"] = "attached";
            storageLookup["added_internal"] = "attached";
            /* lmc
             storageLookup[STORAGE_TYPE_DEV] = "attached";
             storageLookup[STORAGE_TYPE_LVM] = "attached";
             storageLookup[STORAGE_TYPE_NFS] = "nfs";
             storageLookup[STORAGE_TYPE_CIFS] = "cifs";
             storageLookup[STORAGE_TYPE_ISCSI] = "attached";
             storageLookup[STORAGE_TYPE_S3] = "cloud";
             storageLookup[STORAGE_TYPE_GS] = "cloud";
             storageLookup[STORAGE_TYPE_SWIFT] = "cloud";
             */

            function getStorageType(row) {
                var type = 'attached';
                if (row.type == 'nas') {
                    if (typeof row.properties !== 'undefined' && row.properties != null) {
                        type = row.properties.protocol;
                    }
                } else {
                    type = storageLookup[row.type];
                }
                return type;
            }

            var cellFormatter = {
                type: function (value, row) {
                    var usage = usageLookup(row.usage),
                        stoType = getStorageType(row),
                        usageIcon = '<i class="icon-phd-' + usage + ' icon-large text-standard" title="' + usage + '"></i> ',
                        stoIcon = '<i class="icon-phd-bds-' + stoType + ' icon-large text-standard" title="' + stoType + '"></i>';
                    var has_open_alerts = [];
                    var alert_icon = "";
                    if (row.alerts != undefined && Array.isArray(row.alerts) && row.alerts.length > 0) {
                        has_open_alerts = $.grep(row.alerts, function (v) {
                            return v.resolved === false;
                        });
                        if (has_open_alerts.length > 0) {
                            alert_icon = ' <i class="icon-exclamationsign severity-critical" title="has open alerts"></i>';
                        }
                    }
                    return usageIcon + stoIcon + alert_icon;
                },
                mb_free: function (value, row) {
                    var percent_used = row.percent_used,
                        state = "storage-";

                    var used_allocated = "";
                    var color = "";
                    if (row.mb_size !== undefined) {
                        var sizeVal = row.mb_size;
                        var freeVal = row.mb_free;
                        if (sizeVal === 0) {
                            used_allocated = "Not Available";
                            color = "color:gray";
                            percent_used = "100%";
                        } else {
                            var size = PHD.util.formatMB(sizeVal);
                            var used = PHD.util.formatMB(sizeVal - freeVal);
                            used_allocated = "" + used + " / " + size;
                        }
                    }
                    var will_purge = row.mb_to_purge;
                    var to_display = used_allocated;
                    if (sizeVal == 0) {
                        state += "unknown";
                    } else if (freeVal !== undefined && (sizeVal > 0 && (freeVal < (0.1 * sizeVal)))) {
                        state += "warn";
                        //to_display = used_allocated + "; needs to purge for scheduled backup jobs.";
                    } else {
                        state += "ok";
                    }
                    return '<div class="progress-bar" title="' + to_display + '">' +
                        '<div class="progress-meter ' + state + '" style="width:' + percent_used + ';' + color + '"></div></div>';
                },
                status: function (value, row) {
                    var iconClass = value === "online" ? "okcircle" : "off";
                    var statusIcon = '<i class="icon-'+ iconClass +' icon-large" title="' + value + '"></i>';
                    if (row.is_purging === true) {
                        var purge_title = "Purging " + PHD.util.formatMB(row.mb_to_purge);
                        statusIcon = '<i class="icon-phd-sync icon-large" title="' + purge_title + '"></i>';
                    }
                    return '<span class="storage-status">' + statusIcon + '</span>';
                }
            };

            function updateWidget(init) {

                if (!$rootScope.user.loggedIn) {
                    Timer.rerun();
                    return;
                }
                var resp;

                if (init === true) {
                    StorageTable = PHD.DataTable($storageTable, dtOptions);
                    StorageTable.registerFormatters(cellFormatter);
                }
                resp = PHD.Ajax.get("/api/storage");

                resp.done(function (data) {
                    var i;
                    if (typeof data.storage !== 'undefined') {
                        if (init === true) {
                            /*sub_data = data.data;
                             for (i = 0; i < sub_data.length; ++i) {
                             data_state = sub_data[i].state;
                             threshold_lookup[data_state.id] = {
                             warn_threshold: data_state.warn_threshold,
                             stop_threshold: data_state.stop_threshold
                             }
                             } */
                            for (i = 0; i < data.storage.length; ++i) {
                                var id = data.storage[i].id;
                                threshold_lookup[id] = {
                                    warn_threshold: (typeof data.storage[i] !== 'undefined') ? data.storage[i].warn_threshold : 0,
                                    stop_threshold: (typeof data.storage[i] !== 'undefined') ? data.storage[i].stop_threshold : 0
                                }
                            }
							 if(StorageTable !== undefined)
                               StorageTable.load(data.storage);
                        } else {
                            if(StorageTable !== undefined)
   							 StorageTable.update(data.storage, true);
                        }
                    }

                    $storageTable.scrollTable({"height": ".widget-content"});

                    totals = data.totals;
                    if (typeof totals !== 'undefined') {
                        $("[data-source-total]").each(function (index) {
                            var $el = $(this),
                                key = $el.attr("data-source-total");

                            if (totals.hasOwnProperty(key)) {
                                $el.text(totals[key]);
                            }
                        });

                        $storageTotalProgress
                            .attr("title", totals["used_allocated"])
                            .children(".progress-meter").css("width", totals["percent_used"] + "%");
                    }

                    if (init) {
                        Timer.start();
                    } else {
                        Timer.resume();
                    }
                });
            }

            UserPreference = PHD.UserPreference($prefTrigger, {
                name: gettext("Storage Widget"),
                namespace: namespace,
                key: "dash_storage_refresh",
                refresh_name: "Storage refresh",
                defaultValue: refreshRate
            });

            $(document)
                .on("login", function (event) {
                    console.log("=====>Storage, caught LOGIN, name = ", event.name);
                    updateWidget(true);
                    stopAll();
                })
                .on("logout", function (event) {
                    console.log("=====>Storage, LOGOUT");
                    Timer.stop();
                })
                .on("uinav.dashboard", function (event) {
                    console.log("=====>Selected Dashboard");
                })
                .on("uinav.protect", function (event) {
                    console.log("=====>Selected Protect");
                    stopAll();
                })
                .on("uinav.recover", function (event) {
                    console.log("=====>Selected Recover");
                    stopAll();
                })
                .on("uinav.jobs", function (event) {
                    console.log("=====>Selected Jobs");
                    stopAll();
                })
                .on("uinav.configure", function (event) {
                    console.log("=====>Selected Configure");
                    stopAll();
                })
                .on("uinav.reports", function (event) {
                    console.log("=====>Selected Reports");
                    stopAll();
                })
                .on("userpreferencesave." + namespace, function (event, key, value) {
                    console.log("=====>storage preference save rate = " + refreshRate);
                    refreshRate = value;
                    Timer.setInterval(refreshRate).reset();
                })
                .on("timertick." + namespace, function () {
                    updateWidget();
                })
                .on("timersetmanual." + namespace, function (event, $control, PageTimer) {
                    $control.show();
                })
                .on("timerunsetmanual." + namespace, function (event, $control, PageTimer) {
                    $control.hide();
                });

            function stopAll() {
                Timer.stop();
                $(document).off("login");
                $(document).off("logout");
                $(document).off("uinav");
                $(document).off("userpreferencesave." + namespace);
                $(document).off("timertick." + namespace);
                $(document).off("timersetmanual." + namespace);
                $(document).off("timerunsetmanual." + namespace);
                delete $(document);
            }

            /*
             *   DOM Ready Function
             */
            //$(function () {
            function initializeStorage() {

                Timer = PHD.Timer({
                    namespace: namespace,
                    interval: refreshRate,
                    refreshControl: $("#storage-widget-preferences .timer-refresh")
                });

                $storageTotalProgress.tooltip({
                    position: {
                        my: "center bottom",
                        at: "center top"
                    },
                    tooltipClass: "progress-bar-tooltip"
                });

                updateWidget(true);
            }

            initializeStorage();
            //});

        }
    }]);

//})(jQuery, window.PHD || (window.PHD = {}), window);
