(function($, PHD, window, undefined) {
    var $form = $("#retention-form"),
        RetentionForm,
        $btnCreate = $form.find(".btn-create-policy"),
        $btnAdd = $('<button>', {
            "type": "button",
            "class": "btn btn-right btn-add-policy clearleft",
            "html": gettext("Add Policy")
        }),
        $btnCancel = $('<button>', {
            "type": "button",
            "class": "btn btn-right btn-cancel-add-policy clearleft",
            "html": gettext("Cancel")
        }),
        $btnDelete = $('<button>', {
            "type": "button",
            "class": "btn btn-right btn-delete-policy clearfull",
            "html": gettext("Delete")
        }),
        $policySelect = $form.find("#policies"),
        $policies = $form.find(".retention-forms"),
        $policyEditArea = $form.find(".manage-policy"),
        $policyTemplate = $("#policy-template"),
        $formCounter = $form.find("#id_form-TOTAL_FORMS"),
        formCount = parseInt($form.find("#id_form-INITIAL_FORMS").val(), 10),
        spinnerOptions = {
            "min": 0,
            "max": 999,
            "step": 1
        },
        dirty = false,
        staged = false,
        confirmCancelTitle = gettext("Confirm Cancellation of Changes"),
        confirmCancelMessage = gettext("Any policies you have added will lost and changes made to existing policies will not take effect."),
        regex = /__prefix__/gi;

    function clearPolicyForm() {
        $policyEditArea
            .addClass("hidden")
        .find(".active-form")
            .detach()
            .removeClass("active-form")
            .appendTo($policies);

        $policyEditArea.html("");
    }

    function validatePolicy(event) {
        if($form.valid()) {
            return true;
        } else {
            event.preventDefault();
            return false;
        }
    }

    function savePolicyName(id, name) {
        if($(".active-form").length) {
            var id = $(".active-form").data("dbid"),
                name = $(".active-form").find("[name$=name]").val();
            $policySelect.find("[value=" + id + "]").text(name);   
        }
    }

    function editPolicy(event) {
        if($form.valid()) {
            if(!$policySelect.children(":selected").hasClass("deleted")) {
                savePolicyName();
                clearPolicyForm();
                var val = $policySelect.val();
                
                if(val !== "") {
                    var $f = $policies.find("[data-dbid=" + val + "]").detach();
                    $policyEditArea.append($f.addClass("active-form")).append($btnDelete.clone().attr("value", val)).removeClass("hidden");
                }
            } else {
                $policySelect.val("");
            }
        } else {
            event.preventDefault();
            return false;
        }
    }

    function showPolicyCreateForm(event) {
        savePolicyName();
        clearPolicyForm();
        $policySelect.val("");
        var name = "My Retention Policy",
            html = $policyTemplate.html();

        $policyEditArea
            .html(html)
            .append($btnAdd.clone())
            .append($btnCancel.clone())
            .removeClass("hidden")
        .find("[name$=name]")
            .val(name)
            .focus()
            .select();

        $policyEditArea.find(".spinner-group input")
            .spinner(spinnerOptions);

        $policySelect.disable();
        staged = true;
        $("#retention-form-footer").hide();
    }

    function addNewPolicy(event) {
        if($form.valid()) {
            var name = $policyEditArea.find("[name$=name]").val(),
                val = "new" + formCount;

            $policyEditArea
                .addClass("hidden")
            .find(".empty-form")
                .attr("data-dbid", val)
                .removeClass("empty-form")
                .detach()
                .appendTo($policies)
            .find("input, label")
                .each(function() {
                    var $el = $(this), newID, newName;
                    if($el.is("input")) {
                        newID = $el.attr("id").replace(regex, formCount);
                        newName = $el.attr("name").replace(regex, formCount);
                        $el.attr("name", newName).attr("id", newID);
                    } else {
                        newID = $el.attr("for").replace(regex, formCount);
                        $el.attr("for", newID);
                    }
                });

            $policySelect.append('<option value="'+val+'">' + name + '</option>');
            $formCounter.val(++formCount);
            $policyEditArea.html("");

            $policySelect.enable();
            staged = false;
            dirty = true;
            $("#retention-form-footer").show();
        }
    }

    function cancelAddPolicy(event) {
        $policyEditArea
            .addClass("hidden")
            .html("");

        $policySelect.enable();
        staged = false;
        $("#retention-form-footer").show();
    }

    function confirmDeletePolicy(event) {
        var confirm = gettext("Are you sure you want to delete '%s'? VMs using this policy will be set to 'Typical'."),
            message = interpolate(confirm, [getPolicyName($(this).val())]);

        $policySelect.disable();
        PHD.confirmDialog(
            "<p>" + message + "</p>", {
            title: gettext("Confirm Retention Policy Deletion")
        })
        .done($.proxy(deletePolicy, this))
        .fail(function() {$policySelect.enable();});
    }

    function deletePolicy() {
        var trans = gettext("%s will be deleted when changes are saved."),
            message = interpolate(trans, [getPolicyName($(this).val()) ]);
        PHD.throwNotice({
            result: [{
                message: message
            }]
        });
        $policyEditArea.find("[name$=DELETE]").prop("checked", true);
        $policySelect
            .val("")
        .find("option[value=" + $(this).val() + "]")
            .addClass("deleted")
            .append(" [deleted]");
        clearPolicyForm();
        $policySelect.enable();
        dirty = true;
    }

    function getPolicyName(id) {
        return $policySelect.find("[value=" + id + "]").text();
    }

    function getNumRetentionBackups(recent, days, weeks, months, years) {
        if (isNaN(recent = parseInt(recent)))
            return isNaN;
        if (isNaN(days = parseInt(days)))
            return isNaN;
        if (isNaN(weeks = parseInt(weeks)))
            return isNaN;
        if (isNaN(months = parseInt(months)))
            return isNaN;
        if (isNaN(years = parseInt(years)))
            return isNaN;

        var numBackups = days >= recent ? days : recent;
        if (weeks > 0) {
            numBackups += weeks - 1;
        }
        if (months > 0) {
            numBackups += months - 1;
        }
        return numBackups;
    }

    function displayRetentionPolicyDetails(event) {
        var $this = $(this),
            parentGroup = $this.closest(".virtual-full-spinners"),
            textFormat,
            backups,
            chains;
        if (parentGroup.length) {
            backups = getNumRetentionBackups(
                parentGroup.find("[type=text]:eq(0)").val(),
                parentGroup.find("[type=text]:eq(1)").val(),
                parentGroup.find("[type=text]:eq(2)").val(),
                parentGroup.find("[type=text]:eq(3)").val(),
                parentGroup.find("[type=text]:eq(4)").val());
            if (!isNaN(backups)) {
                textFormat = ngettext(RETENTION_VIRTUAL_FILL_SINGULAR, RETENTION_VIRTUAL_FILL_PLURAL, backups);
                parentGroup.find(".virtual-full-details").text(interpolate(textFormat, [backups]));
            } else {
                parentGroup.find(".virtual-full-details").text("");
            }
        } else {
            parentGroup = $this.closest(".full-inc-spinners");
            chains = parseInt($this.val());
            if (!isNaN(chains)) {
                textFormat = gettext(RETENTION_FULL_INC);
                parentGroup.find(".full-inc-details").text(interpolate(textFormat, [chains, chains * 6]));
            } else {
                parentGroup.find(".full-inc-details").text("");
            }
        }
    }

    function initForm() {
        $.validator.addClassRules("retention-int", {
            required: true,
            digits: true,
            min: 0,
            max: 999
        });

        $.validator.addClassRules("retention-name", {
            required: true
        });

        RetentionForm = PHD.FormController($form, {
            submitOnEnter: false,
            debug: false
        });

        RetentionForm
            .validate({
                rules: {

                },
                messages: {

                }
            })
            .on("formcancel", function(event) {
                if(dirty || staged)
                    PHD.confirmDialog("<p>" + confirmCancelMessage + "</p>", { title: confirmCancelTitle })
                        .done(function() { PHD.currentDialog.wizard("close"); });
                else
                    PHD.currentDialog.wizard("close");
            })
            .on("formsubmit", function(event, data) {
                var result = data.result[0];
                if(result.code === AJAX_RESULT_SUCCESS) {
                    PHD.throwNotice(data, {});
                    PHD.currentDialog.wizard("close");
                    $(document).trigger("retentionupdate", [result.data]);
                } else if(result.code === AJAX_RESULT_ERROR) {
                    PHD.throwError(data);
                }
            });

        $policySelect.on("mousedown", validatePolicy);
        $policySelect.on("change", editPolicy);

        $btnCreate.on("click", showPolicyCreateForm);

        $form.on("change spin spinchange", "[type=text]", displayRetentionPolicyDetails);

        $form
            .on("click", ".btn-add-policy", addNewPolicy)
            .on("click", ".btn-cancel-add-policy", cancelAddPolicy)
            .on("click", ".btn-delete-policy", confirmDeletePolicy)
            .one("change spin spinchange", "[type=text]", setDirty)
        .find(".spinner-group input")
            .spinner(spinnerOptions);

        $form.valid();
    }

    function setDirty() {
        //console.log("dirty");
        dirty = true;
    }

    /*
    *   DOM Ready Function
    */
    $(function() {
        // Initialize form validation and event handlers
        initForm();
    });

})(jQuery, window.PHD || (window.PHD = {}), window);
