(function($, PHD, window, undefined) {
    var $form = $("#inventory-settings-form"),
        SettingsForm,
        $inputs = $form.find(".form-inputs input, .form-inputs select"),
        $fields = $form.find(".inventory-settings-forms"),
        $retentionSelect = $("#id_form-__prefix__-retention"),
        $wizard = PHD.currentDialog,
        retentionPolicies,
        explainTemplate = gettext($("#explanation-template").html()),
        explainTemplateFI = gettext($("#explanation-template-fi").html()),
        regex = /form-__prefix__-/,
        RETENTION_KEEP_ALL = 2;

    function presaveForm() {
        $inputs.each(function() {
            var $input = $(this),
                name = $(this).data("name");
            if(!$input.prop("disabled")) {
                $fields.find("[name$=" + name + "]").val($(this).val());
            }
        });
    }

    function initForm() {
        $inputs.each(function() {
            var name = $(this).attr("name").replace(regex, '');
            $(this).data("name", name);
        });

        SettingsForm = PHD.FormController($form, {
            serialize: function($form) {
                presaveForm();
                return $form.serialize();
            }
        });

        SettingsForm
            .validate({
                rules: {
                    "form-__prefix__-app_aware_password": {
                        required: "#id_form-__prefix__-app_aware_username:filled"
                    }
                },
                messages: {
                    "form-__prefix__-app_aware_password": {
                        required: gettext("Password is required if username is provided")
                    }
                }
            })
            .on("formcancel", function(event) {
                $wizard.wizard("close");
            })
            .on("formsubmit", function(event, data) {
                var result = data.result[0];
                if(result.code === AJAX_RESULT_SUCCESS) {
                    PHD.throwNotice(data, {});
                    $(document).trigger("inventorysettingsupdate", [result.data]);
                    $wizard.wizard("close");
                } else if(result.code === AJAX_RESULT_ERROR) {
                    PHD.throwError(data);
                }
            });

        $form.find(".formset-toggle")
            .on("change", checkboxToggleDisabled)
            .trigger("change");

        $form.find(".btn-edit-retention")
            .on("click", launchRetentionDialog);

        $retentionSelect.on("change", updateRetentionExplanation).trigger("change");

        $(document).on("retentionupdate", handleRetentionUpdate);
    }

    function indexPolicies(data) {
        retentionPolicies = {};
        data.forEach(function(policy) {
            //retentionPolicies[policy.state.id] = policy;
		// lmc remove state
            retentionPolicies[policy.id] = policy;
        });
    }

    function updateRetentionExplanation() {
        var dbid = parseInt($(this).val(), 10),
            policy = retentionPolicies[dbid];

        if(dbid === RETENTION_KEEP_ALL) {
            $form.find(".retention-explanation").html(gettext("No backups will be deleted by 'Keep All' retention."));
            $form.find(".retention-explanation-fi").html("");
        } else {
            //$form.find(".retention-explanation").html(interpolate(explainTemplate, policy.data, true));
            //$form.find(".retention-explanation-fi").html(interpolate(explainTemplateFI, policy.data, true));
	    // lmc remove .data
            $form.find(".retention-explanation").html(interpolate(explainTemplate, policy, true));
            $form.find(".retention-explanation-fi").html(interpolate(explainTemplateFI, policy, true));
        }
    }

    function handleRetentionUpdate(event, data) {
        indexPolicies(data.objects);
        var lastval = $retentionSelect.val(),
            options = makeRetentionOptions(data.objects);

        $fields.find("[name$=retention]").html(options);
        $retentionSelect.html(options).val(lastval).trigger("change");
    }

    function makeRetentionOptions(data) {
        var separatorRendered = false;
        return data.reduce(function(memo, policy) {
            if(!policy.data.system && !separatorRendered) {
                memo += '<option value="" disabled>' + BLANK_LISTBOX_TEXT + '</option>';
                separatorRendered = true;
            }                
            //return memo + "<option value=" + policy.state.id + ">" + policy.data.name + "</option>";
		// lmc remove state and data
            return memo + "<option value=" + policy.id + ">" + policy.name + "</option>";
        }, "");
    }

    function launchRetentionDialog(event) {
        event.preventDefault();
        $(document).trigger("launchdialog", {"action": "retention"});
    }

    function checkboxToggleDisabled(event) {
        var $checkbox = $(this),
            checked = $checkbox.prop("checked"),
            $inputs = $checkbox.closest(".modal-form-box").find("select, [type=text], [type=password]");

        $inputs
            .prop("disabled", !checked)
            .each(function() {
                //$fields.find("[name$=" + $(this).data("name") + "]").prop("disabled", !checked);
                $fields.find("[name$=" + $(this).data("name") + "]").prop("disabled", !checked);
            });

        if(checked) $checkbox.closest(".modal-form-box").find(".modal-dismiss").remove();
    }

    /*
    *   DOM Ready Function
    */
    $(function() {
        indexPolicies(policy_data);
        // Initialize form validation and event handlers
        initForm();
    });

})(jQuery, window.PHD || (window.PHD = {}), window);
