(function($, PHD, window, undefined) {
    var $form = $("#backup-job-form"), // The main <form> element
        BackupForm,
        $backupTreeContainer = $("#backup-tree-0"),

        // Inventory table management elements
        $inventoryTable = $("#job-inventory-table"),
        $inventoryCounter = $("#id_joborderoverride_set-TOTAL_FORMS"),
        inventoryRows = $("#id_joborderoverride_set-INITIAL_FORMS").val(),

        re = /__prefix__/gi,
        $inventoryRowTemplate = $("#inventory-row-template"),
        $inventoryRowEdit = $("#inventory-row-edit"),
        $selectedRow, $currentEditRow,

        inventory = {},

        EmailTable,

        recurrenceSpinner,
        $recurrenceCheckbox = $("#recurrence-checkbox"),
        $recurrenceInput = $("#id_recurrence"),
        $recurrencePeriod = $("#recurrence-period"),
        $recurrenceAmount = $("#recurrence-amount"),
        recurrenceAmount = $recurrenceAmount.val(),

        $startTime = $("#cal_incr_start_time"),
        $endTime = $("#id_end_time"),
        $startTimeFull = $("#cal_full_start_time"),
        $endTimeFull = $("#id_run_full_end_time"),
        $endTimeToggle = $("#end-time-toggle"),
        $fullEndTimeToggle = $("#run-full-end-time-toggle"),
        $timeInputs = $(".time-input input"),

        backupStrategy = $("#id_backup_strategy option:selected").val(),
        scheduleType = parseInt($('[name="schedule_type"]:checked').val(), 10) || SCHEDULE_TYPE_SCHEDULED,
        TreeViewData,
        backupJobTreeview, backupJobTreeviewHeader, backupJobTreeviewSearch;

        // Info for adding/saving a job
    var system_id;
    var client_id;
    var instance_id;

    var inventoryRowDefaults = {
        name: "Unknown",
        app_aware: "Not Enabled"
        /*
        verify_type: "1",
        use_cbt: true,
        quiesce: "0",
        vss_truncate_logs: "0",
        post_backup_scripts: "0"
        */
    };

    var savedJobName = $("#id_name").val();
    
    var InventoryRow = function(objectID, rowCount, values) {
        if(this instanceof InventoryRow) {
            this.ID = objectID;
            this.rowCount = rowCount;
            this.values = values || inventoryRowDefaults;
            this.idRegex = /__prefix__/g;
            this.sourceRegex = /joborderoverride_set-(__prefix__|\d{1,3}|__edit__)-/ig;
        } else {
            return new InventoryRow(objectID, rowCount, values).init();
        }
    };

    var AdoptedInventoryRow = function($el, rowCount) {
        var objectID = $el.data("dbid"),
            values = {name: $el.find(".item-name").text().trim()},
            ctrlRegex = /joborderoverride_set-(__prefix__|\d{1,3}|__edit__)-/ig;

        $el.find(".inputs").children().each(function() {
            var $ctrl = $(this),
                source = $ctrl.attr("name").replace(ctrlRegex, "");
            values[source] = InventoryRow.prototype.extractValue.call(this, $ctrl, false);
        });

        return new InventoryRow(objectID, rowCount, values).adopt($el);
    }

    InventoryRow.prototype = {
        init: function() {
            var self = this;
            this.$el = null;
            this.$inputs = null;
            this.rendered = false;
            $inventoryTable.on("backupjobeditall", function(event, $form) {
                self.update($form);
            });
            return this.render();
        },

        getName: function() {
            return this.values.name;
        },

        getValue: function(source) {
            var $ctrl = $inventoryRowEdit.find("[name$=" + source + "]");
            if($ctrl.is("select")) {
                return $ctrl.find("option[value='" + this.values[source] + "']").text();
            } else if($ctrl.is("input:checkbox")) {
                return humanize($ctrl.prop("checked"));
            } else {
                return this.values[source];
            }
        },

        render: function() {
            var self = this,
                html = $inventoryRowTemplate.html().replace(this.idRegex, this.rowCount);

            this.$el = $("<tr>", {
                    "class": "inventory-row"
                })
                .attr("data-dbid", this.ID)
                .html(html);

            this.$el.find("td:not(.row-controls)")
                .each(function() {
                    var $cell = $(this),
                        value = self.getValue($cell.data("source"));
                    $cell.html(value);
                });

            this.$el.find(".item-id input").val(this.ID);

            this.$el.find("[data-source=name]").attr("title", this.values.name);

            this.$el.find("button").data("row", this);
            this.$inputs = this.$el.find(".inputs");

            //handleQuiesceDependency(this.$inputs, true);

            this.persistValues();

            this.rendered = true;
            return this;
        },

        insert: function($container) {
            if(this.$el && $container) {
                $container.append(this.$el);    
            }
            return this;
        },

        edit: function() {
            var self = this.prepareEditDialog($inventoryRowEdit);

            $inventoryRowEdit.dialog({
                title: gettext("Edit settings for ") + self.getName(),
                resizable: false,
                draggable: false,
                modal: true,
                width: 340,
                dialogClass: "wizard-edit wizard-backup-edit",
                buttons: [
                    {
                        text: gettext("Save"),
                        click: function() {
                            self.update($(this));
                            $(this).dialog("close");
                        }
                    },
                    {
                        text: gettext("Cancel"),
                        click: function() {
                            $(this).dialog("close");
                        }
                    }
                ]
            });
        },

        // Given a control (checkbox or select), extract a value optimized
        // for storage in JavaScript or for display
        extractValue: function($ctrl, display) {
            if($ctrl.is("select")) {
                return display ? 
                    $ctrl.find("option:selected").text() : 
                    $ctrl.find("option:selected").val();
            } else if($ctrl.is("input:checkbox")) {
                return display ?
                    humanize($ctrl.prop("checked")) :
                    $ctrl.prop("checked");
            } else {
                return false;
            }
        },

        update: function($form) {
            var self = this;
            return this.saveValues($form.find("input, select"))
                .persistValues();
        },
        // Saves values to self.values[] and in the html for display
        saveValues: function($controls) {
            var self = this;
            $controls.each(function(index) {
                var $ctrl = $(this),
                    source = $ctrl.attr("name").replace(self.sourceRegex, "");

                self.values[source] = self.extractValue($ctrl, false);
                self.$el
                    .find("[data-source=" + source + "]")
                    .html(self.extractValue($ctrl, true));
            });
            return this;
        },
        // Saves values to the row's hidden controls
        persistValues: function($form, values) {
            var values = (typeof values === 'undefined') ? this.values : values,
                $form = (typeof $form === 'undefined') ? this.$inputs : $form;
            $.each(values, function(key, value) {
                var $ctrl = $form.find("[name$=" + key + "]");
                if($ctrl.is("select")) {
                    $ctrl.val(value).trigger("change");
                } else if($ctrl.is(":checkbox")) {
                    $ctrl.prop("checked", value);
                }
            });
            return this;
        },

        prepareEditDialog: function($form) {
            return this.persistValues($form);
        },

        delete: function() {
            this.$el
                .addClass("deleted")
                .find(".delete-box input")
                .prop("checked", true);
            return this;
        },

        undelete: function() {
            this.$el
                .removeClass("deleted")
                .find(".delete-box input")
                .prop("checked", false);
            return this;
        },

        adopt: function($row) {
            var self = this;
            $inventoryTable.on("backupjobeditall", function(event, $form) {
                self.update($form);
            });
            this.$el = $row;
            this.$inputs = $row.find(".inputs");
            this.rendered = true;
            return this;
        }
    };

    function humanize(value) {
        if(value === false) {
            return gettext("No");
        }
        if(value === true) {
            return gettext("Yes");
        }
        return value;
    }


    function treeViewCheckClickHandler(event, $node, simulated) {
        var dbId = $node.data("dbid"),
            type = $node.data("node");

        if(!simulated) {
            $form.validate().element($("#treeview-validator"));
        }

        if($node.hasClass("expl-included")) {
            var data_type = $node.data("type");
            // UEB does not support containers, so we need to selected all instances when a container is selected.
            if (data_type == INV_TYPE_HOST || data_type == INV_TYPE_FOLDER || data_type == INV_TYPE_RESOURCE_POOL) {
                // Add child objects from inventory when a container is checked
                if ($node.find(".children").length) {
                    $node.find(".children .checkable").not(".expl-excluded").each(function(index) {
                        var add_type = $(this).data("type");
                        // Don't explicitly add disks for backup.
                        // UEB only handles VMs and templates.
                        //if (add_type != INV_TYPE_DISK) {
                        if (add_type == INV_TYPE_VM || add_type == INV_TYPE_TEMPLATE) {
                            addInventoryRow($(this), $(this).data("dbid"));
                            addInstance($(this), $(this).data("dbid"));
                        }
                    });
                    $inventoryCounter.val($inventoryTable.find(".inventory-row").length);
                }
            } else {
                // Add the row to inventory table
                addInventoryRow($node, dbId);
                // lmc
                addInstance($node, dbId);
                // keep track of the number of rows in inventory
                $inventoryCounter.val($inventoryTable.find(".inventory-row").length);
            }
        } else if($node.hasClass("impl-included")) {
            // Mark the row for deletion
            removeInventoryRow(dbId);
            // lmc
            removeInstance($node, dbId);
        } else if($node.hasClass("expl-excluded")) {
            removeInventoryRow(dbId);
            // lmc
            removeInstance($node, dbId);
            // Remove child objects from inventory when a container is unchecked
            if($node.find(".children").length && type !== "vm") {
                $node.find(".children .checkable").not(".expl-included").each(function(index) {
                    removeInventoryRow($(this).data("dbid"));
                    // lmc - remove these from the list to protect.
                    removeInstance($(this), $(this).data("dbid"));
                });
            }
        }
        $inventoryTable.scrollTable({"height": ".modal-form-col","resize": false});
    }

    function addInventoryRow($el, dbId) {
        // See if the object is already in the table and marked for deletion. If that is
        // the case, just unmark it and return true
        if($inventoryTable.find('tr[data-dbid="' + dbId + '"]').length > 0) {
            unDeleteInventoryRow(dbId);
            return true;
        }

        // If the object we are adding has an already-included tree parent in inventory,
        // find and store it here so that its settings can be used
        var treeParentId = hasIncludedTreeParent($el, dbId),
            defaults = inventoryRowDefaults;
        if(treeParentId !== false) {
            defaults = inventory[treeParentId].values;
        }

        defaults = $.extend({}, defaults, {name: $el.children(".header").children(".item-name").text()});
        inventory[dbId] = InventoryRow(dbId, inventoryRows, defaults).insert($inventoryTable.children("tbody"));

        // Increment the inventory row counter
        inventoryRows++;
    }

    function addInstance($el, dbid) {
        var i = inventoryRows - 1;
        var rowText = "";
        var a = dbid.split('_');
        if (system_id === undefined) {
            // Only set sid if system_id is not defined first.  It will be with pre-selected items.
            system_id = a[0];
            PHD.appliance_sid = system_id;
        }
        console.log("adding instance dbid = " + dbid);
        if (a.length == 2) {
            if (client_id == 0 || client_id == undefined) {
                client_id = a[1];
            }
            rowText = '<input type="hidden" name="client_id" value="' + client_id + '" id="id_inventory_' + i + '" data-itemtype="number" />';
            console.log("adding client id " + client_id);
        } else  if (a.length > 2) {
            if (client_id == 0 || client_id == undefined) {
                client_id = a[1];
            }
            instance_id = a[a.length - 1];
            rowText = '<input type="hidden" name="instances[' + i + ']" value="' + instance_id + '" id="id_inventory_' + i + '" data-itemtype="number" />';
            console.log("adding instance id " + instance_id + " with client id " + client_id);
        }
        if (rowText != "") {
            $("#backup-job-selections").append(rowText);
        }
    }

    function removeInventoryRow(dbId) {
        if(typeof inventory[dbId] !== 'undefined') {
            inventory[dbId].delete();
        }
    }

    function removeInstance($el, dbid) {
        var a = dbid.split('_');
        if (a.length > 0) {
            instance_id = a[a.length - 1];
        }
        var selections = backupJobTreeview.$selections;
        console.log("removing instance: dbid = " + dbid + " and instance id = " + instance_id);
        var $node = selections.find('[value="' + instance_id + '"]');
        if ($node.length > 0) {
            console.log("Found: node length " + $node.length + ".  Removing instance " + instance_id);
            $node.remove();
        }
    }

    function unDeleteInventoryRow(dbId) {
        inventory[dbId].undelete();
    }

    function enableEditClick(loopIndex) {
        var dbId = $(this).data("dbid");
        inventory[dbId] = AdoptedInventoryRow($(this), loopIndex);
    }
    
    function handleQuiesceDependency($row, init) {
        var $q = $row.find("[name$=quiesce]"),
            quiesce = $q.val();

        if(typeof init !== 'undefined' && init) {
            $q.on("change", function(event) {
                handleQuiesceDependency($row);
            });
        }

        $row.find("[name$=post_backup_scripts]").prop("disabled", quiesce == 1 ? false : true);
    }

    // Recursive; walks up the TreeView and checks each ancenstor of the
    // original node for inclusion in the inventory table
    function hasIncludedTreeParent($el, dbId) {
        var uid = backupJobTreeview.universeID;

        // TODO: handle checking if the node is the 'universe' node
        // this may not always be 1, especially with multi-tenancy
        if(dbId == uid) {
            return false;
        }

        var $tree = backupJobTreeview.$container,
            $treeEl = $tree.find("[data-dbid=" + dbId + "]");
        
        var $elParent = getTreeParent($treeEl),
            parentId = $elParent.data("dbid");

        if(parentId == uid) {
            return $inventoryTable.find("[data-dbid=" + uid + "]").length > 0 ? uid : false;
        }

        if($inventoryTable.find("[data-dbid=" + parentId + "]").length) {
            return parentId;
        } else {
            return hasIncludedTreeParent($elParent, parentId);
        }
    }

    function getTreeParent($el) {
        return $el.parent().closest(".checkable");
    }

    function handleRecurranceState($el) {
        var checked = $el.is(":checked");
        recurrenceSpinner.spinner("option","disabled", !checked);
        $recurrencePeriod.prop("disabled", !checked);
        $endTime.prop("disabled", !checked);
    }

    function setRecurrenceInputValue(enabled) {
        var multiplier = enabled ? $recurrencePeriod.val() : 0;
        $recurrenceInput.val($recurrenceAmount.val() * multiplier);
        if (!enabled) {
            $recurrenceInput.prop("disabled", true);
        }
    }

    function handleScheduleTypeChange(val) {
        var $ctrl = $(".backup-job-type"),
        mode = parseInt(val, 10);

        //console.log("in handle schedule type change, val = " + val);

        $ctrl.find(".schedule-type-help p").hide();
        $ctrl.find(".schedule-type-help .option-" + mode).show();

        switch(mode) {
            case SCHEDULE_TYPE_NOW:
                $(".schedule-ctrl, .full-schedule-ctrl").hide();
                $(".backup-mode").show();
                // when changing to a "now" schedule, default selected to Full
                $("#id_strategy_now_full").prop("checked", true);

                // save current job name, change to "On Demand" if a now.
                savedJobName = $("#id_name").val();
                $("#id_name").val("On-Demand");
                $("#id_name").prop("disabled", true);
                // hide options that don't apply to a now schedule.
                $(".email_job_info").hide();

                break;
            case SCHEDULE_TYPE_SCHEDULED:
                $(".schedule-ctrl").show();
                $(".full-schedule-ctrl").hide();
                $(".backup-mode").hide();
                $(".recurrence-controls").hide();
                if(backupStrategy == BACKUP_MODE_FULL_AND_INC) $(".full-schedule-ctrl").show();

                // recall prior job name if saved.
                $("#id_name").val(savedJobName);
                $("#id_name").prop("disabled", false);
                // bring back options that do apply to a scheduled job.
                $(".email_job_info").show();
                break;
        }
    }

    function handleBackupModeState(val) {
        var $ctrl = $(".backup-strategy"),
            mode = parseInt(val, 10);

        //console.log("in handle backup mode state, val = " + val);

        $ctrl.find(".backup-strategy-help p").hide();
        $ctrl.find(".backup-strategy-help .option-" + mode).show();

        switch(mode) {
            case BACKUP_MODE_VIRTUAL_FULL:
                $(".full-schedule-ctrl").hide();
                // disable so empty input will not be sent to joborder API.
                $(".full-schedule-ctrl").prop('disabled', true);
                break;
            case BACKUP_MODE_FULL_AND_INC:
                if(scheduleType == SCHEDULE_TYPE_SCHEDULED) {
                    $(".full-schedule-ctrl").show();
                    $(".full-schedule-ctrl").prop('disabled', false);
                }
                break;
        }
    }

    function initDateControls() {
        $(".datepicker input:text").datepicker({
            showOn: "both",
            buttonText: '<img src="assets/images/calendar.png" width="24" height="24">',
            dateFormat: DATE_FORMAT,
            minDate: 0,
            yearRange: "c-1:c+2"
        }).attr("placeholder", DATE_DISPLAY_FORMAT);

        var recurrenceOptions = {
            hours: {
                max: 12,
                min: 1,
                step: 1
            },
            minutes: {
                max: 30,
                min: 15,
                step: 15
            }
        };

        if(recurrenceAmount >= 60) {
            $recurrenceAmount.val(recurrenceAmount/60);
            recurrenceSpinner = $("#recurrence-amount").spinner(recurrenceOptions.hours);
        } else {
            recurrenceSpinner = $("#recurrence-amount").spinner(recurrenceOptions.minutes);
        }

        recurrenceSpinner.on("spinchange", function() {
            var val = parseFloat($(this).val()),
                valid = $recurrencePeriod.val() === "60" ? [1,2,3,4,5,6,7,8,9,10,11,12] : [15,30];
            if(!_.contains(valid, val)) {
                val = Math.round(val);
                if(!_.contains(valid, val)) {
                    $(this).spinner("value", valid[0]);
                } else {
                    $(this).spinner("value", val);
                }
            }
        });

        $recurrencePeriod.on("change", function(event) {
            var val = $(this).val();

            switch (val) {
                case "1":
                recurrenceSpinner
                    .spinner("option", "min", 15)
                    .spinner("option", "max", 30)
                    .spinner("option", "step", 15)
                    .val(15);
                break;

                case "60":
                recurrenceSpinner
                    .spinner("option", "min", 1)
                    .spinner("option", "max", 12)
                    .spinner("option", "step", 1)
                    .val(1);
                break;
            }
        });

        $endTime.timeEntry({
                noSeparatorEntry: true,
                spinnerImage: null,
                ampmPrefix: " "
            });
        if(!jobEdit) {
            $endTime.timeEntry('setTime', DEFAULT_DO_NOT_RUN_AFTER_TIME);    
        }
        

        $recurrenceCheckbox.on("change", function(event) {
            handleRecurranceState($(this));
        }).trigger("change");
        //handleRecurranceState($recurrenceCheckbox);
    }

    function initTimeControls() {
        var endTimeOn = $endTimeToggle.prop("checked"),
            fullEndTimeOn = $fullEndTimeToggle.prop("checked"),
            settings = {
                noSeparatorEntry: true,
                spinnerImage: null,
                ampmPrefix: " "
            };

        $startTime.timeEntry(settings);
        //$endTime.timeEntry(settings);
            //.toggle(endTimeOn)
            //.prop("disabled", !endTimeOn);
        $startTimeFull.timeEntry(settings);
        //$endTimeFull.timeEntry(settings)
            //.toggle(fullEndTimeOn)
            //.prop("disabled", !fullEndTimeOn);
        $endTimeFull.prop("disabled", true);

        /*$endTimeToggle.on("change", function(event) {
            endTimeOn = $(this).is(":checked");
            if(endTimeOn) {
                $endTime.timeEntry('setTime', DEFAULT_DO_NOT_RUN_AFTER_TIME);
            }
            $endTime
                //.toggle(endTimeOn)
                .prop("disabled", !endTimeOn);
        });

        $fullEndTimeToggle.on("change", function(event) {
            fullEndTimeOn = $(this).is(":checked");
            if(fullEndTimeOn) {
                $endTimeFull.timeEntry('setTime', DEFAULT_END_TIME);
            }
            $endTimeFull
                //.toggle(fullEndTimeOn)
                .prop("disabled", !fullEndTimeOn);
        });*/
    }

    var storage_targets = [];
    var storageSelect = $("#id_storage")[0];

    function initStorage() {
        var sid = (PHD.appliance_sid === null) ? "" : "?sid=" + PHD.appliance_sid;
        var url = "/api/storage/" + sid;
        var resp = PHD.Ajax.get(url);
        resp.done = function(data) {
            storage_targets = data.storage;
            if (storage_targets) {
                storage_targets.forEach(function(obj, index) {
                    if (obj.usage !== 'archive') {
                        storageSelect.options[index] = new Option(obj.name, obj.name);
                    }
                })
            }

        };
    }

    function initForm() {

        // Handle popovers for definitions.
        $form.find(".definition").popover({
            appendTo: $form
        });


        $('#id_backup_strategy').on("change", function(event) {
            backupStrategy = $(this).val();
            handleBackupModeState(backupStrategy);
        });
        handleBackupModeState($("#id_backup_strategy option:selected").val());

        $('input:radio[name="schedule_type"]').on("change", function(event) {
            scheduleType = parseInt($(this).val(), 10);
            handleScheduleTypeChange(scheduleType);
        });
        handleScheduleTypeChange($('input:radio[name="schedule_type"]:checked').val());

        $.validator.addMethod("validate-future-date", function(value, ilelement) {
                var date = moment(value + " " + $("#cal_incr_start_time").val() + " " + tz, "MM/DD/YYYY hh:mm A Z").utc(),
                    now = moment().utc(),
                    isvalid = now.isBefore(date);
                    
                return isvalid || (scheduleType === SCHEDULE_TYPE_SCHEDULED);
            },
            gettext("Please select a future date and time")
        );

        $.validator.addMethod("validate-future-time", function(value, element) {
                var date = moment($("#cal_incr_start_date").val() + " " + value + " " + tz, "MM/DD/YYYY hh:mm A Z").utc(),
                    now = moment().utc(),
                    isvalid = now.isBefore(date);

                return isvalid || (scheduleType === SCHEDULE_TYPE_SCHEDULED);
            },
            gettext("Please select a future date and time")
        );

        BackupForm = PHD.StepForm($form, {
            namespace: "backupform",
            actions: {
                "submit": function(data) {
                    setRecurrenceInputValue($recurrenceCheckbox.prop("checked"));
                    //console.log('submit clicked, call save of schedule');
                    return this.$form.valid();
                }
            },
            serialize: function($form) {
                $timeInputs.timeEntry('option', 'show24Hours', true);
                return $form.serialize();
            },
            isEditing: jobEdit
        });

        BackupForm
            .validate({
                onkeyup: function(element) {
                    var $el = $(element);
                    if($el.is("input[type=text]")) {
                        this.element(element);
                    }
                    if($el.is(".hasTimeEntry")) {
                        this.element($("#cal_incr_start_time")[0]);
                    }
                },
                onfocusout: function(element, event) {
                    var $el = $(element),
                        self = this;
                    if($el.is("input[type=text]")) {
                        this.element(element);    
                    }
                    if($el.is(".hasTimeEntry")) {
                        this.element($("#cal_incr_start_time")[0]);
                    }
                },
                rules: {
                    name: {
                        required: true
                    },
                    "calendar[0].start_date": {
                        required: true,
                        dateFmt: true,
                        "validate-future-date": true
                    },
                    "calendar[0].start_time": {
                        required: true,
                        time12h: true,
                        "validate-future-time": true
                    },
                    run_full_start_time: {
                        required: true,
                        time12h: true
                    },
                    end_time: {
                        required: function(element) {
                            if ($recurrenceCheckbox.prop("checked")) {
                                return $(element).val() == "";
                            } else {
                                return false;
                            }
                        },
                        time12h: true
                    }
                },
                messages: {
                    name: {
                        required: gettext("Job Name is required")
                    },
                    "calendar[0].start_date": {
                        required: gettext("Start Date is required")
                    },
                    "calendar[0].start_time": {
                        required: gettext("Start Time is required")
                    },
                    run_full_start_time: {
                        required: gettext("Full Schedule Start Time is required")
                    },
                    end_time: {
                        required: gettext("Please enter an end time for recurrence")
                    }
                }
            })
            .on("formcancel", function() {
                PHD.currentDialog.wizard("close");
            })
            .on("formsubmit", function(event, data) {
                PHD.currentDialog.wizard("close");

                scheduleType = parseInt(scheduleType, 10);
                if (scheduleType == SCHEDULE_TYPE_NOW) {
                    backupNowReturn = data.data;
                    PHD.wizard("app/joborders/backup/now_dialog.html", "backup-now", {
                        title: gettext("On-Demand Job Status"),
                        dialogClass: "standalone",
                        height: 290,
                        helpArticle: PHD.App.getHelpLink("backup", true),
                        close: function(event, ui) {
                            // Include the wizard standard close code (from main.js)
                            $("body")
                                .removeClass("dialog-open")
                                .css("overflow", "hidden");
                            $(this).wizard("destroy").remove();

                            var scope = angular.element(document).scope();
                            if(scope.user.showTour) {
                                window.location.href = "#";
                            }
                        }
                    });
                } else {
                    if (data && data.result && data.result[0] && data.result[0].message !== undefined) {
                        // The structure returned on a job is the same as "now" status, so use the same dialog.
                        backupNowReturn = data.result;
                        PHD.wizard("app/joborders/backup/now_dialog.html", "backup-now", {
                            title: gettext("Backup Job Status"),
                            dialogClass: "standalone",
                            height: 290,
                            helpArticle: PHD.App.getHelpLink("backup", true),
                            close: function(event, ui) {
                                // Include the wizard standard close code (from main.js)
                                $("body")
                                    .removeClass("dialog-open")
                                    .css("overflow", "hidden");
                                $(this).wizard("destroy").remove();

                                var scope = angular.element(document).scope();
                                if(scope.user.showTour) {
                                    window.location.href = "#";
                                }
                            }
                        });
                    } else {
                        // lmc - we checked HTTP status earlier, so if we get here, we know the call was successful.
                        if (window.location.href.indexOf("#/protect") !== -1) {
                            console.log("we are in protect, not jobs");
                            var options = {},
                                tab = (parseInt(scheduleType, 10) === SCHEDULE_TYPE_NOW) ? "#0" : "#1";
                            options["buttons"] = [
                                {
                                    text: gettext("View Jobs"),
                                    click: function() {
                                        window.location.href = "#/jobs/" + tab;
                                        $(this).dialog("destroy").remove();
                                    }
                                }
                            ];
                            // check to see if there is a message to be displayed.
                            if (data && data.result && data.result[0]) {
                                data.result[0].message = "Job successfully saved.";
                                PHD.throwNotice(data, options);
                            }
                        }

                        //PHD.currentDialog.wizard("close");
                        var scope = angular.element(document).scope();
                        if(scope.user.showTour) {
                            window.location.href = "#";
                        }
                    }
                }
            });

        BackupForm.submit = function() {
            var self = this;
            var $now = $("#id_job_now");
            var backup_now = $now.is(":checked");
            var save_message = backup_now ? "Starting On-Demand Backup Job..." : "Saving Backup Job...";

            var load = PHD.showLoadingIndicator("body", true, save_message);

            this.$buttons.blur().disable();

            var submitData = form2js(this.$form.attr("id"), '.', false);

            var $editing_id = this.$form.find("#edit_object_id");
            console.log('edit id text = ' + $editing_id.text() + " and appliance id is " + PHD.appliance_sid);
            var sid = (PHD.appliance_sid === null) ? "" : "?sid=" + PHD.appliance_sid;
            var url = this.$form.attr("action");
            var hasItems = false;
            if (backup_now) {
                console.log("Backup Now! submitData is " + JSON.stringify(submitData));
                url = "/api/backups/" + sid;
                var backupObj = {};
                //backupObj.system_id = parseInt(PHD.appliance_sid, 10);
                var calendar = submitData.calendar;
                // see if full or incremental is checked.
                if ($("#id_strategy_now_full").is(":checked")) {
                    backupObj.backup_type = "Full";
                } else {
                    backupObj.backup_type = "Incremental";
                }
                // set verify level for now job
                if ($("#id_verify_level").is(":checked")) {
                    backupObj.verify = "inline";
                } else {
                    backupObj.verify = "none";
                }
                hasItems = !(submitData.instances === undefined && submitData.client_id === undefined);
                if (submitData.instances !== undefined) {
                    backupObj.instances = submitData.instances;
                } else if (submitData.client_id !== undefined) {
                    backupObj.client_id = submitData.client_id;
                }
                if (hasItems) {
                    backupObj = JSON.stringify(backupObj);
                    console.log("Backup now with obj " + backupObj);
                    PHD.Ajax
                        .put(url, backupObj)
                        .done(function(data) {
                            self.$form.trigger("formsubmit", [data, self.$form, self]);
                        })
                        .always(function() {
                            self.$buttons.enable();
                            PHD.hideLoadingIndicator(load);
                        });
                } else {
                    self.$buttons.enable();
                    PHD.hideLoadingIndicator(load);
                    PHD.throwError({"result": [{"message": gettext("You must specify assets to protect.")}]});
                }
            } else {
                // handle verify option in a special way, as it is not boolean:
                // see if verify is selected and if not, set to "none".
                if (submitData !== undefined) {
                    submitData.verify = submitData.verify == true ? "inline" : "none";
                }
                submitData.calendar.forEach(function(obj, index) {
                    if (obj.backup_type == 'Incremental') {
                        obj.start_time = DateTimeHandler.convertTimeToAPI("00:00:00", $("#cal_incr_start_time").val());
                    } else {
                        obj.start_time = DateTimeHandler.convertTimeToAPI("00:00:00", $("#cal_full_start_time").val());
                    }
                });
                hasItems = !(submitData.instances === undefined && submitData.client_id === undefined);
                if (hasItems) {
                    if (validateInventory()) {
                        submitData = JSON.stringify(submitData);
                        console.log('=== Submit form: object = ' + submitData);
                        if ($editing_id.text() != "none") {
                            var myID = $editing_id.text();
                            url += myID + "/" + sid;
                            console.log("IS Editing, url is " + url);
                            PHD.Ajax
                                .put(url, submitData)
                                .done(function(data) {
                                    self.$form.trigger("formsubmit", [data, self.$form, self]);
                                })
                                .always(function() {
                                    self.$buttons.enable();
                                    PHD.hideLoadingIndicator(load);
                                });
                        } else {
                            url += sid;
                            console.log("Is Adding, not Editing, url is " + url);
                            PHD.Ajax
                                .post(url, submitData)
                                .done(function(data) {
                                    self.$form.trigger("formsubmit", [data, self.$form, self]);
                                })

                                .always(function() {
                                    self.$buttons.enable();
                                    PHD.hideLoadingIndicator(load);
                                });
                        }
                    } else {
                        self.$buttons.enable();
                        PHD.hideLoadingIndicator(load);
                        var editJob = jobEdit ? "  When editing a job, all assets must be of the job's current hypervisor/application type." : "";
                        PHD.throwError({"result": [{"message": gettext("You have selected incompatible assets for this backup job.") + "<br/><br/>" +
                            gettext("All assets in the job must be of the same hypervisor type (VMware or Hyper-V), or the same application, and on the same appliance.") +
                            gettext(editJob)}]});
                    }
                } else {
                    self.$buttons.enable();
                    PHD.hideLoadingIndicator(load);
                    PHD.throwError({"result": [{"message": gettext("You must specify assets to protect.")}]});
                }
            }

            return this;
        };

        //initStorage();
    }

    /*
     * Validate the inventory items to see that they match the scheduling rules.
     */
    function validateInventory() {
        var valid_sid;
        var valid_client_id;
        var valid_app_id;

        // Assume valid at first.
        var compatible_sid = true;
        var compatible_client = true;
        var compatible_app = true;

        $inventoryTable.find(".inventory-row").each(function() {
            console.log("dbid = " + $(this).data("dbid"));
            if ($(this).hasClass("deleted")) {
                // skip deleted inventory items.
            } else {
                var id = $(this).data("dbid");
                // id components are "_" delimited, e.g.,
                //
                // system-id_client-id_app-id_instance-id
                // system-id_client-id_app-id_host-guid_instance-id
                // system-id_client-id
                //
                var a = id.split("_");
                if (a.length > 1) {
                    // Ensure system id's are valid and compatible.
                    if (valid_sid === undefined) {
                        valid_sid = a[0];
                    } else if (valid_sid != a[0]) {
                        compatible_sid = false;
                        return false;
                    }
                    if (jobEdit && valid_sid != PHD.appliance_sid) {
                        compatible_sid = false;
                        return false;
                    }
                }
                if (a.length > 2) {
                    // Ensure client id's (if specified) are valid and compatible
                    if (valid_client_id === undefined) {
                        valid_client_id = a[1];
                    } else if (valid_client_id != a[1]) {
                        compatible_client = false;
                        return false;
                    }
                    if (jobEdit && valid_client_id != client_id) {
                        compatible_client = false;
                        return false;
                    }
                    // Ensure app id's (if specified) are valid and compatible
                    if (valid_app_id === undefined) {
                        valid_app_id = a[2];
                    } else if (valid_app_id != a[2]) {
                        compatible_app = false;
                        return false;
                    }
                }
            }
        });

        return (compatible_sid && compatible_client && compatible_app);
    }

    function initTreeView() {
        var load = PHD.showLoadingIndicator($backupTreeContainer);
        console.log('in init treeview, loader is ' + load);

        var hypervisors = $("#backup-job-treeview-header").find("select").map(function() {
                return parseInt($(this).val(), 10);
            }).get(),
            tvTask = PHD.TreeViewBuilder({
                hypervisorView: hypervisors,
                disks: 1,
                checkboxes: true,
                loader: load
            });

        tvTask.done(function(tree) {
            TreeViewData = tree.tree;
            $backupTreeContainer.html(tree.html);

            backupJobTreeview = PHD.TreeView(
                $backupTreeContainer, 
                $("#backup-job-selections")
            );

            backupJobTreeviewHeader = PHD.TreeViewHeader(
                $("#backup-job-treeview-header"), 
                backupJobTreeview, 
                "backupjob"
            );
            backupJobTreeviewSearch = PHD.TreeViewSearch(
                $("#backup-job-treeview-find"), 
                backupJobTreeview, 
                backupJobTreeview.$container, 
                backupJobTreeview.$container.find(".tree-universe")
            );

            // lmc - pre-select any items that were checked
            if (preSelected != null) {
                selectPreselected(backupJobTreeview.$container);
                backupJobTreeview.$selections = $("#backup-job-selections");
            }

            // Attach event handler for Treeview checkbox clicks
            backupJobTreeview.$container.on("checkclick", ".checkbox", treeViewCheckClickHandler);
            var initList = jobEdit;
            backupJobTreeview.initSelections(initList);

            PHD.hideLoadingIndicator(load);

            initializeInventoryTable();
        });

    }

    function updateTreeView(hypervisors) {
        var load = PHD.showLoadingIndicator($backupTreeContainer);
        console.log('in updatetreeview, loader is ' + load);
        var tvTask = PHD.TreeViewBuilder({
                hypervisorView: hypervisors,
                disks: 1,
                checkboxes: true,
                loader: load
            });

        tvTask.done(function(tree) {
            TreeViewData = tree.tree;
            $backupTreeContainer.html(tree.html);
            PHD.hideLoadingIndicator(load);
        });
    }

    function prepareEditControls() {
        $inventoryRowEdit.html(
             $inventoryRowEdit.html().replace(/__prefix__/g, "__edit__")
        );

        $(".edit-all")
            .on("click", function() {
                if($inventoryTable.find(".inventory-row").length) {
                    $inventoryRowEdit.dialog({
                        title: gettext("Edit settings for all inventory"),
                        resizable: false,
                        draggable: false,
                        modal: true,
                        width: 340,
                        dialogClass: "wizard-edit wizard-backup-edit",
                        buttons: {
                            "Save": function() {
                                $inventoryTable.trigger("backupjobeditall", [$(this)]);
                                $(this).dialog("close");
                            },
                            "Cancel": function() {
                                $(this).dialog("close");
                            }
                        }
                    });
                }
            })
            .tooltip({
                position: {
                    my: "left+20 center",
                    at: "left center"
                }
            });
        //handleQuiesceDependency($inventoryRowEdit, true);
    }

    /*
     * If this is an edit/copy, initialize existing inventory rows.
     */
    function initializeInventoryTable() {
        if($inventoryTable.find(".inventory-row").length) {
            $inventoryTable.find(".inventory-row").each(enableEditClick);
            PHD.currentDialog.one("wizardfocus", function() {
                $inventoryTable.scrollTable({"height": ".modal-form-col","resize": false});
         });
        }
    }

    /*
     * Find pre-selected items, and add their nodes to the inventory on the rhs pane.
     */
    function selectPreselected($container) {
        var item_array = preSelected.split('&');
        var notfound_array = [];
        if (item_array.length > 0) {
            //console.log('length is ' + item_array.length);
            for (var i = 0; i < item_array.length; i++) {
                var item = item_array[i];
                var inventory = item.split('=');
                if (inventory.length == 2) {
                    // inventory={id}
                    var id = inventory[1];
                    // split the id into pieces.
                    var a = id.split("_");
                    if (a.length > 1) {
                        system_id = a[0];
                        PHD.appliance_sid = system_id;
                        client_id = a[1];
                    }
                    if (a.length > 3) {
                        instance_id = a[a.length - 1];
                    }
                    var found = false;
                    console.log("trying to find id " + id);
                    var $node = $container.find("[data-dbid=" + id + "]");
                    if ($node.html() == undefined) {
                        // try again, with our id pieces.
                        console.log("first search: undefined");
                        $node = $container.find("[data-dbid^=" + system_id + "][data-dbid$=" + "_" + instance_id + "]");
                        if ($node.html() !== undefined) {
                            found = true;
                            var dbid = $node.data("dbid");
                            var rowText = '<input type="hidden" name="inventory" value="' + dbid + '" id="id_inventory_' + i + '"/>';
                            $("#backup-job-selections").append(rowText);
                            //rowText = '<input type="hidden" name="instances[' + i + ']" value="' + instance_id + '" id="id_inventory_' + i + '" data-itemtype="number" />';
                            //$("#backup-job-selections").append(rowText);
                            //addInventoryRow($node, dbid);
                            console.log("NODE was undefined: node dbid = " + $node.data("dbid") + " and row text is " + rowText);
                        }
                    } else {
                        found = true;
                        var item_id = (instance_id > 0) ? instance_id : client_id;
                        var rowText = '<input type="hidden" name="inventory" value="' + id + '" id="id_inventory_' + i + '"/>';
                        $("#backup-job-selections").append(rowText);
                        if (instance_id > 0) {
                            rowText = '<input type="hidden" name="instances[' + i + ']" value="' + item_id + '" data-itemtype="number" />';
                        } else {
                            rowText = '<input type="hidden" name="client_id" value="' + item_id + '" data-itemtype="number" />';
                        }
                        $("#backup-job-selections").append(rowText);
                        console.log("NODE not undefined:  add id " + id + " and row text is " + rowText);
                        addInventoryRow($node, id);
                    }
                    if (!found) {
                        // Add to the list of items not found.
                        notfound_array.push(item);
                    }
                }
            }
        } else {
            PHD.throwNotice({"result": [{"message": gettext("No items are selected for protection in this joborder.")}]});
        }
        if (notfound_array.length > 0) {
            var verb = notfound_array.length == 1 ? "is " : "are ";
            PHD.throwNotice({"result": [{"message": gettext("There ") + verb + notfound_array.length +
                                                    gettext(" item(s) in this job not available in the inventory of possible items to protect.  ") +
                                                    gettext("A likely cause is that an item was deleted but inventory sync has not run yet.") + "<br/><br/>" +
                                                    gettext("To clear up this warning, go to the Options menu to re-run inventory sync, re-edit and save this job.")}]});
        }
    }

    function setJobOptions() {
        if (jobOptions != null) {
            calendar = jobOptions.calendar;
            console.log("calendar = " + JSON.stringify(calendar));
            // part of details string? <!--<small>(The current time in US/Eastern is July 27, 2014, 10:41 a.m..)</small>-->

            var hasFull = false;
            for (var i = 0; i < calendar.length; i++) {
                var cal = calendar[i];
                if (cal.backup_type == 'Full') {
                    hasFull = true;
                    $("#cal_full_start_date").val(cal.start_date);
                    $("#cal_full_schedule_run").val(cal.schedule_run);
                    var run_on = cal.run_on;
                    console.log("run_on:full " + JSON.stringify(run_on));
                    $("#cal_full_run_on_sun").prop("checked", ($.inArray("0", run_on) != -1));
                    $("#cal_full_run_on_mon").prop("checked", ($.inArray("1", run_on) != -1));
                    $("#cal_full_run_on_tue").prop("checked", ($.inArray("2", run_on) != -1));
                    $("#cal_full_run_on_wed").prop("checked", ($.inArray("3", run_on) != -1));
                    $("#cal_full_run_on_thu").prop("checked", ($.inArray("4", run_on) != -1));
                    $("#cal_full_run_on_fri").prop("checked", ($.inArray("5", run_on) != -1));
                    $("#cal_full_run_on_sat").prop("checked", ($.inArray("6", run_on) != -1));
                    //$("#cal_full_start_time").val(cal.start_time);
                    $("#cal_full_start_time").timeEntry('setTime', DateTimeHandler.convertTimeFromAPI(cal.start_time));
                } else if (cal.backup_type == 'Incremental') {
                    $("#cal_incr_start_date").val(cal.start_date);
                    $("#cal_incr_schedule_run").val(cal.schedule_run);
                    var run_on = cal.run_on;
                    console.log("run_on:full " + JSON.stringify(run_on));
                    $("#cal_incr_run_on_sun").prop("checked", ($.inArray("0", run_on) != -1));
                    $("#cal_incr_run_on_mon").prop("checked", ($.inArray("1", run_on) != -1));
                    $("#cal_incr_run_on_tue").prop("checked", ($.inArray("2", run_on) != -1));
                    $("#cal_incr_run_on_wed").prop("checked", ($.inArray("3", run_on) != -1));
                    $("#cal_incr_run_on_thu").prop("checked", ($.inArray("4", run_on) != -1));
                    $("#cal_incr_run_on_fri").prop("checked", ($.inArray("5", run_on) != -1));
                    $("#cal_incr_run_on_sat").prop("checked", ($.inArray("6", run_on) != -1));
                    //$("#cal_incr_start_time").val(cal.start_time);
                    $("#cal_incr_start_time").timeEntry('setTime', DateTimeHandler.convertTimeFromAPI(cal.start_time));
                    if (cal.recurrence !== undefined) {
                        console.log("Edit: set recurrence: " + cal.recurrence);
                        $recurrenceAmount.val(cal.recurrence);
                    }
                }
            }
            if (hasFull) {
                $("#id_strategy_inc4ever").prop("checked", false);
                $("#id_strategy_full_incr").prop("checked", true);
                handleBackupModeState($("#id_backup_strategy option:selected").val());
            }
            if (jobOptions.verify !== undefined) {
                $("#id_verify_level").prop("checked", jobOptions.verify !== 'none');
            }
            if (jobOptions.email_report !== undefined) {
                $("#id_schedule").prop("checked", jobOptions.email_report == true);
            }
            if (jobOptions.failure_report !== undefined) {
                $("#id_failure").prop("checked", jobOptions.failure_report == true);
            }
            if (jobOptions.storage !== undefined) {
                // set $("#id_storage"), for now disabled.
                //$("#id_storage").prop("disabled", true);
            }
        } else {
            // a new job
            var start_date = moment().format("MM/DD/YYYY");
            $("#cal_incr_start_date").val(start_date);
        }
    }

    /*
    *   DOM Ready Function
    */
    $(function() {
        // Initialize form validation and event handlers
        initForm();

        // Initialize time controls
        initTimeControls();

        // Initialize date controls
        initDateControls();

        // Load, build and initialize the Treeview
        initTreeView();

        // Disable any form fields that are disabled by default
        $(".start-disabled").find("input, select").toggleDisabled();

        EmailTable = PHD.EmailTable($form);

        $inventoryTable.on("click", ".find-item", function() {
            var id = $(this).closest("tr").data("dbid");
            backupJobTreeviewSearch.goTo(id);
        });

        $inventoryTable.on("click", ".edit-row", function() {
            var id = $(this).closest("tr").data("dbid"),
                row = inventory[id];
            row.edit();
        });

        initializeInventoryTable();

        console.log('in dialog.backupjob.js, jobEdit is ' + jobEdit + ' and jobID is ' + jobID);
        // If this is an edit, disable "Now" radio button
        if (jobEdit) {
            //$("#id_schedule_type_0").prop("disabled", true);
            $("#id_job_scheduled").prop("disabled", true);
            $("#id_job_now").prop("disabled", true);
        }

        $inventoryTable.find("th .definition").popover({
            appendTo: $form,
            popoverClass: "backup-job-popover"
        });

        prepareEditControls();

        setJobOptions();

        $(document)
            // Attach event handler to Hypervisor view select box changes
            .on("hypervisorviewchange.backupjob", function(event, views) {
                updateTreeView(views);
                backupJobTreeview.clearSelections();
                $inventoryTable.find(".inventory-row").each(function() {
                    inventory[$(this).data("dbid")].delete();
                });
            });
    });

})(jQuery, window.PHD || (window.PHD = {}), window);
