(function($, PHD, window, undefined) {

    var $form = $("#recovery-job-form"),
        RecoverForm,
        $backupCatalog = $("#recovery-table"),
        $backupFilter = $("#recovery-backup-catalog-filter"),
        backupCatalog, backupFilter,
        backupCatalogOptions = {},

        $inventoryTable = $("#recovery-inventory-table"),
        $inventoryCounter = $("#id_override-TOTAL_FORMS"),
        inventoryRows = 0, //$("#id_override-INITIAL_FORMS").val(),
        inventory = {},

        inventoryRowTemplate = $("#inventory-row-template").html(),
        inventoryDiskTemplate = $("#inventory-disk-template").html(),
        inventoryNICTemplate = $("#inventory-nic-template").html(),

        $vmSuffix = $("#vm-name-suffix"),
        $vmNameSuffixToggle = $("#toggle_id_vm_name_suffix"),
        $vmNameSuffix = $("#id_vm_name_suffix"),
        $vmNameSuffixAction = $("#id_vm_name_suffix_extra"),
        VM_SUFFIX_APPEND = 1,
        VM_SUFFIX_REMOVE = 0,

        EmailTable,

        $startDate = $("#id_start_date"),
        $startTime = $("#id_start_time"),
        $scheduleType = $("[name=schedule_type]"),
        scheduleType = $scheduleType.val();


    var restore_targets = [];
    var restore_targets_initialized = false;
    var restore_networks = [];
    var restore_switches = {};
    var hostSelect = $("#id_host")[0];
    var resourceSelect = $("#id_resource")[0];
    var storageSelect = $("#id_storage")[0];
    var networkSelect = $("#id_network")[0];
    var switchSelect = $("#id_switch")[0];
    var restoreObj = null;
    var applicationType;
    var hostRequired = false;
    var resourcePoolRequired = false;
    var storageRequired = false;
    var networkRequired = false;
    var switchRequired = false;


    var InventoryRow = function(dbid, vmid, info, index, adopted) {
        if(this instanceof InventoryRow) {
            console.log("--> add InventoryRow: dbid is " + dbid + "and vmid = " + vmid + " and info = " + JSON.stringify(info) + " and index = " + index);
            this.dbid = dbid;
            this.vmid = vmid;
            this.info = info;
            this.values = {};
            this.index = index;

            this.overrideRegex = /override-__prefix__/ig;
            this.diskRegex = /disk-__prefix__/ig;
            this.nicRegex = /nic-__prefix__/ig;

            this.prefix = ""; // lmc - no override "override-"+this.index+"-";

            this.$el = null;
            this.adopted = adopted || false;
        } else {
            return new InventoryRow(dbid, vmid, info, index, adopted).init();
        }
    };

    var AdoptedInventoryRow = function($el, dbid, vmid, info, index) {
        return new InventoryRow(dbid, vmid, info, index, true).adopt($el);
    };

    InventoryRow.prototype = {
        init: function() {
            this.render()
                .initControls()
                .initDestinationSelection()
                .initHostSelection()
                .initValues()
                .initDisks()
                .initNICs()
                .initLocation()
                .initPath()
                .update(this.$inputs);

            if(!this.adopted) this.processName($vmNameSuffix.val(), VM_SUFFIX_APPEND);

            return this;
        },

        render: function() {
            if(!this.$el) {
                this.$el = $("<tr>", { "class": "inventory-row" })
                    .attr("data-dbid", this.id)
                    .attr("data-vmid", this.instance_id)
                    .html(inventoryRowTemplate.replace(this.overrideRegex, this.setID("override", this.instance_id)));
                var $name = this.$el.find("[data-source=\"name\"]");
                this.values["name"] = this.info.name;
                $name.text(this.info.name);
            }

            return this;
        },

        initControls: function() {
            this.$cells = this.$el.children(":not(.row-controls)");
            this.$controls = this.$el.children(".row-controls");
            this.$inputs = this.$controls.find(".inputs");

            this.$diskControls = this.$inputs.find(".edit-recovery-disks");
            this.$nicControls = this.$inputs.find(".edit-recovery-nics");

            this.$controls.children(".edit-row")
                .data("row", this)
                .on("click", $.proxy(this.edit, this));

            return this;
        },

        initDestinationSelection: function() {
            var self = this;
            this.$inputs.find(".btn-select-destination")
                .on("click", function(event) {
                    event.preventDefault();
                    var url = "/api/restore/targets/" + self.info.environment + "/" + self.dbid + "/";
                    PHD.miniWizard(url, "recover-destination", {
                        "title": gettext("Select Target Destination"),
                        "width": 400,
                        "height": 500
                    }, false);
                    $(document).one("recoverdestination", $.proxy(self.handleDestinationSelection, self));
                });
            return this;
        },

        initHostSelection: function() {
            var self = this;
            if (typeof this.info !== 'undefined' && typeof this.info.hypervisor !== 'undefined') {
                if (this.info.hypervisor !== HYPERVISOR_VMWARE) {
                    this.$inputs.find("select[name$=target_host]")
                        .html(this.buildHostOptions())
                        .on("change", function () {
                            self.buildHardwareOptions(parseInt($(this).val(), 10));
                        });
                }
            }

            if(!this.adopted) {
                this.buildHardwareOptions(this.info.target_host.id);
            }
            return this;
        },

        initValues: function() {
            if(!this.adopted) {
                this.$controls.children(".meta").find(".item-id input").val(this.dbid);
                this.persistValues(void 0, this.info);
            }
            this.saveValues(this.$inputs.find("input:text, select"));

            // default target network to show not applicable if not an IR job.
            if (!isIRJob && this.$el) {
                this.$el
                    .children("[data-source=" + "target_network" + "]")
                    .html("Not Applicable");
            }
            return this;
        },

        buildHardwareOptions: function(hostID) {
            this.$inputs.find("select[name$=storage]").html(this.buildStorageOptions(hostID));
            this.$inputs.find("select[name$=network]").html(this.buildNetworkOptions(hostID));
        },

        initDisks: function() {
            if (this.info !== undefined && this.info.disks !== null) {
                this.$diskControls.prev(".legend").append(" (" + this.info.disks.length + ")");
                if (!this.adopted) {
                    this.$inputs.find(".disk-management-form [name$=TOTAL_FORMS]").val(this.info.disks.length);
                    this.info.disks.forEach(this.addDiskControl, this);
                }
            }
            return this;
        },

        initNICs: function() {
            if (typeof this.info !== 'undefined' && typeof this.info.nics !== 'undefined') {
                this.$nicControls.prev(".legend").append(" (" + this.info.nics.length + ")");
                if (!this.adopted) {
                    this.$inputs.find(".nic-management-form [name$=TOTAL_FORMS]").val(this.info.nics.length);
                    this.info.nics.forEach(this.addNICControl, this);
                } else {
                    this.info.nics.forEach(this.initNicData, this);
                }
                this.$nicControls.find(".nic-controls").each(this.initMACAddress);
            }
            return this;
        },

        initLocation: function() {
            if (typeof this.info !== 'undefined' && typeof this.info.hypervisor !== 'undefined') {
                if (this.info.hypervisor === HYPERVISOR_VMWARE) {
                    this.$inputs.find("[name$=target_host]").readonly();
                    this.$inputs.find("[name$=target_folder]").readonly();
                    this.$inputs.find("[name$=target_resource]").readonly();
                } else {
                    this.$inputs.find(".btn-select-destination").hide();
                    this.$inputs.find("[name$=target_resource]").closest(".control-group").hide();
                    this.$inputs.find("[name$=target_folder]").closest(".control-group").hide();
                }
            }
            return this;
        },

        initPath: function() {
            if (typeof this.info !== 'undefined' && typeof this.info.hypervisor !== 'undefined') {
                if (this.info.hypervisor !== HYPERVISOR_HYPERV) {
                    this.$inputs.find("[name$=path]").closest(".control-group").hide();
                }
            }
            return this;
        },

        handleDestinationSelection: function(event, selections) {
            this.setValue("target_host", selections.host.id)
                .setValue("target_folder", selections.location.id)
                .setValue("target_resource", selections.resource.id);

            this.buildHardwareOptions(selections.host.id);
            this.persistValues();
        },

        buildHostOptions: function() {
            var hostOptions = this.info.available_hosts.map(function(host) {
                    return '<option value="' + host.id + '">' + host.name + '</option>';
                }).join("");

            return hostOptions;
        },

        buildStorageOptions: function(hostId) {
            hostId = hostId || this.getValue("target_host");
            var host = _.findWhere(this.info.available_hosts, {id: hostId}),
                storageOptions = host !== undefined && host.storage !== undefined && host.storage.map(function(datastore) {
                    return '<option value="' + datastore.id + '">' + datastore.name + '</option>';
                }).join("");

            return storageOptions;
        },

        buildNetworkOptions: function(hostId) {
            hostId = hostId || this.getValue("target_host");
            var host = _.findWhere(this.info.available_hosts, {id: hostId}),
                networkOptions = host !== undefined && host.networks !== undefined && host.networks.map(function(network) {
                    return '<option value="' + network.id + '">' + network.name + '</option>';
                }).join("");

            return '<option value="">'+gettext("Not Connected")+'</option>' + networkOptions;
        },

        addDiskControl: function(value, index, disks) {
            var html = inventoryDiskTemplate.replace(this.diskRegex, this.setID("disk", index)).replace(this.overrideRegex, this.setID("override", this.index)),
                $row = $("<div>", {"class":"disk-controls", html:html});

            this.$diskControls.append($row);
            $row.find(".item-id input").val(value.id);
            $row.find(".include-label label").append(" " + value.name + " (" + value.total_storage + ")");
            $row.find("[name$=storage]").html(this.buildStorageOptions()).val(value.storage);
            $row.find("[name$=path]").val(value.path);
            $row.data("values", value);
        },

        addNICControl: function(value, index, nics) {
            var html = inventoryNICTemplate.replace(this.nicRegex, this.setID("nic", index)).replace(this.overrideRegex, this.setID("override", this.index)),
                $row = $("<div>", {"class":"nic-controls", html:html});

            this.$nicControls.append($row);
            $row.find(".item-id input").val(value.id);
            $row.find(".include-label label").append(" " + value.nic);
            $row.find("[name$=network]").html(this.buildNetworkOptions()).val(value.network);
            $row.find("[name$=mac_address]").val(value.mac_address);
            $row.data("values", value);
        },

        initNicData: function(value) {
            this.$nicControls
                .find(".item-id input[value=" + value.id + "]")
            .closest(".nic-controls")
                .data("values", value);
        },

        initMACAddress: function(index, row) {
            var $row = $(this),
                value = $row.data("values");

            $row.find("[name$=mac_address]").each(function() {
                var $input = $(this).data("mac-user", value.mac_address).data("mac-original", value.mac_address),
                    autoGen = $row.find("[name$=auto_generate_mac]").prop("checked");
                $input.prop("readonly", autoGen);
                if(!autoGen) {
                    $input.val($input.data("mac-user"));
                    $row.find(".mac-auto").show();
                    $row.find(".mac-edit").hide();
                } else {
                    $input.val(gettext("auto-generate"));
                    $row.find(".mac-auto").hide();
                }
            });

            $row.find(".mac-edit").on("click", function() {
                var $input = $(this).hide().nextAll(".mac-auto").show().nextAll("input");
                $row.find("[name$=auto_generate_mac]").prop("checked", false);

                $input.prop("readonly", false)
                    .val($input.data("mac-user"))
                    .addClass("validate-mac-address")
                    .on("blur.valid-mac", function() {
                        var macregex = /^([a-zA-Z0-9]{2}\:){5}[a-zA-Z0-9]{2}$/;
                        if(!$(this).val().match(macregex)) {
                            $(this).addClass("error");    
                        } else {
                            $(this).removeClass("error").data("mac-user", $(this).val());    
                        }
                    });

            });

            $row.find(".mac-auto").on("click", function() {
                var $input = $(this).hide().prevAll(".mac-edit").show().nextAll("input");
                $row.find("[name$=auto_generate_mac]").prop("checked", true);

                $input.prop("readonly", true)
                    .val(gettext("auto-generate"))
                    .removeClass("validate-mac-address")
                    .off("blur.valid-mac");
            });

            $row.find(".mac-undo").on("click", function() {
                var $input = $(this).nextAll("input"),
                    editing = $input.prop("readonly");

                if(!editing) {
                    $input.val($input.data("mac-original")).data("mac-user", $input.data("mac-original"));
                }
            });
        },

        insert: function($container) {
            if(this.$el && $container) {
                $container.append(this.$el);    
            }
            return this;
        },

        edit: function() {
            var self = this;

            this.$inputs.show().dialog({
                title: gettext("Edit settings for ") + this.getValue("name"),
                resizable: false,
                draggable: false,
                modal: true,
                width: 440,
                dialogClass: "wizard-edit wizard-recover-edit",
                buttons: [
                    {
                        text: gettext("Save"),
                        click: function() {
                            self.update($(this));
                            $(this).dialog("close").dialog("destroy").hide();
                        }
                    },
                    {
                        text: gettext("Cancel"),
                        click: function() {
                            self.persistValues();
                            $(this).dialog("close").dialog("destroy").hide();
                        }
                    }
                ]
            });
        },

        // Given a control, extract a value optimized
        // for storage in JavaScript or for display
        extractValue: function($ctrl, display) {
            if($ctrl.is("select")) {
                if(display) {
                    return $ctrl.find("option:selected").text();
                } else if($ctrl.find("option:selected").val()) {
                    return parseInt($ctrl.find("option:selected").val(), 10);
                } else {
                    return "";
                }
            } else if($ctrl.is("input:checkbox")) {
                return display ?
                    humanize($ctrl.prop("checked")) :
                    $ctrl.prop("checked");
            } else if($ctrl.is("input:text")) {
                return $ctrl.val();
            } else {
                return false;
            }
        },

        // Saves values to $form inputs, where by default, $form is a row's hidden inputs,
        // and values are a row's stored values.
        persistValues: function($form, values) {
            var $ctrl;

            values = (values === void 0) ? this.values : values;
            $form = ($form === void 0) ? this.$inputs : $form;    

            $.each(values, function(key, value) {
                if(key !== "id") {
                    $ctrl = $form.find("[name$=" + key + "]");
                    if($ctrl.length) {
                        if(_.isObject(value)) {
                            value = value.id;
                        }
                        if($ctrl.is("select")) {
                            $ctrl.val(value).trigger("change");
                        } else if($ctrl.is(":checkbox")) {
                            $ctrl.prop("checked", value);
                        } else {
                            $ctrl.val(value);
                        }
                    }
                }
            });
            return this;
        },

        // Saves values to self.values[] and in the html for display
        saveValues: function($controls) {
            var self = this;
            $controls.each(function(index) {
                var $ctrl = $(this),
                    name = $ctrl.attr("name"),
                    source;

                if(name) {
                    source = name.replace(self.prefix, "");
                    self.values[name] = self.extractValue($ctrl, false);
                    self.$el
                        .children("[data-source=" + source + "]")
                        .html(self.extractValue($ctrl, true));
                }
            });
            return this;
        },

        collectDisks: function() {
            var storages = _.uniq(this.$diskControls.find("select").map(function() {
                return $(this).children(":selected").text();
            }).get()).join("; ");

            this.$el.children("[data-source=disk_storage]").html(storages);
        },

        collectNetworks: function() {
            var networks = _.uniq(this.$nicControls.find("select").map(function() {
                return $(this).children(":selected").text();
            }).get()).join("; ");

            this.$el.children("[data-source=network]").html(networks);
        },

        update: function($form) {
            this.saveValues($form.find("input:text, select"));
            this.collectNetworks();
            this.collectDisks();
            return this;
        },

        dodelete: function() {
            this.$el
                .addClass("deleted")
                .find(".delete-box input")
                .prop("checked", true);
            return this;
        },

        undelete: function() {
            this.$el
                .removeClass("deleted")
                .find(".delete-box input")
                .prop("checked", false);
            return this;
        },

        setID: function(type, value) {
            return type + "-" + value;
        },

        getValue: function(key, prefix) {
            key = this.prefix + key;
            console.log("in getValue, values are " + JSON.stringify(this.values) + " and for key " + key + " is " + this.values[key]);
            if(this.values.hasOwnProperty(key) && this.values[key]) {
                return this.values[key];
            } else {
                return null;
            }
        },

        setValue: function(key, value) {
            key = this.prefix + key;
            this.values[key] = value;
            return this;
        },

        processName: function(suffix, action) {
            var newName = this.getValue("name");
            if (!isIRJob && applicationType !== undefined && (applicationType.indexOf("Hyper-V") != -1)) {
                $vmNameSuffixAction.prop("disabled", true);
                $vmNameSuffix.val("");
                $("#id_hyperv_warning").show();
            } else {
                $vmNameSuffixAction.prop("disabled", false);
                $("#id_hyperv_warning").hide();
                switch(action) {
                    case VM_SUFFIX_APPEND:
                        newName += suffix;
                        break;
                    case VM_SUFFIX_REMOVE:
                        var rx = new RegExp(suffix + "$", "i");
                        newName = newName.replace(rx, "");
                        break;
                }
            }

            this.setValue("name", newName);
            //this.$el.find("td.item-name").html(newName);
            var $name = this.$el.find("[data-source=\"name\"]");
            $name.html(newName);
            return this.persistValues();
        },

        adopt: function($row) {
            this.$el = $row;
            this.adopted = true;
            return this.init();
        }
    };

    function humanize(value) {
        if(value === false) {
            return "No";
        }
        if(value === true) {
            return "Yes";
        }
        return value;
    }

    function handleSiblings(vmid, dbid) {
        if($inventoryTable.has("[data-vmid=" + vmid + "]").length) {
            $inventoryTable.find("[data-vmid=" + vmid + "]").not("[data-dbid=" + dbid + "]").each(function() {
                removeInventoryRow($(this).attr("data-dbid"));
            });
        }
        return false;
    }

    function handleTreeClick($checkbox) {
        var $node = $checkbox.closest("tr"),
            type = $node.data("type"),
            id = $node.attr(type + "_id"), 
            siblingID = false;

        // ensure the node we work on is a backup
        //console.log("type = " + type);
        if(type === "disk") {
            $node = backupCatalog.getParent($node);
            id = $node.attr("backup_id");
        } else if(type ==="vm") {
            $node = backupCatalog.getChildren($node).eq(0);
            id = $node.attr("backup_id");
        }

        if($checkbox.is(":checked")) {
            handleSiblings($node.attr("vm_id"), id);

            if($inventoryTable.has("[data-dbid=" + id + "]").length) {
                unDeleteInventoryRow(id);
            } else {
                if (!addInventoryItem($node, id)) {
                    $checkbox.prop("checked", false);
                }
            }
        } else {
            removeInventoryRow(id);
        }
    }

    function addInventoryItem($node, id) {
        if (id !== undefined) {
            var sid = $node.attr('system-id');
            // var sid = PHD.appliance_sid;
            //console.log("addInventoryItem: $node for add item is " + $node.html());
            var url = "/api/backups/?sid=" + sid + "&bid=" + id + "/";
            PHD.Ajax
                .get(url)
                .done(function(data) {
                    if (data.BackupStatus !== undefined && Array.isArray(data.BackupStatus)) {
                        data = data.BackupStatus[0];
                        data.name = (data.vm_name != null) ? data.vm_name : (data.database_name != null) ? data.database_name : data.client_name;
                        data.target_location = null;
                        data.target_folder = null;
                        data.target_storage = {};
                        data.target_host = {};
                        data.available_hosts = [];
                        data.disks = null;
                        addInventoryRow($node, id, data);
                    }
                });
            return true;
        } else {
            PHD.throwNotice({"result": [{"message": gettext("This selection is not recognized as a Backup.  Please use the Import to Source button to import a backup copy.")}]});
            return false;
        }
    }

    function addPreselectedInventory($node, id) {
        console.log("add pre-selected inventory, id = " + id);
        $row = addInventoryItem($node, id);

        /*var dbid = parseInt(id, 10),
            data = _.findWhere(backupInfo.backups, {"id": dbid}),
            $row = addInventoryRow($node, id, data);
            */

        return $row;
    }

    function addInventoryRow($node, dbid, data) {
        //console.log('in add Inventory Row, dbid is = ' + dbid + ' and data is = ' + JSON.stringify(data) + "and node = " + $node.html());
        var vmid = data.instance_id,
            disks = data.disks,
            nics = data.nics;

        var values = {
            "name": data.name,
            "target_location": data.target_location,
            "target_folder": data.target_folder,
            "target_storage": data.target_storage,
            "network": "None"
        };

        applicationType = data.app_name;

        inventory[dbid] = InventoryRow(dbid, vmid, data, inventoryRows).insert($inventoryTable.children("tbody"));

        // Increment the inventory row counter
        inventoryRows++;

        // keep track of the number of rows in inventory
        $inventoryCounter.val($inventoryTable.find(".inventory-row").length);

        return inventory[dbid];
    }

    function adoptInventoryItem(index, row) {
        var $row = $(row),
            dbid = parseInt($row.data("dbid"), 10),
            $node = backupCatalog.getNodeById("backup", dbid),
            vmid = backupCatalog.getParent($node).attr("instance_id"),
            info = _.findWhere(backupInfo.backups, {"id": dbid});

        $row.attr("data-vmid", vmid);

        inventory[dbid] = AdoptedInventoryRow($row, dbid, vmid, info, index);
    }

    function removeInventoryRow(dbId) {
        if(typeof inventory[dbId] !== 'undefined') {
            inventory[dbId].dodelete();
        }
    }

    function unDeleteInventoryRow(dbId) {
        if(typeof inventory[dbId] !== 'undefined') {
            inventory[dbId].undelete();
        }
    }

    function getBackupCatalog() {
        var load = PHD.showLoadingIndicator($backupCatalog.parent()),
            task = PHD.BackupCatalogBuilder({
                "view": backupCatalogView,
                "ir": isIRJob,
                "vms": {
                    "show": true,
                    "checkboxes": false
                },
                "disks": {
                    "show": false, //lmc - no disks in a UEB backup
                    "checkboxes": false
                },
                "checked": selections,
                "loader": load
            });

        task
            .done(function(data) {
                buildCatalog(data);
            })
            .always(function() {
                PHD.hideLoadingIndicator(load);
            });
    }

    function buildCatalog(data) {
        backupCatalogOptions[BACKUP_CATALOG_VIEW_VM] = {
            0: { multiselect: true, childselect: "first" },
            1: { multiselect: false, childselect: "all" },
            2: { multiselect: true, childselect: false } 
        };

        backupCatalogOptions[BACKUP_CATALOG_VIEW_DATE] = {
            0: { multiselect: false, childselect: false },
            1: { multiselect: false, childselect: false },
            2: { multiselect: false, childselect: false },
            3: { multiselect: false, childselect: false } 
        };

        backupCatalogOptions[BACKUP_CATALOG_VIEW_BDS] = {
            0: { multiselect: false, childselect: false },
            1: { multiselect: false, childselect: false },
            2: { multiselect: false, childselect: false },
            3: { multiselect: false, childselect: false } 
        };

        backupCatalogOptions[BACKUP_CATALOG_VIEW_BDS] = {
            0: { multiselect: false, childselect: false },
            1: { multiselect: false, childselect: false },
            2: { multiselect: false, childselect: false },
            3: { multiselect: false, childselect: false }
        };

        backupCatalog = PHD.BackupCatalog($backupCatalog, backupCatalogOptions[backupCatalogView]);
        backupFilter = PHD.BackupFilter($backupFilter, backupCatalog).refresh();

        backupCatalog.load(data);
        backupFilter.refresh(data.tree).submitFilter();

        backupCatalog.$table.on("treeselect", ".row-select", function() {
            selections = backupCatalog.getSelectionObject();
            handleTreeClick($(this));
        });

        if(!jobEdit) {
            // If this is not an edit, initialize pre-selections
            initTreeTableSelections();
        } else if($inventoryTable.find(".inventory-row").length) {
            // If this is an edit, initialize existing inventory rows
            $inventoryTable.find(".inventory-row").each(adoptInventoryItem);
        }
        
    }

    function initTreeTableSelections() {
        backupCatalog.getNodesByType("backup").find(".row-select:checked").each(function() {
            var $node = $(this).closest("tr"),
                dbid = $(this).val();

            addPreselectedInventory($node, dbid);
        });
    }

    function updateCatalog() {
        var selections = backupCatalog.getSelectionObject(),
            task = PHD.BackupCatalogBuilder({
                "view": backupCatalogView,
                "ir": isIRJob,
                "date": {
                    "show": true,
                    "checkboxes": false
                },
                "vms": {
                    "show": true,
                    "checkboxes": false
                },
                "disks": {
                    "show": false,
                    "checkboxes": false
                },
                "checked": selections
            });
        task.done(function(data) {
            updateBackupsTable(data);
        });
    }

    function updateBackupsTable(data) {
        backupCatalog.load(data);
        backupCatalog.refresh(backupCatalogOptions[backupCatalogView]);
        backupFilter.refresh(data.tree).submitFilter();
    }

    function initBackupFilter() {
        $backupCatalog.find(".btn-dateview").on("click", function(event) {
            event.preventDefault();
            backupCatalogView = BACKUP_CATALOG_VIEW_DATE;
            updateCatalog();
        });

        $backupCatalog.find(".btn-vmview").on("click", function(event) {
            event.preventDefault();
            backupCatalogView = BACKUP_CATALOG_VIEW_VM;
            updateCatalog();
        });

        $backupCatalog.find(".btn-bdsview").on("click", function(event) {
            event.preventDefault();
            backupCatalogView = BACKUP_CATALOG_VIEW_BDS;
            updateCatalog();
        });

        $backupCatalog.on("afterfilter backupcatalogload", function(event, data) {
            $backupCatalog.scrollTable({"height": "article", "resize": false});
        });
    }

    function dialogVMNames($el, hide) {
        hide = (hide === true);
        if($vmSuffix.is(":visible") || hide === true) {
            $vmSuffix.css("display", "none").off("click.applysuffix");
        } else {
            $vmSuffix
                .css("display", "table-row")
                .on("click.applysuffix", ".btn-apply-suffix", function(e) {
                    bulkEditVMNames($vmNameSuffix.val(), $vmNameSuffixAction.val());
                });
        }
    }

    function bulkEditVMNames(suffix, action) {
        $.each(inventory, function(dbid, row) {
            row.processName(suffix, parseInt(action, 10));
        });
    }

    function initForm() {

        // Handle popovers for definitions.
        $form.find(".definition").popover({
            appendTo: $form
        });

        var timesettings = {
                noSeparatorEntry: true,
                spinnerImage: null,
                ampmPrefix: " "
            };

        $inventoryTable
            .on("click", ".btn-vm-suffix", function(e) {
                e.preventDefault();
                dialogVMNames($(this));
            })
            .on("click", ".btn-hide-vm-suffix", function(e) {
                e.preventDefault();
                dialogVMNames($(this), true);
            });

        $.validator.addMethod("validate-catalog", function(value, element) {
                return $form.find("input[name=backup]:checked").length > 0;
            },
            gettext("Select at least one Backup")
        );

        $.validator.addMethod("validate-inventory", function(value, element) {
                return $inventoryTable.find(".inventory-row:visible").length > 0;
            },
            gettext("Select at least one Backup")
        );

        $.validator.addMethod("validate-future-date", function(value, element) {
                var date = moment(value + " " + $("#id_start_time").val() + " " + tz, "MM/DD/YYYY hh:mm A Z").utc(),
                    now = moment().utc(),
                    isvalid = now.isBefore(date);
                    
                return isvalid || (scheduleType === SCHEDULE_TYPE_SCHEDULED);
            },
            gettext("Please select a future date and time")
        );

        $.validator.addMethod("validate-future-time", function(value, element) {
                var date = moment($("#id_start_date").val() + " " + value + " " + tz, "MM/DD/YYYY hh:mm A Z").utc(),
                    now = moment().utc(),
                    isvalid = now.isBefore(date);

                return isvalid || (scheduleType === SCHEDULE_TYPE_SCHEDULED);
            },
            gettext("Please select a future date and time")
        );

        RecoverForm = PHD.StepForm($form, {
            namespace: "recoverform",
            serialize: function($form) {
                $startTime.timeEntry('option', 'show24Hours', true);
                return $form.serialize();
            },
            isEditing: jobEdit,
            debug: false
        });

        RecoverForm
            .validate({
                ignore: [],                     /* lmc - do not ignore hidden fields */
                onkeyup: function(element) {
                    var $el = $(element);
                    if($el.is("input[type=text]")) {
                        this.element(element);
                    }
                    if($el.is(".hasTimeEntry")) {
                        this.element($("#id_start_date")[0]);
                    }
                },
                onfocusout: function(element, event) {
                    var $el = $(element),
                        self = this;
                    if($el.is("input[type=text]")) {
                        this.element(element);    
                    }
                    if($el.is(".hasTimeEntry")) {
                        this.element($("#id_start_date")[0]);
                    }
                },
                rules: {
                    name: {
                        required: true
                    },
                    start_date: {
                        required: true,
                        dateFmt: true,
                        "validate-future-date": true
                    },
                    start_time: {
                        required: true,
                        time12h: true,
                        "validate-future-time": true
                    },
                    host: {
                        required: function(element) {
                            return hostRequired;
                        }
                    },
                    resource: {
                        required: function(element) {
                            return resourcePoolRequired;
                        }
                    },
                    storage: {
                        required: function(element) {
                            return storageRequired;
                        }
                    },
                    network: {
                        required: function(element) {
                            return networkRequired;
                        }
                    },
                    hypervisor_switch: {
                        required: function(element) {
                            // required for Hyper-V IR jobs not in audit mode.
                            return switchRequired;
                        }
                    }
                },
                messages: {
                    name: {
                        required: gettext("Job Name is required")
                    },
                    start_date: {
                        required: gettext("Start Date is required")
                    },
                    start_time: {
                        required: gettext("Start Time is required")
                    },
                    host: {
                        required: gettext("Target location is required")
                    },
                    resource: {
                        required: gettext("Target resource pool is required")
                    },
                    storage: {
                        required: gettext("Target storage is required")
                    },
                    network: {
                        required: gettext("Target appliance network is required")
                    },
                    hypervisor_switch: {
                        required: gettext("Target hypervisor network switch is required")
                    }
                }
            })
            .on("formcancel", function() {
                PHD.currentDialog.wizard("close");
            })
            .on("formsubmit", function(event, data) {
                data = {};
                var message = "<p>";
                if (isIRJob) {
                    message += gettext("Instant recovery job(s) successfully started.") + "</p>";
                } else {
                    message += gettext("Recovery job(s) successfully started.") + "<br/><br/>" +
                    gettext("After all recovery jobs have completed, please go to the Options menu and re-synchronize the inventory ") +
                    gettext("to see the recovered virtual machine(s).") + "</p>";
                }

                data.result = [ {'message': message } ];
                var options = {};
                if (window.location.href.indexOf("#/jobs") !== -1 || window.location.href.indexOf("#/protect") !== -1) {
                    options.buttons = [
                        {
                            text: gettext("View Jobs"),
                            click: function() {
                                // Differentiate where to go based on IR or full restore job (active jobs).
                                window.location.href = isIRJob ? "#/recover/#2" : "#/jobs/#0";
                                $(this).dialog("destroy").remove();
                            }
                        }
                    ];
                }
                PHD.throwNotice(data, options);
                PHD.currentDialog.wizard("close");
            })
            .on("stepformgoto", function(event, page) {
                if(page === 2) {
                    $inventoryTable.scrollTable({"height": ".job-inventory", "resize": false});
                } else if (page === 1) {
                    var isHyperV = (applicationType != undefined) && (applicationType.indexOf("Hyper-V") != -1);
                    hostRequired = true;
                    storageRequired = true;
                    networkRequired = isIRJob;
                    switchRequired = isIRJob && isHyperV;
                    if (!restore_targets_initialized) {
                        var selections = backupCatalog.getSelectionObject();
                        var sid = selections.system_id;
                        $.each(inventory, function(dbid, row) {
                            console.log("dbid = " + dbid + " and row " + row);
                            var targets_url = "/api/restore/targets/?sid=" + sid;
                            if (isHyperV) {
                                if (isIRJob) {
                                    // Only get the targets that support IR based on instance
                                    targets_url += "&instance_id=" + row.info.instance_id + "&type=ir";
                                } else {
                                    //targets_url += "&backup_id=" + dbid + "&target=original";
                                    // Option that will get original + alternates.
                                    targets_url += "&backup_id=" + dbid + "&type=restore";
                                }
                            } else {
                                // Get targets based on backup.
                                targets_url += "&backup_id=" + dbid;
                            }
                            var load = PHD.showLoadingIndicator($("#id_host"));
                            var resp = PHD.Ajax.get(targets_url, load);
                            resp.done(function(data) {
                                //PHD.hideLoadingIndicator(load);
                                restore_targets = data.targets;
                                hostSelect.options[0] = new Option("Please select host", "");
                                if (restore_targets) {
                                    restore_targets.forEach(function(target, index) {
                                        hostSelect.options[index + 1] = new Option(target.name, target.id);
                                    })
                                }
                            });
                            // If audit-mode, show audit mode and network selection.
                            if (isIRJob) {
                                $(".audit-box").toggleClass("hidden", false);
                                $(".network-box").toggleClass("hidden", false);
                                var resp = PHD.Ajax.get("/api/networks/?sid=" + sid); // PHD.appliance_sid);
                                resp.done(function(data) {
                                    restore_networks = data.networks;
                                    if (restore_networks != undefined) {
                                        if (restore_networks.length > 0) {
                                            networkSelect.options[0] = new Option("Please select network", "");
                                        }
                                        restore_networks.forEach(function(network, index) {
                                            console.log("adding network  " + network.ip);
                                            var target_network = network.id + " - " + network.ip;
                                            networkSelect.options[index + 1] = new Option(target_network, network.ip);
                                        })
                                    }
                                });
                            }
                            restore_targets_initialized = true;
                            return false;
                        });
                    }
                }
                dialogVMNames(null, true);
            });

        $("#id_host").on("change", function (event) {
            $.each(inventory, function(i, value) {
                var $el = value.$el;
                var $host = $el.find("[data-source=\"target_host\"]");
                this.values["target_host"] = $("#id_host option:selected").text();
                $host.text(this.values["target_host"]);
                return this.persistValues();
            });
        });

        $("#id_resource").on("change", function (event) {
            $.each(inventory, function(i, value) {
                var $el = value.$el;
                var $rp = $el.find("[data-source=\"target_group\"]");
                this.values["target_group"] = $("#id_resource option:selected").text();
                $rp.text(this.values["target_group"]);
                return this.persistValues();
            });
        });

        $("#id_storage").on("change", function (event) {
            $.each(inventory, function(i, value) {
                var $el = value.$el;
                var $storage = $el.find("[data-source=\"target_storage\"]");
                this.values["target_storage"] = $("#id_storage option:selected").text();
                $storage.text(this.values["target_storage"]);
                return this.persistValues();
            });
        });

        $("#id_network").on("change", function (event) {
            $.each(inventory, function(i, value) {
                var $el = value.$el;
                var $network = $el.find("[data-source=\"target_network\"]");
                this.values["target_network"] = $("#id_network option:selected").text();
                $network.text(this.values["target_network"]);
                return this.persistValues();
            });
        });

        $("#id_audit").on("change", function (event) {
            if (isIRJob && (applicationType.indexOf("Hyper-V") != -1)) {
                if ($("#id_audit").prop("checked") === true) {
                    $("#id_switch_control_group").hide();
                    switchRequired = false;
                } else {
                    $("#id_switch_control_group").show();
                    switchRequired = true;
                }
            }
        });

        RecoverForm.submit = function() {
            // lmc - TODO - need a good way to customize the standard submit.

            if ($form.valid() == false) {
                return false;
            }

            var self = this,
                load = PHD.showLoadingIndicator("body", true, "Starting Recovery...");

            this.$buttons.blur().disable();

            var submitData = form2js($form.attr("id"), '.', false);

            $.each(inventory, function(i, value) {
                restoreObj = {};
                restoreObj.target = {};
                // If Hyper-V set host name and client_id.
                if (applicationType.indexOf("Hyper-V") != -1) {
                    if (isIRJob) {
                        restoreObj.target.host = $("#id_host option:selected").text();
                    } else {
                        restoreObj.client_id = parseInt($("#id_host").val(), 10);
                    }
                } else {
                    // otherwise, set host by value.
                    restoreObj.target.host = $("#id_host").val();
                }
                //restoreObj.target.host = (applicationType.indexOf("Hyper-V") != -1) ? $("#id_host option:selected").text() : $("#id_host").val();
                restoreObj.target.name = value.getValue('name');
                if (applicationType.indexOf("Hyper-V") != -1) {
                    restoreObj.target.directory =  $("#id_storage").val();
                } else {
                    restoreObj.target.datastore =  $("#id_storage").val();
                    restoreObj.target.metadata = true;
                    if (resourceSelect.options !== undefined && resourceSelect.options.length > 0) {
                        restoreObj.target.group = $("#id_resource").val();
                    } else {
                        restoreObj.target.group = $("#id_resource").val();
                    }
                }
                restoreObj.target.template = value.info !== undefined && value.info.vmware_template;
                console.log("restore as template = " + restoreObj.target.template);
                //restoreObj.backup_id = parseInt(submitData.backup, 10);
                // The first parameter is the dbid, the backup ID associated with each inventory item.
                restoreObj.backup_id = parseInt(i, 10);
                if (isIRJob) {
                    // start in audit mode.
                    restoreObj.audit = $("#id_audit").prop("checked");
                    if (!restoreObj.audit && (applicationType.indexOf("Hyper-V") != -1)) {
                        restoreObj.target.switch = $("#id_switch").val();
                    }
                    restoreObj.address = $("#id_network").val();
                }
                restoreObj = JSON.stringify(restoreObj);
                console.log('=== Submit form: object = ' + restoreObj);
                var selections = backupCatalog.getSelectionObject();
                var sid = selections.system_id;
                sid = "?sid=" + sid;

                //var $editing_id = this.$form.find("#edit_object_id");
                //console.log('edit id text = ' + $editing_id.text() + " and appliance id is " + PHD.appliance_sid);
                //var sid = (PHD.appliance_sid === null) ? "" : "?sid=" + PHD.appliance_sid;
                var url = (isIRJob ? "/api/restore/instant/" : "/api/restore/full/") + sid;
                console.log("POST url is " + url);
                PHD.Ajax
                    .post(url, restoreObj)
                    .done(function(data) {
                        self.$form.trigger("formsubmit", [data, self.$form, self]);
                    })

                    .always(function() {
                        self.$buttons.enable();
                        PHD.hideLoadingIndicator(load);
                    });
            });

            return this;
        };


        //this.$submit = this.$form.find(".filter-submit").on("click", $.proxy(this.submitFilter, this));

        $("#id_host").on("change", function(event) {
            var id = $("#id_host").val();
            console.log("select id = " + id);
            var target = _.find(restore_targets, function(obj) { return obj.id == id });
            console.log("target = " + JSON.stringify(target));
            if (target) {
                if (target.groups && target.groups.VG && Array.isArray(target.groups.VG)) {
                    resourcePoolRequired = true;
                    $("#id_resource_control_group").show();
                    if (target.groups.VG.length > 0) {
                        resourceSelect.options[0] = new Option("Please select resource pool", "");
                        resourceSelect.options[1] = new Option(target.name, target.name);
                    }
                    target.groups.VG.forEach(function(group, index) {
                        console.log("adding resource group " + group.Name);
                        resourceSelect.options[index + 2] = new Option(group.Name, group.Key);
                    })
                } else {
                    $("#id_resource_control_group").hide();
                    resourcePoolRequired = false;
                    resourceSelect.options[0] = new Option("");
                }
                if (target.datastores && Array.isArray(target.datastores)) {
                    if (target.datastores.length > 0) {
                        var storage_name = (applicationType.indexOf("Hyper-V") != -1) ? "volume" : "datastore";
                        storageSelect.options[0] = new Option("Please select " + storage_name, "");
                    }
                    target.datastores.forEach(function(store, index) {
                        console.log("adding datastore " + store.name);
                        storageSelect.options[index + 1] = new Option(store.name, store.name);
                    })
                }
            }
            // If Hyper-V IR, need to get network switches.
            if (isIRJob && (applicationType.indexOf("Hyper-V") != -1)) {
                $("#id_switch_control_group").show();
                var load = PHD.showLoadingIndicator($("#id_switch"));
                var host = $("#id_host option:selected").text();
                var selections = backupCatalog.getSelectionObject();
                var sid = selections.system_id;
                var resp = PHD.Ajax.get("/api/networks/hypervisor-switch/?sid=" + sid + "&hypervisor_type=Hyper-V&hypervisor=" + host, load);
                resp.done(function(data) {
                    //PHD.hideLoadingIndicator(load);
                    restore_switches = data.names;
                    switchSelect.options[0] = new Option("Please select network switch", "");
                    if (restore_switches) {
                        restore_switches.forEach(function(name, index) {
                            switchSelect.options[index + 1] = new Option(name, name);
                        })
                    }
                });
            }
        });

        $startDate.datepicker({
            showOn: "both",
            buttonText: '<img src="assets/images/calendar.png" width="24" height="24">',
            dateFormat: DATE_FORMAT,
            open: function(event, ui) {
                return !$startDate.prop("disabled");
            }
        }).attr("placeholder", DATE_DISPLAY_FORMAT);

        $startTime.timeEntry(timesettings);

        $scheduleType
            .on("change", handleScheduleType)
            .filter(":checked")
            .trigger("change");

        if(jobEdit && jobId) {
            // Disable "Now" radio button
            $("#id_schedule_type_0").prop("disabled", true);
        }

        // set system ID based on instances selected (should be visible).
        //PHD.appliance_sid = 1;

    }

    function handleScheduleType(event) {
        scheduleType = $(this).val();
        // lmc TODO - do not reference schedule/joborder type we don't have.
        var runLater = false;
        //var runLater = (parseInt(scheduleType, 10) === SCHEDULE_TYPE_RUN_ONCE);
        $startDate.prop("disabled", !runLater).closest(".control-group").toggle(runLater);
        $startTime.prop("disabled", !runLater).closest(".control-group").toggle(runLater);
    }

    /*
    *   DOM Ready Function
    */
    $(function() {

        // Initialize form validation and event handlers
        initForm();

        // Initialize the filter
        initBackupFilter();

        // Fetch the backup catalog
        getBackupCatalog();

        EmailTable = PHD.EmailTable($form);
    });

})(jQuery, window.PHD || (window.PHD = {}), window);
