(function($, PHD, window, undefined) {

    var $form = $("#destination"),
        $dialog = $("#dialog-replication-destination"),
        DestinationWizard,
        $inventoryTreeView = $("#target-inventory-treeview"),
        $locationTreeView = $("#target-location-treeview"),
        $hostTreeView = $("#target-host-treeview"),
        $resourceTreeView = $("#target-resource-treeview"),
        LocationTreeView, HostTreeView, ResourceTreeView,
        selectedLocation = selectedHost = selectedResource = null,
        selections = {},
        $btnSave = $form.find("#form-save")
        $btnCancel = $form.find("#form-cancel");


    function selectInventory(environment) {
        var InvTreeView, selectedInventory,
            TV = PHD.DestinationTreeViewBuilder({
                "environment": environment, 
                "slug": "target_view",
                "checkboxes": false
            });
            
            TV.done(function(data) {
                $inventoryTreeView.html(data.html);
                InvTreeView = PHD.SingleSelectTreeView($inventoryTreeView, [INV_TYPE_VM]);
                $inventoryTreeView.on("nodeclick", ".checkable", function(event, dbid) {
                    selectedInventory = dbid;
                    selections['inventory'] = {
                        "id": dbid,
                        "name": InvTreeView.getNodeName(dbid)
                    };
                });
            });


        $btnSave.on("click", function(event) {
            event.preventDefault();
            if(selectedInventory !== null) {
                finish("inventory");
                $dialog.dialog("close");
            }
        });

        $btnCancel.on("click", function(event) {
            event.preventDefault();
            $dialog.dialog("close");
        });
    }

    function selectLocation(environment) {
        var TV = PHD.DestinationTreeViewBuilder({
                "environment": environment, 
                "slug": "location_view",
                "checkboxes": false
            });
            
            TV.done(function(data) {
                $locationTreeView.html(data.html);
                LocationTreeView = PHD.SingleSelectTreeView($locationTreeView, [INV_TYPE_FOLDER, INV_TYPE_DATACENTER]);
                $locationTreeView.on("nodeclick", ".checkable", function(event, dbid) {
                    setLocation(dbid);
                });
            });
    }

    function setLocation(dbid) {
        selectedLocation = dbid;
        selections['location'] = {
            "id": dbid,
            "name": LocationTreeView.getNodeName(dbid)
        };
    }

    function selectHost(folder) {
        var TV = PHD.DestinationTreeViewBuilder({
                "environment": folder, 
                "slug": "host_view",
                "checkboxes": false
            });
            
            TV.done(function(data) {
                $hostTreeView.html(data.html);
                HostTreeView = PHD.SingleSelectTreeView($hostTreeView, [INV_TYPE_HOST]);
                $hostTreeView.on("nodeclick", ".checkable", function(event, dbid) {
                    setHost(dbid);
                });
            });
    }

    function setHost(dbid) {
        selectedHost = dbid;
        selections['host'] = {
            "id": dbid,
            "name": HostTreeView.getNodeName(dbid)
        };
    }

    function selectResource(host) {
        var TV = PHD.DestinationTreeViewBuilder({
                "environment": host, 
                "slug": "resource_view",
                "checkboxes": false
            });
            
            TV.done(function(data) {
                $resourceTreeView.html(data.html);
                ResourceTreeView = PHD.SingleSelectTreeView($resourceTreeView, [INV_TYPE_HOST, INV_TYPE_CLUSTER, INV_TYPE_VAPP, INV_TYPE_RESOURCE_POOL]);
                $resourceTreeView.on("nodeclick", ".checkable", function(event, dbid) {
                    setResource(dbid);
                });
            });
    }

    function setResource(dbid) {
        selectedResource = dbid;
        selections['resource'] = {
            "id": dbid,
            "name": ResourceTreeView.getNodeName(dbid)
        };
    }

    function finish(namespace) {
        $(document).trigger("replication" + namespace, [selections]);
    }

    function initForm() {
        DestinationWizard = PHD.StepPage(
            $form.find(".mini-wizard-container"), {
                "next": function(currentPage) {
                    if(currentPage === 0) {
                        return (selectedLocation !== null);
                    }
                    if(currentPage === 1) {
                        return (selectedHost !== null);
                    }
                    if(currentPage === 2) {
                        return (selectedResource !== null);
                    }
                },
                "prev": function() {
                    return true;
                },
                "jump": function() {
                    if(!PHD.DEBUG) {
                        return true;
                    } else {
                        return true;
                    }
                },
                "close": function() {
                    $dialog.dialog("close");
                },
                "save": function() {
                    if(selectedResource !== null) {
                        finish("destination");
                        $dialog.dialog("close");
                    } else {
                        return false;
                    }
                }
            });

        DestinationWizard.$container
            .on("steppagegoto", function(event, index) {
                if(index === 0) {
                    selectedLocation = null;
                    selectLocation(targetEnvironment);
                }
                if(index === 1) {
                    selectedHost = null;
                    selectHost(LocationTreeView.getSelectionID());
                }
                if(index === 2) {
                    selectedResource = null;
                    selectResource(HostTreeView.getSelectionID());
                }
            });
    }

    /*
    *   DOM Ready Function
    */
    $(function() {
        PHD.currentMiniWizard.one("dialogfocus", function() {
            if(inventory) {
                selectInventory(targetEnvironment);
            } else {
                initForm();
                selectLocation(targetEnvironment);
            }
        });
    });

})(jQuery, window.PHD || (window.PHD = {}), window);
