(function($, PHD, window, undefined) {

    var $form = $("#form-replication-job"),
        ReplicationForm,
        $joborderCtrl = $("#id_backup_joborder"),
        $jobTable = $("#replication-backup-jobs-table"),
        $vmsTable = $("#replication-vm-table"),
        JobTable, VMTable,

        $inventoryTable = $("#replication-inventory-table"),
        $inventoryCounter = $("#id_override-TOTAL_FORMS"),
        inventoryRows = $("#id_override-INITIAL_FORMS").val(),
        inventory = {},

        inventoryRowTemplate = $("#inventory-row-template").html(),
        inventoryDiskTemplate = $("#inventory-disk-template").html(),
        inventoryNICTemplate = $("#inventory-nic-template").html(),

        $vmSuffix = $("#vm-name-suffix"),
        $vmNameSuffixToggle = $("#toggle_id_vm_name_suffix"),
        $vmNameSuffix = $("#id_vm_name_suffix"),
        $vmNameSuffixAction = $("#id_vm_name_suffix_extra"),
        VM_SUFFIX_APPEND = 1,
        VM_SUFFIX_REMOVE = 0,

        EmailTable,

        defaultJobOrderID,
        defaultEnvironmentID,
        defaultHostID,
        defaultNetworkID,
        defaultResourceID,
        defaultStorageID,
        defaultFolderID,
        defaults = {
            host: null,
            resource: null,
            folder: null,
            target_inventory: null,
            storage: null,
            network: null,
            disk_storage: null,
            nic_network: null,
            auto_generate_mac: null,
            include_new_hardware: null
        },

        available_hosts,
        environments,

        $selectEnvironments = $("#id_layer"),

        $selectHosts = $("#id_default_host"),
        $selectFolder = $("#id_default_folder"),
        $selectResource = $("#id_default_resource"),

        $selectedHost = $("#default_host_name"),
        $selectedFolder = $("#default_folder_name"),
        $selectedResource = $("#default_resource_name"),

        $selectStorage = $("#id_default_storage"),
        $selectNetwork = $("#id_default_network"),

        blankOption = '<option value="">---------</option>',
        spinnerOptions = {
            "min": 0,
            "max": 999,
            "step": 1
        };
    
    var InventoryRow = function(dbid, info, index) {
        if(this instanceof InventoryRow) {
            this.dbid = dbid;
            this.info = info;
            this.values = {};
            this.index = index;

            this.overrideRegex = /override-__prefix__/ig;
            this.diskRegex = /disk-__prefix__/ig;
            this.nicRegex = /nic-__prefix__/ig;

            this.prefix = "override-"+this.index+"-";

            this.$el = null;
            this.adopted = false;
        } else {
            return new InventoryRow(dbid, info, index).init();
        }
    };

    var AdoptedInventoryRow = function($el, dbid, info, index) {
        return new InventoryRow(dbid, info, index).adopt($el);
    };

    InventoryRow.prototype = {
        init: function() {
            this.render()
                .initControls()
                .initDestinationSelection()
                .initHostSelection()
                .initValues()
                .initNewHardwareDefaults()
                .initDisks()
                .initNICs()
                .initLocation()
                .initPath()
                .update(this.$inputs);
            
            return this;
        },

        render: function() {
            if(!this.$el) {
                this.$el = $("<tr>", { "class": "inventory-row" })
                    .attr("data-dbid", this.dbid)
                    .html(inventoryRowTemplate.replace(this.overrideRegex, this.setID("override", this.index)));
            }

            return this;
        },

        initControls: function() {
            this.$cells = this.$el.children(":not(.row-controls)");
            this.$controls = this.$el.children(".row-controls");
            this.$inputs = this.$controls.find(".inputs");

            this.$diskControls = this.$inputs.find(".edit-replication-disks");
            this.$nicControls = this.$inputs.find(".edit-replication-nics");

            this.$controls.children(".edit-row")
                .data("row", this)
                .on("click", $.proxy(this.edit, this));

            return this;
        },

        initDestinationSelection: function() {
            var self = this;
            this.$inputs.find(".btn-select-destination")
                .on("click", function(event) {
                    event.preventDefault();
                    var url = "/replication/destination/" + defaultEnvironmentID + "/";
                    PHD.miniWizard(url, "replication-destination", {
                        "title": gettext("Select Target Destination"),
                        "width": 400,
                        "height": 500
                    }, false);
                    $(document).one("replicationdestination", $.proxy(self.handleDestinationSelection, self));
                });

            this.$inputs.find(".btn-select-inventory")
                .on("click", function(event) {
                    event.preventDefault();
                    var url = "/replication/destination/" + defaultEnvironmentID + "/?inventory=1";
                    PHD.miniWizard(url, "replication-destination", {
                        "title": gettext("Select Target Inventory"),
                        "width": 400,
                        "height": 500
                    }, false);
                    $(document).one("replicationinventory", $.proxy(self.handleInventorySelection, self));
                });
            return this;
        },

        initHostSelection: function() {
            var self = this;
            if(this.info.hypervisor !== HYPERVISOR_VMWARE) {
                this.$inputs.find("select[name$=host]")
                    .html(this.buildHostOptions())
                    .on("change", function() {
                        self.buildHardwareOptions(parseInt($(this).val(), 10));
                    });
            }

            if(!this.adopted) {
                this.buildHardwareOptions(this.info.host.id);
            }
            return this;
        },

        initValues: function() {
            if(!this.adopted) {
                this.$controls.children(".meta").find(".item-id input").val(this.dbid);
                this.persistValues(void 0, this.info);
            }
            this.saveValues(this.$inputs.find("input:text, select"));
            return this;
        },

        buildHardwareOptions: function(hostID) {
            this.$inputs.find("select[name$=storage]").html(this.buildStorageOptions(hostID));
            this.$inputs.find("select[name$=network]").html(this.buildNetworkOptions(hostID));
        },

        initNewHardwareDefaults: function() {
            var self = this,
                $hwCtrl = this.$inputs.find("select[name$=include_new_hardware]"),
                $storageCtrl = this.$inputs.find("select[name$=disk_storage]").closest(".control-group"),
                $networkCtrl = this.$inputs.find("select[name$=nic_network]").closest(".control-group");

            $hwCtrl
                .on("change", function() {
                    var val = parseInt($(this).val(), 10);
                    switch(val) {
                        case INCLUDE_NEW_HW_NONE:
                            $storageCtrl.hide().prop("disabled", true);
                            $networkCtrl.hide().prop("disabled", true);
                            break;
                        case INCLUDE_NEW_HW_ALL:
                            $storageCtrl.show().prop("disabled", false);
                            $networkCtrl.show().prop("disabled", false);
                            break;
                        case INCLUDE_NEW_HW_DISKS_ONLY:
                            $storageCtrl.show().prop("disabled", false);
                            $networkCtrl.hide().prop("disabled", true);
                            break;
                        case INCLUDE_NEW_HW_VM_SETTINGS_ONLY:
                            $storageCtrl.hide().prop("disabled", true);
                            $networkCtrl.show().prop("disabled", false);
                            break;
                    }
                }).trigger("change");

            return this;
        },

        initDisks: function() {
            this.$diskControls.prev(".legend").append(" (" + this.info.disks.length + ")");
            if(!this.adopted) {
                this.$inputs.find(".disk-management-form [name$=TOTAL_FORMS]").val(this.info.disks.length);
                this.info.disks.forEach(this.addDiskControl, this);
            }
            return this;
        },

        initNICs: function() {
            this.$nicControls.prev(".legend").append(" (" + this.info.nics.length + ")");
            if(!this.adopted) {
                this.$inputs.find(".nic-management-form [name$=TOTAL_FORMS]").val(this.info.nics.length);
                this.info.nics.forEach(this.addNICControl, this);
            } else {
                this.info.nics.forEach(this.initNICData, this);
            }
            this.$nicControls.find(".nic-controls").each(this.initMACAddress);
            return this;
        },

        initLocation: function() {
            var self = this;
            if(this.info.hypervisor === HYPERVISOR_VMWARE) {
                this.$inputs.find("[name$=host]").readonly();
                this.$inputs.find("[name$=folder]").readonly();
                this.$inputs.find("[name$=resource]").readonly();
            } else {
                this.$inputs.find(".btn-select-destination").hide();
                this.$inputs.find("[name$=resource]").closest(".control-group").hide();
                this.$inputs.find("[name$=folder]").closest(".control-group").hide();
            }

            this.$inputs.find("[name$=target_inventory]").readonly();

            if(this.$inputs.find("[name$=target_inventory]").val()) {
                this.$inputs.find("[id$=location_or_target_1]").prop("checked", true);
            } else {
                this.$inputs.find("[id$=location_or_target_0]").prop("checked", true);
            }

            this.$inputs.find(".control-group.radios input")
                .on("change", function() {
                    var isLocation = (parseInt($(this).val(), 10) === LOCATION_OR_TARGET_LOCATION);
                    self.$inputs.find("[name$=name]").prop("readonly", !isLocation);
                    self.$inputs.find(".target-destination-group").toggle(isLocation);
                    self.$inputs.find(".target-inventory-group").toggle(!isLocation);
                }).filter(":checked").trigger("change");

            return this;
        },

        initPath: function() {
            if(this.info.hypervisor !== HYPERVISOR_HYPERV) {
                this.$inputs.find("[name$=path]").closest(".control-group").hide();
            }
            return this;
        },

        handleDestinationSelection: function(event, selections) {
            this.setValue("host", selections.host.id)
                .setValue("folder", selections.location.id)
                .setValue("resource", selections.resource.id);

            this.buildHardwareOptions(selections.host.id);
            this.persistValues();
        },

        handleInventorySelection: function(event, selections) {
            this.setValue("target_inventory", selections.inventory.id);
            this.$inputs.find("[name$=target_inventory]")
                .html(this.buildInventoryOptions(selections.inventory))
                .val(this.getValue("target_inventory")).trigger("change");
        },

        buildInventoryOptions: function(inv) {
            if(inv) {
                var inventoryOptions = '<option value="' + inv.id + '">' + inv.name + '</option>';
                return blankOption + inventoryOptions;
            } else {
                return blankOption;
            }
        },

        buildHostOptions: function() {
            var hostOptions = available_hosts.map(function(host) {
                    return '<option value="' + host.id + '">' + host.name + '</option>';
                }).join("");

            return hostOptions;
        },

        buildFolderOptions: function(hostId) {
            hostId = hostId || this.getValue("host");
            var host = _.findWhere(available_hosts, {id: hostId}),
                folderOptions = host.folders.map(function(datastore) {
                    return '<option value="' + datastore.id + '">' + datastore.name + '</option>';
                }).join("");

            return folderOptions;
        },

        buildResourceOptions: function(hostId) {
            hostId = hostId || this.getValue("host");
            var host = _.findWhere(available_hosts, {id: hostId}),
                resourceOptions = host.resources.map(function(datastore) {
                    return '<option value="' + datastore.id + '">' + datastore.name + '</option>';
                }).join("");

            return resourceOptions;
        },

        buildStorageOptions: function(hostId) {
            hostId = hostId || this.getValue("host");
            var host = _.findWhere(available_hosts, {id: hostId}),
                storageOptions = host.storage.map(function(datastore) {
                    return '<option value="' + datastore.id + '">' + datastore.name + '</option>';
                }).join("");

            return storageOptions;
        },

        buildNetworkOptions: function(hostId) {
            hostId = hostId || this.getValue("host");
            var host = _.findWhere(available_hosts, {id: hostId}),
                networkOptions = host.networks.map(function(network) {
                    return '<option value="' + network.id + '">' + network.name + '</option>';
                }).join("");

            return '<option value="">'+gettext("Not Connected")+'</option>' + networkOptions;
        },

        addDiskControl: function(value, index, disks) {
            var html = inventoryDiskTemplate.replace(this.diskRegex, this.setID("disk", index)).replace(this.overrideRegex, this.setID("override", this.index)),
                $row = $("<div>", {"class":"disk-controls", html:html});

            this.$diskControls.append($row);
            $row.find(".item-id input").val(value.id);
            $row.find(".include-label label").append(" " + value.name + " (" + value.total_storage + ")");
            $row.find("[name$=storage]").html(this.buildStorageOptions(this.getValue("host"))).val(this.info.storage.id);
            $row.find("[name$=path]").val(value.path);
            $row.data("values", value);
        },

        addNICControl: function(value, index, nics) {
            var html = inventoryNICTemplate.replace(this.nicRegex, this.setID("nic", index)).replace(this.overrideRegex, this.setID("override", this.index)),
                $row = $("<div>", { "class":"nic-controls", "html":html });

            this.$nicControls.append($row);
            $row.find(".item-id input").val(value.id);
            $row.find(".include-label label").append(" " + value.nic);
            $row.find("[name$=auto_generate_mac]").prop("checked", this.info.auto_generate_mac);
            $row.find("[name$=network]").html(this.buildNetworkOptions(this.getValue("host"))).val(this.info.network.id);
            $row.find("[name$=mac_address]").val(value.mac_address);
            $row.data("values", value);
        },

        initNICData: function(value) {
            this.$nicControls
                .find(".item-id input[value=" + value.id + "]")
            .closest(".nic-controls")
                .data("values", value);        
        },

        initMACAddress: function(index, row) {
            var $row = $(this),
                value = $row.data("values");

            $row.find("[name$=mac_address]").each(function() {
                var $input = $(this).data("mac-user", value.mac_address).data("mac-original", value.mac_address),
                    autoGen = $row.find("[name$=auto_generate_mac]").prop("checked");

                $input.prop("readonly", autoGen);
                if(!autoGen) {
                    $input.val($input.data("mac-user"));
                    $row.find(".mac-auto").show();
                    $row.find(".mac-edit").hide();
                } else {
                    value.mac_address = "auto-generate";
                    $input.val(gettext("auto-generate"));
                    $row.find(".mac-auto").hide();
                }
            });

            $row.find(".mac-edit").on("click", function() {
                var $input = $(this).hide().nextAll(".mac-auto").show().nextAll("input");
                $row.find("[name$=auto_generate_mac]").prop("checked", false);

                $input.prop("readonly", false)
                    .val($input.data("mac-user"))
                    .addClass("validate-mac-address")
                    .on("blur.valid-mac", function() {
                        var macregex = /^([a-zA-Z0-9]{2}\:){5}[a-zA-Z0-9]{2}$/;
                        if(!$(this).val().match(macregex)) {
                            $(this).addClass("error");    
                        } else {
                            $(this).removeClass("error").data("mac-user", $(this).val());    
                        }
                    });

            });

            $row.find(".mac-auto").on("click", function() {
                var $input = $(this).hide().prevAll(".mac-edit").show().nextAll("input");
                $row.find("[name$=auto_generate_mac]").prop("checked", true);

                $input.prop("readonly", true)
                    .val(gettext("auto-generate"))
                    .removeClass("validate-mac-address")
                    .off("blur.valid-mac");
            });

            $row.find(".mac-undo").on("click", function() {
                var $input = $(this).nextAll("input"),
                    editing = $input.prop("readonly");

                if(!editing) {
                    $input.val($input.data("mac-original")).data("mac-user", $input.data("mac-original"));
                }
            });
        },

        insert: function($container) {
            if(this.$el && $container) {
                $container.append(this.$el);    
            }
            return this;
        },

        edit: function() {
            var self = this;

            this.$inputs.show().dialog({
                title: gettext("Edit settings for ") + this.getValue("name"),
                resizable: false,
                draggable: false,
                modal: true,
                width: 440,
                dialogClass: "wizard-edit wizard-replication-edit",
                buttons: {
                    "Save": function() {
                        self.update($(this));
                        $(this).dialog("close").dialog("destroy").hide();
                    },
                    "Cancel": function() {
                        self.persistValues();
                        $(this).dialog("close").dialog("destroy").hide();
                    }
                }
            });
        },

        // Given a control (checkbox or select), extract a value optimized
        // for storage in JavaScript or for display
        extractValue: function($ctrl, display) {
            if($ctrl.is("select")) {
                if(display) {
                    return $ctrl.find("option:selected").text();
                } else if($ctrl.find("option:selected").val()) {
                    return parseInt($ctrl.find("option:selected").val(), 10);
                } else {
                    return "";
                }
            } else if($ctrl.is("input:checkbox")) {
                return display ?
                    humanize($ctrl.prop("checked")) :
                    $ctrl.prop("checked");
            } else if($ctrl.is("input:text")) {
                return $ctrl.val();
            } else {
                return false;
            }
        },

        // Saves values to $form inputs, where by default, $form is a row's hidden inputs,
        // and values are a row's stored values.
        persistValues: function($form, values) {
            var $ctrl;
            values = (values === void 0) ? this.values : values;
            $form = ($form === void 0) ? this.$inputs : $form;
                

            $.each(values, function(key, value) {
                if(key !== "id") {
                    $ctrl = $form.find("[name$=" + key + "]");
                    if($ctrl.length) {
                        if(_.isObject(value)) {
                            value = value.id;
                        }
                        if($ctrl.is("select")) {
                            $ctrl.val(value).trigger("change");
                        } else if($ctrl.is(":checkbox")) {
                            $ctrl.prop("checked", value);
                        } else {
                            $ctrl.val(value);
                        }
                    }
                }
            });
            return this;
        },

        // Saves values to self.values[] and in the html for display
        saveValues: function($controls) {
            var self = this;
            $controls.each(function(index) {
                var $ctrl = $(this),
                    name = $ctrl.attr("name"),
                    source;

                if(name) {
                    source = name.replace(self.prefix, "");
                    self.values[name] = self.extractValue($ctrl, false);
                    self.$el
                        .children("[data-source=" + source + "]")
                        .html(self.extractValue($ctrl, true));
                }
            });
            return this;
        },

        collectDisks: function() {
            var storages = _.uniq(this.$diskControls.find("select").map(function() {
                return $(this).children(":selected").text();
            }).get()).join("; ");

            this.$el.children("[data-source=disk_storage]").html(storages);
        },

        collectNetworks: function() {
            var networks = _.uniq(this.$nicControls.find("select").map(function() {
                return $(this).children(":selected").text();
            }).get()).join("; ");

            this.$el.children("[data-source=network]").html(networks);
        },

        update: function($form) {
            this.saveValues($form.find("input:text, select"));
            this.collectNetworks();
            this.collectDisks();
            return this;
        },

        dodelete: function() {
            this.$el
                .addClass("deleted")
                .find(".delete-box input")
                .prop("checked", true);
            return this;
        },

        undelete: function() {
            this.$el
                .removeClass("deleted")
                .find(".delete-box input")
                .prop("checked", false);
            return this;
        },

        setID: function(type, value) {
            return type + "-" + value;
        },

        getValue: function(key, prefix) {
            key = this.prefix + key;
            if(this.values.hasOwnProperty(key) && this.values[key]) {
                return this.values[key];
            } else {
                return null;
            }
        },

        setValue: function(key, value) {
            key = this.prefix + key;
            this.values[key] = value;
            return this;
        },

        processName: function(suffix, action) {
            var newName = this.getValue("name");
            switch(action) {
                case VM_SUFFIX_APPEND:
                    newName += suffix;
                    break;
                case VM_SUFFIX_REMOVE:
                    var rx = new RegExp(suffix + "$", "i");
                    newName = newName.replace(rx, "");
                    break;
            }
            this.setValue("name", newName);
            this.$el.find("td.item-name").html(newName);
            return this.persistValues();
        },

        adopt: function($row) {
            this.$el = $row;
            this.adopted = true;
            return this.init();
        }
    };

    function humanize(value) {
        if(value === false) {
            return "No";
        }
        if(value === true) {
            return "Yes";
        }
        return value;
    }

    function addInventoryItem(data) {
        var dbid = data.id,
            values = $.extend(true, data, defaults);

        if($inventoryTable.find("tbody tr[data-dbid=" + dbid + "]").not(".deleted").length) {
            inventory[dbid].persistValues(void 0, values);
        } else {
            addInventoryRow(dbid, values);
        }
    }

    function adoptInventoryItem(index, row) {
        var dbid = parseInt($(row).data("dbid"), 10),
            info = _.findWhere(inventoryInfo.inventory, {"id": dbid});

        inventory[dbid] = AdoptedInventoryRow($(row), dbid, info, index);
    }

    function addInventoryRow(dbid, values) {
        inventory[dbid] = InventoryRow(dbid, values, inventoryRows).insert($inventoryTable.children("tbody"));

        // Increment the inventory row counter
        inventoryRows++;

        // keep track of the number of rows in inventory
        $inventoryCounter.val($inventoryTable.find(".inventory-row").length);

        return inventory[dbid];
    }

    function deleteInventoryRow(dbid) {
        if(inventory[dbid] !== void 0) {
            inventory[dbid].dodelete();
        }
    }

    function unDeleteInventoryRow(dbid) {
        if(inventory[dbid] !== void 0) {
            inventory[dbid].undelete();
        }
    }

    function dialogVMNames($el, hide) {
        hide = (hide === true);
        if($vmSuffix.is(":visible") || hide === true) {
            $vmSuffix.css("display", "none").off("click.applysuffix");
        } else {
            $vmSuffix
                .css("display", "table-row")
                .on("click.applysuffix", ".btn-apply-suffix", function(e) {
                    bulkEditVMNames($vmNameSuffix.val(), $vmNameSuffixAction.val());
                });
        }
    }

    function bulkEditVMNames(suffix, action) {
        $.each(inventory, function(dbid, row) {
            row.processName(suffix, parseInt(action, 10));
        });
    }

    function initForm() {
        if(jobEdit) {
            defaultEnvironmentID = parseInt($selectEnvironments.val(), 10);
        }

        $form.find(".definition").popover({
            appendTo: $form,
            position: {
                my: "center bottom",
                at: "center top-15"
            }
        });

        $inventoryTable
            .on("click", ".btn-vm-suffix", function(e) {
                e.preventDefault();
                dialogVMNames($(this));
            })
            .on("click", ".btn-hide-vm-suffix", function(e) {
                e.preventDefault();
                dialogVMNames($(this), true);
            });

        $.validator.addMethod("validate-inventory", function(value, element) {
                return $vmsTable.find("tbody :checked").length > 0;
            },
            gettext("Select at least one VM to replicate")
        );

        $.validator.addMethod("validate-mac-address", function(value, element) {
                var macregex = /^([a-zA-Z0-9]{2}\:){5}[a-zA-Z0-9]{2}$/;
                return value.match(macregex);
            },
            gettext("Invalid Mac Address")
        );

        ReplicationForm = PHD.StepForm($form, {
            namespace: "replicationform",
            isEditing: jobEdit
        });

        ReplicationForm
            .validate({
                rules: {
                    name: {
                        required: true
                    },
                    layer: {
                        required: true
                    },
                    default_host: {
                        required: true
                    },
                    default_folder: {
                        required: true
                    },
                    default_resource: {
                        required: true
                    },
                    default_storage: {
                        required: true
                    },
                    default_network: {
                        required: true
                    },
                    restore_points: {
                        required: true,
                        min: 0,
                        max: 99,
                        digits: true
                    }
                },
                messages: {
                    name: {
                        required: gettext("Job Name is required")
                    },
                    layer: {
                        required: gettext("Target Environment is required")
                    },
                    default_host: {
                        required: gettext("Default Host is required")
                    },
                    default_folder: {
                        required: gettext("Default Folder is required")
                    },
                    default_resource: {
                        required: gettext("Default Resource is required")
                    },
                    default_storage: {
                        required: gettext("Target Storage is required")
                    },
                    default_network: {
                        required: gettext("Target Network is required")
                    },
                    restore_points: {
                        required: gettext("Please enter a positive restore points to keep"),
                        min: gettext("Please enter a positive restore points to keep"),
                        max: gettext("Restore points to keep may not exceed 99"),
                        digits: gettext("Restore points to keep must be a number")
                    }
                }
            })
            .on("formcancel", function() {
                PHD.currentDialog.wizard("close");
            })
            .on("formsubmit", function(event, data) {
                if(data.result[0].code === AJAX_RESULT_SUCCESS) {
                    var options = {};
                    if(PHD.App.url !== '/jobs/') {
                        options.buttons = [
                            {
                                text: gettext("View Jobs"),
                                click: function() {
                                    window.location.href = "/jobs/#1";
                                }
                            }
                        ];
                    }
                    PHD.throwNotice(data, options);
                    PHD.currentDialog.wizard("close");
                } else if(data.result[0].code === AJAX_RESULT_ERROR) {
                    PHD.throwError(data);
                }
            })
            .on("stepformgoto", function(event, page) {
                if(page==1) {
                    if(jobEdit) {
                        if(environments[defaultEnvironmentID].translated_hypervisor === HYPERVISOR_VMWARE) {
                            enableTargetSelection(defaultEnvironmentID);    
                        } else {
                            enableHostSelection();
                        }
                        toggleRestorePointControls((environments[defaultEnvironmentID].translated_hypervisor === HYPERVISOR_HYPERV));
                    }
                }
                if(page === 2) {
                    protectTheEnvironment();
                    gatherDefaults();
                    initOverrides();
                }
                dialogVMNames(null, true);
            });

        $selectEnvironments.on("change", function(event) {
            var val = $(this).val();

            if($inventoryTable.find(".inventory-row").length) {
                _.each(inventory, function(inv, dbid) {
                    inv.dodelete();
                    delete inventory[dbid];
                });
            }

            if(val) {
                defaultEnvironmentID = parseInt(val, 10);
                setupTargetFields();
                // Get host info and enable host selection
                getHostInfo();
            } else {
                disableTargetSelection();
            }
        });

        $form.find(".spinner-group input").spinner(spinnerOptions);
    }

    function setupTargetFields() {
        if(environments[defaultEnvironmentID].translated_hypervisor === HYPERVISOR_VMWARE) {
            enableTargetSelection(defaultEnvironmentID);
            $selectFolder.closest(".control-group").removeClass("hidden");
            $selectResource.closest(".control-group").removeClass("hidden");
            if(!jobEdit) {
                $selectStorage.prop("disabled", true).html(blankOption);
                $selectNetwork.prop("disabled", true).html(blankOption);
            }
        } else {
            $("#btn-select-host").addClass("hidden");
            $selectFolder.closest(".control-group").addClass("hidden");
            $selectResource.closest(".control-group").addClass("hidden");
        }
    }

    function protectTheEnvironment() {
        $selectEnvironments
            .readonly()
            .off("mousedown.confirm")
            .on("mousedown.confirm", confirmEnvironmentChange);
    }

    function unProtectTheEnvironment() {
        $selectEnvironments
            .readonly("disable")
            .off("mousedown.confirm");
    }

    function confirmEnvironmentChange() {
        PHD.confirmDialog(
            "<p>"+gettext("If you change the target environment, you will lose all job settings. This action cannot be undone.")+"</p>" + 
            "<p><strong>"+gettext("Are you sure you want to change the target environment?")+"</strong></p>",
            { title: gettext("Confirm Target Environment Change") }
        ).done(unProtectTheEnvironment);
    }

    function initOverrides() {
        var load = PHD.showLoadingIndicator($inventoryTable.parent()),
            url = "/inventory_info/",
            existing = Object.keys(inventory),
            checked = $vmsTable.find("tbody input:checked").map(function(){ return "" + $(this).closest("tr").data("dbid"); }).get(),
            toDelete = _.difference(existing, checked),
            toFetch = _.difference(checked, existing).map(function(dbid) { return $vmsTable.find("tr[data-dbid="+dbid+"] :checkbox").val(); }),
            qs = toFetch.length ? "?id=" + toFetch.join("&id=") : "";

        PHD.Ajax
            .get(url + qs)
            .done(function(data) {
                data.forEach(function(vm) {
                    addInventoryItem(vm);
                });
            })
            .always(function() {
                checked.forEach(function(dbid) {
                    inventory[dbid].undelete();
                });
                toDelete.forEach(function(dbid) {
                    inventory[dbid].dodelete();
                });
                PHD.hideLoadingIndicator(load);
                $inventoryTable.scrollTable({"height": ".job-inventory", "resize": false});
            });
    }

    function adoptOverrides() {
        $inventoryTable.find(".inventory-row").each(adoptInventoryItem);
    }

    function getHostInfo() {
        return PHD.Ajax
            .get("/hosts_in_env/" + defaultEnvironmentID + "/")
            .done(function(data) {
                available_hosts = data;
                if(environments[defaultEnvironmentID].translated_hypervisor !== HYPERVISOR_VMWARE) {
                    enableHostSelection(jobEdit);
                }
                toggleRestorePointControls((environments[defaultEnvironmentID].translated_hypervisor === HYPERVISOR_HYPERV));
            });
    }

    function toggleRestorePointControls(hide) {
        $("#id_restore_points").closest(".control-group").toggle(!hide);
    }

    function enableTargetSelection(environment) {
        $("#btn-select-host")
            .prop("disabled", false)
            .removeClass("disabled")
            .off("click")
            .on("click", function(event) {
                event.preventDefault();
                var url = "/replication/destination/" + environment + "/";
                PHD.miniWizard(url, "replication-destination", {
                    "title": gettext("Select Target Destination"),
                    "width": 400,
                    "height": 500
                }, false);
                $(document).one("replicationdestination", handleDefaultDestinationSelection);
            });

        $selectHosts.readonly();
        $selectFolder.readonly();
        $selectResource.readonly();
    }

    function enableHostSelection(isEditing) {
        $selectHosts
            .prop("disabled", false)
            .html(blankOption + buildOptions(available_hosts))
            .val(defaultHostID)
            .on("change", function() {
                defaultHostID = parseInt($(this).val(), 10);
                var host = _.findWhere(available_hosts, {"id": defaultHostID});
                if(host) {
                    $selectStorage
                        .html(blankOption + buildOptions(host.storage));
                    $selectNetwork
                        .html(blankOption + buildOptions(host.networks));
                }
            });
    }

    function disableTargetSelection(environment) {
        $("#btn-select-host")
            .prop("disabled", true)
            .addClass("disabled")
            .off("click");
    }

    function handleDefaultDestinationSelection(event, selections) {
        defaultHostID = parseInt(selections.host.id, 10);
        var host = _.findWhere(available_hosts, {"id": defaultHostID});

        $selectHosts
            .prop("disabled", !(host.storage.length > 0))
            .html(blankOption + buildOptions(available_hosts)).val(selections.host.id).trigger("change");
        $selectFolder
            .prop("disabled", !(host.folders.length > 0))
            .html(blankOption + buildOptions(host.folders)).val(selections.location.id).trigger("change");
        $selectResource
            .prop("disabled", !(host.resources.length > 0))
            .html(blankOption + buildOptions(host.resources)).val(selections.resource.id).trigger("change");
        $selectStorage
            .prop("disabled", !(host.storage.length > 0))
            .html(blankOption + buildOptions(host.storage));
        $selectNetwork
            .prop("disabled", !(host.networks.length > 0))
            .html(blankOption + buildOptions(host.networks));
    }

    function gatherDefaults() {
        $(".replication-target-defaults, .replication-hardware-defaults").find("select, [type=checkbox], [type=hidden]").each(function() {
            var $el = $(this),
                name = $el.attr("name").replace("default_", ""),
                val;

            if(defaults.hasOwnProperty(name)) {
                if($el.is("select")) {
                    val = parseInt($el.val(), 10);
                    if(val !== null && val !== void 0) {
                        defaults[name] = {"id": val, "display": $el.find(":selected").text()};    
                    } else {
                        defaults[name] = "";
                    }
                } else if($el.is("[type=hidden]")) {
                    val = parseInt($el.val(), 10);
                    defaults[name] = {"id": val, "display": $el.prev("input").val()};
                } else {
                    defaults[name] = $el.prop("checked");
                }
            }

            if(defaults.hasOwnProperty("disk_" + name)) {
                defaults["disk_" + name] = {"id": parseInt($el.val(), 10), "display": $el.find(":selected").text()};
            }

            if(defaults.hasOwnProperty("nic_" + name)) {
                defaults["nic_" + name] = {"id": parseInt($el.val(), 10), "display": $el.find(":selected").text()};
            }
        });
    }

    function initTables() {
        $jobTable
            .on("loadfinished", function() {
                $jobTable.scrollTable({"resize": false});
            })
            .on("rowselect", function(event, dbid, $row) {
                defaultJobOrderID = dbid;
                $joborderCtrl.val(defaultJobOrderID);
                // Fetch vms in job
                PHD.Ajax
                    .get("/vms_in_joborder/" + dbid + "/")
                    .done(function(data) {
                        VMTable.load(data);
                    });

                // fetch environment for job
                PHD.Ajax
                    .get("/avail_envs_for_backup_joborder/" + defaultJobOrderID + "/")
                    .done(function(data) {
                        environments = _.indexBy(data, "id");
                        $selectEnvironments.html(blankOption + buildOptions(data));  
                        if(jobEdit) {
                            $selectEnvironments.val(defaultEnvironmentID);
                            getHostInfo().done(adoptOverrides);
                            setupTargetFields();
                        }
                    });
            });

        $vmsTable
            .on("change", ".check-select-all", function() {
                $vmsTable.find("tbody :checkbox").prop("checked", $(this).prop("checked"));
            })
            .on("loadfinished", function(event, newRows) {
                $vmsTable.scrollTable({"resize": false});
                if(jobEdit) {
                    selections.forEach(function(vm) {
                        if(vm.selected) {
                            $vmsTable.find("input[value=" + vm.id + "]").prop("checked", true);
                        }
                    });
                }
            });
            
        JobTable = PHD.DataTable($jobTable, {
            "select": !jobEdit
        }, backupJobs);

        VMTable = PHD.DataTable($vmsTable, {
            "select": false
        }).registerFormatters(VMTableFormatter);

        // If joborderCtrl has a value (edit or preselect), select that row
        JobTable.selectByID($joborderCtrl.val());
    }

    function buildOptions(data) {
        var options = data.map(function(item) {
                return '<option value="' + item.id + '">' + item.name + '</option>';
            }).join("");

        return options;
    }

    /*
    *   DOM Ready Function
    */
    $(function() {
        // Initialize form validation and event handlers
        initForm();

        PHD.currentDialog.one("wizardfocus", function() {
            initTables();
            if(jobEdit) {
                defaultHostID = parseInt($selectHosts.val(), 10);
                protectTheEnvironment();
            }
        });

        EmailTable = PHD.EmailTable($form);
    });

    var VMTableFormatter = {
        controls: function(value, row) {
            return '<input type="checkbox" value="' + row.state.exp_item_id + '">';
        }
    };

})(jQuery, window.PHD || (window.PHD = {}), window);
