//(function($, PHD, window, undefined) {
function processJobInventory(PHD, window) {

    /*
     *   Builds the Job Inventory Details table
     */

    var DETAIL_TABLE_HEIGHT = 250,
        defaults = {
            select: false,
            multiselect: false,
            className: "",
            cellClasses: null,
            sortable: false,
            sortableCells: null,
            scroll: false,
            filter: false,
            resize: false,
            paged: false,
            placeholder: gettext("There is no data to display")
        };

    /* 
     *   Object to convert raw values to display-friendly values
     */
    var defaultFormatter = {
        getValue: function (key, value, row) {
            if ((typeof value === 'undefined') || value === null)
                value = EMPTY_CELL_TEXT;
            return this.hasOwnProperty(key) ?
                this[key](value, row) :
                value;
        },
        inv_name: function (value, row) {
            var type = row.data.icon,
                icon = '<i class="icon-hv-' + row.data.icon + ' icon" title="' + row.state.tooltip + '"></i>',
                name = '<span class="truncates item-name" title="' + value + '">' + value;

            if (type === "drive") {
                name += ' <span class="disk-size">(' + PHD.util.formatBytes(row.data.total_storage) + ')</span>';
            } else if (type !== "task") {
                icon = '<i class="expand-ctrl"></i>' + icon;
            }
            name += '</span>';
            return icon + name;
        },
        job_status: function (value, row) {
            var status = value.toLowerCase(),
                icon = '<i class="icon-phd-' + status + ' icon"></i>';

            return '<span class="job-status-display">' + icon + value + '</span>';
        },
        percent_complete: function (value, row) {
            if (value == 100) {
                return this.getValue('job_status', row.data.job_status, row);
            } else {
                return '<div class="progress-bar" title="' + value + '%">' +
                    '<div class="progress-meter" style="width:' + value + '%;"></div></div><span class="progress-count">' + value + '%</span>';
            }
        },
        data_written: function (value) {
            return PHD.util.formatBytes(value);
        },
        current_speed: function (value) {
            return PHD.util.formatBytes(value) + "/s";
        }
    };
    defaultFormatter.speed = defaultFormatter.average_speed = defaultFormatter.current_speed;

    var lookup = PHD.StateLookup;

    var JobInventory = function JobInventory($table, TreeTable, options) {
        if (this instanceof JobInventory) {
            this.$table = $table;
            this.$thead = this.$table.children("thead");
            this.$tbody = this.$table.children("tbody");
            this.$tfoot = this.$table.children("tfoot");

            this.TreeTable = TreeTable;

            this.options = $.extend({}, defaults, options);

            this.data = null;
            this.cols = 0;
            this.rows = {};
            this.dbids = [];

            this.$selected = $();
            this.selectedID = null;

            this.formatter = $.extend({}, defaultFormatter);
        } else {
            return new JobInventory($table, TreeTable, options).init();
        }
    };

    JobInventory.prototype = Object.create(PHD.DataTable.prototype);

    JobInventory.prototype.build = function (data) {
        var self = this, parentRow;

        console.log('in build, data = ' + data);
        // lmc add this.
        this.data = data;

        this.data.forEach(function (row) {
            var dbid = row.id;
            row.depth = 0;
            parentRow = self.rows[dbid] = JobInventoryRow(self, row, self.options.className, self.options.cellClasses, null);
            self.$table.trigger("rowadd", [self.rows[dbid], dbid, self]);
            row.data.disks.forEach(function (disk) {
                var diskdbid = disk.state.id;
                disk.depth = 1;
                self.dbids.push(diskdbid);
                self.rows[diskdbid] = JobInventoryRow(self, disk, self.options.className, self.options.cellClasses, parentRow);
                self.$table.trigger("rowadd", [self.rows[diskdbid], diskdbid, self]);
            });
        });

        this.$placeholder.toggleClass("hidden", (this.data && this.data.length > 0));
        this.TreeTable.refresh();
        this.$table.trigger("loadfinished", [this]);
        return this;
    };

    JobInventory.prototype.update = function (data) {
        var self = this, parentRow;

        this.data = data;
        if (data.length) {
            this.hidePlaceholder();
            this.data.forEach(function (row) {
                var dbid = row.id;
                if (_.contains(self.dbids, dbid)) {
                    parentRow = self.rows[dbid].update(row);
                    self.$table.trigger("rowupdate", [self.rows[dbid], dbid, self]);
                } else {
                    //console.log("adding vm row", dbid);
                    self.dbids.push(dbid);
                    row.depth = 0;
                    parentRow = self.rows[dbid] = JobInventoryRow(self, row, self.options.className, self.options.cellClasses, null);
                    self.$table.trigger("rowadd", [self.rows[dbid], dbid, self]);
                }
                row.data.disks.forEach(function (disk) {
                    var diskdbid = disk.state.id;
                    disk.depth = 1;
                    //console.log("processing disk row", diskdbid, "for vm", dbid);
                    if (_.contains(self.dbids, diskdbid)) {
                        //console.log("updating disk row", diskdbid, "for vm", dbid);
                        //console.log("parent of disk is", parentRow);
                        self.rows[diskdbid].update(disk, true);
                        self.$table.trigger("rowupdate", [self.rows[diskdbid], diskdbid, self]);
                    } else {
                        //console.log("adding disk row", diskdbid, "for vm", dbid);
                        //console.log("parent of disk is", parentRow);
                        self.dbids.push(diskdbid);
                        self.rows[diskdbid] = JobInventoryRow(self, disk, self.options.className, self.options.cellClasses, parentRow);
                        self.$table.trigger("rowadd", [self.rows[diskdbid], diskdbid, self]);
                    }
                });
            });
        }

        this.TreeTable.refresh();

        this.$table.trigger("updatefinished", [this]);
        return this;
    };

    PHD.JobInventory = JobInventory;

    var JobInventoryRow = function JobInventoryRow(Table, data, rowClass, cellClasses, parentRow) {
        if (this instanceof JobInventoryRow) {
            this.Parent = Table;
            this.schema = this.Parent.schema;
            this.data = data;
            this.rowClass = rowClass;
            this.cellClasses = cellClasses;
            this.parentRow = parentRow;

            this.$el = $();
        } else {
            return new JobInventoryRow(Table, data, rowClass, cellClasses, parentRow).init();
        }
    };

    JobInventoryRow.prototype = Object.create(PHD.DataRow.prototype);

    JobInventoryRow.prototype.tr = function () {
        var self = this,
            tds = this.schema.map($.proxy(this.td, this)).join(""),
            type = this.data.data.icon,
            dataType = type === "drive" ? "disk" : type,
            className = "level-" + this.data.depth,//(type === "drive" || type === "task") ? "sub-row closed level-1" : "open visible level-" + this.data.depth,
            idAttr = type === "drive" ? "disk_id" : "vm_id",
            attrs = {
                "data-dbid": self.data.state.id,
                "data-depth": self.data.depth,
                "data-type": dataType,
                "parent-id": self.data.state.parent_id
            },
            $precedes, $row;

        if (type === "drive" || type === "task") {
            className += " sub-row";

            if (this.parentRow.$el.find(".expand-ctrl").hasClass("expanded")) {
                className += " open visible";
            } else {
                className += " closed";
            }
        } else {
            className += "open visible";
        }

        attrs["class"] = className;
        attrs[idAttr] = this.data.state.id;

        if (this.parentRow) {
            attrs["parent-id"] = this.parentRow.data.state.id;
        }

        $row = $("<tr>", attrs).html(tds);

        //this.$el = $row.classify(this.data.state, lookup);
        // remove data and state, use status.
        this.$el = $row.classify(this.status, lookup);
        this.parentID = self.data.state.parent_id;

        if (this.parentRow) {
            $precedes = this.Parent.TreeTable.getLastChild(this.parentRow.$el);
            if (!$precedes.length) $precedes = this.parentRow.$el;
            this.$el.insertAfter($precedes);
        } else {
            this.$el.appendTo(this.Parent.$tbody);
        }
        self.enrich();
        return this;
    };

    JobInventoryRow.prototype.td = function (key, index, schema) {
        var value = this.data.data[key],
            cell = '<td data-source="' + key + '">';

        return cell + this.format(key, value) + '</td>';
    };

    JobInventoryRow.prototype.update = function (data, disk) {
        var self = this;
        this.data = data;

        this.$el.find("td").each(function () {
            var $cell = $(this),
                key = $cell.data("source");

            if (key && key !== "inv_name") {
                var value = self.data.data.hasOwnProperty(key) ? self.data.data[key] : null;
                $cell.html(self.format(key, value));
            }
        });

        //self.$el.classify(this.data.state, lookup);
        // lmc use status
        self.$el.classify(this.data.status, lookup);
        self.enrich();

        return this;
    };

}
//})(jQuery, window.PHD || (window.PHD = {}), window);
