var jobApp = angular.module('jobs', ['download', 'angulartics', 'angulartics.google.analytics']);

jobApp.factory('jobFactory', function($http, DownloadService) {
	return {
		async: function(url, start_date, end_date, report_name, extra_params) {
			var report_name = report_name || null;
			var extra_params = extra_params || null;
			var params = {};
			params.ts = Date.now();
			if (extra_params != null) {
				params = $.extend(params, extra_params);
			}
			var promise = $http({
				url: url,
				method: "get",
				params: params
			}).success(function(data, status, headers) {
				console.log("success");
				// This should be encapsulated in a reusable function so that other areas (outside reports) can export to CSV or PDF.
				headers = headers();
				var contentType = headers['content-type'];
				if (contentType.indexOf('text/csv') > -1) {
					var filename = report_name == 'null' ? 'report' + Date.now + ".csv" : report_name.replace(" ", "-") + ".csv";
					var blob = new Blob([data], {type: contentType});
					DownloadService.downloadFile(blob, filename);
				}
			}).error(function(){
				window.PHD.throwError({"result": [{"message": gettext("Requested job data cannot be exported.")}]});
				PHD.hideLoadingIndicator(load);
			});  //1. this returns promise
			return promise;
		}
	};
});

jobApp.controller('JobsCtrl', ['$scope', '$rootScope', '$http', 'Page', 'jobFactory', 'DownloadService', '$analytics',
		function($scope, $rootScope, $http, Page, jobFactory, DownloadService, $analytics) {

		Page.setTitle('Jobs');
		$scope.Test = "Lisa";

		$scope.$on('$viewContentLoaded', function() {
			processJobsPage(PHD, window, $rootScope, $analytics);
			processJobInventory(PHD, window);
		});

		$scope.exportHistory = function() {
			$analytics.eventTrack('CSV Export', {  category: 'Jobs', label: 'Recent' });
			var load = PHD.showLoadingIndicator($(document));
			jobFactory.async("/api/jobs/history/?format=csv", null, null, 'Job History', null).then(function(result) {
				PHD.hideLoadingIndicator(load);
			})
		};

		$scope.exportAudits = function() {
			$analytics.eventTrack('CSV Export', {  category: 'Jobs', label: 'System' });
			var load = PHD.showLoadingIndicator($(document));
			jobFactory.async("/api/jobs/system/?format=csv", null, null, 'System Jobs', null).then(function(result) {
				PHD.hideLoadingIndicator(load);
			})
		};

	}]
)
