
function processJobsPage(PHD, window, $rootScope, $analytics) {

    var $activeJobCount = $("#active-job-count"),
        $postProcessCount = $("#post-process-count"),
        UserPreference, JobsTimer,
        tabs = [],
        initialTab = $.cookie("initial_tab_jobs") || 0,
        currentTabIndex = 0,
        DETAIL_TABLE_HEIGHT = 256,
        slugs = ['active', 'manage', 'history', 'system', 'post_processing'],
        jobTypeIcon = [];

    jobTypeIcon["replication"] = "backup-copy";
    jobTypeIcon["restore"] = "recover";
    jobTypeIcon["instant recovery"] = "instant-recover";
    jobTypeIcon["backup"] = "backup";
    jobTypeIcon["archive"] = "backup-copy";
    jobTypeIcon['backup copy'] = "backup-copy";

    function getJobIcon(jobType) {
        var icon = jobTypeIcon[jobType];
        var html;
        switch (icon) {
            case "backup-copy":
                html = '<i class="icon-uui-backup-copy">' + '</i> ';
                break;
            case "instant recovery":
                html = '<i class="icon-uui-instant-recover">' + '</i> ';
                break;
            case "backup":
            case "recover":
            case "restore":
            default:
                html = '<i class="icon-uui-' + jobTypeIcon[jobType] + '"></i> ';
                break;
        }
        return html;
    }

    function getJobMode(mode){
        var html = "";
        // don't put colon if undefined or archive as mode (since generic)
        if (mode !== undefined && mode != 'archive' && mode != 'Backup Copy' && mode != 'replication' && mode != 'Backup Copy (optimized)' ) {
           html = ": " + mode;
        }
        return html;
    }

    function getJobType(type){
        var html = "";
        if (type !== undefined) {
            switch (type) {
                case "archive":
                case "backup copy":
                    html = "Backup Copy";
                    break;
                case "replication":
                    html = "Backup Copy (optimized)";
                    break;
                default:
                    html = type;
                    break;
            }
        }
        return html;
    }

    var itemMap = [];

    $rootScope.JobsTimer.refreshControl = $("#jobs-page-preferences .timer-refresh");
    JobsTimer = $rootScope.JobsTimer;

    itemMap[APP_ID_VMWARE] = "VMs";
    itemMap[APP_ID_HYPERV_2012] = "VMs";

    function setPageRefresh(interval) {
        refreshRate = interval;
        tabs.forEach(function (tab) {
            tab.setTimerInterval(refreshRate);
        });
    }

    function setActiveJobs(num) {
        $activeJobCount.html("(" + num + ")");
    }

    function setPostProcessCount(num) {
        $postProcessCount.html("(" + num + ")");
    }

    function initTabs() {
        $(".ui-tabs-panel").each(function (index) {
            tabs[index] = new JobTab(index, $(this), slugs[index]).init();
        });

        tabs[currentTabIndex].activate();
    }

    $(document)
        .on("uinav.dashboard", function (event) {
            console.log("=====>Selected Dashboard");
            stopAll();
        })
        .on("uinav.protect", function (event) {
            console.log("=====>Selected Protect");
            stopAll();
        })
        .on("uinav.recover", function (event) {
            console.log("=====>Selected Recover");
            stopAll();
        })
        .on("uinav.jobs", function (event) {
            console.log("=====>Selected Jobs");
        })
        .on("uinav.configure", function (event) {
            console.log("=====>Selected Configure");
            stopAll();
        })
        .on("uinav.reports", function (event) {
            console.log("=====>Selected Reports");
            stopAll();
        })
        .on("userpreferencesave.jobspage", function (event, key, value) {
            setPageRefresh(value);
        })
        .on("timersetmanual.jobspage", function (event, $control) {
            $control.show();
        })
        .on("timerunsetmanual.jobspage", function (event, $control) {
            $control.hide();
        });

    function stopAll() {
        JobsTimer.stop();
        $(document).off("uinav");
        $(document).off("userpreferencesave.jobspage");
        $(document).off("timertick.jobspage");
        $(document).off("timersetmanual.jobspage");
        $(document).off("timerunsetmanual.jobspage");
        delete $(document);
    }

    // lmc - move this up.
    var expandedItemsFormatter = {
        instances: function (value, row) {
           //var item_type = row.app_id != undefined && row.app_id != APP_ID_FILE_LEVEL ? (itemMap[row.app_id] + ":") : EMPTY_CELL_TEXT;
            var names = "";
            if (row.instances !== undefined) {
                if ($.isArray(row.instances)) {
                    names = (row.app_id == APP_ID_VMWARE) ?  _.pluck(row.instances, "secondary_name") : _.pluck(row.instances, "primary_name");
                    names = names.toString();
                } else {
                    names = row.instances;
                }
            }
            return names;
        },
        message: function(value, row) {
            return '<span class="truncates item-name" title="' + value + '">' + value + '</span>';
        }
    };

    var JobTab = function (index, panel, slug) {
        this.index = index;
        this.$panel = panel;
        this.$table = this.$panel.find(".active-table");
        this.$buttons = this.$panel.find(".button-bar");
        this.$inventory = this.$panel.find(".job-detail");
        this.$inventoryTable = this.$inventory.find("table");

        this.$dragHandle = this.$panel.find(".handle");
        this.namespace = "jobs" + slug;

        this.dataTable = PHD.DataTable(this.$table, {
            select: this.index != JOB_TAB_SYSTEM ? 'click' : false,
            scrollSelect: this.index != JOB_TAB_HISTORY,  // lmc - added to prevent scroll on row selection.
            placeholder: gettext("There are no jobs to display"),
            prefix: "job_",
            integerIDs: (this.index == JOB_TAB_SYSTEM || this.index == JOB_TAB_HISTORY),
            sortable: true,
            cellClasses: this.index == JOB_TAB_SYSTEM ? [null, "numerical", null, null, "long-text", null] :
                         this.index == JOB_TAB_MANAGER ? [null, null, null, "long-text", null, null] : null
        }).registerFormatters(cellFormatter);

        /*if(this.index === JOB_TAB_ACTIVE || this.index === JOB_TAB_HISTORY || this.index === JOB_TAB_SYSTEM) {
         // Enable TreeTable and DataTable for job inventory detail table
         this.JobInventoryTreeTable = PHD.TreeTable(this.$inventory, {});
         this.JobInventory = PHD.JobInventory(this.$inventoryTable, this.JobInventoryTreeTable, {});
         } else {
         this.JobInventory = PHD.DataTable(this.$inventoryTable, {"select": false});
         this.JobInventory.registerFormatters(expandedItemsFormatter);
         } */
        // lmc - no hierarchy
        var inventoryTableDefaults = {
            select: false,
            sortable: true,
            cellClasses: ["numerical", null, null, "long-text"]
        };
        this.JobInventory = PHD.DataTable(this.$inventoryTable, inventoryTableDefaults);
        this.JobInventory.registerFormatters(expandedItemsFormatter);

        this.$buttons.find(".jobcreate-btn-group").flyout();

        this.$selected = this.$table.find(".current-row");
        this.selectedID = null;
        this.rowData = null;

        this.isActive = false;
        this.timestamp = 0;
        this.inventoryTimestamp = 0;
        this.inventoryOpen = false;
        this.sort = null;
        this.showSystem = true;

        //lmc - if Job manager, use joborders resource; otherwise use jobs resource.
        this.inventoryTableHeight = DETAIL_TABLE_HEIGHT;
        if (slug == 'manage') {
            this.url = "/api/joborders";
        } else {
            if (slug == 'history'){
                this.url = "/api/jobs/history/?include_on_demand";
            } else {
                this.url = "/api/jobs/" + slug;
            }
        }
    };

    JobTab.prototype.init = function () {
        var self = this;
        this.$inventory.hide();

        this.initButtons();

        // Initialize table sorting
        this.$table.sortTable({
            sort_two_dates: sort_two_dates,
            sort_duration: sort_duration
            //sort_dedupe: sort_dedupe
        });

        $(document)
            .on("timertick.jobspage", $.proxy(this.handleTimerTick, this));

        // Handle events thrown by DataTable
        this.$table
            .on("rowselect", "tr.clickable", $.proxy(this.handleRowSelect, this))
            .on("rowdelete", $.proxy(this.handleRowDelete, this))
            .on("updatefinished", $.proxy(this.handleRowAddition, this));

        this.$dragHandle.dragdelta({
            autoHide: true,
            disabled: true,
            min: 360,
            max: window.innerHeight - 90
        }).on("dragdeltastop", function (event, ui) {
            //console.log(ui.delta);
            self.adjustJobDetails(ui.delta);
        });

        return this;
    };

    JobTab.prototype.handleTimerTick = function (event, timer, interval) {
        if (this.isActive) {
            this.update();
        }
    };

    JobTab.prototype.update = function (resetTable, resetTimer) {
        var self = this,
            reset = (this.timestamp === 0) || (resetTable === true),
            tableProcess, inventoryProcess;

        tableProcess = this.updateTable(reset);
        inventoryProcess = this.updateInventory(reset, this.selectedID);

        $.when(tableProcess, inventoryProcess).done(function () {
            if (reset || resetTimer) {
                JobsTimer.start();
            } else {
                JobsTimer.resume();
            }
            self.$table.scrollTable();
        });
    };

    JobTab.prototype.deleteRow = function (jobID) {
        this.dataTable.deleteRow(jobID);
    }

    JobTab.prototype.updateTable = function (reset) {
        var self = this,
            url = this.getUpdateUrl(reset),
            process = $.Deferred();

        var load = (reset) ? PHD.showLoadingIndicator(this.$panel) : null;
        var request = PHD.Ajax.get(url, load);

        request.done(function (data) {
            // Only get active job count when active job API is called.
            if (currentTabIndex == JOB_TAB_ACTIVE) {
                var jobs = data.data;
                var active = (jobs !== undefined && jobs !== null) ? jobs.length : 0;
                var postprocess = 0;
                setActiveJobs(active);
                setPostProcessCount(postprocess);
            } else if (currentTabIndex == JOB_TAB_HISTORY) {
                var data = data.data;
                var results = [];
                if (data !== undefined) {
                    var n = 0;
                    data.forEach(function(obj, index) {
                        var backups = obj.backups;
                        if (backups !== undefined) {
                            backups.forEach(function(backup, index) {
                                backup.server = (obj.app_name == "VMware") ? backup.primary_name : obj.client_name;
                                backup.instance = (obj.app_name == "VMware") ? backup.secondary_name :
                                    (obj.app_name.indexOf("Hyper-V") != -1) ? backup.primary_name : "N/A";
                                backup.app_name = (obj.app_name);
                                backup.name = obj.name;
                                if (backup.message !== undefined) {
                                    if (backup.message.indexOf('Fail') != -1) {
                                        backup.fake_status = 'Error';
                                        backup.expanded_error = backup.message;
                                    } else if (backup.message.indexOf('Warning') != -1) {
                                        backup.fake_status = 'Warning';
                                        backup.expanded_error = backup.message;
                                    } else {
                                        backup.fake_status = 'Success';
                                    }
                                }
                                backup.id = backup.backup_id === "N/A" ? n++ : backup.backup_id;
                                backup.system_name = obj.system_name;
                                backup.system_id = obj.system_id;
                                backup.mode = backup.type;
                                backup.type = obj.type;
                                results.push(backup);
                            })
                        }
                    })
                }
                data = {};
                data.data = results;
            }

            if (data.timestamp === self.timestamp) {
                self.timestamp += 1;
            } else {
                self.timestamp = data.timestamp;
            }

            // lmc - sort?
            //self.setTableSortHeaderForServerSideSort();

            if (reset) {
                self.dataTable.load(data.data);
                self.$table.columnResize();
                PHD.hideLoadingIndicator(load);
            } else {
                self.dataTable.update(data.data, true);
            }

            if (!self.showSystem) {
                self.$table.find(".system-true").addClass("hidden");
            }

            self.selectRowByID();
            self.updateButtons();

            process.resolve();
        });

        return process;
    };

    JobTab.prototype.updateInventory = function (reset, jobID) {
        var process = $.Deferred();

        if (this.inventoryOpen && this.selectedID != null && this.selectedID == jobID) {
            var self = this;
            var data;
            if (this.index === JOB_TAB_HISTORY) {
                data = [];
                if (self.rowData.backups !== undefined) {
                    console.log("in UPDATE INVENTORY");
                    data = self.rowData.backups;
                    data.forEach(function(obj, index) {
                        //obj.system_name = self.rowData.system_name;
                        obj.server = (self.rowData.app_name == "VMware") ? obj.primary_name : self.rowData.client_name;
                        obj.instance = (self.rowData.app_name == "VMware") ? obj.secondary_name :
                            (self.rowData.app_name.indexOf("Hyper-V") != -1) ? obj.primary_name : "N/A";
                        obj.app_name = (self.rowData.app_name);
                    });
                    self.JobInventory.load(data);
                    self.$inventoryTable.scrollTable({height: self.inventoryTableHeight - 6});
                    self.$inventoryTable.sortTable();
                    self.$inventoryTable.columnResize();
                    self.updateButtons();
                }
                process.resolve();
            } else if (self.rowData.type === 'replicationxx') {
                console.log("<<<<  type is replication");
                data = [];
                var o = {};
                o.system_name = self.rowData.system_name;
                o.instances = "Open Job to see details";
                o.client_name = "Open Job to see details";
                o.description = "Replication Job";
                o.calendar_str = "No Schedule";
                data.data = [o];
                self.JobInventory.load(data.data);
                self.$inventoryTable.scrollTable({height: self.inventoryTableHeight - 6});
                self.$inventoryTable.columnResize();
                self.updateButtons();
                process.resolve();
            } else {
                var load = (reset) ? PHD.showLoadingIndicator(this.$inventory) : null;
                var url = this.getInventoryUrl();
                var request = PHD.Ajax.get(url, load);

                request.done(function (data) {
                    //if (reset) {
                    //    PHD.hideLoadingIndicator(load);
                    //}
                    // get the one active object for the details.
                    var found = false;
                    if (data.data) {
                        for (var i = 0; i < data.data.length; i++) {
                            var o = data.data[i];
                            if (o.id == self.selectedID) {
                                // copy relevant items from main row.
                                if (self.rowData != null) {
                                    var row = self.rowData;
                                    o.system_name = (row.system_name != undefined) ? row.system_name : "";
                                    o.description = (typeof row.description != 'undefined') ? row.description : "";
                                    o.next_time = (typeof row.next_time != 'undefined') ? row.next_time : "";
                                    o.calendar_str = (typeof row.calendar_str != 'undefined') ? row.calendar_str : "";
                                    o.application = (row.application_name != undefined) ? row.application_name :
                                                    (row.application != undefined) ? row.application : "";
                                    // lmc - TODO Hyper-V?
                                    if (o.application == "VMware") {
                                        if (o.instances != undefined && o.instances[0] != undefined && o.instances[0].primary_name != undefined) {
                                            o.client_name = o.instances[0].primary_name;
                                        }
                                    } else if (o.application == "Hyper-V") {
                                        o.client_name = "Hyper-V Host"; // lmc TODO - temporary, need results from API (UNIBP-1856).
                                    }
                                    if(row.type === "replication"){
                                        o.appliance = row.target;
                                    }

                                }
                                data.data = [o];
                                found = true;
                                break;
                            }
                        }
                    }
                    if (!found) {
                        data.data = [];
                    }
                    self.inventoryTimestamp = data.timestamp;
                    if (reset) {
                        self.JobInventory.load(data.data);
                    } else {
                        self.JobInventory.update(data.data);
                    }

                    self.$inventoryTable.scrollTable({height: self.inventoryTableHeight - 6});
                    self.updateButtons();
                    process.resolve();
                });
            }
        } else {
            process.resolve();
        }

        return process;
    };

    JobTab.prototype.queueSelection = function (dbid) {
        var self = this;
        this.$table.one("loadfinished", function () {
            if (self.dataTable.selectByID(dbid)) {
                self.$buttons.find(".btn-view-details").trigger("click");
            }
        });
    }

    JobTab.prototype.handleRowSelect = function (event, dbid, $row, DataTable, DataRow) {
        this.$selected = $row;
        this.selectedID = dbid;
        this.rowData = DataRow.data;
        //console.log("in handleRowSelect, id = " + dbid + " and rowData = " + JSON.stringify(this.rowData));
        this.updateButtons();
        if (this.inventoryOpen) {
            console.log("Inventory is OPEN");
            this.showJobDetails();
        }
    };

    JobTab.prototype.handleRowDelete = function (event, dbid, DataTable) {
        if (this.selectedID === dbid) {
            this.$selected = $();
            this.selectedID = null;
            this.$inventory.find("table tbody tr").remove();
            this.hideJobDetails();
        }
    };

    JobTab.prototype.handleRowAddition = function (event, newRows, DataTable) {
        if (newRows) {
            this.$table.sortTable();
        }
    };

    JobTab.prototype.selectRowByID = function (id) {
        id = id || this.selectedID;
        if (!id) {
            return false;
        }
        this.dataTable.selectByID(id);
    };

    /*JobTab.prototype.serverSideSort = function(event) {
     var $column = $(event.target).closest("th"),
     sortBy = $column.data("source");

     if(this.sort === sortBy) {
     this.sort = "-" + this.sort;
     } else if(this.sort === ("-" + sortBy)) {
     this.sort = this.sort.substr(1);
     } else {
     this.sort = sortBy;
     }
     this.updateTable(true);
     }; */

    /*
     JobTab.prototype.setTableSortHeaderForServerSideSort = function() {
     if(this.$table.is("#history-jobs")) {
     var sortDirection = "asc",
     sortBy = this.sort;
     if(sortBy.substr(0,1) === "-") {
     sortDirection = "desc";
     sortBy = sortBy.substr(1);
     }
     var $headers = this.$table.find("th").removeClass("sort-asc sort-desc");
     $headers.filter("th[data-source=" + sortBy + "]").addClass("sort-" + sortDirection);
     }
     };
     */

    JobTab.prototype.getUpdateUrl = function (reset) {
        //var timestamp = reset ? 0 : this.timestamp,
       //     url = this.url + "?timestamp=" + timestamp;
        return this.url;
       // return url;
    };

    JobTab.prototype.getInventoryUrl = function () {
        var url = "/api/jobs";
        switch (this.index) {
            case JOB_TAB_ACTIVE:
                url += "/active";
                break;
            case JOB_TAB_HISTORY:
                url += "/history/";
                break;
            case JOB_TAB_SYSTEM:
                url += "/system/";
                break;
            case JOB_TAB_MANAGER:
                url = "/api/joborders/" + this.selectedID;
                break;
        }
        return url;
    };

    JobTab.prototype.initButtons = function () {
        var tab = this;
        this.$buttons.find("a").each(function (index) {
            var $but = $(this);
            PHD.buttonLogic($but, null, tab, null, null, $analytics);
        });
    };

    JobTab.prototype.updateButtons = function ($button) {
        var $row = this.$selected,
            id = this.selectedID,
            rowData = this.rowData;
        tab = this;

        if ($button !== void 0) {
            PHD.buttonLogic($button, $row, tab, id, rowData, $analytics);
        } else {
            this.$buttons.find("a").each(function (index) {
                var $but = $(this);
                PHD.buttonLogic($but, $row, tab, id, rowData, $analytics);
            });
        }
    };

    JobTab.prototype.adjustJobDetails = function (delta) {
        this.inventoryTableHeight -= delta;
        if (!this.$inventory.is(":hidden")) {
            this.$inventoryTable.adjustTableHeight(this.inventoryTableHeight - 6);
            this.$table.offsetTableHeight(delta);
        }
    };

    JobTab.prototype.showJobDetails = function () {
        this.inventoryTimestamp = 0;
        if (this.$inventory.is(":hidden")) {
            this.$table
                .adjustTableHeight(this.$table.getAvailableVerticalSpace(".full-height") - this.inventoryTableHeight);
            this.$inventory.show("slide", {direction: "down"});
            this.inventoryOpen = true;
            this.updateInventory(true, this.selectedID);
            this.$dragHandle.dragdelta("option", "disabled", false);
        } else {
            this.updateInventory(true, this.selectedID);
        }
        this.updateButtons(this.$panel.find(".btn-view-details"));
    };

    JobTab.prototype.hideJobDetails = function () {
        var self = this;
        this.$dragHandle.dragdelta("option", "disabled", true);
        this.$inventory.hide("slide", {direction: "down"}, function () {
            self.$table
                .adjustTableHeight(self.$table.getAvailableVerticalSpace(".full-height"));
            self.inventoryOpen = false;
            self.updateButtons(self.$panel.find(".btn-view-details"));
        });
    };

    JobTab.prototype.toggleSystemJobs = function (show) {
        var self = this;
        this.$table.find(".system-true").toggleClass("hidden", !show);
        this.$table.scrollTable();
        this.showSystem = show;
    };

    JobTab.prototype.setTimerInterval = function (interval) {
        JobsTimer.setInterval(refreshRate);
        if (this.$panel.is(":visible")) {
            JobsTimer.reset();
        }
    };

    JobTab.prototype.activate = function () {
        this.isActive = true;
        this.$dragHandle.dragdelta("option", "max", window.innerHeight - 90);
        this.update(false, true);
    };

    JobTab.prototype.deactivate = function () {
        this.isActive = false;
        JobsTimer.stop();
    };

    var sort_two_dates = function (a, b) {
        var x, y;
        if (a.value === EMPTY_CELL_TEXT) {
            return -1;
        }
        if (b.value === EMPTY_CELL_TEXT) {
            return 1;
        }
        if (_.isArray(a.value)) {
            x = a.value.map(function (val) {
                return new Date(val);
            });
            x = x[0] < x[1] ? x[0] : x[1];
        } else {
            x = new Date(a.value)
        }
        if (_.isArray(b.value)) {
            y = b.value.map(function (val) {
                return new Date(val);
            });
            y = y[0] < y[1] ? y[0] : y[1];
        } else {
            y = new Date(b.value)
        }
        return (x < y) ?
            -1 : (x > y) ?
            1 : 0;
    };

    var sort_duration = function (a, b) {
        if (a.value === EMPTY_CELL_TEXT || b.value === null) {
            return 1;
        }
        if (b.value === EMPTY_CELL_TEXT || a.value === null) {
            return -1;
        }
        var aVals = a.value.split(":").map(function (val) {
                return parseInt(val, 10);
            }),
            bVals = b.value.split(":").map(function (val) {
                return parseInt(val, 10);
            });

        a = (aVals[0] * 60 * 60) + (aVals[1] * 60) + aVals[2];
        b = (bVals[0] * 60 * 60) + (bVals[1] * 60) + bVals[2];
        return a - b;
    };

    /*var sort_dedupe = function (a, b) {
        if (a.value === 'Infinite' || b.value === null) {
            return 1;
        }
        if (b.value === 'Infinite' || a.value === null) {
            return -1;
        }
        var x = parseInt(a.value.split(":")[0].replace(',', ''), 10),
            y = parseInt(b.value.split(":")[0].replace(',', ''), 10);

        return x - y;
    };*/

    // Object to convert raw values to display-friendly values
    var cellFormatter = {
        name: function (value, row) {
            var display = typeof row.environment !== "undefined" ? value + " (" + row.environment + ")" : value;
            return '<span class="truncates item-name" title="' + display + '">' + display + '</span>';
        },
        // for system jobs
        username: function (value, row) {
            var icon = '</i>';
            var icon_class = value == 'System' ? 'icon-managedhosting' :
                             value.indexOf('bypass') !== -1 ? 'icon-logprograms' :  'icon-useralt' ;
            icon = '<i class="' + icon_class + ' icon">' + icon;
            return '<span class="job-type-display">' + icon + value + '</span>';
        },
        // lmc
        type: function (value, row) {
            var icon = getJobIcon(row.type.toLowerCase());
            var type = getJobType(value);
            var mode = getJobMode(row.mode);
            return '<span class="job-type-display">' + icon + type + mode + '</span>';
        },
        calendar_str: function (value, row) {
            return '<span class="truncates item-name" title="' + value + '">' + value + '</span>';
        },
        joborderbackup__schedule_type: function (value, row) {
            return Array.isArray(value) ? value.join("<br>") : value;
        },
        job_status: function (value, row) {
            var status = getStatusIcon(row);
            icon = '<i class="icon-phd-' + status + ' icon"></i>';
            
            return '<span class="job-status-display">' + icon + value + '</span>';
        },
        fake_status: function (value, row) {
            var icon = "</i>",
                val = row.fake_status,
                status = row.fake_status,
                title = row.expanded_error !== undefined ? row.expanded_error : row.message;

            if (row.fake_status == 'Error') {
                icon = '<i class="icon-phd-error icon">' + icon;
            } else if (row.fake_status == 'Warning') {
                icon = '<i class="icon-phd-warning icon">' + icon;
            } else {
                icon = '<i class="icon-phd-success icon">' + icon;
            }
            return '<span class="joborder-status-display" title="' + title + '">' + icon + val + '</span>';
        },
        start_date_unf: function (value, row) {
            if (value === EMPTY_CELL_TEXT) {
                return value;
            }
            return moment(value).format("MMM D, YYYY h:mm A");
        },
        data_written: function (value, row) {
            return PHD.util.formatBytes(value);
        },
        current_speed: function (value, row) {
            return PHD.util.formatBytes(value) + "/s";
        },
        instances: function (value, row) {
            return _.pluck(value, "names");
        }
    };
    cellFormatter.vm_name = cellFormatter.name;
    //cellFormatter.last_time = cellFormatter.next_time;
    cellFormatter.total_storage = cellFormatter.data_written;
    cellFormatter.speed = cellFormatter.average_speed = cellFormatter.current_speed;
    cellFormatter.job_type_extra = cellFormatter.job_type;
    // lmc - add these
    cellFormatter.start_date = cellFormatter.start_date_unf;
    cellFormatter.finish_date = cellFormatter.start_date_unf;
    cellFormatter.status = cellFormatter.job_status;
    cellFormatter.comment = cellFormatter.calendar_str;
    cellFormatter.message = cellFormatter.comment;
    // lmc - end of add these

    var lookupIcon = [];
    lookupIcon['on hold'] = "";
    lookupIcon["queued"] = "";
    lookupIcon["connecting"] = "active";
    lookupIcon["active"] = "active";
    lookupIcon["running"] = "active";
    lookupIcon["db update"] = "active";
    lookupIcon["cancelled"] = "";
    lookupIcon["warning"] = "";
    lookupIcon["successful"] = "";
    lookupIcon['error'] = "";
    lookupIcon["complete"] = "";
    lookupIcon["process terminated"] = "";
    lookupIcon["unknown"] = "unknown";
    lookupIcon["incomplete/cancelled"] = "";
    lookupIcon["available"] = "";
    lookupIcon["idle"] = ""; // clear this from idle, as flag doesn't change properly when enabled/disaabled.
    
    function getStatusIcon(row) {
        var status;
        var raw_status = row.status;
        if (row.status !== undefined) {
            status = lookupIcon[raw_status.toLowerCase()];
        }
        /*if (row.enabled == false) { // could be 0 or false, use ==.
            status = 'disabled';
        } else {
            var raw_status = row.status;
            if (row.status !== undefined) {
                status = lookupIcon[raw_status.toLowerCase()];
            }
        }*/
        return status;
    }


    function handlePlugin() {
        var params = PHD.util.getParams();
        if (params) {
            if (params.hasOwnProperty("action")) {
                $(document).trigger("launchdialog", [params]);
            }

            if (params.hasOwnProperty("select")) {
                // lmc - add this to kill the other timers.
                var e = $.Event('uinav.protect');
                $(document).trigger(e);
                tabs[0].queueSelection(params.select);
            }
            // lmc - don't do this
            //window.history.replaceState({}, "", window.location.origin + window.location.pathname);
        }
    }

    function handleTabChange() {
        $(document).trigger("tabchange", ["jobs" + slugs[currentTabIndex]]);
    }

    /*
     *   DOM Ready Function
     */
    $(function () {
        // Quick and dirty way to change what tab opens by default via URL
        if (!isNaN(PHD.util.getHashInt())) {
            initialTab = PHD.util.getHashInt();
        }

        UserPreference = PHD.UserPreference($("#jobs-page-preferences .preference-trigger"), {
            name: gettext("Jobs Page"),
            namespace: "jobspage",
            key: "page_jobs_refresh",
            defaultValue: refreshRate
        });

        /*JobsTimer = PHD.Timer({
            namespace: "jobspage",
            interval: refreshRate,
            refreshControl: $("#jobs-page-preferences .timer-refresh")
        }); */

        // Initialize the tabs
        $('.tabs-wrapper').tabs({
            active: initialTab,
            create: function (event, ui) {
                currentTabIndex = ui.tab.index();
                initTabs();
                handleTabChange();
            },
            activate: function (event, ui) {
                currentTabIndex = ui.newTab.index();
                $.cookie("initial_tab_jobs", currentTabIndex, {expires: 365, path: '/'});
                tabs[ui.oldTab.index()].deactivate();
                tabs[currentTabIndex].activate();
                handleTabChange();
            }
        });

        // Handle actions from plugin
        handlePlugin();
    });

//})(jQuery, window.PHD || (window.PHD = {}), window);
}
