(function($, PHD, window, undefined) {
    
    /*
    *   Handles CRUD and events for widget- or page-based User Preferences
    */

    var defaults = {
        name: "PHD Virtual Backup",
        namespace: "phduserpreference",
        key: 0,
        defaultValue: 0,
        modal: false
    };

    var dialogOptions = {
        autoOpen: true,
        modal: false,
        show: { effect: "slide", direction: "up" },
        hide: { effect: "scale", percent: 0, origin: ["top", "right"] },
        dialogClass: "user-preference-dialog",
        resizable: false,
        draggable: false,
        width: 262,
        buttons: {}
    };

    var UserPreference = function UserPreference($trigger, options) {
        if(this instanceof UserPreference) {
            options = options || {};
            this.options = $.extend({}, defaults, options);

            this.$trigger = $trigger;
        } else {
            return new UserPreference($trigger, options).init();
        }
    };

    UserPreference.prototype = {
        init: function() {
            var self = this;

            this.$dialog = $("<div>", {
                "id": "dialog-" + this.options.namespace,
                "class": "dialog-box user-preference"
            });

            this._dialogOptions = $.extend({}, dialogOptions);
            this._dialogOptions.modal = this.options.modal;
            this._dialogOptions.position ={
                my: "right top",
                at: "right top",
                of: this.$trigger.parent()
            };
            this._dialogOptions.title = this.options.name + " Preferences";

            //this._dialogOptions.buttons["Save"] = function() {$(document).trigger("prefdialogsave." + this.namespace);};
            //this._dialogOptions.buttons["Cancel"] = function() {$(document).trigger("prefdialogcancel." + this.namespace);};
            //console.log(this._dialogOptions);

            this.$trigger.on("click", function(event) {
                event.preventDefault();
                event.stopPropagation();
                self.edit();
            });

            this.$dialog
                .on("click", ".btn-pref-save", $.proxy(this._save, this))
                .on("click", ".btn-pref-cancel", $.proxy(this._cancel, this));

            //$(document).on("prefdialogsave." + this.namespace, function() {
            //    self._save();
            //});

            return this;
        },

        edit: function() {
            var self = this;
	        // lmc - build page by loading template and variable value.
            /*
            PHD.Ajax.getHTML("/preferences/edit/" + this.options.key + "/")
		    .done(function(html) {
                    self.$dialog.html(html);
                    self.$dialog.find("#user-preference-name").text(self.options.name);
                    self.$dialog.insertAfter(self.$trigger).dialog(self._dialogOptions);    
                });
		    */

            var scope = $("[ng-controller=GlobalCtrl]").scope();
            var username = scope.user.name;

            PHD.Ajax.get("/api/preferences/" + username + "/" + this.options.key + "/")
		    .done(function(data) {
                var preferences = data.data;
			    rate = preferences[0][self.options.key];
            	PHD.Ajax.getHTML("app/preferences/userpreference.html")
				   .done(function(html) {
                 		self.$dialog.html(html);
                   		self.$dialog.find("#user-preference-name").text(self.options.name);
                   		self.$dialog.find("#refresh-name").text(self.options.refresh_name);
                   		self.$dialog.find("#id_refresh").val(rate);
                   		self.$dialog.find("#user-preference-rate").text(rate);
                   		self.$dialog.insertAfter(self.$trigger).dialog(self._dialogOptions);
			});
		});
        },

        _save: function() {
            var self = this,
                $form = this.$dialog.find("form"),
                key = this.options.key,
                values = $form.serializeArray(),
                //value = _.findWhere(values, {"name": key}),
		        // lmc - differences in form creation.
                value = _.findWhere(values, {"name": "refresh_rates"});
                //putData = $.param({"key": key, "value": value.value});

            var putData = {};
            putData[key] = value.value;

            var scope = $("[ng-controller=GlobalCtrl]").scope();
            var username = scope.user.name;

            PHD.Ajax.put('/api/preferences/' + username + '/', JSON.stringify(putData))
                .done(function(data) {
                    //$(document).trigger("userpreferencesave." + self.options.namespace, [key, data.value]);
		            // lmc refresh rate in sec vs ms, value not returned in data
		            var rate = parseInt(value.value) * 1000;
                    console.log("trigger namespace " + self.options.namespace + " and rate = ", rate);
                    $(document).trigger("userpreferencesave." + self.options.namespace, [key, rate]);
                    self._close();
                })
                .fail(function(data) {
                    $(document).trigger("userpreferencefail." + self.options.namespace, [key, value, data]);
                    self._close();
                });
        },

        _cancel: function() {
            this._close();
        },

        _close: function() {
            this.$dialog.dialog("destroy").html("").detach();
        }

    };

    PHD.UserPreference = UserPreference;

    var isInt = function(n) {
        return !isNaN(parseInt(n, 10));
    };

})(jQuery, window.PHD || (window.PHD = {}), window);
