(function($, PHD, window, undefined) {

    var $form = $("#backup-copy-recovery-form"),
        ImportForm;

    var $jobLogTable = $("#backup-copy-table"),
        LogTable;

    var ids_to_import;

    function initForm() {

        var sid = selections.system_id;
        console.log("selections = " + JSON.stringify(selections));
        var bids = selections.backups;
        var instance = instance_id;


        LogTable = PHD.DataGrid($jobLogTable, {
            dynamic: true,
            tableScrollOptions: {"height": "article"},
            select: false,
            sortable: true,
            sortOnLoad: true,
            sortOnHeader: true,
            cellClasses: [null, null, 'long-text truncate', null, null, null, null, null, null, null],
            classifiers: ['success']
        }).registerFormatters(cellFormatter);

        updateTable();

        function updateTable() {
            var related_url = "/api/archive/related/" + bids[0] + "/?sid=" + sid + "&instance=" + instance_id;
            var resp = PHD.Ajax.get(related_url);
            resp.done(function(data){
                var status_url = "/api/archive/status/?sid=" + sid;
                ids_to_import = data.ids;
                if (ids_to_import !== undefined) {
                    var idJSON = {};
                    idJSON.archive_ids = ids_to_import.split(',').map(function(id) {
                         return parseInt(id, 10);
                    });
                    idJSON = JSON.stringify(idJSON);
                    PHD.Ajax
                        .post(status_url, idJSON)
                        .done(function(data) {
                            if (data.status === undefined || data.status.length == 0) {
                                var obj = {};
                                obj.id = 0;
                                obj.success = 'none';
                                obj.backup_date = 'unknown';
                                obj.instance_description = 'The backup copies no longer exist on the appliance.';
                                data.status = [obj];
                            }
                            LogTable.load(data.status);
                        });
                } else {
                    var obj = {};
                    obj.id = 0;
                    obj.success = 'none';
                    obj.backup_date = 'unknown';
                    obj.instance_description = 'The backup copies no longer exist on the appliance.';
                    data.status = [obj];
                    LogTable.load(data.status);
                }
            });
        }

        ImportForm = PHD.FormController($form, {
            namespace: "importform"
        });

        ImportForm
            .on("formcancel", function(event) {
                PHD.currentDialog.wizard("close");
            })
            .on("formsubmit", function(event, data) {
                data = {};
                var message = "<p>" + gettext("Importing of Backup Copies successfully started.") + "<br/><br/>" +
                    gettext("After all recovery jobs have completed, please return to the Recover page to recover the imported backups.") + "</p>";
                data.result = [ {'message': message}];
                var options = {};
                if (window.location.href.indexOf("#/jobs") === -1)  {
                    console.log("options ", options);
                    options.buttons = [
                        {
                            text: gettext("View Jobs"),
                            click: function() {
                                window.location.href = "#/jobs/#0";
                                $(this).dialog("destroy").remove();
                            }
                        }
                    ];
                }
                PHD.throwNotice(data, options);
                PHD.currentDialog.wizard("close");
            });

        ImportForm.submit = function() {
            var self = this,
                load = PHD.showLoadingIndicator("body", true, "Starting Import...");

            this.$buttons.blur().disable();

            //var submitData = form2js(this.$form.attr("id"), '.', false);

            PHD.appliance_sid = selections.system_id;

            var restoreObj = {};
            restoreObj.archive_ids = ids_to_import;
            restoreObj.association = "dr";              // associates with an instance if it exists, creates one if it does not.
            restoreObj = JSON.stringify(restoreObj);
            console.log('=== Submit form: object = ' + restoreObj);

            var sid = (PHD.appliance_sid === null) ? "" : "?sid=" + PHD.appliance_sid;
            var url = "/api/restore/archive/full/" + sid;
            console.log("POST url is " + url);
            PHD.Ajax
                .post(url, restoreObj)
                .done(function(data) {
                    self.$form.trigger("formsubmit", [data, self.$form, self]);
                })

                .always(function() {
                    self.$buttons.enable();
                    PHD.hideLoadingIndicator(load);
                });

            return this;
        };
    }

    var cellFormatter = {
        instance_description: function(value) {
            var title = _.escape(value);
            return '<span class="truncates item-name" title="' + title + '">' + value + '</span>';
        },
        success: function (value, row) {
            var iconClass = value == undefined ? 'warning' : value === true ? "success" : "error";
            return '<i class="icon-phd-'+ iconClass +' icon-large" title="' + iconClass + '"></i>';
            //return '<span class="storage-status">' + statusIcon + '</span>';
        },
        encrypted: function (value, row) {
            return (value === true) ? "Yes" : "No";
        },
        size: function (value, row) {
            return(value/1024).toFixed(2);
        }
    };

    /*
    *   DOM Ready Function
    */
    $(function() {
        initForm();
    });

})(jQuery, window.PHD || (window.PHD = {}), window);
