function processBackupFilter(PHD, window) {

        /*
         *   BackupFilter defines behavior for the controls used to filter backups
         *   from the backup catalog
         */

        var BackupFilter = function BackupFilter($form, catalog) {
            if(this instanceof BackupFilter) {
                this.$form = $form;
                this.catalog = catalog;
                this.data = catalog.tree;
                this.trueForm = this.$form.is("form");
            } else {
                return new BackupFilter($form, catalog).init();
            }
        };

        var errorTooltipOptions = {
            disabled: true,
            position: {
                my: "right-10 center",
                at: "left center"
            },
            tooltipClass: "error-tooltip",
            track: false
        };

        BackupFilter.prototype = {
            init: function() {
                var self = this;

                this.$name = this.$form.find(".filter-name").data("fid", "name");
                this.$environments = this.$form.find(".filter-environment").data("fid", "env");
                this.$hosts = this.$form.find(".filter-host").data("fid", "host");
                this.$types = this.$form.find(".filter-type").data("fid", "type");
                this.$recent = this.$form.find(".filter-most-recent").data("fid", "recent");
                this.$pinned = this.$form.find(".filter-pinned").data("fid", "pin");
                this.$success = this.$form.find(".filter-success").data("fid", "status");
                //this.$bdstype = this.$form.find(".filter-bds-type").data("fid", "bds-type");
                this.$backupType = this.$form.find(".filter-backup-type").data("fid", "backup-type");

                this.$fromDate = this.$form.find(".filter-date-from").datepicker({
                    showOn: "both",
                    buttonText: '<img src="assets/images/calendar.png" width="24" height="24">',
                    dateFormat: DATE_FORMAT
                }).data("fid", "from").attr("placeholder", DATE_DISPLAY_FORMAT);

                this.$toDate = this.$form.find(".filter-date-to").datepicker({
                    showOn: "both",
                    buttonText: '<img src="assets/images/calendar.png" width="24" height="24">',
                    dateFormat: DATE_FORMAT
                }).data("fid", "to").attr("placeholder", DATE_DISPLAY_FORMAT);

                this.$submit = this.$form.find(".filter-submit").on("click", $.proxy(this.submitFilter, this));

                this.$clear = this.$form.find(".filter-clear").on("click", $.proxy(this.clearFilter, this));

                this.$form.on("keydown", "input, select", function(e) {
                    e.stopPropagation();
                    if(e.keyCode == $.ui.keyCode.ENTER) {
                        self.submitFilter();
                    }
                    if(e.keyCode == $.ui.keyCode.ESCAPE) {
                        self.clearFilter();
                    }
                });

                this.$recent.on("change", $.proxy(this.handleRecent, this));

                this.$toDate.closest(".control-group")
                    .tooltip(errorTooltipOptions);

                this.$fromDate.closest(".control-group")
                    .tooltip(errorTooltipOptions);

                return this;
            },

            handleRecent: function() {
                var checked = this.$recent.prop("checked");
                this.$toDate.prop("disabled", checked);
                this.$fromDate.prop("disabled", checked);
            },

            refresh: function(data) {
                this.data = data;
                return this.collectOptions();
            },

            collectOptions: function() {
                //this.resetDropdown(this.$environments, this.catalog.getEnvironments());
                //this.resetDropdown(this.$hosts, this.catalog.getHosts());
                this.resetDropdown(this.$types, this.catalog.getTypes());
                return this;
            },

            serializeForm: function() {
                return this.$form.find("input, select").serialize();
            },

            submitFilter: function() {
                var fns = [],
                    datesValid = this.validateDates();

                console.log('in submit filter function');
                if(!datesValid) return;

                fns.push(function(node) {
                    if(node.parent && node.parent.filtered.length) {
                        node.filtered.push("filtered-parent");
                        return true;
                    }
                    return false;
                });

                /*
                if (this.$bdstype.val()) {
                    var keep = this.$bdstype.val().map(function(i){return parseInt(i, 10);});
                    fns.push(function(node) {
                        if (node.type === 'backup'&& keep.indexOf(node.bds_type) === -1) {
                            node.filtered.push("filtered-bds-type");
                            return true;
                        } else {
                            return false;
                        }
                    });
                }
                */

                if (this.$backupType.val()) {
                    var keep = this.$backupType.val();
                    fns.push(function(node) {
                        if ((node.type === 'backup' || node.type === 'archive' || node.type === 'imported') && keep.indexOf(node.type) === -1) {
                            node.filtered.push("filtered-backup-type");
                            return true;
                        } else {
                            return false;
                        }
                    });
                }

                if(this.$pinned.is(":checked")) {
                    fns.push(function(node) {
                        if(node.type === 'backup' && !node.legal_hold) {
                            node.filtered.push("filtered-pinned");
                            return true;
                        } else {
                            return false;
                        }
                    });
                }

                if(this.$success.is(":checked")) {
                    fns.push(function(node) {
                        if ((node.type === 'backup' || node.type === 'archive' || node.type === 'imported') && (node.status == 'Failed' || node.status == 'Warning')) {
                            node.filtered.push("filtered-successes");
                            return true;
                        } else {
                            return false;
                        }
                    });
                }

                if(this.$recent.is(":checked")) {
                    fns.push(function(node) {
                        if(node.type === 'backup' && !node.last) {
                            node.filtered.push("filtered-recent");
                            return true;
                        } else {
                            return false;
                        }
                    });
                }

                if(this.$name.val() !== undefined && this.$name.val() !== "") {
                    var nameQuery = this.$name.val().toLowerCase();
                    fns.push(function(node) {
                        if(node.type === 'vm' || node.type === 'compute_resource') {
                            var subject = node.name.toLowerCase();
                            if(subject.indexOf(nameQuery) === -1) {
                                node.filtered.push("filtered-name");
                                return true;
                            } else {
                                return false;
                            }
                        } else {
                            return false;
                        }
                    });
                }

                if (this.$environments.val() && this.$environments.val() !== "") {
                    var envQuery = this.$environments.val().toLowerCase();
                    fns.push(function(node) {
                        if (node.type === 'backup' || node.type === 'archive' || node.type === 'imported') {
                            var subject = node.system_name.toLowerCase();
                            if(subject.indexOf(envQuery) === -1) {
                                node.filtered.push("filtered-env");
                                return true;
                            } else {
                                return false;
                            }
                        } else {
                            return false;
                        }
                    });
                }

                if (this.$hosts.val() !== undefined && this.$hosts.val() !== "") {
                    var hostQuery = this.$hosts.val().toLowerCase();
                    fns.push(function(node) {
                        if(node.type === 'backup') {
                            var subject = node.host ? node.host.toLowerCase() : "";
                            if(subject.indexOf(hostQuery) === -1) {
                                node.filtered.push("filtered-host");
                                return true;
                            } else {
                                return false;
                            }
                        } else {
                            return false;
                        }
                    });
                }

                if (this.$types.val() !== undefined && this.$types.val() !== "") {
                    var typeQuery = this.$types.val().toLowerCase().trim();
                    fns.push(function(node) {
                        if (node.type === 'backup' || node.type === 'archive' || node.type === 'imported') {
                            var subject = node.backup_mode.toLowerCase().trim();
                            if(subject !== typeQuery) {
                                node.filtered.push("filtered-type");
                                return true;
                            } else {
                                return false;
                            }
                        } else {
                            return false;
                        }
                    });
                }

                if (this.$fromDate.val() !== undefined  && this.$fromDate.val() !== "" && !this.$fromDate.prop("disabled")) {
                    var fromQuery = moment(this.$fromDate.val());
                    fns.push(function(node) {
                        if(node.type === 'backup' || node.type === 'archive' || node.type === 'imported') {
                            var subject = moment(node.start_date);
                            if(subject.isBefore(fromQuery)) {
                                node.filtered.push("filtered-datefrom");
                                return true;
                            } else {
                                return false;
                            }
                        } else {
                            return false;
                        }
                    });
                }

                if (this.$toDate.val() !== undefined && this.$toDate.val() !== "" && !this.$toDate.prop("disabled")) {
                    var toQuery = moment(this.$toDate.val());
                    //3.14.15:sab:UNIBBP-1859.
                    toQuery.add(1, 'days').subtract(1, 'seconds');
                    fns.push(function(node) {
                        if(node.type === 'backup' || node.type === 'archive' || node.type === 'imported') {
                            var subject = moment(node.start_date);
                            if(subject.isAfter(toQuery)) {
                                node.filtered.push("filtered-dateto");
                                return true;
                            } else {
                                return false;
                            }
                        } else {
                            return false;
                        }
                    });
                }

                this.runFilter(this.data, fns);
                this.catalog.filter();
            },

            runFilter: function(nodes, fns) {
                var self = this,
                    all = true;

                if(fns.length) {
                    nodes.forEach(function(node) {
                        var filtered = false;
                        node.filtered = [];
                        fns.forEach(function(fn) {
                            if(fn(node)) filtered = true;
                        });
                        if(node.children) {
                            if(self.runFilter(node.children, fns)) {
                                node.filtered.push("filtered-children");
                                filtered = true;
                            }
                        }
                        if(!filtered) {
                            all = false;
                        }
                    });
                    return all;
                }
                return false;
            },

            clearFilter: function() {
                this.$form.find("[type=text]").val("");
                this.$form.find("[type=checkbox]").prop("checked", false);
                this.$success.prop("checked", true);
                this.$form.find("select").prop("selectedIndex", 0);
                //this.$bdstype.find("option").prop("selected", true);
                this.$backupType.find("option").prop("selected", true);
                this.clearDates();
                this.submitFilter();
            },

            clearDates:  function(){
                this.$toDate
                    .removeClass("error")
                    .closest(".control-group")
                    .attr("title", "")
                    .tooltip("option", "disabled", true)
                    .removeClass("error");
                this.$fromDate
                    .removeClass("error")
                    .closest(".control-group")
                    .attr("title", "")
                    .tooltip("option", "disabled", true)
                    .removeClass("error");
            },

            resetDropdown: function($select, values) {
                $select.html('<option value="" selected="selected">'+gettext("All")+'</option>');
                if(values && _.isArray(values)) {
                    var options = values.map(function(value) {
                        return value === "" ? null : '<option value="' + value + '">' + value + '</option>';
                    }).join("");
                    $select.append($(options));
                }
                return this;
            },

            buildDropdown: function($select, items) {
                items.forEach(function(item) {
                    $select.append($("<option/>", { value: item.id, html: item.name }));
                });
                return this;
            },

            validateDates: function() {
                var to = this.$toDate.val(),
                    from = this.$fromDate.val(),
                    mTo, mFrom, valid = true;

                if(to) {
                    if(PHD.util.isValidDate(to)) {
                        mTo = moment(to);
                        this.$toDate
                            .removeClass("error")
                            .closest(".control-group")
                            .attr("title", "")
                            .tooltip("option", "disabled", true)
                            .removeClass("error");
                    } else {
                        this.$toDate
                            .addClass("error")
                            .closest(".control-group")
                            .attr("title", "Enter a valid date")
                            .tooltip("option", "disabled", false)
                            .addClass("error");
                        valid = false;
                    }
                }

                if(from) {
                    if(PHD.util.isValidDate(from)) {
                        mFrom = moment(from);
                        this.$fromDate
                            .removeClass("error")
                            .closest(".control-group")
                            .attr("title", "")
                            .tooltip("option", "disabled", true)
                            .removeClass("error");
                    } else {
                        this.$fromDate
                            .addClass("error")
                            .closest(".control-group")
                            .attr("title", "Enter a valid date")
                            .tooltip("option", "disabled", false)
                            .addClass("error");
                        valid = false;
                    }
                }

                if(valid && mTo && mFrom) this.swapDates(from, to, mFrom, mTo);

                return valid;
            },

            swapDates: function(from, to, mFrom, mTo) {
                if(mTo.isBefore(mFrom)) {
                    /*
                     this.$toDate.val(from);
                     this.$fromDate.val(to);
                     */

                    this.$toDate.add(this.$fromDate)
                        .addClass("error")
                        .closest(".control-group")
                        .attr("title", "'From Date' must be before 'To Date'")
                        .tooltip("option", "disabled", false)
                        .addClass("error");
                }
            }
        }

        PHD.BackupFilter = BackupFilter;
}
