(function($, PHD, window, undefined) {
    var $form = $("#rollback-job-form"),
        RollbackForm, 
        $optionToggle = $("#rollback-options-toggle"),
        $backupToggle = $("#rollback-backups-toggle"),
        $jobOptions = $("#rollback-job-form-content"),
        $jobBackups = $("#rollback-job-backup-catalog"),

        $backupCatalog = $("#rollback-job-backup-table"),
        backupCatalog,
        backupCatalogOptions = {},

        $startDate = $("#id_start_date"),
        $startTime = $("#id_start_time"),
        $scheduleType = $("[name=schedule_type]"),
        scheduleType = $scheduleType.val(),
        $snapshot = $("#id_use_snapshot");

    function initForm() {
        var timesettings = {
                noSeparatorEntry: true,
                spinnerImage: null,
                ampmPrefix: " "
            };

        $.validator.addMethod("validate-backups", function(value, element) {
                return $("input[name=backups]:checked", $jobBackups).length > 0;
            },
            gettext("Please select at least one backup")
        );

        $.validator.addMethod("validate-future-date", function(value, element) {
                var date = moment(value + " " + $("#id_start_time").val() + " " + tz, "MM/DD/YYYY hh:mm A Z").utc(),
                    now = moment().utc(),
                    isvalid = now.isBefore(date);
                    
                return isvalid || (scheduleType === SCHEDULE_TYPE_SCHEDULED);
            },
           gettext( "Please select a future date and time")
        );

        $.validator.addMethod("validate-future-time", function(value, element) {
                var date = moment($("#id_start_date").val() + " " + value + " " + tz, "MM/DD/YYYY hh:mm A Z").utc(),
                    now = moment().utc(),
                    isvalid = now.isBefore(date);

                return isvalid || (scheduleType === SCHEDULE_TYPE_SCHEDULED);
            },
            gettext("Please select a future date and time")
        );

        $.validator.addMethod("time12h", function(value, element) {
            return this.optional(element) || /^((0?[1-9]|1[012])(:[0-5]\d){1,2}(\ ?[AP]M))$/i.test(value);
        }, gettext("Please enter a valid time"));


        RollbackForm = PHD.FormController($form, {
            serialize: function($form) {
                $startTime.timeEntry('option', 'show24Hours', true);
                return $form.serialize();
            },
            isEditing: jobEdit
        });

        RollbackForm
            .validate({
                rules: {
                    name: {
                        required: true
                    },
                    start_date: {
                        required: true,
                        date: true,
                        "validate-future-date": true
                    },
                    start_time: {
                        required: true,
                        time12h: true,
                        "validate-future-time": true
                    }
                },
                messages: {
                    name: {
                        required: gettext("Job Name is required")
                    },
                    start_date: {
                        required: gettext("Start Date is required")
                    },
                    start_time: {
                        required: gettext("Start Time is required")
                    }
                }
            })
            .on("formcancel", function() {
                PHD.currentDialog.wizard("close");
            })
            .on("formsubmit", function(event, data) {
                if(data.result[0].code === AJAX_RESULT_SUCCESS) {
                    var options = {};
                    if(PHD.App.url !== '/jobs/') {
                        var tab = (parseInt(scheduleType, 10) === SCHEDULE_TYPE_NOW) ?
                            "#0" : "#1";
                        options.buttons = [
                            {
                                text: gettext("View Jobs"),
                                click: function() {
                                    window.location.href = "/jobs/" + tab;
                                }
                            }
                        ];
                    }
                    PHD.throwNotice(data, options);
                    PHD.currentDialog.wizard("close");
                } else if(data.result[0].code === AJAX_RESULT_ERROR) {
                    $startTime.timeEntry('option', 'show24Hours', false);
                    PHD.throwError(data);
                }
            });

        $startDate.datepicker({
            showOn: "both",
            buttonText: '<img src="' + STATIC_URL + 'images/calendar.png" width="24" height="24">',
            dateFormat: DATE_FORMAT,
            minDate: 0,
            yearRange: "c-1:c+2",
            open: function(event, ui) {
                return !$startDate.prop("disabled");
            }
        }).attr("placeholder", DATE_DISPLAY_FORMAT);

        $startTime.timeEntry(timesettings);

        $scheduleType
            .on("change", handleScheduleType)
        .filter(":checked")
            .trigger("change");

        $snapshot.on("change", function() {
            if(!$(this).prop("checked")) {
                var confirm = PHD.confirmDialog(
                    "<p>"+gettext("Snapshots allow you to revert to your VM’s current state in the event the Rollback process does not complete due to error or interruption.") +
                    "<strong> "+gettext("Without a snapshot, data may be lost.")+"</strong> " +
                    "<br>"+gettext("Are you sure you want to disable Use Snapshot?")+"</p>", {
                        title: gettext("Confirm Snapshot Bypass")
                    });

                confirm.done(function() {
                    $snapshot.prop("checked", false);
                });

                confirm.fail(function() {
                    $snapshot.prop("checked", true);
                });
            }
            return true;
        });

        if(jobEdit) {
            // Disable "Now" radio button
            $("#id_schedule_type_0").prop("disabled", true);
        }
    }

    function handleScheduleType() {
        scheduleType = $(this).val();
        // lmc TODO - do not reference schedule/joborder type we don't have.
        //var runLater = (parseInt(scheduleType, 10) === SCHEDULE_TYPE_RUN_ONCE);
        var runLater = false;
        $startDate.prop("disabled", !runLater).closest(".control-group").toggle(runLater);
        $startTime.prop("disabled", !runLater).closest(".control-group").toggle(runLater);
    }

    function getCatalog() {
        var task = PHD.BackupCatalogBuilder({
            "view": BACKUP_CATALOG_VIEW_ROLLBACK,
            "vms": {
                "show": true,
                "checkboxes": false
            },
            "disks": {
                "show": true,
                "checkboxes": false
            },
            "checked": selections
        });
        task.done(buildCatalog);
    }

    function buildCatalog(data) {
        var options = {
            0: { multiselect: true, childselect: "first" },
            1: { multiselect: false, childselect: "all" },
            2: { multiselect: true, childselect: false } 
        };

        backupCatalog = PHD.BackupCatalog($backupCatalog, options);
        backupCatalog.load(data);

        backupCatalog.$table.on("treeselect", ".row-select", function() {
            $form.valid();
        });
        $backupCatalog.scrollTable({"height":".modal-form-col.full-height", "resize": false});
    }

    /*
    *   DOM Ready Function
    */
    $(function() {

        PHD.DOMReady();

        // Initialize form validation and event handlers
        initForm();

        getCatalog();

    });

})(jQuery, window.PHD || (window.PHD = {}), window);
