
var reportApp = angular.module('reports', ['ui.grid', 'ui.bootstrap','download', 'angulartics', 'angulartics.google.analytics','ui.grid.cellNav','ngTouch', 'ui.grid', 'ui.grid.pagination', 'ui.grid.edit', 'ui.grid.resizeColumns', 'ui.grid.pinning', 'ui.grid.selection', 'ui.grid.moveColumns', 'ui.grid.exporter', 'ui.grid.importer', 'ui.grid.grouping']);
var configParams;
var isSetInfo = false;
var enviornmentId = 0;

/* Creating factory for reportApp
**Input Params:
	*url
	*start_date
	*end_date
**Output 
	*JSON object from requested URL
*/
reportApp.factory('reportFactory', function($http, DownloadService) {
  return {
  async: function(url, start_date, end_date, enviornmentID, report_name, extra_params, analytics) {
	  var report_name = report_name || null;
	  var extra_params = extra_params || null;
	  var $analytics = analytics || false;
	  $analytics && $analytics.eventTrack(report_name, {  category: 'Reports', label: 'Reports' });
	  var params = {};
	  // only add sid if environmentID is not all.
	  if (enviornmentID != 0) {
		  params.sid = enviornmentID;
	  }
	  params.start_date = start_date;
	  params.end_date = end_date;
	  params.ts = Date.now();
	  if (extra_params != null) {
		  params = $.extend(params, extra_params);
	  }
    var promise = $http({
       url: url, 
       method: "get",
       params: params
    }).success(function(data, status, headers) {
		console.log("success");
		// This should be encapsulated in a reusable function so that other areas (outside reports) can export to CSV or PDF.
		headers = headers();
		var contentType = headers['content-type'];
		if (contentType.indexOf('text/csv') > -1) {
			var filename = report_name == 'null' ? 'report' + Date.now + ".csv" : report_name.replace(" ", "-") + ".csv";
			var blob = new Blob([data], {type: contentType});
			DownloadService.downloadFile(blob, filename);
		}else if (data.pdf_url !== undefined) {
			var filename = report_name == 'null' ? 'report' + Date.now + ".pdf" : report_name.replace(" ", "-") + ".pdf";
			var pdf_location = String(data.pdf_url).indexOf('/');
			var pdf = String(data.pdf_url).slice(pdf_location);
			DownloadService.saveAs(pdf, filename);
			PHD.hideLoadingIndicator(load);
		}
	}).error(function(data, status, headers){
		if(data && data.result[0].message != null && data.result[0].message != "") {
			window.PHD.throwError(data);
		} else {
			window.PHD.throwError({"result": [{"message": gettext("Requested data can't be retrieved for this report.")}]});
		}
		PHD.hideLoadingIndicator(load);
	});  //1. this returns promise
	 return promise;
	}
  };
});

/* Creating Controller for reportApp
*All the business logic goes here
*/

var ARCHIVE_SET_ID = "Archives in Set";
/* reportApp.controller('ReportsCtrl', ['reportFactory','$scope', '$routeParams', '$http', 'Page', '$analytics',
		function(reportFactory, $scope, $routeParams, $http, Page, $analytics) { */
		
reportApp.controller('ReportsCtrl', ['reportFactory','$scope', '$routeParams', '$http', 'Page','$analytics','$timeout', '$interval', 'uiGridConstants', function(reportFactory, $scope, $routeParams, $http, Page, $analytics, $timeout, $interval, uiGridConstants) {

		 $scope.dataGridOption={};
         var paginationOptions = {
            pageNumber: 1,
            pageSize: 25,
            sort: null
          };
		 $scope.heading= [];
		 Page.setTitle('Reports');
		 $scope.tempDataArray = [];
		 $scope.allDataArray = [];
		 $scope.parsedBackendData = [];
		
		 $scope.totalServerItemsArray=[];
		 
		 $scope.gridData = [];
		
		
		 var cacheParseDataArray = [];
		 var index = 0;
		 var gridReportData = "";
		 var GridItems = [];
         var reportId = 0;
		 var reportDays = 0;
         var report_sid = 0;

		$scope.dataGrids = [];
		$scope.LEGAL_HOLD = 1;
		$scope.UNPROTECTED = 2;
		$scope.PROTECTION_SUMMARY = 3;
		$scope.BACKUP_HISTORY = 4;
		$scope.STORAGE_DETAILS = 5;
		$scope.UPDATES = 6;
		$scope.REPLICATION = 7;
		$scope.ARCHIVE = 8;
		$scope.REPLICATION_HISTORY = 9;
		$scope.ARCHIVE_HISTORY = 10;
		$scope.VM_STORAGE = 11;
		$scope.SYSTEM_LOAD = 12;
		$scope.REPLICATION_REPORT = 13;
		$scope.RESTORE_HISTORY=14;
		$scope.STORAGE_DATAREDUCTION=15;
		$scope.SYSTEM_CAPACITY = 16;
		$scope.REPLICATION_FOOTPRINT = 17;
		$scope.BACKUP_FAILURE = 18;
		
		
		/*Main Report Id's.*/
		
		$scope.BACKUP_REPORT = 1;
		$scope.RESTORE_REPORT = 2;	
		$scope.REPLICATION_MAIN_REPORT = 3;
		$scope.ARCHIVE_REPORT = 4;
		$scope.SYSTEM_REPORT = 5;
		$scope.STORAGE_REPORT = 6;
		
		/*
		 * Array to show the Date field, appliance combo and header depends upon report requirement
		 */
		$scope.REPORTS_WITH_NO_DATE_SELECTION = [ // Report with no Date Selection
			$scope.STORAGE_DETAILS,
			$scope.REPLICATION_REPORT,
			$scope.SYSTEM_CAPACITY,
			$scope.REPLICATION,
			$scope.REPLICATION_FOOTPRINT
		];
		
		$scope.REPORTS_WITH_NO_HEADER_INFO = [ // Report with no header info
			$scope.STORAGE_DETAILS,
			$scope.REPLICATION_REPORT
		];
		$scope.REPORTS_WITH_NO_APPLIANCE_SELECTION = [ // Report with appliance combo
			$scope.STORAGE_DETAILS,
			$scope.REPLICATION_REPORT
		];
		
		$scope.names = [
			"",
			"Legal Hold Backups",
			"Unprotected VMs",
			"Protection Summary",
			"Backup History",
			"Storage",
			"Update History",
			"Replication Capacity",
			"Archive",
			"Replication History",
			"Archive History",
			"VM Storage Report",
			"Load Report",
			"Replication Report",
			"Recovery History" ,
			"Data Reduction",
			"Capacity Report",
			"Storage Footprint Report",
			"Backup Failures"
		]
		
		$scope.descriptions = [
			"",
			"All backups currently on legal hold.",
			"Assets that were not backed up within the selected date range.",
			"A graphical and tabular view of protected and unprotected assets within the selected date range.",
			"All backup results by asset within the selected date range.",
			"Details for all configured storage.",
			"Updates for the specified appliance, that occurred during the selected date range.",
			"The amount of data protected by the backups on a replication target.",
			"The most recent archive for each protected asset within the selected time frame.",
			"Backups that have been replicated to the selected target.",
			"All archives by asset within the selected date range.",
			"Cumulative per-protected asset backup size statistics.",
			"A view of the load on the appliance over the selected date range.",
			"Successful, active, and queued replication jobs in the last 24 hours.",
			"Recovery operations that occurred within the selected date range.  File-level recovery and Instant recovery operations are not included." ,
			"A report of storage space saved by the appliance using compression.",
			"The protected capacity, the amount of data used by Full backups, listed by protected asset.",
			"The amount of space replicating sources are using on a target.",
			"Failed backups over the selected date range."
			
		];

		/*
		 * If a report ID was passed in, set it here.
		 */
		$scope.alreadySelected = false;
		if ($routeParams.id != undefined) {
			reportId = $routeParams.id;
			console.log("report id is + " + reportId);
			$scope.alreadySelected = true;
			reportDays = $routeParams.days;
            report_sid = $routeParams.sid !== undefined ? $routeParams.sid : 0;
		}
					
		angular.element(document).ready(function() {
			processReportsPage();
		});

		$scope.onBackupFailureRowClick = function(row){
                
                console.log($scope.dataGrids[0].selectedItems);
                console.log($scope.dataGrids[0]);
				console.log(report_sid);

				//GET Appliance ID for selected report
				$http({
			       url: "/api/systems", 
			       method: "get",
			       params: {}
    			}).success(function(data, status, headers) {
    				if(data!==undefined && data!==null && data!=""){
    					if(data.appliance!==undefined && data.appliance!==null && data.appliance!=""){
							var applianceId = 0;
    						for(var i=0; i<data.appliance.length; i++){
    							if(data.appliance[i].name === $scope.dataGrids[0].selectedItems[0].system_name){
    								applianceId = data.appliance[i].id;
    								break;
    							}	
    						}
							PHD.appliance_sid = applianceId;
							backup_ids = $scope.dataGrids[0].selectedItems[0].id;
							PHD.wizard("app/inventory/details/status.html", "view-status", {
								title: gettext(" Backup Failure Status: "),
								dialogClass: "standalone",
								height: 500,
								helpArticle: PHD.App.getHelpLink("backup", true)
							});
    					}
    				}
    			});
			
            }
		$scope.onRecoveryHistoryReportRowClick  = function(row){
               $http({
			       url: "/api/systems", 
			       method: "get",
			       params: {}
    			}).success(function(data, status, headers) {
    				if(data!==undefined && data!==null && data!=""){
    					if(data.appliance !==undefined && data.appliance !==null && data.appliance !=""){
							var applianceId = 0;
    						for(var i=0; i<data.appliance.length; i++){
    							if(data.appliance[i].name === $scope.dataGrids[0].selectedItems[0].system_name){
    								applianceId = data.appliance[i].id;
    								break;
    							}	
    						}
							PHD.appliance_sid = applianceId;
							backup_ids = $scope.dataGrids[0].selectedItems[0].id;
							var wzd = PHD.wizard("app/inventory/details/status.html", "view-status", {
								title: gettext(" Recovery History Status: "),
								dialogClass: "standalone",
								height: 500,
								helpArticle: PHD.App.getHelpLink("backup", true)
							});
    					}
    				}
    			});
        }

		/*
		 * Wrap the existing jQuery in a function. Call after Angular loads the page.
		 */
		function processReportsPage() {
		
			var $reportCategories = $(".report-categories"), 
			    $reportForm = $("#reports-form"),
				$reportCategories = $(".report-categories"),
				$reportCategoriesSubElems = $(".report-categories > div"),
				$reportCategory = $(".report-category"),
				$curCategory = $(".reports-cat-backup"),
				$reportCategorySubElems = $(".report-category h2"),
				$reportContainer = $(".report-container"),
				$reportBack = $("#report-back"),
				$availableReports = $("#available-reports"),
				$availableReportsHeader = $availableReports.children("h1"),
				$availableReportsSubElems = $availableReports.find("div"),
				$currentReport = $("#current-report"),
				$reportListbox = $("#id_name");
				$holderID = $("#flot_holder");
				$divTitleId = $(".divTitle");
				
			
				/* Catching current selection of Appliance drop down */
				   $scope.enviornment = { // for reports which having  "All" as well as other appliances .
						currentActivity: {
							id: 0,
							name: "All" 
						}
					};
				    $scope.enviornmentModelWithoutAll = {// for reports which having no "All" appliances.
				        currentActivityData: {
						
						}
				   }; 
            
				//Catching list of all appliance from Backend.
				$scope.envornments =[{"id" : 0,"name": "All"}];//Adding All as default object in json, From backend data not coming as all.
				$scope.enviornmentDataWithoutAll =[{"id" : 0,"name": "All"}];
                reportFactory.async('/api/systems').then(function(resultData) {
				// $('#applianceId-'+reportId+'').prepend($('#applianceloadMask').html(''));
                    var holdList = {};
                    for(var i = 0; i< resultData.data.appliance.length; i++){
                        holdList = {
                                    "id":resultData.data.appliance[i].id,
                                    "name":resultData.data.appliance[i].name
                                   }
                        $scope.envornments.push(holdList);//Adding all list from json
						$scope.enviornmentDataWithoutAll.push(holdList);//Adding all list from json
                    }
					$scope.enviornment.currentActivity = $scope.envornments[0];
					console.log($scope.enviornment.currentActivity);
					
				 	 $scope.enviornmentModelWithoutAll.currentActivityData = $scope.enviornmentDataWithoutAll[1]; 
					 $scope.enviornmentDataWithoutAll.splice(0, 1);
					 console.log($scope.enviornmentModelWithoutAll.currentActivityData);
					
				});
				
				/*************** Main report Category  array of object***************************/
				$scope.categoryArray = [{id:1,category:"BACKUP",catImgClass:"icon-uui-backup icon-large"},
										{id:2,category:"RECOVER",catImgClass:"icon-uui-recover icon-large"},
										/* {id:3,category:"REPLICATION",catImgClass:"icon-backup icon-large"}, sab: TODO:  uncomment after CE
										{id:4,category:"ARCHIVE",catImgClass:"icon-phd-backup icon-large"}, */
										{id:5,category:"APPLIANCE",catImgClass:"icon-uui-unitrends-appliance icon-large"},
										{id:6,category:"STORAGE",catImgClass:"icon-uui-storage icon-large"}];
		 

				
				/**** Intalization of SubReport data for Backup Report at very first time ***/
				$scope.subCategoryArray =[{id:$scope.PROTECTION_SUMMARY,description:"A summary of protection details and lists of all protected and unprotected assets within the selected date range.",label: 'PROTECTION SUMMARY', subCatImgClass:"icon-clipboard icon-large",dropDownWithAll:false,open: ($scope.alreadySelected && (reportId== $scope.PROTECTION_SUMMARY)) ? true:false},
								 {id:$scope.BACKUP_HISTORY,description:"All backup results by protected asset within the selected date range.", label: 'BACKUP HISTORY', subCatImgClass:"icon-uui-backup icon-large",dropDownWithAll:false,open: ($scope.alreadySelected && (reportId== $scope.BACKUP_HISTORY)) ? true:false},
								/*  {id:$scope.LEGAL_HOLD,description:"A report of all backups currently on legal hold.", label: 'LEGAL HOLD BACKUPS', subCatImgClass:"icon-backup icon-large", dropDownWithAll:false,open: ($scope.alreadySelected && (reportId== $scope.LEGAL_HOLD)) ? true:false}, sab: TODO:  uncomment after CE
								  */{id:$scope.BACKUP_FAILURE,description:"A report of all failed backups.", label: 'BACKUP FAILURES', subCatImgClass:"icon-phd-error icon-large", dropDownWithAll:false,open: ($scope.alreadySelected && (reportId== $scope.BACKUP_FAILURE)) ? true:false}
								];
				
				/**** Intalization of array on click of subreport ************************************* ***/
			
					$scope.getSubReportCategory = function(id){
						    //Adding and removing CSS on click of report category.
							$("a.reportCat").removeClass("reportSelected");
							$("#test-"+id+"").addClass("reportSelected");
							
						switch(id) {
						case $scope.BACKUP_REPORT:
								$scope.subCategoryArray =[ {id:$scope.PROTECTION_SUMMARY,description:"A summary of protection details and lists of all protected and unprotected assets within the selected date range.",label: 'PROTECTION SUMMARY', subCatImgClass:"icon-clipboard icon-large",dropDownWithAll:false,open: false},
												  {id:$scope.BACKUP_HISTORY,description:"All backup results by protected asset within the selected date range.", label: 'BACKUP HISTORY', subCatImgClass:"icon-uui-backup icon-large",dropDownWithAll:false,open:false},
												 /*  {id:$scope.LEGAL_HOLD,description:"A report of all backups currently on legal hold.", label: 'LEGAL HOLD BACKUPS', subCatImgClass:"icon-backup icon-large",dropDownWithAll:false, open:false},   sab: TODO:  uncomment after CE
												   */{id:$scope.BACKUP_FAILURE,description:"A report of all failed backups.", label: 'BACKUP FAILURES', subCatImgClass:"icon-phd-error icon-large",dropDownWithAll:false, open:false}
												];
						break;
						case $scope.RESTORE_REPORT:
								$scope.subCategoryArray =[ {id:$scope.RESTORE_HISTORY,description:"Recovery operations that occurred within the selected date range. File-level recovery and instant recovery operations are not included.",label: 'RECOVERY HISTORY', subCatImgClass:"icon-circlearrowup icon-large",open:false}
												];
						break;
						case $scope.REPLICATION_MAIN_REPORT:
								$scope.subCategoryArray =[ {id:$scope.REPLICATION,description:"The amount of data protected by the backups on a replication target.", label: 'REPLICATION CAPACITY',subCatImgClass:"icon-clipboardalt icon-large", dropDownWithAll:false,open:false},
												  {id:$scope.REPLICATION_HISTORY,description:"A report of backups replicated to the selected target.", label: 'REPLICATION HISTORY',subCatImgClass:"icon-phd-replicate icon-large", dropDownWithAll:false,open:false},
												  {id:$scope.REPLICATION_REPORT,description:"Display a report of successful, active, and queued replication jobs in the last 24 hours.", label: 'REPLICATION REPORT',subCatImgClass:"icon-clipboard icon-large", dropDownWithAll:false,open:false},
												  {id:$scope.PROTECTION_SUMMARY,description:"A summary of protection details and lists of all protected and unprotected assets within the selected date range.", label: 'PROTECTION SUMMARY',subCatImgClass:"icon-clipboard icon-large", dropDownWithAll:false,open:false},
												  {id:$scope.REPLICATION_FOOTPRINT,description:"A report of the amount of space replicating sources are using on a target.", label: 'STORAGE FOOTPRINT REPORT',subCatImgClass:"icon-phd-replicate icon-large",dropDownWithAll:false, open:false}
												];						
						break;
						case $scope.ARCHIVE_REPORT:
								$scope.subCategoryArray =[ {id:$scope.PROTECTION_SUMMARY,description:"A summary of protection details and lists of all protected and unprotected assets within the selected date range.",  label: 'PROTECTION SUMMARY',subCatImgClass:"icon-clipboard icon-large", dropDownWithAll:false,open:false},
												  {id:$scope.ARCHIVE,description:"The most recent archive for each protected asset within the selected time frame.",  label: 'ARCHIVE',subCatImgClass:"icon-clipboardalt icon-large",dropDownWithAll:false, open:false}
												];
						break;
						case $scope.SYSTEM_REPORT:
								$scope.subCategoryArray =[ {id:$scope.UPDATES, description:"Updates for specified appliances, based on the selected date range.",label: 'UPDATE HISTORY',subCatImgClass:"icon-circlearrowup icon-large",dropDownWithAll:false, open:false},
												  {id:$scope.SYSTEM_CAPACITY, description:"The maximum capacity the system can use to store new backups, prior to purging.",label: 'CAPACITY REPORT',subCatImgClass:"icon-uui-storage icon-large",dropDownWithAll:false, open:false},
												  {id:$scope.SYSTEM_LOAD, description:"Generate a report showing appliance load over time.",label: 'LOAD REPORT',subCatImgClass:"icon-barchart icon-large",dropDownWithAll:true, open:false}
												];
						break;
						case $scope.STORAGE_REPORT:
								$scope.subCategoryArray =[
												  {id:$scope.STORAGE_DETAILS, description:"Details for all configured storage.", label: 'STORAGE',subCatImgClass:"icon-uui-storage icon-large",dropDownWithAll:false, open:false},
												  {id:$scope.STORAGE_DATAREDUCTION, description:"A report of space saved by deduplication and compression.", label: 'DATA REDUCTION',subCatImgClass:"icon-uui-storage icon-large", dropDownWithAll:true,open:false}
												];
						break;
				    }		
					
			       }
				/*** on click  of individual sub report of main report on right hand side****/
				$scope.selectSubReport = function(reportId,reportExpand,subCategoryArray,index){
				    var mainReportCategoryId = document.getElementById("catId").value
				    if(mainReportCategoryId == 1){
					   $('#appliance-combo2-'+reportId+'').val(0);
					}
					if(subCategoryArray[index].open != true){
						subCategoryArray[index].open = false;		
                    }else{					
					  
						for(var i=0; i< subCategoryArray.length; i++){
							
							if(subCategoryArray[i].open){
								subCategoryArray[i].open =false;
								console.log(subCategoryArray[i].id);
								$("a.reportSubCat-"+subCategoryArray[i].id+"").removeClass("reportSelected");
								$("a.reportSubCat-"+subCategoryArray[i].id+"").addClass("reportUnSelected");
						       
								$("#expandId-"+subCategoryArray[i].id+"").removeClass("reportExpanded");
								$("#expandId-"+subCategoryArray[i].id+"").addClass("reportExpand");
								
							} 
							 subCategoryArray[index].open = true;
							
						}
					}					
					
					if(reportExpand){
					    $("a.reportSubCat-"+reportId+"").removeClass("reportUnSelected");
						$("a.reportSubCat-"+reportId+"").addClass("reportSelected");
						
						$("#expandId-"+reportId+"").removeClass("reportExpand");
					    $("#expandId-"+reportId+"").addClass("reportExpanded");
					}else{
					    $("a.reportSubCat-"+reportId+"").removeClass("reportSelected");
						$("a.reportSubCat-"+reportId+"").addClass("reportUnSelected");
						
						$("#expandId-"+reportId+"").removeClass("reportExpanded");
						$("#expandId-"+reportId+"").addClass("reportExpand");
					  
					}
					//Initialization o f objects.
				    $reportListbox = $("#id_name-"+reportId+"");
					$reportForm = $("#reports-form-"+reportId+"");
					$holderID = $("#flot_holder");
					$reportListbox.val(reportId);
					
					// Initialization of date Picker.
					console.log($holderID);
					$(".datepicker input:text").datepicker({
						showOn: "both",
						buttonText: '<img src="assets/images/calendar.png" width="24" height="24">',
						dateFormat: DATE_FORMAT
					}).attr("placeholder", DATE_DISPLAY_FORMAT);
					
					selectReport(reportId,$reportListbox); // call for select report 
					
				    $reportForm.validate({  // Date field validation 
						rules: {
							start_date: {
								dateFmtNotRequired: true,
								"validate-start-date": true
							},
							end_date: {
								dateFmtNotRequired: true
							}
						}
					});
					
			    }
				
			 var loadApplianceData;
			 var reportIndex = 0;
			 function selectReport(reportId,$reportListbox) {
			  		reportIndex = reportId;
				
				$("#report-description-"+reportId+"").text($scope.descriptions[reportIndex]);
				$("#report-filter-options-"+reportId+"").css("display", ($.inArray(reportIndex, $scope.REPORTS_WITH_NO_DATE_SELECTION) == -1) ? "" : "none");
				$("#report-appliance-options-"+reportId+"").css("display", ($.inArray(reportIndex, $scope.REPORTS_WITH_NO_APPLIANCE_SELECTION) == -1) ? "" : "none");
				$(".report-info").css("display", ($.inArray(reportIndex, $scope.REPORTS_WITH_NO_HEADER_INFO) == -1) ? "" : "none");
		 	}
			
			$reportBack.click(
				function () {
				    //added
					$(".reportCategory").removeClass("hidden");
					$(".report-section-right").removeClass("hidden");
					
					$(this).addClass("hidden");
					$("#report-data").addClass("hidden");
					$("#flot_holder").empty();
					 $holderID.addClass("hidden");
								
					$reportCategories.removeClass("hidden");
					$reportContainer.removeClass("hidden");
					$availableReports.removeClass("hidden");
					
					//From dashboard when we select any backup,failure,protected or unprotected report.
					if (!$scope.alreadySelected) $reportForm.removeClass("hidden");
						if($scope.alreadySelected){
						   $scope.selectSubReport();
						   $scope.getSubReportCategory(reportId);
						    if (reportDays !== "0") {
							var start_date = moment().subtract('days', reportDays).format('MM/DD/YYYY');
							    $("#id_start_date_"+reportId+"").val(start_date);
							} else {
								$("#id_start_date_"+reportId+"").val(null);
							}
						}
					
				}
			);
		
		
			$availableReportsHeader.click(
				function () {
					$availableReportsHeader.removeClass("report-hover");
					$availableReportsSubElems.addClass("hidden");
					$curCategory.removeClass("hidden");
					$reportContainer.addClass("hidden");
				}
			);
			
		
			/*Function for initialize all array and variables to default which were
			 *used for creation of dynamic ng-grids
		     */	
				$scope.intializeGrid = function(){
					    angular.forEach($scope.dataGrids,function(value,index){
						 //$scope.dataGrids[index].grid ={};
						 $scope.parsedBackendData[index]=[]; 
						 $scope.tempDataArray[index]=[];//TODO
						 $scope.allDataArray[index] = [];
									 
						});
						 $scope.dataGrids=[];
						 $scope.parsedBackendData=[];
						 $scope.tempDataArray=[];
						 $scope.allDataArray = [];
						 $scope.pagingOptions = [];
						 $scope.totalServerItemsArray=[];
						 pagingOptionsObj={};
						 cacheParseDataArray = []; 
						 index = 0;
						 gridReportData = "";
				};
				
			  /*Function for calculating date range text which are present in header of each report
			   *start_date_val: Selected Start date
			   *end_date_val: Selected end date
		       */		
				$scope.getDateRange = function( start_date_val,end_date_val){
					  if(start_date_val == null && end_date_val== null){
					   $scope.report_date_range = "Showing All";
					  }else if(start_date_val != null && end_date_val== null){
					   $scope.report_date_range = "equal to or after "+start_date_val;
					  }else if(start_date_val == null && end_date_val != null){
					   $scope.report_date_range = "equal to or before "+end_date_val;
					  }else{
					    $scope.report_date_range = start_date_val+" - "+end_date_val;
					  }
					  return $scope.report_date_range;
				};
					
					
			     /*Function for creating report*/		
				$scope.generateReport= function (output_format,subReportID){
					   
				       var reportFormValidFlag = true;
					   var enviornmentName="All";
					   
					   $(".reportCategory").addClass("hidden");
					   
					    if($scope.alreadySelected){
							
							reportFormValidFlag =true;
						}else{
							reportFormValidFlag = $reportForm.valid();
						}
				   if(reportFormValidFlag) {
						if (typeof output_format === 'undefined' || output_format === null || output_format === '') {
							$scope.systemCapacityReport = false;
							$scope.isHide = false;
							
							$scope.intializeGrid(); // For initalizing all grid scope variables.
							
							if(output_format != '' ){reportId = $reportListbox.val();
							 }else{reportId = subReportID;}
						
						    if($scope.STORAGE_DATAREDUCTION == reportId  || reportId == $scope.SYSTEM_LOAD ){// for reports which having no "All" appliances.
								enviornmentId = $scope.enviornmentModelWithoutAll.currentActivityData[reportId].id;
								enviornmentName = $scope.enviornmentModelWithoutAll.currentActivityData[reportId].name;
							}else{ // for reports which having "All" as well as other appliances.
							    if($scope.enviornment.currentActivity[reportId] != undefined){
									enviornmentId = $scope.enviornment.currentActivity[reportId].id;
									enviornmentName = $scope.enviornment.currentActivity[reportId].name;
								}
							}
							 
							if(reportId == $scope.SYSTEM_CAPACITY && enviornmentName != "All"){
								$scope.systemCapacityReport = true;
							}
							
							if($scope.alreadySelected){
								if (reportDays !== "0") {
									var start_date = moment().subtract('days', reportDays).format('MM/DD/YYYY');
									start_date_val = start_date;
									end_date_val = null;
								} else {
								    start_date_val = null;
									end_date_val = null;
								}
						  
							}else{
							 start_date_val=($("#id_start_date_"+reportId+"").val() == "") ? null : $("#id_start_date_"+reportId+"").val() ;
							 end_date_val=($("#id_end_date_"+reportId+"").val() == "") ? null : $("#id_end_date_"+reportId+"").val() ;
							}
	
							 if(reportId == $scope.STORAGE_DETAILS ||
								reportId == $scope.REPLICATION_REPORT ||
								reportId == $scope.SYSTEM_CAPACITY ||
								reportId == $scope.REPLICATION ||
								reportId == $scope.REPLICATION_FOOTPRINT ){
							   start_date_val = null;
							   end_date_val = null;
							   $scope.isHide = true;
							 }
							 /* For Report header */
							 $scope.reportdesc  = $scope.descriptions[reportId];
							 $scope.reportTitle = $scope.names[reportId];
							 var dateRange = $scope.getDateRange( start_date_val,end_date_val);							 
							 $scope.report_date_range = dateRange;
							 $scope.current_time = getCurrentTimeAndDate();
							 $scope.applianceName = enviornmentName;
						}	 /*end*/
							
							 $reportContainer.removeClass("hidden");
							 $("#report-data").removeClass("hidden");
							 load = PHD.showLoadingIndicator($reportContainer);

							configParams = getReportConfigs($scope, reportId,$scope.isCE);
							 $scope.heading = configParams[2];
							if (typeof output_format !== 'undefined') {
								extra_params = { format: output_format };
							} else {
								extra_params = null;
							}
							
							 reportFactory.async(configParams[0],start_date_val,end_date_val,enviornmentId,$scope.names[reportId],extra_params,$analytics).then(function(result){
								 PHD.hideLoadingIndicator(load);
                             if (typeof output_format === 'undefined' || output_format === null || output_format === '') {
								 if (reportId != $scope.ARCHIVE && reportId != $scope.REPLICATION_FOOTPRINT)
								 {
                                    var gbSize=0;
                                    for(var i=0; i<result.data.data.length; i++){
                                        if(result.data.data[i].size !==undefined){
                                            gbSize = (result.data.data[i].size / 1024).toFixed(2);
                                            result.data.data[i].size = gbSize;    
                                        }
                                    }
									 $scope.resultData = result.data.data;
								 }
								 else{
									 $scope.resultData = result.data;
								 }
								
								  if (configParams[1] == "PieChart" ){ //TODO
									   console.log("pieChart");
									   $holderID.removeClass("hidden");
									   PieChartRenderProtSummary(result, $scope.isCE);
									   $scope.colDefArray = configParams[3]; //getColumnArray
									   $scope.populateGridData($scope.resultData,$scope.colDefArray); 
								   }
								   else if (configParams[1] == "NoGrid")
								   {
									   urlToOpen=result.data.html;
									   var wnd = window.open("about:blank", "", "_blank");
									 /*Checking whether pop-up is blocked or not in User browser*/
									  if(!wnd) {
									   alert('Seems like a pop-up blocker is enabled. Please disable');
									  } 
									 else
									  { wnd.document.write(urlToOpen);}
									  $reportBack.trigger( "click" );
								 
								   }
								   else if (configParams[1] == "Pie-BarChart" ){ 
										$holderID.removeClass("hidden");
										PieChartForBackupStorage(result);
										createBarChartData(result);
										$scope.colDefArray = configParams[3]; //getColumnArray
										console.log($scope.colDefArray);
										$scope.populateGridData($scope.resultData,$scope.colDefArray); 
								   }
								   else if (configParams[1] == "Bar-Line" ){ //TODO
									if (reportId == $scope.SYSTEM_LOAD)
									   {
										   $holderID.removeClass("hidden");
										   plotSytemLoadGraph(result.data);
									   }
									   else
									   {
										$holderID.removeClass("hidden");
										lineOverBarDataReduction(result.data, $scope.isCE);
									   }
									
								   }else{
								   $scope.colDefArray = configParams[3]; //getColumnArray
								   $scope.populateGridData($scope.resultData,$scope.colDefArray); 
								   }
							}
							}); 
								$reportCategories.addClass("hidden");
								$availableReports.addClass("hidden");
								$curCategory.addClass("hidden");
								if(!$scope.alreadySelected) $reportForm.addClass("hidden");
								$reportBack.removeClass("hidden");
					}
				}
					/*function to concatenate  JSON object
                     
                     */
                
                        $.concat||$.extend({concat:function(b,c){var a=[];for(x in arguments)a=a.concat(arguments[x]);return a}});
				
				
				  /*Function for manipulating JsonData data
				   *Note: 
				  */

			  	$scope.populateGridData= function (jsonData,columnDefArray){
					$scope.jsonData=jsonData;
					 	 if(reportId == $scope.PROTECTION_SUMMARY){
							for (key in jsonData)
							{
								for (childkey in jsonData[key]){
									 if (jsonData[key][childkey]  instanceof Array){

										 if(key == 'clients')
										 {
											 $scope.tempDataArray.push($.concat(jsonData[key][childkey],jsonData["vms"][childkey]));
											 $scope.allDataArray.push($.concat(jsonData[key][childkey],jsonData["vms"][childkey]));
											 $scope.parsedBackendData = $scope.tempDataArray;
											 cacheParseDataArray.push($.concat(jsonData[key][childkey],jsonData["vms"][childkey]));
										 }
										 else if(key == 'backups' || key == 'archives')
										 {

											 if(childkey == 'clients')
											 {
												 $scope.tempDataArray.push($.concat(jsonData[key][childkey],jsonData[key]["vms"]));
												 $scope.allDataArray.push($.concat(jsonData[key][childkey],jsonData[key]["vms"]));
												 $scope.parsedBackendData = $scope.tempDataArray;
												 cacheParseDataArray.push($.concat(jsonData[key][childkey],jsonData[key]["vms"]));
											 }

										 }
										 // until replication supported, we have to do this, as the data array is position dependent.
										 else if(key != 'vms' && key !== 'replicated_backups')
										 {
											 $scope.tempDataArray.push(jsonData[key][childkey]);
											 $scope.allDataArray.push(jsonData[key][childkey]);
											 $scope.parsedBackendData = $scope.tempDataArray;
											 cacheParseDataArray.push(jsonData[key][childkey]);
										 }
									}

								 }

							}
					    $scope.createReportGrid(columnDefArray);
								
						}else if (reportId == $scope.ARCHIVE){
      					   $scope.heading=[];
                           var columnDefArrayList = [];
                           for (var i =0; i<jsonData.data.length; i++){
                                var archiveList=[];
                                 var archiveSetInfolist=[];
                                var archiveSetInfo ={
                                    "archive_set_id": null,
                                        "available": null,
                                        "date": null,
                                        "description": null,
                                        "status": null,
                                        "target": null,
                                        "is_imported": null
                                        
                                }
                                                                
                                archiveSetInfo.archive_set_id= jsonData.data[i].archive_set_id;
                                archiveSetInfo.available= jsonData.data[i].available;
                                archiveSetInfo.date= jsonData.data[i].date;
                                archiveSetInfo.description= jsonData.data[i].description;
                                archiveSetInfo.status= jsonData.data[i].status;
                                archiveSetInfo.target= jsonData.data[i].target;
                                archiveSetInfo.is_imported= jsonData.data[i].is_imported;
                                   
                                archiveSetInfolist.push(archiveSetInfo);
                                   
                                    $scope.tempDataArray.push(archiveSetInfolist);
                                   	$scope.allDataArray.push(archiveSetInfolist);
                                    $scope.parsedBackendData = $scope.tempDataArray;
                                    cacheParseDataArray.push(archiveSetInfolist);
                                
                                    $scope.tempDataArray.push(jsonData.data[i].archives);
                                    $scope.allDataArray.push(jsonData.data[i].archives);
                                    $scope.parsedBackendData = $scope.tempDataArray;
                                    cacheParseDataArray.push(jsonData.data[i].archives);
                                    columnDefArrayList.push(columnDefArray[0]);
                                    columnDefArrayList.push(columnDefArray[1]);
                                   
							var archiveSetHeading = 'Archive Set Created' +': '+ jsonData.data[i].date;
							var archivesHeading   =  'Archives in Set';// + jsonData.data[i].set.archive_set_id;
                                 
								$scope.heading.push(archiveSetHeading);
								$scope.heading.push(archivesHeading);
                               
                                    
                            }
							$scope.createReportGrid(columnDefArrayList);
						}
						else if (reportId == $scope.SYSTEM_CAPACITY){
                        var system_capacityData = [];
                        var capictyReportData=[];
                        var totalLastFull = 0.0;
                        var lastfull =0.0 ;
                        var lastFullArray =[];
                          $scope.usedSpace = jsonData[0].used.toFixed(2);
                         for ( i =0 ; i <jsonData.length ; i++)
                            {
                                         
                               for (key in jsonData[i])
                                {
                                    if (jsonData[i][key]  instanceof Array){
                                         for(var k=0; k<jsonData[i][key].length;k++){
                                              jsonData[i][key][k]["system_name"] = jsonData[i].system_name; 
                                         }
                                    }
                                }
                            }
                        for ( i =0 ; i <jsonData.length ; i++)
                            {
                               for (key in jsonData[i])
                                {
                                    if (jsonData[i][key]  instanceof Array){
                                        
                                        if(key == 'clients')
											 {
                                                 
                                                 
												  system_capacityData.push($.concat(jsonData[i][key],jsonData[i]["vms"]));
                                            
			  									   
											 }
                                        
                                    }
                                }
                            }

						 for(var i = 0; i < system_capacityData.length; i++)
						 {
							 capictyReportData = capictyReportData.concat(system_capacityData[i]);

						 }

						 for (var k  = 0 ; k < capictyReportData.length ; k++  )
						 {
							 lastfull = capictyReportData[k].last_full;
							 totalLastFull = parseFloat(totalLastFull) + parseFloat(lastfull);
						 }
                         parseFloat(totalLastFull) != 0 ? lastFullArray.push(parseFloat(totalLastFull)) : lastFullArray.push("0.0");
                         $scope.lastFullArray = lastFullArray;
                         cacheParseDataArray = [capictyReportData];
                         $scope.tempDataArray.push(capictyReportData);
                         $scope.parsedBackendData = $scope.tempDataArray;
                         $scope.allDataArray.push(capictyReportData);
                         $scope.createReportGrid(columnDefArray);
                        
                    }else if (reportId == $scope.REPLICATION_FOOTPRINT){
                            for (key in jsonData)
							{
								if (jsonData[key]  instanceof Array){
									for(var k=0; k<jsonData[key].length;k++){
										jsonData[key][k]["local_system_name"] = jsonData.local_system_name; 
										jsonData[key][k]["local_system_version"] = jsonData.local_system_version; 
										jsonData[key][k]["report_date"] = jsonData.report_date; 
									
									}
                                    $scope.tempDataArray.push(jsonData[key]);
                                    $scope.allDataArray.push(jsonData[key]);
									$scope.parsedBackendData = $scope.tempDataArray;
                                    cacheParseDataArray.push(jsonData[key]);
								}
							}
                            $scope.createReportGrid(columnDefArray);
						}
						else if(reportId == $scope.STORAGE_DETAILS){
                             		  $scope.tempDataArray.push(jsonData);
									   $scope.allDataArray.push(jsonData);
									  $scope.parsedBackendData = $scope.tempDataArray;
									  cacheParseDataArray.push(jsonData);	
									  $scope.createReportGrid(columnDefArray);
						}else if(reportId == $scope.RESTORE_HISTORY){
									if(jsonData != null && jsonData !=undefined){
										for(var i =0; i<jsonData.length;i++){

											if(jsonData[i].instance_name != null && 
												jsonData[i].instance_name != undefined &&
												jsonData[i].instance_name != ""){
											jsonData[i].client_name = jsonData[i].instance_name
											}
										}
									}
                             		$scope.allDataArray.push(jsonData);
									$scope.tempDataArray.push(jsonData);
									$scope.parsedBackendData = $scope.tempDataArray;
									cacheParseDataArray.push(jsonData);
									$scope.createReportGrid(columnDefArray);
						}
						else{ 
								if (jsonData  instanceof Array){
									  console.log("in else11");
									  $scope.allDataArray.push(jsonData);
									  $scope.tempDataArray.push(jsonData);
									  $scope.parsedBackendData = $scope.tempDataArray;
									  cacheParseDataArray.push(jsonData);
								}
								$scope.createReportGrid(columnDefArray);
						}
								
				}
				
			 $scope.dataGridOption.rowIdentity = function(row) {
                return row.id;
            };
            $scope.dataGridOption.getRowIdentity = function(row) {
                return row.id;
            };
					 /*Function for creating grid using manipulated data
				      *columnDefArray: column definition array
				     */
				

				$scope.createReportGrid = function (columnDefArray){
                  var index = 0;
                    angular.forEach($scope.parsedBackendData, function(elem) {
                        if((columnDefArray[index]) != undefined){
                        
							$scope.dataGridOption={  
								data:$scope.parsedBackendData[index], 
								paginationPageSizes: [25, 50, 100],
								paginationPageSize: 25,
								enableFiltering : true,
								enableGridMenu : true,
								showGridFooter : true,
								exporterPdfPageSize: 'LETTER',
								exporterPdfMaxGridWidth: 500,
								exporterPdfDefaultStyle: {fontSize: 8},
								exporterPdfTableStyle: {margin: [5, 5, 5, 5]},
								exporterPdfTableHeaderStyle: {fontSize: 8, bold: true, italics: true, color: 'black'},
								exporterPdfHeader: { text: $scope.heading[index] , style: 'headerStyle' },
								exporterPdfFooter: function ( currentPage, pageCount ) {
								  return { text: currentPage.toString() + ' of ' + pageCount.toString(), style: 'footerStyle' };
								},
								exporterPdfCustomFormatter: function ( docDefinition ) {
								  docDefinition.styles.headerStyle = { fontSize: 10, bold: true, color: 'grey' }; 
								  docDefinition.styles.footerStyle = { fontSize: 8, bold: true }; 
								  return docDefinition;
								},
							  //  exporterCsvLinkElement: angular.element(document.querySelectorAll(".custom-csv-link-location")),

								enableColumnResizing : true,
								columnDefs: columnDefArray[index],
								onRegisterApi: function(gridApi) {
									$scope.gridApi = gridApi;
									gridApi.core.on.columnVisibilityChanged( $scope, function (column) {
									  // do something
										/* console.log("***************");
										
										console.log(column);
										console.log(column.colDef.visible);
										console.log(columnDefArray);
										console.log(gridApi); */
									  //  console.log($scope.dataGridOption.id);
									   
									});
								   									
									$scope.gridApi.core.on.sortChanged($scope, function(grid, sortColumns) {
										if (sortColumns.length == 0) {
											paginationOptions.sort = null;
										} else {
											paginationOptions.sort = sortColumns[0].sort.direction;
										}
									});
								  gridApi.pagination.on.paginationChanged($scope, function (newPage, pageSize) {
										paginationOptions.pageNumber = newPage;
										paginationOptions.pageSize = pageSize;
								  });
								}       
						};     
						
						$scope.dataGrids.push($scope.dataGridOption);
						
                        if(reportId == $scope.STORAGE_DETAILS || reportId == $scope.ARCHIVE){
                                console.log("");
                              console.log($scope.dataGrids[index]);
                              $scope.dataGrids[index].columnDefs[0].visible = true;
                            }
                            else {
                              $scope.dataGrids[index].columnDefs[0].visible = $scope.showSystemName();
                           }

                                /*   $scope.$on('ngGridEventData', function (e,s) {
                                 noDataRecordEventHandler($scope.parsedBackendData,"reportGridID","No data found");
                             }); */
						  ++index;

                        }
					});
				}
                $scope.currentSelections= [];
				$scope.showSystemName = function()
				 {
				    var enviornmentName ="";
					if($scope.STORAGE_DATAREDUCTION == reportId  || reportId == $scope.SYSTEM_LOAD ){
				       enviornmentName = $scope.enviornmentModelWithoutAll.currentActivityData[reportId][reportId].name;
					 }else{
					    if($scope.enviornment.currentActivity[reportId] != undefined){
					     enviornmentName = $scope.enviornment.currentActivity[reportId].name;
					    }else{
					     enviornmentName = 'All';
					   }
					 }
					 console.log(enviornmentName.toUpperCase());
					 
					 if (enviornmentName == 'All')
					 {
					   return true;
						 
					 }
					 else{
						 return false;
					 }
				 }	
				

			if ($scope.alreadySelected) {
				$(".report-section-right").addClass("hidden");
				$reportCategories.addClass("hidden");
				$reportCategory.addClass("hidden");
				$availableReports.addClass("hidden");
				//$reportForm.addClass("hidden");
				$reportBack.removeClass("hidden");
				$reportContainer.removeClass("hidden");
				//$currentReport.html($("[data-report-id='" + $reportListbox.val() + "']").html());
				// lmc - TODO, need report name and icon.
				$currentReport.html($scope.names[reportId]);
				$availableReportsSubElems.removeClass("hidden");
				$availableReportsHeader.addClass("report-hover");
				/* alert("in alreadySelected"+reportId); */
				$scope.generateReport('',reportId);
			}
			selectReport();
		}
	}]
);

