var graphs = angular.module('ChartAngular', []);

graphs.directive('linegraph', function() {

    var graphs = {}, r;
    var lineClasses = ["line-blue", "line-green", "line-red"];
    var NUM_POINTS = 7;

    function buildGraph(target, r, values, options, tooltips) {
        var lc, c = 0;
        var options = $.extend({}, {
            width: 100,
            height: 100,
            colors: ['#204363', '#80bc3d', '#bc3d41']
        }, options);

        r = Raphael(target, options.width, options.height);
        graphs[target] = r;
        lc = r.linechart(0, 0, options.width, options.height,
            [0, 1, 2, 3, 4, 5, 6], values, {
                smooth: false,
                symbol: 'circle', colors: options.colors, width: 3
            });

        lc.each(function () {
            var node = $(this.symbol.node),
                color = node.attr("fill"),
                arrIndex = Math.floor(c / NUM_POINTS);

            $(this.symbol.node)
                .attr("fill", "#fff")
                .attr("stroke", color)
                .css("cursor", "pointer")
                .tooltip({
                    content: tooltips[c],
                    items: "circle",
                    track: true,
                    tooltipClass: lineClasses[arrIndex],
                    position: {
                        my: "center bottom-5",
                        at: "center top"
                    }
                });

            c++;
        });
        lc.symbols.attr({r: 3, "stroke-weight": 1});
        lc.container = target;
        lc.values = values;
        return lc;
    }

    return {
        restrict: 'E',
        scope: {
            options: '=',
            values: '=',
            tooltips: '='
        },
        replace: true,
        template: '<div></div>',
        link: function (scope, element, attrs) {
            var id = attrs.id;
            scope.$watchCollection('[values, options, tooltips]', function (newValues, oldValues) {
                var r;
                return buildGraph(id, r, newValues[0], newValues[1], newValues[2]);
            });
        }
    }
});

graphs.directive('barchart', function() {

    var graphs = {}, r;
    var NUM_POINTS = 7;

    function buildGraph(target, r, values, options, tooltips) {
        var bc, c = 0;
        var options = $.extend({}, {
            stacked: true,
            type: 'soft',
            width: 100,
            height: 100,
            colors: ['#3d80bc', '#80bc3d', '#bc3d41']
        }, options);

        var barOver = function () {
            var col = 0;
            if (this.bar !== undefined && bc.bars !== undefined) {
                var bars = bc.bars[0];
                for (var i = 0; i < bars.length; i++) {
                    if (bars[i].y == this.bar.y) {
                        col = i;
                        break;
                    }
                }
                bc.flag = r.popup(this.bar.x, this.bar.y, tooltips[col]).insertBefore(this);
            }
        };

        var barOut = function () {
            bc.flag.animate({opacity: 0}, 300, function () {this.remove();});
        };

        r = Raphael(target, options.width, options.height);
        //graphs[target] = r;
        bc = r.barchart(0, 0, options.width, options.height,
            values, {
                smooth: false,
                symbol: 'circle', colors: options.colors, width: 3
            });

        bc.container = target;
        bc.values = values;
        //bc.hoverColumn(barOut, barOver);
        //bc.hoverColumn(barOver, barOut);
        bc.hover(barOver, barOut);
        return bc;
    }

    return {
        restrict: 'E',
        scope: {
            options: '=',
            values: '=',
            tooltips: '='
        },
        replace: true,
        template: '<div></div>',
        link: function (scope, element, attrs) {
            var id = attrs.id;
            scope.$watchCollection('[values, options, tooltips]', function (newValues, oldValues) {
                var r;
                return buildGraph(id, r, newValues[0], newValues[1], newValues[2]);
            });
        }
    }
});
