(function($, PHD, window, undefined) {
    
    /*
    *   DataGrid is a better DataTable
    */

    var doc = window.document;

    var defaults = {
        rowClass: "",
        cellClasses: null,
        classifiers: [],

        // Selectable Rows
        select: false, // 'click', 'check', `false`

        // Scrolling
        tableScroll: true,
        tableScrollOptions: {},

        // Sorting
        sortable: false,
        sortOnLoad: false,
        // header sorting
        sortOnHeader: false,
        sortMethods: {},
        // single-method sorting
        sortKey: 'id',
        sortMethod: 'string',
        sortDir: 'asc',

        // Column Filtering
        filter: false,

        // Column Resizing
        columnResize: false,

        // Dynamic loading
        dynamic: false,
        rowHeight: 30,
        rowBuffer: 8,
        rowsToRender: 30,

        // Auto resize
        resize: false,
        paged: false,
        integerIDs: true,
        prependData: false,
        increments: 0,
        placeholder: gettext("There is no data to display"),
        loading: '<i class="icon-restart icon-spin"></i>'
    };

    var lookup = PHD.StateLookup;

    var DataGrid = function DataGrid($table, options) {
        if(this instanceof DataGrid) {
            this.$table = $table;
            this.$thead = this.$table.children("thead");
            this.$tbody = this.$table.children("tbody");
            this.$tfoot = this.$table.children("tfoot");

            this.options = $.extend({}, defaults, options);

            this._data = null;
            this.data = null;

            this.rows = {};
            this.dbids = [];

            this._insert = Array.prototype.push;
            this._isLoading = false;
            this.cursor = 0;

            this.formatter = $.extend({}, defaultFormatter);
        } else {
            return new DataGrid($table, options).init();
        }
    };

    DataGrid.prototype = {
        init: function() {
            var self = this,
                $cols = this.$thead.find("tr:last-child th"),
                cols = $cols.length;

            this.schema = $cols.map(dataSource).get();

            if(this.options.select) {
                this._initRowSelection();
            }

            if(this.options.sortable) {
                this.options.sortMethods = $.extend({}, defaultSortMethods, this.options.sortMethods);
                this._initSorting();
            }

            if(this.options.filter) {
                this._initFilter();
            }

            if(this.options.dynamic) {
                this._initDynamicRendering(cols);
            } else if(this.options.increments) {
                this._initIncrementalRendering(cols);
            }

            if(this.options.prependData) {
                this._insert = Array.prototype.unshift;
            }

            if(this.options.columnResize) {
                this.$table.columnResize();
            }

            this.$tbody
                .on("mousedown", "tr", function(event) {
                    if(event.which === 2) {
                        console.dir(self.rows[$(this).data("dbid")].$el.data("model"));
                    }
                });

            $(window).on('beforeunload', function() {
                self.$tbody.scrollTop(0);
            });
            
            return this;
        },

        _initFilter: function() {
            var self = this;

            this.$tfoot.find("[data-filter]").each(function() {
                var $input = $(this),
                    key = $input.data("filter"),
                    method = $input.data("filter-type");

                if($input.is("[type=text]")) {
                    $input.val("");
                    $input.data("lastFilterValue", "");
                    $input.on("keyup", PHD.util.debounce($.proxy(self.handleFilterText, self), 250));
                } else if($input.is("select")) {
                    $input[0].selectedIndex = 0;
                    $input.data("lastFilterValue", $input.val());
                    $input.on("keyup change", PHD.util.debounce($.proxy(self.handleFilterSelect, self), 100));
                }
            });
        },

        handleFilterText: function(event) {
            var self = this,
                $input = $(event.currentTarget),
                query = $input.val().trim(),
                key = $input.data("filter"),
                method = $input.data("filter-type"),
                haschanged = (query !== $input.data("lastFilterValue"));
            
            if(haschanged) {
                if(query) {
                    //console.log("running text filter");
                    this.data = filters[method](this._data, query, key);
                } else {
                    this.data = this._data;
                }
                //console.log("rebuilding table");
                $input.data("lastFilterValue", query);
                this.dbids = getIDs(this.data);
                if(this.selected.length) {
                    _.difference(this.selected, this.dbids).forEach(function(dbid) {
                        self.rows[dbid].deselect();
                    });
                    this.selected = _.intersection(this.selected, this.dbids);
                }
                this.$table.trigger("aftercolumnfilter");
                return this.buildDynamic();
            }
            return this;
        },

        handleFilterSelect: function(event) {
            var self = this,
                $input = $(event.currentTarget),
                query = $input.val(),
                key = $input.data("filter"),
                method = $input.data("filter-type"),
                haschanged = (query !== $input.data("lastFilterValue"));
            
            if(haschanged) {
                if(query && query !== "all") {
                    //console.log("running select filter");
                    this.data = filters[method](this._data, query, key);
                } else {
                    this.data = this._data;
                }
                //console.log("rebuilding table");
                $input.data("lastFilterValue", query);
                this.dbids = getIDs(this.data);
                if(this.selected.length) {
                    _.difference(this.selected, this.dbids).forEach(function(dbid) {
                        self.rows[dbid].deselect();
                    });
                    this.selected = _.intersection(this.selected, this.dbids);
                }
                this.$table.trigger("aftercolumnfilter");
                return this.buildDynamic();
            }
            return this;
        },

        _initIncrementalRendering: function() {
            if(this.options.tableScroll) {
                this.$table.scrollTable(this.options.tableScrollOptions);
            }

            this.$tbody.on("scroll", PHD.util.debounce($.proxy(this.handleScrollIncremental, this)));
        },

        _initDynamicRendering: function(cols) {
            var i, pre, post;

            pre = doc.createElement('tr');
            pre.className = 'dg-pre-buffer dg-buffer';
            this.preBuffer = pre;

            post = doc.createElement('tr');
            post.className = 'dg-post-buffer dg-buffer';
            this.postBuffer = post;
            
            for(i=0;i<cols;i++) {
                pre.appendChild(doc.createElement('td'));
                post.appendChild(doc.createElement('td'));
            }

            if(this.options.tableScroll) {
                this.$table.scrollTable(this.options.tableScrollOptions);
            }

            this.$tbody.on("scroll", PHD.util.debounce($.proxy(this.handleScrollDynamicRender, this), 5));
            this.$tbody.on("scroll", PHD.util.debounce($.proxy(this.handleScrollDynamicScroll, this), 250));
        },

        updateDynamic: function() {
            return this.buildDynamic();
        },

        _initEmptyRow: function() {
            var row = doc.createElement('tr');
            this.schema.forEach(function() {
                row.appendChild(doc.createElement('td'));
            });
            return row;
        },

        buildDynamic: function() {
            var i, options = this.options;

            //this.sp = this.$tbody.scrollTop();
            this.$tbody.empty();
            this.renderedRows = [];
            this.lastScrollPos = 0;
            
            this.vpHeight = this.$tbody[0].clientHeight;
            this.scrollMax = this.contentHeight - this.vpHeight;
            this.contentHeight = options.rowHeight * this.data.length;
            this.renderedHeight = options.rowHeight * options.rowsToRender;

            options.rowsToRender = Math.floor((this.vpHeight / options.rowHeight) + (options.rowBuffer * 2));

            for(i=0;i<options.rowsToRender;i++) {
                this.renderedRows.push(this._initEmptyRow());
            }

            this.preBufferHeight = 0;
            this.postBufferHeight = Math.max(0, this.contentHeight - this.renderedHeight);

            this.preBuffer.style.height = this.preBufferHeight + 'px';
            this.$tbody[0].appendChild(this.preBuffer);

            this.createRows(0, this.dbids.length);

            this.lastStart = null;
            this.buildDynamicRows(0, options.rowsToRender, true);

            this.postBuffer.style.height = this.postBufferHeight + 'px';
            this.$tbody[0].appendChild(this.postBuffer);

            if(options.tableScroll) {
                this.$table.scrollTable(options.tableScrollOptions);
            }
            
            if(this.options.columnResize) {
                this.$table.columnResize();
            }
            console.log("scroll area height:", this.contentHeight);
            return this;
        },

        buildDynamicRows: function(start, end, insert) {
            var self = this;
            if(start !== this.lastStart) {
                if(start < 0) start = 0;

                _.each(this.rows, function(row, dbid) {
                    row._inDOM = null;
                });

                this.dbids.slice(start, end).forEach(function(dbid, idx) {
                    var row = self.rows[dbid],
                        renderedRow = self.renderedRows[idx];

                    if(!row.rendered) row.render();
                    copyRow(row.el, renderedRow);
                    row._inDOM = renderedRow;
                    if(insert === true) self.$tbody[0].appendChild(renderedRow);
                });

                this.lastStart = start;
            }
        },

        handleScrollDynamicRender: function(event) {
            var sp = this.$tbody.scrollTop(),
                options = this.options,
                renderStart = Math.floor(sp / options.rowHeight),
                start, end;

            if(sp !== this.lastScrollPos) {
                this.lastScrollPos = sp;

                this.preBufferHeight = PHD.util.clamp(sp - (sp % options.rowHeight) - (options.rowHeight * options.rowBuffer), 0, this.contentHeight - this.renderedHeight);
                this.postBufferHeight = PHD.util.clamp(this.contentHeight - this.renderedHeight - this.preBufferHeight, 0, this.contentHeight - this.renderedHeight);

                this.preBuffer.style.height = this.preBufferHeight + 'px';
                this.postBuffer.style.height = this.postBufferHeight + 'px';

                start = PHD.util.clamp(renderStart, 0, this.dbids.length - options.rowsToRender);
                end = PHD.util.clamp(renderStart+options.rowsToRender, 0, this.dbids.length);
                
                this.buildDynamicRows(start, end);
            }
        },

        handleScrollDynamicScroll: function(event) {
            if(this.options.tableScroll && this.lastScrollPos !== this.$tbody.scrollTop()) {
                this.$table.scrollTable();
            }
        },

        _initRowSelection: function() {
            if(this.options.select === 'click') {
                this.selected = null;
                this.options.rowClass = "clickable " + this.options.rowClass;

                this.$tbody
                    .on("click", ".clickable", $.proxy(this.handleRowClick, this))
                    // Keyboard controls
                    .on("keydown", function(event) {
                        switch ( event.keyCode ) {
                            case $.ui.keyCode.UP:
                                event.preventDefault();
                                if(self.hasSelectedRow()) {
                                    self.selectPreviousRow();
                                }
                                break;
                            case $.ui.keyCode.DOWN:
                                event.preventDefault();
                                if(self.hasSelectedRow()) {
                                    self.selectNextRow();
                                }
                                break;
                        }
                    });
            }

            if(this.options.select === 'check') {
                this.selected = [];
                this.$tbody
                    .on("change", ".row-select", $.proxy(this.handleRowCheck, this));
                this.$thead
                    .on("change", ".check-select-all", $.proxy(this.handleRowCheckAll, this));

                this.getSelectionQueryString = function(key) {
                    key = key || "inventory";
                    if(this.selected && this.selected.length) {
                        return this.selected.map(function(t) { return key + "=" + t}).join("&");
                    }
                    return null;
                };
            }

            return this;
        },

        handleRowCheck: function(event) {
            var checked = event.currentTarget.checked,
                dbid = parseInt(event.currentTarget.value);

            if(checked) {
                this.selectByID(dbid);
            } else {
                this.deselectByID(dbid);
            }
        },

        handleRowCheckAll: function(event) {
            var checked = event.currentTarget.checked;

            if(checked) {
                this.dbids.forEach($.proxy(this.selectByID, this));
            } else {
                this.dbids.forEach($.proxy(this.deselectByID, this));
            }

            this.$table.trigger("selectall");
        },

        selectByID: function(dbid) {
            if(!_.contains(this.selected, dbid) && this.rows.hasOwnProperty(dbid)) {
                this.selected.push(dbid);
                this.rows[dbid].select();
            }
            return this;
        },

        deselectByID: function(dbid) {
            this.selected = _.without(this.selected, dbid);
            if(this.rows.hasOwnProperty(dbid)) {
                this.rows[dbid].deselect();
                return this;
            }
        },

        select: function($row) {
            if(this.options.select === 'check') {
                return this._selectCheck;
            }

            if(this.options.select === 'click') {
                return this._selectClick;
            }
        },

        _selectClick: function($row) {
            if($row.is(".current-row")) {
                return false;
            }

            if(this.$selected.length) {
                this.$tbody.find(".current-row").removeClass("current-row");
            }
            this.$selected = $row.addClass("current-row").focus();
            
            if(this.options.integerIDs) {
                this.selectedID = parseInt(this.$selected.data("dbid"), 10);    
            } else {
                this.selectedID = this.$selected.data("dbid");
            }

            this.$selected.trigger("rowselect", [this.selectedID, $row, this, this.getGridRow(this.selectedID)]);
            return this;
        },

        _selectCheck: function(dbid) {
            console.warn("Not implemented");
        },

        _initSorting: function() {
            if(this.options.sortOnHeader) {
                var self = this,
                    $sortOn = this.$thead.find("th[data-sort-dir=asc], th[data-sort-dir=desc]").first(),
                    defaultSortDir = $sortOn.data("sort-dir");

                this.sortKey = $sortOn.data("source");
                this.sortMethod = this.options.sortMethods[$sortOn.data("sort")];
                this.sortDir = defaultSortDir;

                $sortOn.addClass("sort-" + defaultSortDir);

                this.$thead.on("click", "th.sortable", function() {
                    var $header = $(this),
                        methodName = $header.data("sort") || null,
                        direction = $header.data("sort-dir") || null,
                        sortMethod;

                    if(methodName === null)
                        return;

                    sortMethod = self.options.sortMethods[methodName];
                    self.$table.resetSortIndicators();

                    if(direction) {
                        direction = direction === "desc" ? "asc" : "desc";
                    } else {
                        direction = "asc";
                    }

                    $header
                        .data("sort-dir", direction)
                        .addClass("sort-"+direction);

                    self.sortKey = $header.data("source");
                    self.sortMethod = sortMethod;
                    self.sortDir = direction;

                    self.sort();

                    self.cursor = 0;
                    self.$tbody.scrollTop(0);
                    self.buildDynamic(0, self.options.rowsToRender);

                    self.$table.trigger("aftertablesort", [$header, direction]);
                });
            } else {
                this.sortKey = this.options.sortKey;
                this.sortMethod = this.options.sortMethods[this.options.sortMethod];
                this.sortDir = this.options.sortDir;
            }
            
        },

        sort: function() {
            sortOnKey(this.data, this.sortKey, this.sortMethod, (this.sortDir === "desc"));
            this.dbids = getIDs(this.data);
            return this;
        },

        empty: function() {
            this.$tbody.find("tr:not(.dg-buffer)").remove();
            this.$tbody.find(".dg-buffer").css("height", "0px");
            this.data = this._data = [];
            this.rows = {};
            this.dbids = [];
            this.selected = [];
            return this;
        },

        load: function(data) {
            this.empty();
            this._data = data;
            this.dbids = getIDs(this._data);

            this.data = data;

            if(this.options.sortable && this.options.sortOnLoad) this.sort();

            if(this.options.dynamic) this.buildDynamic();
            else this.build();

            this.$table.trigger("loadfinished", [(this.data.length > 0), this]);
            return this;
        },

        build: function() {
            var self = this,
                cursorEnd = (this.cursor + this.options.increments) || this.dbids.length;

            if(this.data.length) {
                this.hidePlaceholder();

                this.createRows(this.cursor, cursorEnd);
                this.insertRows(this.cursor, cursorEnd);
                this.cursor = cursorEnd;
            } else {
                this.showPlaceholder();
            }

            if(this.options.tableScroll) {
                this.$table.scrollTable(this.options.tableScrollOptions);
            }

            return this;
        },

        update: function(data, deleteMissing) {
            if(this.data === null) return this.load(data);

            var self = this,
                newRows = false;

            if(deleteMissing === true) {
                this.deleteRows(_.difference(this.dbids, getIDs(data)));
            }

            if(data.length) {
                data.forEach(function(rowData) {
                    var dbid = rowData.id;
                    if(_.contains(self.dbids, dbid) && self.rows.hasOwnProperty(dbid)) {
                        self.rows[dbid].update(rowData);
                    } else {
                        self._insert.call(self.data, rowData);
                        self._insert.call(self.dbids, dbid);
                        self.rows[dbid] = self.createRow(rowData);
                    }
                });
            }

            if(this.options.sortable) this.sort();
            if(this.options.dynamic) this.updateDynamic();
            if(this.options.tableScroll) this.$table.scrollTable();

            return this;
        },

        createRow: function(rowData) {
            return GridRow(rowData, this.schema, this.formatter, this.options.rowClass, this.options.cellClasses, this.options.classifiers);
        },

        createRows: function(start, end) {
            var self = this;
            this.data.slice(start, end).forEach(function(rowData) {
                var dbid = rowData.id;
                if(!self.rows[dbid]) {
                    self.rows[dbid] = self.createRow(rowData);
                }
            });
        },

        insertRow: function(row) {
            if(!row.rendered) row.render();
            this.$tbody[0].appendChild(row.el);
            this.$table.trigger("rowadd", [row, row.dbid, this]);
        },

        insertRows: function(start, end) {
            var self = this;
            this.dbids.slice(start, end).forEach(function(dbid, idx) {
                self.insertRow(self.rows[dbid]);
            });
        },

        testLoad: function(data, deleteMissing) {
            var self = this,
                tb = this.$tbody[0],
                df = doc.createDocumentFragment();

            data.forEach(function(row) {
                var r = doc.createElement('tr');
                self.schema.forEach(function(key) {
                    var td = doc.createElement('td');
                    td.innerHTML = "t";//row.data[key];
                    r.appendChild(td);
                });
                df.appendChild(r);
            });
            tb.appendChild(df);
        },

        handleRowClick: function(event) {
            //event.stopPropagation();
            if(!this._isLoading) this.select($(event.currentTarget));
        },

        handleScrollIncremental: function(event) {
            if(this.$tbody.isScrolled()) {
                this.createRows(this.cursor, this.cursor + this.options.increments);
                this.insertRows(this.cursor, this.cursor + this.options.increments);
                this.cursor = this.cursor + this.options.increments;
                if(this.options.tableScroll) {
                    this.$table.scrollTable();
                }
            }
        },

        selectNextRow: function() {
            var next = this.$selected.next("tr");
            return next.length ? this.select(next) : false;
        },

        selectPreviousRow: function() {
            var prev = this.$selected.prev("tr");
            return prev.length ? this.select(prev) : false;
        },

        deselect: function() {
            this.$selected.removeClass("current-row");
            this.$selected = $();
            this.selectedID = null;
            return this;
        },

        hasSelectedRow: function() {
            return (this.selectedID !== null && this.$selected.length);
        },

        hasRow: function(dbid) {
            return this.rows.hasOwnProperty(dbid);
        },

        getRow: function(dbid) {
            return this.rows[dbid].$el;
        },

        getGridRow: function(dbid) {
            return this.rows[dbid];
        },

        getRows: function() {
            return this.$tbody.find("tr:not(.placeholder)");
        },

        getSelectedRow: function() {
            if(this.hasSelectedRow()) {
                return this.$selected;
            }
        },

        count: function() {
            return this.dbids.length;
        },

        deleteRows: function(rowIDs) {
            rowIDs.forEach($.proxy(this.deleteRow, this));
            if(!this.dbids.length) {
                this.empty();
            }
            return this;
        },

        deleteRow: function(dbid) {
            var index = this.dbids.indexOf(dbid);

            if(dbid === this.selectedID) {
                this.selectedID = null;
                this.$selected = $();
            }

            if(this.rows.hasOwnProperty(dbid)) {
                if(!this.options.dynamic) this.rows[dbid].el.remove();
                delete this.rows[dbid];
            }
            
            if(index !== -1) {
                this.data.splice(index, 1);
                this.dbids.splice(index, 1);
            } else {
                console.error("Could not find item %d in data", dbid);
            }
            
            this.$table.trigger("rowdelete", [dbid, this]);
            return this;
        },

        registerFormatters: function(formatters) {
            $.extend(this.formatter, formatters);
            return this;
        },

        getData: function(dbid, key) {
            dbid = parseInt(dbid, 10);
            if(this.hasRow(dbid)) {
                return this.rows[dbid].data[key];
            } else {
                return "Row Does Not Exist";
            }

        },

        on: function(eventName, callback) {
            this.$table.on(eventName, callback);
            return this;
        },

        off: function(eventName) {
            this.$table.off(eventName);
            return this;
        },

        showPlaceholder: function() {
            //this.$placeholder.prependTo(this.$tbody);
            return this;
        },

        hidePlaceholder: function() {
            //this.$placeholder.detach();
            return this;
        }
    };

    PHD.DataGrid = DataGrid;


    var GridRow = function GridRow(data, schema, formatter, rowClass, cellClasses, classifiers) {
        if(this instanceof GridRow) {
            this.data = data;
            this.schema = schema;
            this.formatter = formatter;
            this.rowClass = rowClass;
            this.cellClasses = cellClasses || [];
            this.classifiers = classifiers;
        } else {
            return new GridRow(data, schema, formatter, rowClass, cellClasses, classifiers).init();
        }
    };

    GridRow.prototype = {
        init: function() {
            var self = this;

            // local state
            this.dbid = this.data.id;
            this.rendered = false;
            this.selected = false;
            this.filtered = false;
            this.fitlers  = [];

            // dom elements
            this.el = doc.createElement('tr');
            this.el.setAttribute('data-dbid', self.data.id);
            this.el.setAttribute('class', this.rowClass);

            this.$el = $(this.el);
            this.$el.data("model", this.data);

            this._inDOM = null;

            this.schema.forEach(function(key) {
                var td = doc.createElement('td');
                td.setAttribute('data-source', key);
                self.el.appendChild(td);
            });

            //this.setSortValues();

            return this;
        },
        render: function() {
            var self = this;

            this.schema.forEach(function(key, index) {
                var td = self.el.childNodes[index];
                if(self.cellClasses && self.cellClasses[index]) {
                    td.className = self.cellClasses[index] || "";
                }
                td.innerHTML = self.format(key, self.data[key]);
            });

            this.$el.classifyGrid(this.data, this.classifiers, lookup);
            //this.enrich();

            this.rendered = true;
            return this;
        },
        update: function(data) {
            this.data = data;
            //this.setSortValues();

            if(this.rendered) this.render();

            return this;
        },
        html: function() {
            return this.el.innerHTML;
        },
        select: function() {
            //console.log("GridRow.select()", this.dbid);
            this.selected = true;

            if(this.rendered) {
                var checkbox = this.el.querySelector('.row-select');
                checkbox.defaultChecked = true;

                if(this._inDOM) {
                    this._inDOM.querySelector('.row-select').checked = true;
                }
            }
        },
        deselect: function() {
            //console.log("GridRow.deselect()", this.dbid);
            this.selected = false;

            if(this.rendered) {
                var checkbox = this.el.querySelector('.row-select');
                checkbox.defaultChecked = false;

                if(this._inDOM) {
                    this._inDOM.querySelector('.row-select').checked = false;
                }
            }
        },
        setSortValues: function() {
            var self = this;
            if(!this.data.hasOwnProperty('sort')) this.data.sort = {};
            Object.keys(this.data).forEach(function(key) {
                if(!self.data.sort.hasOwnProperty(key)) self.data.sort[key] = self.data[key];
            });
        },
        format: function(key, value, data) {
            return this.formatter.getValue(key, value, data || this.data, this);
        },
        enrich: function() {
            $(this.el).find(".progress-bar, .table-icon").tooltip({
                position: {
                    my: "center bottom",
                    at: "center top"
                },
                tooltipClass: "progress-bar-tooltip"
            });
        }
    };

    PHD.GridRow = GridRow;

    var re = /_i$/;

    $.fn.classifyGrid = function(data, keys, lookup) {
        var trueKey;
        return this.each(function() {
            var $el = $(this);
            keys.forEach(function(key) {
                trueKey = key.replace(re, "");
                $el.alterClass(trueKey.replace("_", "-") + "-*", lookup.get(trueKey, data[key]));
            });
        });
    };

    /*
    *   Utility Functions
    */

    // Extract a list of IDs from a typical array of result objects
    function getIDs(data) {
        return _.pluck(data, "id");
    }

    function sortOnKey(data, key, method, reverse) {
        var mult = reverse ? -1 : 1;
        data.sort(function(a, b) {
            return method(a[key], b[key]) * mult;
        });
    }

    function copyRow(from, to) {
        to.innerHTML = from.innerHTML;
        to.className = from.className;
        $(to).data("dbid", from.dataset.dbid);
    }

    function dataSource(idx, obj) {
        return $(obj).data("source");
    }

    function getDataAtPath(obj, path) {
        if(path.indexOf('.') === -1) {
            return obj[path];
        }

        path = path.split('.');
        var i = -1;
        while(++i < path.length && obj !== void 0) {
            obj = obj[path[i]];
        }
        return obj;
    }

    /* 
    *   Default sorting methods
    */
    var defaultSortMethods = {
        "int": function(a, b) {
            return parseInt(a, 10) - parseInt(b, 10);
        },
        "float": function(a, b) {
            return parseFloat(a) - parseFloat(b);
        },
        "string": function(a, b) {
            if(_.isArray(a)) {
                x = a.join("").toLowerCase();
            } else {
                x = a.toLowerCase();    
            }
            if(_.isArray(b)) {
                y = b.join("").toLowerCase();
            } else {
                y = b.toLowerCase();    
            }
            
            return (x < y) ?
                -1 : (x > y) ?
                    1 : 0;
        },
        "date": function(a, b) {
            if(a === EMPTY_CELL_TEXT || a === void 0) {
                return -1;
            }
            if(b === EMPTY_CELL_TEXT || a === void 0) {
                return 1;
            }
            x = new Date(a);
            y = new Date(b);
            return (x < y) ?
                -1 : (x > y) ?
                    1 : 0;
        }
    };

    var filters = {
        contains: function(data, query, key) {
            return data.filter(function(item) {
                var val = item[key];
                return (val.toLowerCase().indexOf(query.toLowerCase()) !== -1);
            });
        },
        startswith: function(data, query, key) {
            return data.filter(function(item) {
                var val = item[key];
                return (val.toLowerCase().indexOf(query.toLowerCase()) === -0);
            });
        },
        matches: function(data, query, key) {
            return data.filter(function(item) {
                var val = item[key];
                return (val === query || isNullish(val) && query === EMPTY_CELL_TEXT);
            });
        },
        inarray: function(data, query, key) {
            return data.filter(function(item) {
                var val = item[key];
                if(Array.isArray(val)) return (val.indexOf(query) !== -1);
                return false;
            });
        },
        regex: function(data, query, key) {
            query = new RegExp(query);
            return data.filter(function(item) {
                var val = item[key];
                return query.test(val);
            });
        }
    };

    function isNullish(value) {
        return (value === null || value === void 0);
    }

    /* 
    *   Object to convert raw values to display-friendly values
    *   Contains a few common formatters — the rest are provided by
    *   the view that instantiates the DataGrid
    */
    var defaultFormatter = {
        getValue: function(key, value, data, row) {
            if ((typeof value === 'undefined') || value === null)
                value = EMPTY_CELL_TEXT;
            return this.hasOwnProperty(key) ? 
                this[key](value, data, row) :
                value;
        },
        _controls: function(value, data, row) {
            var hvclass = getHypervisorClass(data.hypervisor),
                checked = row.selected ? " checked " : " ";

            return '<input class="row-select '+hvclass+'" type="checkbox"'+checked+'name="inventory" value="' + data.id + '">';
        },
        name: function(value) {
            return '<span class="truncates item-name" title="' + value + '">' + value + '</span>';
        },
        percent_complete: function(value) {
            return '<div class="progress-bar" title="' + value + '%">' +
                '<div class="progress-meter" style="width:' + value + '%;"></div></div><span class="progress-count">'+ value + '%</span>';
        },
        message: function(value) {
            return '<span class="truncates">' + value + '</span>';
        }
    };

    function getHypervisorClass(hvtype) {
        switch(hvtype) {
            case HYPERVISOR_HYPERV:
                return "hv-type-hyperv";
            case HYPERVISOR_VMWARE:
                return "hv-type-vmware";
            case HYPERVISOR_XEN:
                return "hv-type-xen";
            default:
                return "hv-type-unknown";
        }
    }

})(jQuery, window.PHD || (window.PHD = {}), window);
