(function($, PHD, window, undefined) {

    var defaults = {
        namespace: "phdform",
        url: null,
        actions: {
            "submit": function() {
                return this.$form.valid();
            }, 
            "cancel": function() {
                return true;
            }
        },
        serialize: function($form) {
            return $form.serialize();
        },
        disableButtonsOnSubmit: true,
        submitOnEnter: true,
        isEditing: false,
        debug: false
    };

    var defaultValidator = {
        errorLabelContainer: "#dialog-error-list",
        errorContainer: "#dialog-errors",
        showErrors: function(errorMap, errorList) {
            $("#dialog-errors").find(".summary")
                .html(gettext("Correct the following errors:"));
            this.defaultShowErrors();
        }
    };

    var FormController = function FormController($form, options) {
        if(this instanceof FormController) {
            this.$form = $form;
            this.$actions = this.$form.find(".actions");
            this.$buttons = this.$actions.find("[type=button]");

            this.validator = null;

            this.options = $.extend(true, {}, defaults, options);
            this.namespace = this.options.namespace;
            this.url = this.options.url || $form.attr("action");
            this.serialize = this.options.serialize;

            this.buttons = {};
        } else {
            return new FormController($form, options).init();
        }
    };

    FormController.prototype = Object.create(Object.prototype);

    FormController.prototype.init = function() {
        var self = this;

        this._initActions();

        return this;
    };

    FormController.prototype._initActions = function() {
        var self = this;
        //_.each(this.options.actions, this._initAction, this);

        this.$buttons
            .each(function() {
                self.buttons[$(this).data("action")] = $(this);
            })
            .on("click", function(event) {
                event.preventDefault();
                var action = $(this).data("action");
                if(self._callAction(action)) {
                    !!self[action] ? self[action]() : self.$form.trigger("form"+action, [self]);
                }
            });

        if(this.options.submitOnEnter) {
            this.$form.on("keydown", "input, select", function(event) {
                event.stopPropagation();
                if(event.keyCode == $.ui.keyCode.ENTER) {
                    event.preventDefault();
                    if(self._callAction("submit")) {
                        self.submit();
                    }
                }
            });
        }

        this.$form.on("submit", function(event){event.preventDefault()});
        return this;
    };

    FormController.prototype._initAction = function(fn, action) {
        var self = this;
        this.$actions
            .find("[data-action=" + action + "]")
            .on("click", function(event) {
                event.preventDefault();
                if(self._callAction(action)) {
                    !!self[action] ? self[action]() : self.$form.trigger("form"+action, [self]);
                }
            });
    };

    FormController.prototype._callAction = function(action) {
        if(this.options.actions.hasOwnProperty(action)) {
            return this.options.actions[action].call(this);
        } else {
            return true;
        }
    };

    FormController.prototype.submit = function(submitData) {
        submitData = submitData || null;
        if(this.options.debug) {
            console.log(this.serialize.call(this, this.$form).split("&").join("\n"));
        } else {
            var self = this,
                load = PHD.showLoadingIndicator("body", true, "Saving...");

            this.$buttons.blur().disable();

            //var payload = this.serialize.call(this, this.$form);
            if (submitData == null) {
                submitData = form2js(this.$form.attr("id"), '.', false);
            }
            submitData = JSON.stringify(submitData);
            console.log('=== Submit form: object = ' + submitData);

            var $editing_id = this.$form.find("#edit_object_id");
            console.log('edit id text = ' + $editing_id.text() + " and appliance id is " + PHD.appliance_sid);
            var sid = (PHD.appliance_sid === null) ? "" : "?sid=" + PHD.appliance_sid;
            if ($editing_id.text() != "none") {
                var myID = $editing_id.text();
                var url = this.url + myID + "/" + sid;
                console.log("IS Editing, url is " + url);
                PHD.Ajax
                    .put(url, submitData)
                    .done(function(data) {
                        self.$form.trigger("formsubmit", [data, self.$form, self]);
                    })
                    .always(function() {
                        self.$buttons.enable();
                        PHD.hideLoadingIndicator(load);
                    });
            } else {
                var url = this.url + sid;
                console.log("Is Adding, not Editing, url is " + url);
                PHD.Ajax
                    .post(url, submitData)
                    .done(function(data) {
                        self.$form.trigger("formsubmit", [data, self.$form, self]);
                    })

                    .always(function() {
                        self.$buttons.enable();
                        PHD.hideLoadingIndicator(load);
                    });
            }
        }

        return this;
    };

    FormController.prototype.validate = function(options) {
        this.validator = this.$form.validate($.extend(true, {}, defaultValidator, options));
        return this;
    };

    FormController.prototype.on = function(eventname, callback) {
        this.$form.on(eventname, callback);
        return this;
    };

    FormController.prototype.off = function(eventname) {
        this.$form.off(eventname);
        return this;
    };

    PHD.FormController = FormController;

    var stepformdefaults = $.extend(true, {}, defaults, {
        actions: {
            "submit": function() {
                return this.$form.valid();
            },
            "prev": function() {
                //return this.$form.valid();
                this.validator.resetForm();
                return true;
            },
            "next": function() {
                return this.$form.valid();
            },
            "jump": function() {
                return this.$form.valid();
            },
            "cancel": function() {
                return true;
            }
        }
    });

    var StepForm = function StepForm($form, options) {
        if(this instanceof StepForm) {
            this.$form = $form;
            this.$nav = $form.find(".breadcrumbs");
            this.$pages = $form.find(".form-step");
            this.$actions = $form.find(".actions");
            this.$buttons = this.$actions.find("[type=button]");

            this.options = $.extend(true, {}, stepformdefaults, options);
            this.namespace = this.options.namespace;
            this.url = this.options.url || $form.attr("action");
            this.serialize = this.options.serialize;

            this.breadcrumbs = [];
            this.pages = [];
            this.buttons = {};

            this.currentPage = {
                index: 0,
                page: null
            };

        } else {
            return new StepForm($form, options).init();
        }
    };

    StepForm.prototype = Object.create(FormController.prototype);

    StepForm.prototype.init = function() {
        var self = this;

        this._initPages();
        this._initBreadcrumbs();
        this._initActions();

        this._updateActions();

        return this;
    };

    StepForm.prototype._initPages = function() {
        var self = this;
        this.$pages.each(function() {
            self.pages.push($(this).hide());
        });

        this.currentPage.page = this.pages[this.currentPage.index].show();
        return this;
    };

    StepForm.prototype._initBreadcrumbs = function() {
        var self = this;

        this.$nav.find("li").each(function() {
            self.breadcrumbs.push($(this));
        });

        this.breadcrumbs.forEach(function($link, index) {
            $link.find("a").on("click", function(event) {
                event.preventDefault();
                self.jump(index);
            });
        });

        return this;
    };

    StepForm.prototype._initActions = function() {
        var self = this;
        //_.each(this.options.actions, this._initAction, this);

        this.$buttons
            .each(function() {
                self.buttons[$(this).data("action")] = $(this);
            })
            .on("click", function(event) {
                event.preventDefault();
                var action = $(this).data("action");
                if(self._callAction(action)) {
                    !!self[action] ? self[action]() : self.$form.trigger("form"+action, [self]);
                }
            });

        if(this.options.submitOnEnter) {
            this.$form.on("keydown", "input, select", function(event) {
                event.stopPropagation();
                if(event.keyCode == $.ui.keyCode.ENTER) {
                    event.preventDefault();
                    if((self.currentPage.index === (self.pages.length - 1))) {
                        if(self._callAction("submit")) {
                            self.submit();
                        }   
                    } else {
                        var action = "next";
                        if(self._callAction(action)) {
                            !!self[action] ? self[action]() : self.$form.trigger("form"+action, [self]);
                        }
                    }
                }
            });
        }

        this.$form.on("submit", function(event){event.preventDefault()});
        return this;
    };

    StepForm.prototype.next = function() {
        var newIndex = this.currentPage.index + 1;
        if(newIndex > this.pages.length) return this;
        return this._goto(newIndex);
    };

    StepForm.prototype.prev = function() {
        var newIndex = this.currentPage.index - 1;
        if(newIndex < 0) return this;
        return this._goto(newIndex);
    };

    StepForm.prototype.jump = function(index) {
        if(index >= 0 && index <= this.pages.length) {
            return this._goto(index);
        }
        return this;
    };

    StepForm.prototype._goto = function(pageIndex) {
        this.breadcrumbs[this.currentPage.index].removeClass("active").addClass("done");
        this.currentPage.page.hide();
        this.currentPage.page = this.pages[pageIndex].show();
        this.currentPage.index = pageIndex;
        this.breadcrumbs[this.currentPage.index].addClass("active");
        this.$form.trigger("stepformgoto", [this.currentPage.index]);
        return this._updateActions();
    };

    StepForm.prototype._updateActions = function() {
        var firstPage = (this.currentPage.index === 0),
            lastPage = (this.currentPage.index === (this.pages.length - 1));

        this.buttons["prev"]
            .prop("disabled", firstPage)
            .toggleClass("hidden", firstPage);
        this.buttons["next"]
            .prop("disabled", lastPage)
            .toggleClass("hidden", lastPage);
        this.buttons["submit"]
            .prop("disabled", (!lastPage && !this.options.isEditing));

        return this;
    };

    StepForm.prototype.setEditing = function(isEditing) {
        this.options.isEditing = (isEditing === true);
        return this._updateActions();
    };

    StepForm.prototype.addAction = function(action, callback) {
        this.options.actions[action] = callback;
        return this;
    };

    StepForm.prototype.disableAction = function(action) {
        this.buttons[action].prop("disabled", true).toggleClass("hidden", (action !== "submit" && action !== "cancel"));
        return this;
    };

    StepForm.prototype.enableAction = function(action) {
        this.buttons[action].prop("disabled", false).removeClass("hidden");
        return this;
    };

    PHD.StepForm = StepForm;


    var StepPage = function StepPage($container, callbacks) {
        if(this instanceof StepPage) {
            this.$container = $container;
            this.$pages = this.$container.find(".form-step");
            this.$actions = this.$container.find(".actions");

            this.pages = [];
            this.buttons = {};

            if(typeof callbacks == 'undefined') {
                this.callbacks = {};
            } else {
                this.callbacks = callbacks;
            }

            this.currentPage = {
                index: 0,
                page: null
            };

        } else {
            return new StepPage($container, callbacks).init();
        }
    };

    StepPage.prototype = {
        init: function() {
            var self = this;

            this.$pages.each(function(index, Element) {
                self.pages.push($(Element).hide());
            });

            this.currentPage.page = this.pages[this.currentPage.index].show();

            this.$actions.find("[type=button]").each(function(index, Element) {
                var $but = $(Element);
                self.buttons[$but.data("action")] = $but.data("disabled", false);
            });

            this.initActions();
            this.updateActions();

            return this;
        },

        next: function() {
            var run = true;
            if(this.callbacks.hasOwnProperty("next")) {
                run = this.callbacks["next"].call(this, this.currentPage.index);
            }
            if(run) {
                var newIndex = this.currentPage.index + 1;
                if(newIndex > this.pages.length) {
                    return false;
                }
                return this.goTo(newIndex);
            } else {
                return this
            }
        },

        prev: function() {
            var run = true;
            if(this.callbacks.hasOwnProperty("prev")) {
                run = this.callbacks["prev"].call(this, this.currentPage.index);
            }
            if(run) {
                var newIndex = this.currentPage.index - 1;
                if(newIndex < 0) {
                    return false;
                }
                return this.goTo(newIndex);
            } else {
                return this;
            }
        },

        goTo: function(pageIndex) {
            this.currentPage.page.hide();
            this.currentPage.page = this.pages[pageIndex].show();
            this.currentPage.index = pageIndex;

            this.$container.trigger("steppagegoto", [this.currentPage.index]);
            return this.updateActions();
        },

        close: function() {
            if(this.callbacks.hasOwnProperty("close")) {
                this.callbacks["close"].apply(this, this.callbacks["close"].arguments);
            } else {
                this.$container.closest(".dialog-box").dialog("close");
            }
        },

        save: function() {
            if(this.callbacks.hasOwnProperty("save")) {
                this.callbacks["save"].apply(this, this.callbacks["save"].arguments);
            } else {
                this.$container.closest(".dialog-box").dialog("close");
            }
        },

        initActions: function() {
            var self = this;
            this.buttons["cancel"].on("click", function(event) {
                if(!$(this).data("disabled"))
                    self.close();
            });

            this.buttons["prev"].on("click", function(event) {
                if(!$(this).data("disabled"))
                    self.prev();
            });

            this.buttons["next"].on("click", function(event) {
                if(!$(this).data("disabled"))
                    self.next();
            });

            this.buttons["save"].on("click", function(event) {
                if(!$(this).data("disabled"))
                    self.save();
            });
        },

        updateActions: function() {
            var firstPage = (this.currentPage.index === 0),
                lastPage = (this.currentPage.index === (this.pages.length - 1));

            this.buttons["prev"]
                .prop("disabled", firstPage)
                .toggleClass("hidden", firstPage);
            this.buttons["next"]
                .prop("disabled", lastPage)
                .toggleClass("hidden", lastPage);
            this.buttons["save"]
                .prop("disabled", !lastPage);

            return this;
        },

        addCallback: function(buttonId) {
            self.buttons.callbacks[buttonId] = buttonId;
        },

        disableAction: function($button) {
            $button.data("disabled", true);
        },

        enableAction: function($button) {
            $button.data("disabled", false);
        }
    };

    PHD.StepPage = StepPage;

})(jQuery, window.PHD || (window.PHD = {}), window);
