(function($) {

    $.fn.columnResize = function(options) {
        return this.each(function() {
            var $table = $(this).addClass("resizable-table"),
                $thead = $table.find("thead"),
                $tbody = $table.find("tbody"),
                $tfoot = $table.find("tfoot"),
                $columns = $thead.find("tr:last-child th"),
                $bColumns = $tbody.find("tr:not(.placeholder) td"),
                $fColumns = $tfoot.find("tr:first-child th"),
                numcols = $columns.length,
                height = $thead.find("tr:last-child").height(),
                widths = $columns.map(function() { return this.offsetWidth; }).get(),
                totalWidth = widths.reduce(function(prev,curr,index,arr){return (prev + curr);}, 0),
                minColWidth = 15,
                $handleContainer, $handles, 
                running = 0;

            // If this plugin has already been set on a table, just remeasure
            if($table.data("columnResize")) {
                $handleContainer = $table.data("handleContainer");
                $handles = $table.data("handles");
                $columns.each(function(idx, el) {
                    var width = $(el).css("width");

                    running += el.offsetWidth;

                    if(idx < numcols-1) {
                        var next = $columns.eq(idx+1),
                            leftMin = $(el).offset().left + minColWidth,
                            rightMax = next.offset().left + next[0].offsetWidth - minColWidth;
                        $table.data("handles").eq(idx)
                            .css({ "left": (running-6) + "px" })
                            .data("leftMin", leftMin)
                            .data("rightMax", rightMax);
                    }

                    $tbody.find("tr:not(.placeholder) td:nth-child(" + (idx+1) + ")").css({
                        "width": width,
                        "max-width": width,
                        "min-width": width
                    });
                });
            } else {
                $handleContainer = $('<div class="handles"></div>')
                    .css({
                        "height": 0,
                        "width": totalWidth + "px",
                        "position": "relative"
                    })
                    .insertBefore($table);

                $columns.each(function(idx, el) {
                    var width = $(el).css("width");

                    running += el.offsetWidth;

                    if(idx < numcols-1) {
                        var next = $columns.eq(idx+1),
                            leftMin = $(el).offset().left + minColWidth,
                            rightMax = next.offset().left + next[0].offsetWidth - minColWidth;

                        $handleContainer.append(
                            $("<div>", {
                                "class": "handle"
                            }).css({
                                "cursor": "e-resize",
                                "height": height + "px",
                                "width": "6px",
                                "position": "absolute",
                                "top": 0,
                                "left": (running-6) + "px"
                            }).data("leftMin", leftMin).data("rightMax", rightMax)
                        );
                    }

                    $tbody.find("tr:not(.placeholder) td:nth-child(" + (idx+1) + ")").css({
                        "width": width,
                        "max-width": width,
                        "min-width": width
                    });
                });

                $handles = $handleContainer.find('.handle');

                $table
                    .data("columnResize", true)
                    .data("handleContainer", $handleContainer)
                    .data("handles", $handles);
            }

            $handles.each(function(index) {
                var $handle = $(this),
                    $lCells, $rCells,
                    leftMin = $handle.data("leftMin"),
                    rightMax = $handle.data("rightMax");

                $lCells = 
                    $columns.eq(index)
                    .add($bColumns.filter(":nth-child(" + (index+1) + ")"))
                    .add($fColumns.filter(":nth-child(" + (index+1) + ")"));
                $rCells = 
                    $columns.eq(index+1)
                    .add($bColumns.filter(":nth-child(" + (index+2) + ")"))
                    .add($fColumns.filter(":nth-child(" + (index+2) + ")"));

                $handle.draggable({
                    axis: "x",
                    containment: [leftMin, 0, rightMax, 0],
                    cursor: "e-resize",
                    helper: "clone",
                    stop: function(event, ui) {
                        var idx = index,
                            newPos = parseInt(ui.position.left, 10),
                            newOffset = parseInt(ui.offset.left, 10),
                            delta = parseInt(ui.originalPosition.left, 10) - newPos,
                            leftCol = parseInt($columns.eq(idx).css("width"), 10),
                            rightCol = parseInt($columns.eq(idx+1).css("width"), 10),
                            newLeft, newRight;

                        if(delta !== 0) {
                            // reposition this handle to the helper's final location
                            var $h = $(this).css("left", ui.helper.css("left"));

                            // adjust sibling handles' containment parameters
                            if($h.prev(":not(.ui-draggable-dragging)").length) {
                                $h.prev().setBounds([null, newOffset-minColWidth]);
                            }
                            if($h.next(":not(.ui-draggable-dragging)").length) {
                                $h.next().setBounds([newOffset+minColWidth, null]);
                            }

                            newLeft = leftCol - delta;
                            newRight = rightCol + delta;

                            if(newLeft < 0) {
                                newLeft = 0;
                            }
                            if(newRight < 0) {
                                newRight = 0;
                            }

                            // style cells in the columns on either side of the handle
                            $lCells.css({
                                "width": newLeft,
                                "max-width": newLeft,
                                "min-width": newLeft
                            });
                            $rCells.css({
                                "width": newRight,
                                "max-width": newRight,
                                "min-width": newRight
                            });
                        }
                        
                    }
                });
            });

            $table.on("afterperpagechange afterpagechange aftercolumnfilter afterscrolltable", resizeHandles);

            function resizeHandles() {
                var height = $thead.find("tr:last-child").height();
                $handles.css({"height": height});
            }

        });
    };


    // Store and set drag handle containment boundaries
    $.fn.setBounds = function(bounds) {
        var leftMin = bounds[0] !== null ? bounds[0] : this.data("leftMin"),
            rightMax = bounds[1] !== null ? bounds[1] : this.data("rightMax");
        return this.each(function() {
            $(this)
                .data("leftMin", leftMin)
                .data("rightMax", rightMax)
                .draggable("option", "containment", [leftMin, 0, rightMax, 0]);
        });
    };

})(jQuery);