(function($) {
    $.widget("phd.dragdelta", $.ui.mouse, {
        version: "1.10.3",
        widgetEventPrefix: "dragdelta",
        options: {
            autoHide: false,
            containment: false,
            //disabled: false,
            direction: "vertical",
            helper: true,
            max: Infinity,
            min: 0,

            // callbacks
            start: null,
            stop: null,
            drag: null
        },

        _create: function() {
            var self = this;

            this.element.addClass("drag-handle").disableSelection();

            if(this.options.direction === "vertical") {
                this.cursor = "ns-resize";
                this.element.addClass("drag-handle-v");
                this.prop = "pageY";
                this.cssProp = "top";
            } else {
                this.cursor = "ew-resize";
                this.element.addClass("drag-handle-h");
                this.prop = "pageX";
                this.cssProp = "left";
            }
            
            this.dragging = false;

            this.startPos = 0;
            this.lastPos = 0;
            this.currentPos = 0;
            this.endPos = 0;

            if(this.options.helper) {
                this.helper = $("<div>", {
                    "class": "drag-helper drag-helper-" + this.options.direction.substr(0,1)
                });
            } else {
                this.helper = null;
            }

            if(this.options.autoHide) {
                this.element
                    .addClass("drag-handle-autohide")
                    .on("mouseenter", function() {
                        if(self.options.disabled) return;
                        $(this).removeClass("drag-handle-autohide");
                    })
                    .on("mouseleave", function(){
                        if(self.options.disabled) return;
                        if(!self.dragging) $(this).addClass("drag-handle-autohide");
                    });
            }

            this._mouseInit();
        },

        _mouseCapture: function(event) {
            this.startPos = this.lastPos = this.currentPos = this.endPos = event[this.prop];
            return true;
        },

        _mouseStart: function(event) {
            this.dragging = true;

            $("body").css("cursor", this.cursor);

            if(this.helper) {
                this.helper
                    .appendTo("body")
                    .css({
                        "height": this.element.height(),
                        "width": this.element.width()
                    })
                    .position({
                        "my": "top center",
                        "at": "top center",
                        "of": this.element
                    });
            }

            this.element.addClass("dragging");
            this._propagate("start", event);
            return true;
        },

        _mouseStop: function(event) {
            this.dragging = false;
            this.endPos = this.currentPos;

            $("body").css("cursor", "auto");

            if(this.helper) {
                this.helper.detach();
            }

            this.element.removeClass("dragging");
            this._propagate("stop", event);

            if(this.options.autoHide) this.element.addClass("drag-handle-autohide");

            return false;
        },

        _mouseDrag: function(event) {
            var current = Math.min(Math.max(event[this.prop], this.options.min), this.options.max);
            this.lastPos = this.currentPos;
            this.currentPos = current;
            
            if(this.helper) {
                this.helper.css(this.cssProp, this.currentPos + "px");
            }

            this._propagate("drag", event);
            return false;
        },

        _destroy: function() {
            this._mouseDestroy();

            this.element
                .removeClass("drag-handle drag-handle-autohide dragging");

            if(this.helper) this.helper.remove();
            return this;
        },

        _propagate: function(name, event) {
            (name !== "drag" && this._trigger(name, event, this.ui()));
        },

        ui: function() {
            return {
                element: this.element,
                helper: this.helper,
                start: this.startPos,
                last: this.lastPos,
                current: this.currentPos,
                end: this.endPos,
                delta: this.currentPos - this.startPos
            };
        }

        
    });
})(jQuery);
