(function($) {

    var pageTableDefaults = {
        "perPage": 25,
        "amounts": [10,25,50,100]
    };

    var pageTableControls = _.template(
        '<div class="page-table-controls clearfix <%= placement %>">' +
            '<% if(placement === "top") { %>' +
                '<label for="per-page-<%= placement %>">Rows per page</label>' +
                '<select id="per-page-<%= placement %>">' +
                '<% amounts.forEach(function(val) { %>' +
                    '<option value="<%= val %>"><%= val %></option>' +
                '<% }); %>' +
                '</select>' +
            '<% } %>' +
            '<% if(placement === "bottom") { %>' +
                '<span class="column-filter-legend">Showing ' +
                    '<span class="page-first-row"><%= pageFirstRow %></span> - ' +
                    '<span class="page-last-row"><%= pageLastRow %></span> of ' +
                    '<span class="visible-rows"><%= visible %></span>' +
                    '<span class="filter-count">' +
                        ' (filtered from <span class="total-rows"><%= total %></span> total)' +
                    '</span>' +
                '</span>' +
            '<% } %>' +
            '<nav class="page-table-nav">' +
                '<button type="button" class="page-table-prev">'+gettext('Prev')+'</button>' +
                '<span>Page <span class="current-page">1</span> of ' +
                '<span class="total-pages"><%= pages %></span></span>' +
                '<button type="button" class="page-table-next">'+gettext('Next')+'</button>' +
            '</nav>' +
        '</div>'
    );

    $.fn.pageTable = function(options) {
        return this.each(function() {
            var $table = $(this).addClass("scroll-table"),
                $thead = $table.find("thead"),
                $tbody = $table.find("tbody"),
                $rows = $tbody.find("tr"),
                rows = $rows.length,
                currentPage, perPage, totalPages,
                options = options || {};

            options = $.extend({}, pageTableDefaults, options);
            perPage = options.perPage;
            totalPages = Math.ceil(rows / perPage);
            amounts = options.amounts;
            currentPage = 1;

            var $pageControls = $(pageTableControls({
                placement: "top",
                amounts: amounts,
                pages: totalPages
            })).insertBefore($table);

            $pageControls = $pageControls.add($(pageTableControls({
                placement: "bottom",
                amounts: amounts,
                pages: totalPages,
                pageFirstRow: 1,
                pageLastRow: perPage,
                visible: rows,
                total: rows,
                filtered: false
            })).insertAfter($table));

            var $currentPage = $pageControls.find(".current-page"),
                $totalPages = $pageControls.find(".total-pages"),
                $pageFirstRow = $pageControls.find(".page-first-row"),
                $pageLastRow = $pageControls.find(".page-last-row"),
                $visibleRows = $pageControls.find(".visible-rows"),
                $filterCount = $pageControls.find(".filter-count");

            var $prevBut = $pageControls.find(".page-table-prev")
                    .on("click", function() {
                        if(currentPage-1) {
                            --currentPage;
                            showCurrentPage();
                        }
                    });
            var $nextBut = $pageControls.find(".page-table-next")
                    .on("click", function() {
                        if(currentPage<totalPages) {
                            ++currentPage;
                            showCurrentPage();
                        }
                    });


            if($pageControls.find("[value=" + perPage + "]").length) {
                $pageControls.find("[value=" + perPage + "]").prop("selected", true);
            } else {
                $pageControls.children().first().prop("selected", true);
            }

            $pageControls
                .find("select")
                .on("change", function(e) {
                    $table.trigger("beforeperpagechange");
                    currentPage = 1;
                    perPage = parseInt($(this).val(), 10);
                    totalPages = Math.ceil(rows / perPage);
                    showCurrentPage();
                    $table.trigger("afterperpagechange");
                })
                .trigger("change");

            $table.data("pageTable", true);

            function showCurrentPage() {
                $table.trigger("beforepagechange");

                var $totalRows = $tbody.find("tr"),
                    $shownRows = $totalRows.not("[class*=filtered-]"),
                    total = $totalRows.length,
                    rows = $shownRows.length;

                totalPages = Math.ceil(rows / perPage);

                var showStart = (currentPage-1) * perPage,
                    showEnd = showStart + (perPage);

                $shownRows.addClass("paged-out");
                $shownRows.slice(showStart, showEnd).removeClass("paged-out");
                $currentPage.text(currentPage);
                $totalPages.text(totalPages);
                $pageFirstRow.text((currentPage-1)*perPage+1);
                $pageLastRow.text(currentPage === totalPages ? rows : (currentPage*perPage));
                $visibleRows.text(rows);
                $filterCount.toggle((rows < total));                

                $nextBut.prop("disabled", (currentPage === totalPages));
                $prevBut.prop("disabled", (currentPage === 1));
                
                $table.trigger("afterpagechange");

                if($table.data("scrollTable")) {
                    $table.lockRows();
                }
            }

            function goTo(page) {
                currentPage = 1;
                showCurrentPage();
            }

            $table.on("aftertablesort", function(e, $header, dir) {
                goTo(1);
            });

            $table.on("aftercolumnfilter", function(e, $header, dir) {
                goTo(1);
            });
        });
    };

})(jQuery);