(function($, undefined) {

    $.widget("phd.popover", {
        options: {
            anchor: null,
            appendTo: document.body,
            popoverClass: '',
            position: {
                my: "center top+5",
                at: "center bottom"
            },
            zIndex: 2000,
            autoOpen: false,
            destroyOnClose: false,
            // callbacks
            open: null,
            close: null
        },

        _create: function() {
            var self = this,
                options = self.options,
                defaultClass = 'ui-popover',
                popoverId = this.element.uniqueId().attr("id");

            this.anchor = options.anchor || this.element.prev() || this.element.parent();

            this.container = $('<div/>', {
                    "id": popoverId + "-container",
                    "class": defaultClass + " " + options.popoverClass,
                })
                .css({ zIndex: options.zIndex })
                .hide()
                .appendTo(options.appendTo);

            this.element
                .addClass(defaultClass + "-content")
                .appendTo(this.container)
                .show()
                .data('popover', this);

            this.arrow = $('<div/>', { "class": "ui-popover-arrow" })
                .on("click", function() {self.close()})
                .appendTo(this.container);

            this._id = popoverId;
            this._containerId = popoverId + "-container";

            this._isOpen = false;

            if(!this.options.position.hasOwnProperty("of")) {
                this.options.position.of = this.anchor;
            }

            this.anchor
                .data('popover', this)
                .on('popopen', function(event) {
                    if(!$(this).hasClass('ui-state-disabled')) {
                        self.open();
                    }
                })
                .on('click.' + self._id, function(event) {
                    if(!$(this).hasClass('ui-state-disabled')) {
                        event.stopPropagation();
                        self.toggle();
                        return false;
                    }
                })
                .on("keydown." + self._id, function(event) {
                    if(event.keyCode === $.ui.keyCode.ESCAPE) {
                        event.stopPropagation();
                        self.close();
                    }
                });

            $(document).on("popoveropen", function(event, element) {
                if(self._isOpen) {{
                    self.close();
                }}
            });

            if(options.autoOpen) {
                this.open();
            }

        },

        widget: function() {
            return this.container;
        },

        isOpen: function() {
            return this._isOpen;
        },

        toggle: function() {
            if(this._isOpen) {
                this.close();
            } else {
                this.open();
            }
        },

        _clickOff: function(event) {
            if($(event.target).closest("#" + this._containerId).length === 0 &&
                $(event.target).closest(this.anchor).length === 0 ) {
                if(this.options.destroyOnClose) {
                    this.destroy();
                } else {
                    this.close();
                }
            }
        },

        position: function(position) {
            this.container.position(position);
        },

        open: function() {
            if(this._isOpen) {
                return;
            }

            $(document).trigger("popoveropen", [this.element]);

            if($.isFunction(this.options.open)) {
                this.options.open(this);
            }

            var self = this;

            this.container
                .show()
                .position(this.options.position);
            
            this.anchor.addClass('ui-state-active').focus();

            $(document).on('click.' + this._id, function(event) { self._clickOff(event) });

            this._isOpen = true;
            //this.container.trigger('popoveropen');
        },

        close: function() {
            if(!this._isOpen) {
                return;
            }

            if($.isFunction(this.options.close)) {
                this.options.close(this);
            }

            this.container.hide();

            this.anchor.removeClass('ui-state-active');

            $(document).off('click.' + this._id);

            this._isOpen = false;
            //this.container.trigger('popoverclose');
        },
        
        destroy: function() {
            this.close();
            this.anchor.removeData('popover');
            this.element.removeData('popover');
        },

    });

}(jQuery));