(function($) {

    var scrollTableDefaults = {
        height: ".full-height",
        resize: true
    };

    var fullBlock = {"display": "block", "width": "100%"};

    $.fn.scrollTable = function(options) {
        return this.each(function() {
            var $table = $(this).addClass("scroll-table"),
                $thead = $table.children("thead"),
                $tbody = $table.children("tbody"),
                $tfoot = $table.children("tfoot"),
                $columns = $thead.children("tr:last-child").children("th"),
                $placeholder = $table.data("placeholder") || $tbody.children(".placeholder"),
                numcols = $columns.length,
                tableheight, tbodyheight, widths;

            // Get the maximum height and column widths
            if($table.data("scrollTable") === void 0) {
                // first call
                options = $.extend({}, scrollTableDefaults, options);
                widths = $thead.find("tr:last-child th").map(function(){return $(this).outerWidth();}).get();
                if(options.resize) {
                    $(document).on("navshift", function() { $table.adjustTableWidth(); });
                    $(window).on("resize", debounce(handleTableResize));
                }
            } else {
                // subsequent calls
                if(options === void 0) {
                    options = $table.data("options");
                } else {
                    options = $.extend({}, $table.data("options"), options);
                }                
                widths = resetTable();
            }

            tableheight = $.isNumeric(options.height) ? options.height : $table.getAvailableVerticalSpace(options.height);
            tbodyheight = tableheight - $thead.outerHeight() - $tfoot.outerHeight();
            styleTable();

            $table
                .data("options", options)
                .data("widths", widths)
                .data("scrollTable", true);

            function styleTable() {
                if($table.children("colgroup").length) {
                    $table.children("colgroup").remove();
                }

                // Set the widths of each column in the thead, tbody and tfoot
                $thead.children("tr:last-child").children("th").each(function(index) {
                    $(this).css("width", widths[index]);
                    $(this).css("max-width", widths[index]);
                    if(index != (numcols - 1) || $table.data("columnResize")) {
                        $(this).css("min-width", widths[index]);
                    }
                });

                $tbody.children("tr:visible:not(.placeholder)").children("td").each(function(index) {
                    $(this).css("width", widths[index % numcols]);
                    $(this).css("max-width", widths[index % numcols]);
                    if((index % numcols) != (numcols - 1) || $table.data("columnResize")) {
                        $(this).css("min-width", widths[index % numcols]);
                    }
                });

                $tfoot.children("tr:visible").first().children("th").each(function(index) {
                    var w = widths[index],
                        span = $(this).attr("colspan");
                    if(span) {
                        var spanwidth = sumArray(widths.slice(index, index+span)),
                            padwidth = (this.offsetWidth - $(this).width()) * (span - 1);

                        w = spanwidth + padwidth;
                    }
                    $(this).css("width", w);
                    $(this).css("max-width", w);
                    if(index != (numcols - 1) || $table.data("columnResize")) {
                        $(this).css("min-width", w);
                    }
                });

                // Apply the CSS needed to make the table scroll
                $thead.css({
                    "display": "block"
                });

                $tbody.css({
                    "height": tbodyheight,
                    "max-height": tbodyheight,
                    "display": "block",
                    "overflow-x": "hidden",
                    "overflow-y": "auto"
                });

                $tfoot.css({
                    "display": "block"
                });

                if($placeholder && $placeholder.length) {
                    $placeholder.css(fullBlock);
                    $placeholder.children().css(fullBlock);
                }

                $table.trigger("afterscrolltable");
            }

            // Remove applied styles from thead, tbody and tfoot and return 
            // an array of the natively-calculated widths
            function resetTable() {
                if(!$table.data("columnResize")) {
                    $thead
                        .removeAttr("style")
                        .find("tr:last-child td, tr:last-child th")
                            .removeAttr("style");

                    $tbody
                        .css("display", "")
                        .find("tr td, tr th")
                            .removeAttr("style");

                    $tfoot
                        .removeAttr("style")
                        .find("tr td, tr th")
                            .removeAttr("style");
                }

                return $thead.find("tr:last-child th").map(function(){return $(this).outerWidth();}).get();
            }

            function handleTableResize(event) {
                if($table.is(":visible")) {
                    var newheight = $.isNumeric(options.height) ? options.height : $table.getAvailableVerticalSpace(options.height)
                    $table.adjustTableHeight(newheight);
                    $table.adjustTableWidth();
                }
            }

        });
    };

    $.fn.adjustTableWidth = function(width) {
        return this.each(function() {
            var $table = $(this),
                $columns = $table.find("thead tr:last-child th"),
                numcols = $columns.length,
                tableWidth = $table.blockTableWidth(),
                width = (width === void 0) ? $table.parent().width() : width,
                widths = $table.columnWidths(),
                difference = width - tableWidth,
                eachdiff = parseInt(difference / numcols, 10),
                remainder = difference % numcols,
                newWidths = widths.map(function(value, index) {
                    if(index === (numcols-1)) {
                        return value + eachdiff + remainder;
                    } else {
                        return value + eachdiff;
                    }
                });

            if($table.data("columnResize")) {
                $columns.each(function(idx, el) {
                    $table.find("thead tr:last-child th:nth-child(" + (idx+1) + ")").css({
                        "width": newWidths[idx],
                        "max-width": newWidths[idx],
                        "min-width": newWidths[idx]
                    });
                    $table.find("tbody td:nth-child(" + (idx+1) + ")").css({
                        "width": newWidths[idx],
                        "max-width": newWidths[idx],
                        "min-width": newWidths[idx]
                    });
                    $table.find("tfoot tr:first-child th:nth-child(" + (idx+1) + ")").css({
                        "width": newWidths[idx],
                        "max-width": newWidths[idx],
                        "min-width": newWidths[idx]
                    });
                });

                $table.columnResize();
            } else if($table.data("scrollTable")) {
                $table.scrollTable();
            }
        });
    };

    $.fn.adjustTableHeight = function(height) {
        //console.log("adjusting table height to " + height);
        if(!this.find(".placeholder:visible").length) {
            return this.each(function() {
                var $table = $(this),
                    $tbody = $table.find("tbody"),
                    $thead = $table.find("thead"),
                    $tfoot = $table.find("tfoot");
                    
                if(height < 0) {
                    height = $table.outerHeight() - (height * -1);
                }
                tbodyheight = height;
                tbodyheight -= $thead.outerHeight();
                tbodyheight -= $tfoot.outerHeight();
                $tbody
                    .css("display", "block")
                    .animate({"max-height": tbodyheight, "height": tbodyheight}, 250);
            });
        } else {
            return this;
        }
    };

    $.fn.offsetTableHeight = function(delta) {
        return this.each(function() {
            var $tbody = $(this).find("tbody"),
                tbodyheight = $tbody.height() + delta;

            $tbody
                .css("display", "block")
                .animate({"max-height": tbodyheight, "height": tbodyheight}, 250);
        });
    };

    $.fn.columnWidths = function() {
        return this.first().find("thead tr:last-child th").map(function(){return this.offsetWidth;}).get();
    };

    $.fn.columnSizes = function() {
        return this.first().find("thead tr:last-child th").map(function(){return $(this).width();}).get();
    };

    $.fn.blockTableWidth = function() {
        return sumArray(this.columnWidths());
    };

    $.fn.getAvailableVerticalSpace = function(fullHeightContainer) {
        var $table = this,
            $parent = $table.parent(),
            $container = this.closest(fullHeightContainer);

        if(!$container.length) {
            return $parent.outerHeight();
        }

        var containerHeight = $container.outerHeight(),
            parentExtraHeight = $parent.outerHeight() - $parent.outerHeight(),
            siblingHeight = 0;

        /*console.log("container height: " + containerHeight);
        console.log("parent extra height: " + parentExtraHeight);
        console.log("table height: " + this.height());
        console.log("table head height: " + this.children("thead").height());
        console.log("table foot height: " + this.children("tfoot").height());
        console.log("table body height: " + this.children("tbody").height());*/
        $container.children().each(function() {
            if(this != $table[0] && this != $parent[0] && $(this).is(":visible")) {
                siblingHeight += $(this).outerHeight(true);
            }
        });

        /*console.log("sibling height:  " + siblingHeight);
        console.log("available space is " + (containerHeight - siblingHeight));*/
        return containerHeight - siblingHeight - parentExtraHeight;
    };

    function sumArray(arr) {
        return arr.reduce(function(mem, val) { return mem + val; }, 0);
    }

    function debounce(func, wait, immediate) {
        var timeout, result;
        return function() {
            var context = this, args = arguments;
            var later = function() {
                timeout = null;
                if (!immediate) result = func.apply(context, args);
            };
            var callNow = immediate && !timeout;
            clearTimeout(timeout);
            timeout = setTimeout(later, wait || 100);
            if (callNow) result = func.apply(context, args);
            return result;
        };
    }

})(jQuery);
