(function($, PHD, window, undefined) {
    
    var $form = $("#advanced-settings-form"),
        $dialog,
        ConnectorForm,
        retentionPolicies,
        $retentionSelect = $("#id_orphan_retention"),
        explainTemplate = gettext($("#explanation-template").html()),
        explainTemplateFI = gettext($("#explanation-template-fi").html()),
        RETENTION_KEEP_ALL = 2;

    function initForm() {
        ConnectorForm = PHD.FormController($form, {})
            .validate({})
            .on("formcancel", function(event) {
                $dialog.wizard("close");
            })
            .on("formsubmit", function(event, data) {
                if(data.result[0].code === AJAX_RESULT_SUCCESS) {
                    var returnData = {
                        "connector_method": $form.find("[name=connector_method] option:selected").text(),
                        "orphan_retention": $retentionSelect.find(":selected").text()
                    };
                    $(document).trigger("advancedsettingssave", returnData);
                    $dialog.wizard("close");
                } else if(data.result[0].code === AJAX_RESULT_ERROR) {
                    PHD.throwError(data);
                }
            });

        $form.find(".btn-edit-retention")
            .on("click", launchRetentionDialog);

        $retentionSelect.on("change", updateRetentionExplanation).trigger("change");

        $(document).on("retentionupdate", handleRetentionUpdate);
    }

    function indexPolicies(data) {
        retentionPolicies = {};
        data.forEach(function(policy) {
            retentionPolicies[policy.state.id] = policy;
        });
    }

    function updateRetentionExplanation() {
        var dbid = parseInt($(this).val(), 10),
            policy = retentionPolicies[dbid];

        if(dbid === RETENTION_KEEP_ALL) {
            $form.find(".retention-explanation").html(gettext("No backups will be deleted by 'Keep All' retention."));
            $form.find(".retention-explanation-fi").html("");
        } else {
            $form.find(".retention-explanation").html(interpolate(explainTemplate, policy.data, true));
            $form.find(".retention-explanation-fi").html(interpolate(explainTemplateFI, policy.data, true));
        }
    }

    function handleRetentionUpdate(event, data) {
        indexPolicies(data.objects);
        var lastval = $retentionSelect.val(),
            options = makeRetentionOptions(data.objects);

        $form.find("[name$=orphan_retention]").html(options);
        $retentionSelect.html(options).val(lastval).trigger("change");
    }

    function makeRetentionOptions(data) {
        return data.reduce(function(memo, policy) {
            return memo + "<option value=" + policy.state.id + ">" + policy.data.name + "</option>";
        }, "");
    }

    function launchRetentionDialog(event) {
        event.preventDefault();
        $(document).trigger("launchdialog", {"action": "retention"});
    }

    /*
    *   DOM Ready Function
    */
    $(function() {
        PHD.currentDialog.one("wizardfocus", function() {
            $dialog = PHD.currentDialog;
            indexPolicies(policy_data);
            initForm();
        });
    });

})(jQuery, window.PHD || (window.PHD = {}), window);
