(function($, PHD, window, undefined) {

    var $thisDialog = $("#dialog-support-files"),
        csrf = $thisDialog.find("[name=csrfmiddlewaretoken]").val(),
        $btnDownload = $thisDialog.find(".btn-supportfile-download"),
        $btnSend = $thisDialog.find(".btn-supportfile-send"),
        $sendContainer = $btnSend.parent(),
        $requestTable = $thisDialog.find("#support-request-table"),
        $btnClose = $thisDialog.find("#support-close"),
        $sendForm = $thisDialog.find("#supportfile-send-form"),
        requestTable, updateLoop;

    var cellFormatter = {
        supportfile_status: function(value, row) {
            if(row.data.message !== null) {
                var icon = '<i class="icon-notes icon-large"></i>',
                    link = '<a href="#" class="btn-view-message" title="View Message">' + icon + " " + value + '</a>';
                return link;
            }
            return value;
        },
        date_requested: function(value) {
            return moment(value).format("MM/DD/YYYY hh:mm A");
        },

        filename: function(value, row) {
            if(row.state.supportfile_type === SUPPORTFILE_TYPE_DOWNLOAD && 
                row.state.download_status === SUPPORTFILE_STATUS_READY) {
                var icon = '<i class="icon-downloadalt icon-large"></i>',
                    link = '<a href="/supportfiles/' + value + '">' + icon + " " + value + '</a>';
                return (value === EMPTY_CELL_TEXT) ? value : link;
            }
            return value;
        },
        incident_number: function(value) {
            var icon = '<i class="icon-share icon-large"></i>',
                link = '<a target="phdvbhelp" href="https://phdtechsupport.zendesk.com/tickets/' + value + '">' + icon + " " + value + '</a>';
            return (value === EMPTY_CELL_TEXT) ? value : link;
        },
        supportfile_actions: function(value, row) {
            if(row.state.supportfile_type === SUPPORTFILE_TYPE_DOWNLOAD && 
                row.state.download_status === SUPPORTFILE_STATUS_READY) {
                return '<a href="/supportfile/delete/" class="btn-delete-supportfile"><i class="icon-delete icon-large"></i></a>';    
            }
            return "";
        },
    };

    var sortdate = function(a,b) {
        x = new Date(a.value);
        y = new Date(b.value);
        return (x < y) ?
            -1 : (x > y) ?
                1 : 0;
    };

    function updateSupportFiles(init) {
        var init = (init === true),
            resp = PHD.Ajax.get("/supportfile/requests/");

        resp.done(function(data) {
            if(init) {
                requestTable.load(data.objects);
            } else {
                requestTable.update(data.objects);
            }
            $requestTable.scrollTable({"height": 200, "resize": false});
            if(data.pending > 0 && $thisDialog.is(":visible")) {
                updateLoop = setTimeout(updateSupportFiles, 10000);
            }
        });
    }

    function showMessage($link, $row) {
        var dbid = $row.data("dbid"),
            $message = $("<div>"),
            message = requestTable.getData(dbid, "message");

        $message.html("<p>" + message + "</p>")
            .popover({
                anchor: $link,
                autoOpen: true,
                popoverClass: 'supportfile-message',
                destroyOnClose: true,
                position: {
                    my: "center bottom",
                    at: "center top-20"
                }
            });
    }

    $(function() {
        requestTable = PHD.DataTable($requestTable, {
            select: false,
            placeholder: gettext("You have no support requests")
        }).registerFormatters(cellFormatter);
        updateSupportFiles(true);

        $requestTable.on("loadfinished updatefinished", function(event, newRows) {
            if(newRows) {
                $requestTable.sort(1,sortdate,'desc');
            }
        });
    });

    $requestTable
        .on("click", ".btn-delete-supportfile", function(e) {
            e.preventDefault();
            var $row = $(this).closest("tr"),
                dbid = $row.data("dbid"),
                resp = PHD.Ajax.post("/supportfile/delete/", {
                    csrfmiddlewaretoken: csrf,
                    id: dbid
                });

            resp.done(function(data) {
                requestTable.update(data.objects);
                if(data.pending > 0 && $thisDialog.is(":visible")) {
                    updateLoop = setTimeout(updateSupportFiles, 10000);
                }
            });
        })
        .on("click", ".btn-view-message", function(e) {
            e.preventDefault();
            e.stopPropagation();
            showMessage($(this), $(this).closest("tr"));
        });

    $btnDownload.on("click", function(e) {
        var resp = PHD.Ajax.post("/supportfile/download/", {
            csrfmiddlewaretoken: csrf
        });

        resp.done(function(data) {
            requestTable.update(data.objects);
            if(data.pending > 0 && $thisDialog.is(":visible")) {
                updateLoop = setTimeout(updateSupportFiles, 10000);
            }
        });
    });

    $btnSend.on("click", function(e) {
        $(this).parent()
            .hide({
                effect: "fade", 
                duration: "fast", 
                complete: function() {
                    $sendForm.show({effect: "slide", direction: "left", duration: "fast"});
                    enableSendForm();
                }
            });
    });

    $btnClose.on("click", function() {
        clearTimeout(updateLoop);
        $thisDialog.wizard("close");
    });

    function enableSendForm() {
        $sendForm
            .on("click.sendcancel", ".btn-supportfile-send-form-cancel", function() {
                disableSendForm();
            })
            .on("submit", function(e) {
                e.preventDefault();

                var url = $sendForm.attr("action");

                if($sendForm.valid()) {
                    var resp = PHD.Ajax.post(url, $sendForm.serialize());

                    resp.done(function(data, textStatus, jqXHR) {
                        if(data.result[0].code === AJAX_RESULT_SUCCESS) {
                            updateSupportFiles();
                            disableSendForm();
                        } else if(data.result[0].code === AJAX_RESULT_ERROR) {
                            PHD.throwError(data);
                        }
                    });
                }
            });
    }

    function disableSendForm() {
        $sendForm
            .off("click.sendcancel")
            .off("submit")
            .hide({
                effect: "fade", 
                duration: "fast", 
                complete: function() {
                    $sendContainer.show({effect: "slide", direction: "left", duration: "fast"});
                }
            });
    }

    $sendForm
        .validate({
            errorLabelContainer: "#dialog-error-list",
            errorContainer: "#dialog-errors",
            showErrors: function(errorMap, errorList) {
                $("#dialog-errors").find(".summary")
                    .html(gettext("Correct the following errors:"));
                this.defaultShowErrors();
            },
            rules: {
                customername: {
                    required: true,
                },
                incidentnumber: {
                    required: true,
                    digits: true
                }
            },
            messages: {
                customername: {
                    required: gettext("Your name is required")
                },
                incidentnumber: {
                    required: gettext("Please enter a valid incident number"),
                    digits: gettext("Please enter a valid incident number")
                }
            }
        });

})(jQuery, window.PHD || (window.PHD = {}), window);