(function($, PHD, window, undefined) {

    var DETAIL_TABLE_HEIGHT = 250,
        $filter = $("#sync-filter form"),
        pageTableLegend = gettext("Showing %(start_index)s - %(end_index)s of %(count)s tasks"),
        pageTableLabel = gettext("Page %(page_number)s of %(page_count)s"),
        tab;

    function resizeTable(event) {
        this.adjustTableHeight(this.getAvailableVerticalSpace(".full-height"));
        this.adjustTableWidth();
    }

    var SyncExplorerTab = function(index, panel, slug) {
        this.index = index;
        this.$panel = panel;
        this.$buttons = this.$panel.find(".button-bar");
        this.$btnExport = this.$buttons.find(".btn-export-csv");
        this.slug = slug;
        this.namespace = "syncexplorer" + this.slug;

        this.$table = this.$panel.find("#sync-explorer-table");

        this.SyncTable = PHD.DataTable(this.$table, {
            select: false,
            placeholder: gettext("There are no sync tasks that match your filter"),
            sortable: true
        }).registerFormatters(cellFormatter);

        this.sort = '-backup_date';
        this.params = "";
        this.url = "/sync/explore/data/";

        // Pager details
        this.$pager = this.$table.find(".page-table-row");
        this.Pager = new TablePager(this.$pager, {}).init();
    };

    SyncExplorerTab.prototype.init = function() {
        this.initButtons();

        $filter.on("syncexplorerfilter", $.proxy(this.handleFilter, this));

        // Handle events thrown by DataTable
        this.$table
            .on("click", "th.sortable", $.proxy(this.handleSort, this))
            .on("rowdelete", $.proxy(this.handleRowDelete, this));

        this.$pager
            .on("pagernext", $.proxy(this.handlePager, this))
            .on("pagerprev", $.proxy(this.handlePager, this));

        $(window)
            .off("resize." + this.namespace)
            .on("resize." + this.namespace, PHD.util.debounce($.proxy(resizeTable, this.$table)));

        return this.initFilter();
    };

    SyncExplorerTab.prototype.getUpdateUrl = function(csv) {
        var url = this.url;
        if(csv) {
            url += "?csv=1";
        } else {
            url += "?page=" + this.Pager.page();
        }
        if(this.params) url += "&" + this.params;
        url += "&sort=" + this.sort;
        return url;
    };

    SyncExplorerTab.prototype.update = function() {
        var self = this,
            url = this.getUpdateUrl(),
            resp = PHD.Ajax.get(url);

        resp.done(function(data) {
            self.setTableSortHeader();
            self.SyncTable.load(data.objects);
            self.Pager.update(data.pager);
            self.$table
                .scrollTable();
            self.updateButtons(this.$btnExport);
        });

        return this;
    };

    SyncExplorerTab.prototype.handlePager = function(event) {
        this.update();
    };

    SyncExplorerTab.prototype.handleRowDelete = function(event, dbid) {

    };

    SyncExplorerTab.prototype.handleSort = function(event) {
        var $column = $(event.target).closest("th"),
            sortBy = $column.data("sort");

        if(this.sort === sortBy) {
            this.sort = "-" + this.sort;
        } else if(this.sort === ("-" + sortBy)) {
            this.sort = this.sort.substr(1);
        } else {
            this.sort = sortBy;
        }
        this.update();
    };

    SyncExplorerTab.prototype.handleFilter = function(event, params) {
        this.params = params;
        this.update();
    };

    SyncExplorerTab.prototype.setTableSortHeader = function() {
        var sortDirection = "asc",
            sortBy = this.sort;
        if(sortBy.substr(0,1) === "-") {
            sortDirection = "desc";
            sortBy = sortBy.substr(1);
        }
        var $headers = this.$table.find("th").removeClass("sort-asc sort-desc");
        $headers.filter("th[data-sort=" + sortBy + "]").addClass("sort-" + sortDirection);
    };

    SyncExplorerTab.prototype.initButtons = function() {
        var tab = this;
        this.$buttons.find("a").each(function(index) {
            var $but = $(this);
            PHD.buttonLogic($but, null, tab);
        });
    };

    SyncExplorerTab.prototype.updateButtons = function($button) {
        if($button !== void 0) {
            PHD.buttonLogic($button, null, this);
            return this;
        }

        var tab = this;
        this.$buttons.find("a").each(function(index) {
            PHD.buttonLogic($(this), null, tab);
        });
    };

    SyncExplorerTab.prototype.initFilter = function() {
        this.filter = {};
        this.filter.fields = $filter.find("input, select").map(function() {return $(this).attr("name");}).get();

        var self = this,
            params = getURLParams(window.location.search),
            submit = false;

        if(params) {
            _.forEach(params, function(value, key) {
                var $input = $filter.find('[name=' + key + ']');
                if($input.length) {
                    $input.val(value);
                    submit = true;
                    self.params += makeURLParams(key, value);
                }
            });
        }
        if(submit) {
            //console.log(this.params);
        }
        return this;
    };

    SyncExplorerTab.prototype.showFilter = function() {
        $filter.dialog({
            title: gettext("Filter Sync History"),
            dialogClass: "sync-fitler-dialog explore-filter-dialog",
            draggable: true,
            resizable: false,
            modal: false,
            width: 444,
            height: "auto"
        });
    };

    function getURLParams(search) {
        if(!search) return null;
        var output = {},
            params = search.slice(1).split('&');

        params.forEach(function(str) {
            var kv = str.split('=', 2),
                key = kv[0],
                value = kv[1];

            if(value.startsWith('[') && value.endsWith(']')) {
                value = value.slice(1,-1).split(',');
            }

            output[key] = value;
        });

        return output;
    }

    function makeURLParams(key, value) {
        if(_.isArray(value)) {
            return "&" + value.map(function(val) { return key + "=" + val; }).join("&");
        }
        return "&" + key + "=" + value.toString();
    }

    var pagerControlTmpl = _.template(
        '<span class="page-table-legend">' +
            '<%= pageTableLegend %>' +
        '</span>' +
        '<nav class="page-table-nav">' +
            '<button <% if(!has_prev) { %>disabled<% } %> title="Previous Page" type="button" data-page="<%= page_number - 1 %>" class="btn-page-prev">Prev</button>' +
            '<span class="page-table-label">' +
                '<%= pageTableLabel %>' +
            '</span>' +
            '<button <% if(!has_next) { %>disabled<% } %> title="Next Page" type="button" data-page="<%= page_number + 1 %>" class="btn-page-next">Next</button>' +
        '</nav>'
    );

    var TablePager = function($row, pager) {
        this.$row = $row;
        this.pager = pager || {};

        this.$controls = $row.find(".page-table-controls");
        this.$btnNext = $row.find(".btn-page-next");
        this.$btnPrev = $row.find(".btn-page-prev");
        this.$legendStart = $row.find(".start-index");
        this.$legendEnd = $row.find(".end-index");
        this.$legendTotal = $row.find(".count");
        this.$pageNumber = $row.find(".page-number");
        this.$pageCount = $row.find(".page-count");
    };

    TablePager.prototype.init = function() {
        var self = this;

        this.$controls.on("click", ".btn-page-next", function() {
            ++self.pager.page_number;
            self.$row.trigger("pagernext");
        });

        this.$controls.on("click", ".btn-page-prev", function() {
            --self.pager.page_number;
            self.$row.trigger("pagerprev");
        });

        return this;
    };

    TablePager.prototype.update = function(pager) {
        this.pager = pager;
        this.pager.pageTableLegend = interpolate(pageTableLegend, pager, true);
        this.pager.pageTableLabel = interpolate(pageTableLabel, pager, true);
        this.$controls.html(pagerControlTmpl(this.pager));
    };

    TablePager.prototype.page = function() {
        return this.pager.page_number || 1;
    };

    $("#sync-filter-submit").on("click", function() {
        $filter.trigger("syncexplorerfilter", $filter.serialize());
        $filter.dialog("close");
    });

    $filter.find(".has-datepicker").datepicker({
        showOn: "both",
        buttonText: '<img src="' + STATIC_URL + 'images/calendar.png" width="24" height="24">',
        dateFormat: DATE_FORMAT
    }).attr("placeholder", DATE_DISPLAY_FORMAT);

    /*
    *   DOM Ready Function
    */
    $(function() {
        tab = new SyncExplorerTab(0, $("#sync-explorer-panel") , "default").init();
        tab.update();
    });

    // Object to convert raw values to display-friendly values
    var cellFormatter = {
        name: function(value, row) {
            return '<span class="truncates item-name" title="' + value + '">' + value + '</span>';
        },
        backup_date: function(value, row) {
            if(value && value !== EMPTY_CELL_TEXT) {
                return moment(value).zone(tz).format("M/D/YYYY h:mmA"); 
            }
            return value;
        },
        started: function(value, row) {
            if(value && value !== EMPTY_CELL_TEXT) {
                return moment.unix(value).zone(tz).format(DATE_DISPLAY_FORMAT + " h:mmA"); 
            }
            return value;
        },
        ended: function(value, row) {
            if(value && value !== EMPTY_CELL_TEXT) {
                return moment.unix(value).zone(tz).format(DATE_DISPLAY_FORMAT + " h:mmA"); 
            }
            return value;
        }
    };

})(jQuery, window.PHD || (window.PHD = {}), window);
