(function($, PHD, window, undefined) {

    var steps = ["account", "hypervisor", "storage"],
        status_progress = [QUICKSTART_ACCOUNT, QUICKSTART_HYPERVISOR, QUICKSTART_STORAGE, QUICKSTART_COMPLETE],
        ping = [false, true, true, false],
        host = window.location.hostname,
        ready = $.Deferred(),
        fetchSync = false,
        interfaceURL = "/",
        $buttons = $(".btn"),
        $load;

    var goTo = function(path) {
        window.location.replace("http://" + host + path);
    };

    var Quickstart = {
        next: function() {
            var self = this;
            this.updateStatus(status_progress[progress])
                .done(function() {
                    var nextStep = steps[++progress];

                    if(scenario === SCENARIO_SMALL) {
                        if(nextStep) {
                            window.location.replace("/" + steps[progress] + "/");
                        } else {
                            self.finish();
                        }
                    } else if(nextStep === "storage") {
                        window.location.replace("/summary/" + scenario + "/");
                    } else {
                        window.location.replace("/" + steps[progress] + "/");
                    }
                });
        },
        prev: function() {
            progress--;
            window.location.replace("/" + steps[progress] + "/");
        },
        finish: function() {
            var self = this;

            if($load && $load.length) PHD.hideLoadingIndicator($load);

            $load = PHD.showLoadingIndicator("body", true, gettext("Loading Unitrends Virtual Backup"));
            $buttons.prop("disabled", true);
            this.updateStatus(QUICKSTART_COMPLETE)
                .done(function() {
                    PHD.Ajax
                        .get("/qs-finish/")
                        .done(function(data) {
                            if(data.result[0].code === AJAX_RESULT_SUCCESS) {
                                interfaceURL = data.result[0].data.url;
                                setTimeout(self.goToInterface, 5000);
                            } else {
                                PHD.hideLoadingIndicator($load);
                                $buttons.prop("disabled", false);
                                PHD.throwError(data);
                            }
                        });
                });
        },
        goToInterface: function() {
            goTo(interfaceURL);
        },
        goToDashboard: function() {
            goTo("/");
        },
        goToConfig: function() {
            goTo("/config/");
        },
        reload: function() {
            window.location.replace("http://" + host + "/");
        },
        skip: function(fn) {
            fn = fn || $.proxy(this.finish, this);
            $buttons.prop("disabled", true);
            var message = gettext("Are you sure you want to exit Quick Setup? You will need to finish the configuration manually using the web interface before you can begin using UVB."),
                confirm = PHD.confirmDialog(
                    "<p>"+message+"</p>",
                    { title: gettext("Exit Quick Setup")}
                );
            
            confirm.done(fn).fail(function() {$buttons.prop("disabled", false);});
        },
        updateStatus: function(progress) {
            return PHD.Ajax.get('/qs-status/progress/' + progress + '/');
        },
        resetStatus: function(flag) {
            PHD.Ajax
                .get('/qs-status/reset/' +  flag + '/')
                .done(function(data) {
                    if(!data.result) {
                        console.log("Error resetting", flag);
                    } else {
                        console.log(data.result);
                    }
                });
        }

    };

    function pingStatus() {
        var url = "/qs-status/";
        if(fetchSync === true) url += "?sync=1";

        PHD.Ajax
            .get(url)
            .done(function(data) {
                pingFunctions[progress](data.result.data, data.sync);
            });
    }

    function pingUntilReady() {
        pingStatus();

        ready
            .done(function() {
                Quickstart.next();
            })
            .fail(function(data, flag) {
                //console.log("Error", data);
                PHD.hideLoadingIndicator($load);
                $buttons.prop("disabled", false);
                Quickstart.resetStatus(flag);
                PHD.throwError({"result": [{"message":data.message}]});
            });
    }

    function initializeQuickStart() {
        var $form = $(".quickstart-form");

        if($form.length) {
            $form.validate(validators[progress]);

            $form.on("keydown", "input, select", function(event) {
                event.stopPropagation();
                if(event.keyCode == $.ui.keyCode.ENTER) {
                    event.preventDefault();
                    $(".btn-next").trigger("click");
                }
            });

            $(".btn-next").on("click", function() {
                if($form.valid()) {
                    var message = '<span class="loading-message">' + loadingMessages[progress] + "</span>";

                    $load = PHD.showLoadingIndicator("body", true, message);
                    $buttons.prop("disabled", true);

                    PHD.Ajax
                        .post($form.attr("action"), $form.serialize())
                        .done(function(data) {
                            if(data.result[0].code === AJAX_RESULT_SUCCESS) {
                                if(ping[progress]) {
                                    pingUntilReady();
                                } else {
                                    Quickstart.next();
                                }
                            } else {
                                PHD.hideLoadingIndicator($load);
                                $buttons.prop("disabled", false);
                                PHD.throwError(data);
                            }
                        })
                        .fail(function() {
                            PHD.hideLoadingIndicator($load);
                            $buttons.prop("disabled", false);
                        });
                }
            });

            $(".btn-next-exit").on("click", function() {if($form.valid()) {
                    var message = "<span>" + loadingMessages[progress] + "</span>";

                    $load = PHD.showLoadingIndicator("body", true, message);
                    $buttons.prop("disabled", true);

                    PHD.Ajax
                        .post($form.attr("action"), $form.serialize())
                        .done(function(data) {
                            if(data.result[0].code === AJAX_RESULT_SUCCESS) {
                                Quickstart.finish();
                            } else {
                                PHD.hideLoadingIndicator($load);
                                $buttons.prop("disabled", false);
                                PHD.throwError(data);
                            }
                        })
                        .fail(function() {
                            PHD.hideLoadingIndicator($load);
                            $buttons.prop("disabled", false);
                        });
                }
            });

        } else {
            $(".btn-next").on("click", function() {
                Quickstart.next();
            });
            $(".btn-next-exit").on("click", function() {
                Quickstart.finish();
            });
        }

        // Not Implemented
        $(".btn-prev").on("click", function() {
            Quickstart.prev();
        });

        $(".btn-skip").on("click", function() {
            Quickstart.skip();
        });

        if(initializers[progress]) {
            initializers[progress]($form);
        } else if($("body").is(".summary")) {
            initSummary();
        }
    }

    var pingHypervisor = function(data, sync) {
        if($load.length && sync) {
            $load.find(".loading-message").text("Initializing UVB Management Appliance (" + sync.data.progress + "%)");
        }
        
        if(data.mgmt_sync === true && data.inventory_sync === true && data.eng_sync === true) {
            ready.resolve();
        } else if(data.mgmt_sync === false) {
            ready.reject(data, "mgmt_sync");
        } else if(data.inventory_sync === false) {
            ready.reject(data, "inventory_sync");
        } else if(data.eng_sync === false) {
            ready.reject(data, 'eng_sync');
        } else {
            setTimeout(pingStatus, 2500);
        }
    };

    var pingStorage = function(data) {
        if(data.bds_sync === true && data.catalog_sync === true) {
            ready.resolve();
        } else if(data.bds_sync === false) {
            ready.reject(data, 'bds_sync');
        } else if(data.catalog_sync === false) {
            ready.reject(data, 'catalog_sync');
        } else {
            setTimeout(pingStatus, 2500);
        }
    };

    var pingFunctions = [null, pingHypervisor, pingStorage, null];

    var validators = [
        { // Account
            rules: {
                username: {
                    required: true,
                    maxlength: 30
                },
                group: {
                    required: true
                },
                password: {
                    required: true,
                    minlength: 6
                },
                password_confirm: {
                    required: true,
                    equalTo: "#id_password"
                },
                email: {
                    required: true,
                    email: true
                },
                eula: {
                    required: true
                }
            },
            messages: {
                username: {
                    required: gettext("Username is required"),
                    maxlength: gettext("Username must not exceed than 30 characters")
                },
                group: {
                    required: gettext("Company Name is required")
                },
                password: {
                    required: gettext("Password is required"),
                    minlength: gettext("Password must be at least 6 characters")
                },
                password_confirm: {
                    required: gettext("Your new password must be entered the same in both fields"),
                    equalTo: gettext("Your new password must be entered the same in both fields")
                },
                email: {
                    required: gettext("An email address is required")
                },
                eula: {
                    required: gettext("You must accept the End User License Agreement")
                }
            }
        },
        { // Hypervisor
            rules: {
                name: {
                    required: true,
                    maxlength: 100
                },
                server: {
                    required: true
                },
                username: {
                    required: true
                },
                password: {
                    required: true
                }
            },
            messages: {
                name: {
                    required: gettext("Friendly Name is required"),
                    maxlength: gettext("Friendly Name must not exceed 100 characters")
                },
                server: {
                    required: gettext("Server is required")
                },
                username: {
                    required: gettext("Username is required")
                },
                password: {
                    required: gettext("Password is required")
                }
            }
        },
        { // Storage
            rules: {
                name: {
                    required: true
                },
                disks: {
                    required: true
                }
            },
            messages: {
                name: {
                    required: gettext("Please enter a name for your Backup Data Store")
                },
                disks: {
                    required: gettext("Please select at least one Disk")
                }
            }
        }
    ];

    var loadingMessages = [
        gettext("Initializing UVB Presentation Appliance..."),
        gettext("Initializing UVB Management Appliance..."),
        gettext("Initializing UVB Engine Appliance..."),
        gettext("Welcome to Unitrends Virtual v7")
    ];

    var initAccount = function($form) {
        $(".btn-view-eula").on("click", function(event) {
            event.preventDefault();
            event.stopPropagation();
            PHD.popOut('/eula/', gettext("Unitrends Virtual Backup End User License Agreement"), 'eula', 640, 600);
            /*PHD.wizard('/eula/', 'eula', {
                title: gettext("Unitrends Virtual Backup End User License Agreement"),
                width: 600,
                height: 700
            });*/
        });
    };

    var initHypervisor = function($form) {
        //console.log("Initializing Hypervisor");
        //console.log(pingFunctions[progress]);
        $form.find(".definition").popover({
            position: {
                my: "left top",
                at: "right+20 top"
            }
        });

        fetchSync = true;
    };

    var initStorage = function($form) {
        //console.log("Initializing Storage");
        var $refresh = $(".btn-refresh-disks"),
            $table = $(".available-disks"),
            $placeholder = $table.find(".placeholder"),
            rowTmpl = _.template(
                '<tr class="clickable" data-dbid="<%= data.name %>">' +
                    '<td><input type="checkbox" value="<%= data.name %>" name="disks"></td>' +
                    '<td><%= data.name %></td>' +
                    '<td><%= data.size %></td>' +
                '</tr>'
            );

        var refreshStorage = function(e) {
            $refresh.off("click").addClass("disabled");

            PHD.Ajax
                .get("/qs-disks/")
                .done(function(data) {
                    $refresh.on("click", refreshStorage).removeClass("disabled");

                    if(data.result.length) {
                        $placeholder.detach();
                        var names = _.pluck(_.pluck(data.result, "data"), "name"),
                            $disks = $table.find("tbody tr:not(.placeholder)");

                        $disks.each(function() {
                            if(!_.contains(names, $(this).data("dbid"))) {
                                $(this).remove();
                            }
                        });

                        data.result.forEach(function(value, index, arr) {
                            var uid = value.data.name;
                            if(!$table.find("[data-dbid='" + uid + "']").length) {
                                var html = rowTmpl(value);
                                $(html).appendTo($table.children("tbody"));
                            }
                        });
                    } else {
                        $table.find("tbody tr:not(.placeholder)").remove();
                        $placeholder.appendTo($table.children("tbody"));
                    }
                })
                .always(function() {
                    $table.scrollTable({"height": 120, resize: false});
                });
        };

        $refresh.on("click", refreshStorage);
        $table.scrollTable({"height": 120, resize: false});
    };

    var initSummary = function() {
        $(".btn-complete").on("click", function() {
            Quickstart.finish();
        });
    };

    var initializers = [initAccount, initHypervisor, initStorage, initSummary];
    
    $(function() {
        $buttons.prop("disabled", false);
        if(void 0 !== progress && progress >= 0) {
            initializeQuickStart();    
        } else {
            // Login page
            // Set up event handlers for fancy text inputs
            $('body')
                .on('keydown paste change', ".control-group.fancy input", function() { 
                    $(this).each(function() { PHD.toggleLabel.call(this, 20); });
                })
                .on('focusin', ".control-group.fancy input", function() {
                    $(this).prev('span').animate({ opacity: .25 }, { duration: 350 });
                })
                .on('focusout', ".control-group.fancy input", function() {
                    $(this).prev('span').animate({ opacity: 1 }, { duration: 350 });
                });
        }
        
    });

})(jQuery, window.PHD || (window.PHD = {}), window);
