var BCData, BCView,
    vtmpl, btmpl, dtmpl, datetmpl, bdstmpl,
    options, checked,
    output = "", 
    dupes = [], environments = [], hosts = [], types = [],
    BACKUP_CATALOG_VIEW_VM = 0,
    BACKUP_CATALOG_VIEW_DATE = 1,
    BACKUP_CATALOG_VIEW_ROLLBACK = 2,
    BACKUP_CATALOG_VIEW_BDS = 3,
    BACKUP_CATALOG_VIEW_APPLIANCE = 4,
    BACKUP_STATUS_WARNING = 1,
    BACKUP_STATUS_FAILURE = 2,

    BACKUP_MODE_FULL_AND_INC = 2,
    BACKUP_MODE_FULL_AND_INC_FULL = 3,
    BACKUP_MODE_VIRTUAL_FULL = 1,
    MODE_SUBTYPE_FULL = 0,
    MODE_SUBTYPE_INC = 1,
    BDS_TYPE_BACKUP = (1 << 0),
    BDS_TYPE_REPLICATION = (1 << 1),
    BDS_TYPE_CLOUD_CACHE = (1 << 2),
    BDS_TYPE_IR_CACHE = (1 << 3),
    BDS_TYPE_ARCHIVE = (1 << 4),
    BDS_TYPE_SYNC = (1 << 5),
    BACKUP_SUBMODE_CHOICES = {},
    HYPERVISOR_NA = 0,
    HYPERVISOR_XEN = 1,
    HYPERVISOR_VMWARE = 3,
    HYPERVISOR_HYPERV = 4;

self.addEventListener('message', function(e) {
    importScripts('../../vendor/underscore/underscore-min.js');
    initTemplates();
    BCData = e.data.backups;
    BCView = e.data.options.view;
    options = e.data.options;
    checked = e.data.options.checked;

    TRANS_VIRTUAL_FULL = e.data.trans.vf;
    BACKUP_SUBMODE_CHOICES[MODE_SUBTYPE_FULL] = e.data.trans.full;//"Full";
    BACKUP_SUBMODE_CHOICES[MODE_SUBTYPE_INC] = e.data.trans.inc;//"Incremental";

    output = render(BCData, BCView);
    self.postMessage({
        "table_body": output, 
        "tree": BCData,
        "environments": _.uniq(environments),
        "hosts": _.uniq(hosts),
        "types": _.uniq(types)
    });
}, false);

function render(objects, view) {
    var html = "";
    
    if(!objects.length)
        return html;

    if(view === BACKUP_CATALOG_VIEW_DATE) {
        objects.forEach(function(date, dateIndex) {
            dupes = [];
            date = formatDate(date, dateIndex);
            html += date.html = datetmpl(date);
            date.instances.forEach(function(vm, vmIndex) {
                vm = formatVM(date, vm, 1, vmIndex);
                html += vm.html = vtmpl(vm);
                if(options.backups.show && vm.backups) {
                    vm.backups.forEach(function(backup, backupIndex) {
                        backup = formatBackup(vm, backup, 2, backupIndex);
                        html += backup.html = btmpl(backup);
                        if(options.disks.show && backup.disks) {
                            backup.disks.forEach(function(disk, diskIndex) {
                                disk = formatDisk(vm, backup, disk, 3, diskIndex);
                                html += disk.html = dtmpl(disk);
                            });
                        }
                    });
                }
            });
        });
    } else if(view === BACKUP_CATALOG_VIEW_BDS) {
        objects.forEach(function(bds, bdsIndex) {
            dupes = [];
            bds = formatBDS(bds, bdsIndex);
            html += bds.html = bdstmpl(bds);
            bds.vms.forEach(function(vm, vmIndex) {
                vm = formatVM(bds, vm, 1, vmIndex);
                html += vm.html = vtmpl(vm);
                if(options.backups.show && vm.backups) {
                    vm.backups.forEach(function(backup, backupIndex) {
                        backup = formatBackup(vm, backup, 2, backupIndex);
                        html += backup.html = btmpl(backup);
                        if(options.disks.show && backup.disks) {
                            backup.disks.forEach(function(disk, diskIndex) {
                                disk = formatDisk(vm, backup, disk, 3, diskIndex);
                                html += disk.html = dtmpl(disk);
                            });
                        }
                    });
                }
            });
        });
    } else {
        objects.forEach(function(vm, vmIndex) {
            vm = formatVM(null, vm, 0, vmIndex);
            html += vm.html = vtmpl(vm);
            if(options.backups.show && vm.backups) {
                vm.backups.forEach(function(backup, backupIndex) {
                    backup = formatBackup(vm, backup, 1, backupIndex);
                    html += backup.html = btmpl(backup);
                    if(options.disks.show && backup.disks) {
                        backup.disks.forEach(function(disk, diskIndex) {
                            disk = formatDisk(vm, backup, disk, 2, diskIndex);
                            html += disk.html = dtmpl(disk);
                        });
                    }
                });
            }
        });
    }
    return html;
}

function hasChecked(backups) {
    return _.find(backups, function(backup) {
        // lmc
        if (checked !== null && typeof checked.backups !== 'undefined') {
            return checked.backups.indexOf(backup.id) !== -1;
        } else {
            return false;
        }
    });
}

function formatBDS(bds, index) {
    bds.id = index;
    bds.type = "bds";
    bds.depth = 0;
    bds.visibility = "visible open";
    bds.expanded = "";
    bds.children = bds.vms;
    bds.filtered = [];

    bds.icon = getBackupIcon(bds.bds_type);
    bds.bds_class = getBackupClass(bds.bds_type);

    return bds;
}

function formatDate(date, index) {
    date.id = index;
    date.type = "date";
    date.depth = 0;
    date.visibility = "visible open";
    date.expanded = "";
    //date.children = date.vms;
    date.children = date.instances;
    date.filtered = [];
    return date;
}

function formatVM(parent, vm, depth) {
    if (vm.database_name !== "") {
        vm.id = vm.instance_id;
        if (vm.id === '') {
            vm.id = vm.instance_name;
        }
        vm.type = "vm";
        vm.name = vm.database_name;
        if (vm.app_name.indexOf('VMware') !== -1) {
            vm.host = vm.instance_name.split('|')[0];
        } else if (vm.app_name.indexOf('Hyper-V' !== -1)) {
            vm.host = vm.client_name;
        }
    } else {
        vm.id = vm.client_id;
        vm.type = "compute_resource";
        vm.name = vm.client_name;
        vm.host = vm.client_name;
    }
    vm.depth = depth;
    vm.visibility = "visible open";
    vm.expanded = hasChecked(vm.backups) ? "expanded" : "";
    if(vm.expanded && parent) parent.expanded = "expanded";
    // lmc
    //vm.checked = (checked.vms.indexOf(vm.id) !== -1) ? " checked" : "";
    vm.checked = (checked != null && typeof checked.vms !== 'undefined' && checked.vms.indexOf(vm.id) !== -1) ? " checked" : "";
    vm.display = vm.name;
    /*
    if(_.findWhere(dupes, {"name": vm.name, "env": vm.environment_ip})) {
        vm.display = vm.name + " (" + vm.moref + ")";
    } else {
        vm.display = vm.name;
        dupes.push({"name":vm.name, "env": vm.environment_ip});
    }
    */

    vm.parent = parent;
    vm.children = vm.backups;
    vm.filtered = [];

    return vm;
}

function formatBackup(vm, backup, depth, index) {
    // Was this backup imported from a backup copy (archive)?
    var imported = backup.imported_from_archive;

    backup.index = index;
    backup.backup_mode = backup.type;
    backup.type = backup.set_id ? "archive" : imported ? "imported" : "backup";
    // If this archive was imported from anohter appliance, use the name as the instance-id.
    backup.instance_id = backup.set_id && backup.is_imported ? vm.instance_name : vm.id;
    backup.client_id = vm.client_id;
    //backup.environment_ip = vm.environment_ip;
    backup.system_name = vm.system_name;
    backup.system_id = vm.system_id;
    backup.bds_type = backup.replicated === true ? BDS_TYPE_REPLICATION : backup.type == "archive" ? BDS_TYPE_ARCHIVE : BDS_TYPE_BACKUP;

    backup.icon = getBackupIcon(backup.bds_type, imported);
    backup.title = getBackupTitle(backup.bds_type, imported);
    backup.bds_class = getBackupClass(backup.bds_type, imported);
    backup.hv_class = getHypervisorClass(vm.app_name);

    backup.name = backup.start_date;
    backup.host = vm.host;
    backup.depth = depth;
    
    backup.pinned = backup.legal_hold == true ? "pin" : "";
    switch (backup.type) {
        case 'backup':
        case 'imported':
            backup.status = backup.status == BACKUP_STATUS_FAILURE ? "Failed" :
                backup.status == BACKUP_STATUS_WARNING ? "Warning" : "";
            break;
        case 'archive':
            backup.status = backup.success === false ? "Failed" : "";
            break;
    }
    // If a backup was not successful, add the status in parenthesis.
    if (backup.status != "") {
        backup.name += " (" + backup.status + ")";
    }

    //backup.backup_mode = backup.type; //getBackupMode(backup.backup_mode, backup.mode_subtype);
    // lmc
    //backup.checked = (checked.backups.indexOf(backup.id) !== -1) ? " checked" : "";
    backup.checked = (checked != null && typeof checked.backups !== 'undefined' &&
			checked.backups.indexOf(backup.id) !== -1) ? " checked" : "";
    backup.visibility = (vm.expanded === "") ? "" : "visible open";
    backup.expanded = (options.disks.checkboxes && backup.checked) ? "expanded" : "";
    backup.template = vm.template;

    backup.plural = BCView === BACKUP_CATALOG_VIEW_ROLLBACK ? "s" : "";

    backup.parent = vm;
    backup.children = backup.disks;
    backup.filtered = [];

    if(backup.environment_ip) environments.push(backup.environment_ip);
    if(backup.host) hosts.push(backup.host);
    if(backup.backup_mode) types.push(backup.backup_mode);
    return backup;
}


function formatDisk(vm, backup, disk, depth) {
    disk.type = "disk";
    disk.vid = vm.id;
    disk.bid = backup.id;
    disk.total_storage = formatBytes(disk.total_storage);
    disk.depth = depth;
    disk.visibility = (backup.expanded === "") ? "" : "visible open";
    // lmc
    //disk.checked = (checked.backups.indexOf(backup.id) !== -1) ? " checked" : "";
    disk.checked = (typeof checked.backups !== 'undefined' &&
			checked.backups.indexOf(backup.id) !== -1) ? " checked" : "";

    disk.plural = BCView === BACKUP_CATALOG_VIEW_ROLLBACK ? "s" : "";

    disk.parent = backup;
    disk.children = null;
    disk.filtered = [];
    return disk;
}

function formatBytes(bytes) {
    var bytes = parseInt(bytes, 10);
    if(bytes <= 0 || isNaN(bytes)) {
        return "0b";
    }

    var base = Math.log(bytes) / Math.log(1024),
        unit = ['B', 'KB', 'MB', 'GB', 'TB' , 'PB' , 'EB'],
        result = Math.pow(1024, base - Math.floor(base)),
        precision = Math.round(result).toString().length === 1 ? 2 : 1;

    return result.toFixed(precision) + ' ' + unit[Math.floor(base)];
}

function getBackupIcon(bdstype, imported) {
    switch(bdstype) {
        case BDS_TYPE_ARCHIVE:
            return "icon-uui-backup-copy";
        case BDS_TYPE_SYNC:
            return "icon-phd-sync";
        case BDS_TYPE_BACKUP:
        default:
            return imported ? "icon-uui-backup-imported" : "icon-uui-backup";
    }
}

function getBackupClass(bdstype, imported) {
    switch(bdstype) {
        case BDS_TYPE_ARCHIVE:
            return "backup-type-archive";
        case BDS_TYPE_SYNC:
            return "backup-type-sync";
        case BDS_TYPE_BACKUP:
        default:
            return imported ? "backup-type-imported" : "backup-type-backup";
    }
}

function getBackupTitle(bdstype, imported) {
    switch(bdstype) {
        case BDS_TYPE_ARCHIVE:
            return "Backup Copy";
        case BDS_TYPE_SYNC:
            return "Sync Backup";
        case BDS_TYPE_BACKUP:
        default:
            return imported ? "Imported Backup" : "Backup";
    }
}

function getHypervisorClass(app_name) {
    var type = "unknown";
    if (app_name != undefined && app_name != "") {
        if (app_name.indexOf("VMware") != -1) {
            type = "vmware";
        } else if (app_name.indexOf("Hyper-V") != -1) {
            type = "hyperv";
        } else if (app_name.indexOf("Xen") != -1) {
            type = "xen";
        }
    }
    return "hv-type-" + type;
}

function initTemplates() {
    /*
    vtmpl = _.template(
        '<tr data-type="vm" data-depth="<%= depth %>" data-moref="<%= moref %>" vm_id="<%= id %>" class="level-<%= depth %> <%= visibility %>">' +
            '<td>' +
                '<div class="expand-ctrl <%= expanded %>"></div>' +
                '<% if(options.vms.checkboxes) { %><input type="checkbox" name="vm" value="<%= moref %>" class="row-select"<%= checked %>><% } %>' +
                '<i class="icon-hv-vm icon" alt="vm" title="vm"></i>' +
                '<span class="item-name"><%= display %></span>' +
            '</td>' +
            '<td><%= environment_ip %></td>' +
            '<td><%= host %></td>' +
            '<td></td>' +
        '</tr>'
        );
     datetmpl = _.template(
         '<tr data-type="date" data-depth="<%= depth %>" date_id="<%= id %>" class="level-<%= depth %> <%= visibility %>">' +
         '<td>' +
         '<div class="expand-ctrl <%= expanded %>"></div>' +
         '<% if(options.dates.checkboxes) { %>' +
         '<input type="checkbox" name="date" value="0" class="row-select">' +
         '<% } %>' +
         '<i class="icon-notesdate icon" alt="Date" title="Date"></i>' +
         '<span class="item-name"><%= day %></span>' +
         '</td>' +
         '<td></td>' +
         '<td></td>' +
         '<td></td>' +
         '</tr>'
     );
     btmpl = _.template(
         '<tr data-type="backup" data-date="<%= iso_date %>" data-depth="<%= depth %>"'+
         'data-moref="<%= moref %>" vm_id="<%= vid %>" backup_id="<%= id %>"'+
         'class="level-<%= depth %> <%= visibility %> <%= pinned %> <%= bds_class %> <%= hv_class %>">' +
         '<td>' +
         '<div class="expand-ctrl <%= expanded %>"></div>' +
         '<% if(options.backups.checkboxes) { %><input type="checkbox" name="backup<%= plural %>" value="<%= id %>" class="row-select"<%= checked %>><% } %>' +
         '<i class="<%= icon %> icon" alt="backup" title="<%= title %>"></i>' +
         '<span class="item-name"><%= name %></span>' +
         '</td>' +
         '<td><%= environment_ip %></td>' +
         '<td><%= host %></td>' +
         '<td><%= backup_mode %></td>' +
         '</tr>'
         );
         bdstmpl = _.template(
         '<tr data-type="bds" data-depth="<%= depth %>" bds_id="<%= bds_id %>" class="level-<%= depth %> <%= bds_class %> <%= visibility %>">' +
         '<td>' +
         '<div class="expand-ctrl <%= expanded %>"></div>' +
         '<% if(options.bds.checkboxes) { %>' +
         '<input type="checkbox" name="bds" value="0" class="row-select">' +
         '<% } %>' +
         '<i class="<%= icon %> icon" alt="BDS" title="BDS"></i>' +
         '<span class="item-name"><%= bds_name %></span>' +
         '</td>' +
         '<td></td>' +
         '<td></td>' +
         '<td></td>' +
         '</tr>'
        );
    */
    vtmpl = _.template(
        '<tr data-type="vm" data-depth="<%= depth %>" data-id="<%= id %>" class="level-<%= depth %> <%= visibility %>">' +
        '<td>' +
        '<div class="expand-ctrl <%= expanded %>"></div>' +
        '<% if(options.vms.checkboxes) { %><input type="checkbox" name="vm" value="<%= id %>" class="row-select"<%= checked %>><% } %>' +
        '<i class="icon-hv-<%= type %> icon" alt="<%=type %>" title="<%=type %>"></i>' +
        '<span class="item-name"><%= display %></span>' +
        '</td>' +
        '<td><%= system_name %></td>' +
        '<td><%= host %></td>' +
        '<td></td>' +
        '</tr>'
    );
    /*         ' backup_id="<%= id %>" '+ */
    btmpl = _.template(
        '<tr data-type="backup" data-date="<%= start_date %>" data-depth="<%= depth %>"'+
        '<% if(type !== "archive") { %> backup_id="<%=id %>" <% } else { %> archive_id="<%=id %>"<% }%>' +
        ' system-id="<%= system_id %>" '+
        ' instance-id="<%= instance_id %>" '+
        'class="level-<%= depth %> <%= visibility %> <%= pinned %> <%= status %> <%= bds_class %> <%= hv_class %>">' +
            '<td>' +
                '<div class="no-expand-ctrl"></div>' +
                '<% if(options.backups.checkboxes) { %><input type="checkbox" name="backup<%= plural %>" value="<%= id %>" class="row-select"<%= checked %>><% } %>' +
                '<i class="<%= icon %> icon" alt="backup" title="<%= title %>"></i>' +
                '<span class="item-name"><%= name %></span>' +
            '</td>' +
            '<td><%= system_name %></td>' +
            '<td><%= host %></td>' +
            '<td><%= backup_mode %></td>' +
        '</tr>'
    );
    /* lmc - TODO - this will need work to support archive_ids */
    dtmpl = _.template(
        '<tr data-type="disk" data-depth="<%= depth %>" vm_id="<%= vid %>" disk_id="<%= id %>" backup_id="<%= bid %>" class="level-<%= depth %> <%= visibility %>">' +
            '<td>' +
                '<% if(options.disks.checkboxes) { %>' +
                    '<input type="checkbox" name="disk<%= plural %>" value="<%= id %>" class="row-select"<%= checked %>>' + 
                '<% } else if(BCView === BACKUP_CATALOG_VIEW_ROLLBACK) { %>' +
                    '<input type="checkbox" name="disk<%= plural %>" value="<%= id %>" class="row-select hidden"<%= checked %>>' + 
                '<% } %>' +
                '<i class="icon-hv-drive icon" alt="Disk" title="Disk"></i>' +
                '<span class="item-name"><%= name %> <span class="disk-size">(<%= total_storage %>)</span></span>' +
            '</td>' +
            '<td></td>' +
            '<td></td>' +
            '<td></td>' +
        '</tr>'
        );
    datetmpl = _.template(
        '<tr data-type="date" data-depth="<%= depth %>" date_id="<%= id %>" class="level-<%= depth %> <%= visibility %>">' +
            '<td>' +
                '<div class="expand-ctrl <%= expanded %>"></div>' +
                '<% if(options.dates.checkboxes) { %>' +
                    '<input type="checkbox" name="date" value="0" class="row-select">' + 
                '<% } %>' +
                '<i class="icon-notesdate icon" alt="Date" title="Date"></i>' +
                '<span class="item-name"><%= day %></span>' +
            '</td>' +
            '<td></td>' +
            '<td></td>' +
            '<td></td>' +
        '</tr>'
        );
    bdstmpl = _.template(
        '<tr data-type="bds" data-depth="<%= depth %>" bds_id="<%= id %>" class="level-<%= depth %> <%= bds_class %> <%= visibility %>">' +
            '<td>' +
                '<div class="expand-ctrl <%= expanded %>"></div>' +
                '<% if(options.bds.checkboxes) { %>' +
                    '<input type="checkbox" name="bds" value="0" class="row-select">' + 
                '<% } %>' +
                '<i class="<%= icon %> icon" alt="BDS" title="BDS"></i>' +
                '<span class="item-name"><%= bds_name %></span>' +
            '</td>' +
            '<td></td>' +
            '<td></td>' +
            '<td></td>' +
        '</tr>'
        );
}

function getBackupMode(mode, submode) {
    if(mode === BACKUP_MODE_VIRTUAL_FULL) {
        return TRANS_VIRTUAL_FULL;//"Virtual Full";
    } else {
        return BACKUP_SUBMODE_CHOICES[submode];
    }
}
