
(function ($, PHD, window, undefined) {

    /*
     *
     *   Defines logic for page-level buttons
     *   @param $but - the button element that was clicked
     *   @param $row - the element or element(s) that are considered "selected" by the user
     *   @param tab - the object that owns $but and $row - typically this is the object that called buttonlogic()
     *
     */

    var jobTypeRegex = /[\s|^]job-type-([a-z]+)(?:\s)?/;

    var jobEditSelections = function(rowData, data) {
        var selections = "";
        if (rowData != null) {
            var inventory = "inventory=" + ((rowData.sid != undefined) ? rowData.sid : "1");
            var selection = "";
            var client_id = 0;
            if (data.client_id != undefined) {
                client_id = data.client_id;
            }
            inventory += "_" + client_id;
            if (data.instances != null) {
                //console.log("data is " + JSON.stringify(data));
                var base_inventory = inventory;
                if (data.instances.length > 0) {
                    var app_id = data.app_id;
                    console.log("app id is " + app_id);
                    for (var i = 0; i < data.instances.length; i++) {
                        var instance = data.instances[i];
                        inventory = base_inventory + "_" + app_id + "_" + instance.id;
                        if (i > 0) {
                            selection += "&";
                        }
                        selection += inventory;
                    }
                }
            } else {
                selection = inventory;
            }
            selections += selection;
        }
        console.log("job edit selections: " + selections);
        return selections;
    };

    var getSelectedBackups = function(tab) {
        var selections_obj = {};
        selections_obj.backups = [];
        var backups = [];
        if (tab.TreeTable !== undefined) {
            var selectedBackups = tab.TreeTable.getSelectionsExtra();
            if (selectedBackups.length > 0) {
                //console.log("selected backups = " + JSON.stringify(selectedBackups));
                var backups_per_instance = new Array();
                selectedBackups.forEach(function (obj, index) {
                    if (obj.backup_id !== undefined) {
                        var instance_id = obj.instance_id;
                        if (!$.isArray(backups_per_instance[instance_id])) {
                            backups_per_instance[instance_id] = new Array();
                        }
                        backups_per_instance[instance_id].push(obj.backup_id);
                    }
                });
                for (var instance in backups_per_instance) {
                    var b = backups_per_instance[instance];
                    b.sort();
                    backups.push(b[b.length - 1]);
                }
            }
            selections_obj.backups = backups;
        }
        //console.log("in: getSelectedBackups: selections object = " + JSON.stringify(selections_obj));
        return selections_obj;
    };

    PHD.buttonLogic = function ($but, $row, tab, id, row_data, analytics) {
        var rowData = row_data || null;
        var editpath = "", view = "", action = "";
        var $analytics = analytics || false;

        $but.off("click click.autoblur").on("click.autoblur", function () {
            $(this).blur();
        });

        if ($but.is(".btn-add-backup")) {
            $but.attr("href", "app/joborders/backup/backup.html")
                .on("click", function (event) {
                    event.preventDefault();
                    $analytics && $analytics.eventTrack('Add Backup Job', {  category: 'Backup', label: 'Jobs' });
                    PHD.wizard($but.attr("href"), "add-job", {
                        title: gettext("Create Backup Job"),
                        dialogClass: "dialog-add-job dialog-modal-form",
                        helpArticle: PHD.App.getHelpLink("backup", true),
                        // lmc
                        open: function () {
                            jobEdit = false;
                            jobID = 0;
                            preSelected = $but.data('selections');
                            var dateFormat = $.datepicker.formatDate(DATE_FORMAT, new Date());
                            $("#id_start_date").val(dateFormat);
                            $("#id_name").val("Backup Job");
                        },
                        beforeClose: function (event, ui) {
                            $(".backup-job-popover").remove();
                            $(".datepicker input:text").datepicker("destroy");
                            $(".ui-spinner-input").spinner("destroy");
                            
                            if(resetTour){
                                refreshPage();
                                setTimeout(function(){
                                    var $scope = $("[ng-controller=GlobalCtrl]").scope();
                                    $scope.user.showTour = true;
                                    window.location.href = "#/dashboard/";  
                                    resetTour = false;
                                    $(".btn-pref-cancel").trigger('click');
                                }, 1000); 
                            }
                        }
                    });
                });
            return true;
        }

        if ($but.is(".btn-edit-job")) {
            if ($row !== null && $row.length) {
                editpath = jobTypeRegex.exec($row.attr("class"))[1];

                var selectedJob = rowData;
                var url_sid = (selectedJob === null) ? "/" : ("/?sid=" + (selectedJob.system_id !== undefined ? selectedJob.system_id : selectedJob.sid));

                $but.attr("href", "app/joborders/" + editpath + "/" + editpath + ".html")
                    .removeClass("disabled")
                    .on("click", function (event) {
                        event.preventDefault();
                        $analytics && $analytics.eventTrack('Edit Job', {  category: editpath, label: 'Jobs' });

                        $url = "/api/joborders/" + $row.data("dbid") + url_sid;
                        // lmc - need get to fill in html
                        var load = PHD.showLoadingIndicator($row.find("[data-source=\"status\"]"));
                        PHD.Ajax.get($url, load)
                            .done(function (data) {
                                //PHD.hideLoadingIndicator(load);
                                var data = data.data[0];    // API returns 1 object.
                                var selections = jobEditSelections(rowData, data);
                                var options = rowData;


                                PHD.wizard($but.attr("href"), "edit-job", {
                                    title: gettext("Edit Job"),
                                    dialogClass: "dialog-add-job dialog-edit-job dialog-modal-form",
                                    helpArticle: PHD.App.getHelpLink(editpath, true),
                                    // lmc
                                    open: function () {
                                        jobEdit = true;
                                        jobID = id;
                                        jobOptions = data;
                                        console.log('jobID is ' + jobID + " and name = " + data.name);
                                        $("#id_name").val(data.name);
                                        $("#edit_object_id").text(jobID);
                                        preSelected = selections;
                                        system_id = $row.data("sid");
                                    },
                                    beforeClose: function (event, ui) {
                                        if (editpath === "backup") {
                                            $(".backup-job-popover").remove();
                                            $(".datepicker input:text").datepicker("destroy").datepicker("widget").remove();
                                            $(".ui-spinner-input").spinner("destroy");
                                        }
                                    }
                                }, true);

                            });

                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-copy-job")) {
            if ($row !== null && $row.length && !$row.is(".job-type-rollback") && !$row.is(".job-type-replication")) {
                editpath = jobTypeRegex.exec($row.attr("class"))[1];

                var selectedJob = rowData;
                var url_sid = (selectedJob === null) ? "/" : ("/?sid=" + (selectedJob.system_id !== undefined ? selectedJob.system_id : selectedJob.sid));
                $but.attr("href", "/api/joborders/copy/" + $row.data("dbid") + url_sid)
                    .removeClass("disabled")
                    .on("click", function (event) {
                        event.preventDefault();
                        $analytics && $analytics.eventTrack('Copy Job', {  category: 'All', label: 'Jobs' });
                        var load = PHD.showLoadingIndicator($row.find("[data-source=\"status\"]"));
                        PHD.Ajax.post($but.attr("href"), null, load)
                            .done(function (data) {
                                tab.update();
                            });
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-pause-job")) {
            if ($row !== null && $row.length && !$row.is(".system-true")) {
                $but.removeClass("disabled")
                    .off("click");
                var selectedJob = rowData;
                var url_sid = (selectedJob === null) ? "/" : ("/?sid=" + (selectedJob.system_id !== undefined ? selectedJob.system_id : selectedJob.sid));
                if ($row.is(".job-status-paused")) {
                    $but.attr("href", "/api/jobs/resume/" + $row.data("dbid") + url_sid)
                        .html('<i class="icon-play icon-large"></i>Resume');
                } else if ($row.is(".job-status-queued")) {
                    $but.attr("href", "/api/jobs/suspend/" + $row.data("dbid") + url_sid)
                        .html('<i class="icon-pause icon-large"></i>Pause');
                } else {
                    $but.disableButton();
                    return true;
                }
                $but.on("click", function (event) {
                    event.preventDefault();
                    var load = PHD.showLoadingIndicator($row.find("[data-source=\"status\"]"));
                    PHD.Ajax.put($but.attr("href"), null, load)
                        .done(function (data) {
                            //PHD.hideLoadingIndicator(load);
                            tab.update();
                        });
                });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-cancel-job")) {
            //if ($row !== null && $row.length && !$row.is(".job-state-canceling") && !$row.is(".system-true")) {
            if ($row !== null && $row.length && !$row.is(".job-status-finished") && !$row.is(".job-status-canceled")) {
                var selectedJob = rowData;
                var url_sid = (selectedJob === null) ? "/" : ("/?sid=" + (selectedJob.system_id !== undefined ? selectedJob.system_id : selectedJob.sid));
                $but.removeClass("disabled")
                    .off("click")
                    .attr("href", "/api/jobs/" + $row.data("dbid") + url_sid)
                    .on("click", function (event) {
                        event.preventDefault();
                        var load = PHD.showLoadingIndicator($row.find("[data-source='status']"));
                        PHD.Ajax.delete($but.attr("href"), null, load)
                            .done(function (data) {
                                //PHD.hideLoadingIndicator(load);
                                tab.update();
                            });
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-view-details")) {
            if ($row !== null && $row.length) {
                $but.removeClass("disabled");
                $but.off("click");
                if (!tab.inventoryOpen) {
                    $but
                        .html('<i class="icon-tasks icon-large"></i>' + gettext('View Details'))
                        .on("click", function (event) {
                            event.preventDefault();
                            tab.showJobDetails();
                        });
                } else {
                    $but
                        .html('<i class="icon-eyeclose icon-large"></i>' + gettext('Hide Details'))
                        .on("click", function (event) {
                            event.preventDefault();
                            tab.hideJobDetails();
                        });
                }
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-pop-out")) {
            if ($row !== null && $row.length && tab.index == JOB_TAB_HISTORY) {
                $but.removeClass("disabled")
                    .off("click")
                    .on("click", function (event) {
                        event.preventDefault();
                        var sid = rowData.system_id;
                        //var bidArray = _.filter(rowData, function(val) {return val.backup_id != "";});
                        //var bids = _.pluck(bidArray, 'backup_id');
                        //if (bids.length == 0) {
                        //    bids = _.pluck(bidArray, 'archive_id');
                        //}
                        //if (bids.length > 0) {
                        var bids = rowData.backup_id;
                        if (!isNaN(bids)) {
                            var name = rowData.name;
                            PHD.appliance_sid = sid;
                            backup_ids = bids.toString();
                            PHD.wizard("app/inventory/details/status.html", "view-status", {
                                title: gettext("Backup Statuses for Job: " + name),
                                dialogClass: "standalone",
                                height: 500,
                                helpArticle: PHD.App.getHelpLink("backup", true)
                            });
                        } else {
                            var message = gettext("There are no backups for this job.");
                            if (rowData.expanded_error !== undefined) {
                                message += "<br/><br/>" + rowData.expanded_error + ".";
                            }
                            PHD.throwError({"result": [{"message": message}]});
                        }
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-hide-system-jobs")) {
            $but.on("click", function (event) {
                event.preventDefault();
                if ($but.data("hide") === undefined || $but.data("hide") === true) {
                    $but.data("hide", false);
                    $but.html('<i class="icon-eye icon-large"></i> ' + gettext('Show System Jobs'));
                    tab.toggleSystemJobs(false);
                } else {
                    $but.data("hide", true);
                    $but.html('<i class="icon-eyeclose icon-large"></i> ' + gettext('Hide System Jobs'));
                    tab.$table.find(".system-true").toggleClass("hidden");
                    tab.toggleSystemJobs(true);
                }
            });
            return true;
        }

        if ($but.is(".btn-disable-job")) {
            if ($row !== null && $row.length && !$row.is(".save-status-pending") && !$row.is(".save-status-errors") && !$row.is(".schedule-type-on-demand")) {

                var selectedJob = rowData;
                var url_sid = (selectedJob === null) ? "/" : ("/?sid=" + (selectedJob.system_id !== undefined ? selectedJob.system_id : selectedJob.sid));

                if ($row.is(".job-enabled-true")) {
                    $but.attr("href", "/api/joborders/disable/" + $row.data("dbid") + url_sid)
                        .html('<i class="icon-off icon-large"></i> ' + gettext('Disable'));
                } else if ($row.is(".job-enabled-false")) {
                    $but.attr("href", "/api/joborders/enable/" + $row.data("dbid") + url_sid)
                        .html('<i class="icon-restart icon-large"></i> ' + gettext('Enable'));
                }
                $but.removeClass("disabled")
                    .on("click", function (event) {
                        event.preventDefault();
                        $analytics && $analytics.eventTrack('Disable Job', {  category: 'Manage', label: 'Jobs' });
                        var load = PHD.showLoadingIndicator($row.find("[data-source=\"status\"]"));
                        PHD.Ajax.put($but.attr("href"), null, load)
                            .done(function (data) {
                                //PHD.hideLoadingIndicator(load);
                                tab.update();
                            });
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-delete-job")) {
            if ($row !== null && $row.length && !$row.is(".job-type-replication")) {
                var selectedJob = rowData;
                var url_sid = (selectedJob === null) ? "/" : ("/?sid=" + (selectedJob.system_id !== undefined ? selectedJob.system_id : selectedJob.sid));
                $but.removeClass("disabled")
                    .attr("href", "/api/joborders/" + $row.data("dbid") + url_sid)
                    .on("click", function (event) {
                        event.preventDefault();
                        $analytics && $analytics.eventTrack('Delete Job', {  category: 'Manage', label: 'Jobs' });
                        var url = $but.attr("href"),
                            confirm = PHD.confirmDialog(
                                "<p>" + gettext("Are you sure you want to delete this job?") + "</p>",
                                { title: gettext("Confirm Job Deletion") }
                            );

                        confirm.done(function () {
                            var load = PHD.showLoadingIndicator($row.find("[data-source=\"status\"]"));
                            var jobID = $row.data("dbid");
                            PHD.Ajax.delete($but.attr("href"), null, load)
                                .done(function (data) {
                                    //PHD.hideLoadingIndicator(load);
                                    tab.deleteRow(jobID);
                                    tab.update();
                                });
                        });
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-run-job")) {
            if ($row !== null && $row.length && !$row.is(".save-status-pending") && !$row.is(".save-status-errors") &&
                !$row.is(".job-enabled-false") && !$row.is(".job-type-replication")) {
                var selectedJob = rowData;
                var url_sid = (selectedJob === null) ? "/" : ("/?sid=" + (selectedJob.system_id !== undefined ? selectedJob.system_id : selectedJob.sid));

                $but.removeClass("disabled")
                    .attr("href", "/api/joborders/run/" + $row.data("dbid") + url_sid)
                    .on("click", function (event) {
                        event.preventDefault();
                        $analytics && $analytics.eventTrack('Run Job', {  category: 'Manage', label: 'Jobs' });
                        var load = PHD.showLoadingIndicator($row.find("[data-source=\"status\"]"));
                        PHD.Ajax.put($but.attr("href"), null, load)
                            .done(function (data) {
                                //PHD.hideLoadingIndicator(load);
                                tab.update();
                            });
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-edit-vm-settings")) {
            if ((typeof $but.data("selections") !== 'undefined') && $but.data("selections") !== null && $but.data("selections").length) {
                qs = "?" + $but.data("selections");

                console.log("but data " + JSON.stringify($but.data("selections")));

                function getTitle() {
                    var title = "Edit settings for ";
                    var item_array = $but.data("selections").split('&');
                    if (item_array.length > 0) {
                        title += item_array.length + " protected asset(s)";
                    }
                    return title;
                }
                $but.removeClass("disabled")
                    //.attr("href", "/inventory/settings/" + qs)
                    // lmc
                    .attr("href", "app/inventory/settings.html")
                    .on("click", function (event) {
                        event.preventDefault();
                        PHD.wizard($but.attr("href"), "edit-vm-settings", {
                            title: "View/Edit Protected Asset Settings",
                            helpArticle: PHD.App.getHelpLink("inventorysettings", true),
                            dialogClass: "dialog-edit-vm-settings dialog-modal-form",
                            width: 1024,
                            height: 700,
                            // lmc
                            open: function (event, ui) {
                                //var name = self.$selectedAppliance.find("[data-source=\"name\"]").text();
                                // var ip = self.$selectedAppliance.find("[data-source=\"host\"]").text();
                                //var user = self.$selectedAppliance.find("[data-source=\"username\"]").text();
                                // var id = self.selectedID;
                                var name = getTitle();
                                console.log("name is " + name);
                                $("#id_name").val(name);
                                //$("#id_ip").val(ip);
                            }
                        });
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-restore")) {
            if ($row !== null && $row.length && $row.hasClass('backup-type-archive')) {
                $but.disableButton();
                return true;
            }

            var catalogView = $but.data("catalog-view") || BACKUP_CATALOG_VIEW_VM;
            var selections_obj = getSelectedBackups(tab);

            $but.removeClass("disabled")
                .attr("href", "app/joborders/recover/recover.html")
                .on("click", function (event) {
                    event.preventDefault();
                    $analytics && $analytics.eventTrack('Recover - Full', {  category: 'Backup', label: 'Full' });
                    PHD.wizard($but.attr("href"), "recover-backups", {
                        title: gettext("Create Recover Job"),
                        helpArticle: PHD.App.getHelpLink("restore", true),
                        dialogClass: "dialog-restore dialog-modal-form",
                        // lmc
                        open: function (event, ui) {
                            //$("#id_name").val("Recover Job");
                            // Recover Jobs are only on-demand and show as such in the active jobs list.
                            $("#id_name").val("On-Demand");
                            $("#id_name").prop("disabled", true);
                            $("#instant_show_time").remove();
                            $("#id_label_seed").remove();
                            $("#id_start_seed").remove();
                            backupInfo = { "backups": [] };
                            isIRJob = false;
                            selections = selections_obj;
                            backupCatalogView = catalogView;
                        }
                    });
                });
            return true;
        }

        if ($but.is(".btn-add-replication")) {
            if ($row !== null && $row.is(".job-type-backup") && tab.namespace === "jobsmanage") {
                qs = "?backup_joborder_id=" + $row.data("dbid");
            }
            // lmc - need get to fill in html
            $but.removeClass("disabled")
                .attr("href", "app/joborders/replication/replication.html")
                .on("click", function (event) {
                    event.preventDefault();
                    PHD.wizard($but.attr("href"), "replicate-vms", {
                        title: gettext("Create Replication Job"),
                        helpArticle: PHD.App.getHelpLink("replication", true),
                        dialogClass: "dialog-replication dialog-modal-form"
                    });
                });
            return true;
        }

        if ($but.is(".btn-recover-files")) {
            if ($row && (($row.filter("[data-type=backup]").length > 1) || $row.hasClass('backup-type-archive'))) {
                $but.disableButton();
                return true;
            } else {
                var catalogView = $but.data("catalog-view") || BACKUP_CATALOG_VIEW_VM;
                var selections_obj = getSelectedBackups(tab);

                $but.removeClass("disabled")
                    .attr("href", "app/recover/flr/filerecovery.html")
                    .on("click", function (event) {
                        event.preventDefault();
                        $analytics && $analytics.eventTrack('Recover - Files', {  category: 'Backup', label: 'Files' });
                        PHD.wizard($but.attr("href"), "recover-files", {
                            title: gettext("Create File-Level Recovery Job"),
                            helpArticle: PHD.App.getHelpLink("filerecovery", true),
                            dialogClass: "dialog-recover-files dialog-modal-form",
                            // lmc
                            open: function (event, ui) {
                                backupInfo = {"backups": []};
                                isIRJob = false;
                                selections = selections_obj;
                                objects = $but.data('objects');
                                view = catalogView;
                            }
                        })
                    });
            }
            return true;
        }

        if ($but.is(".btn-import-backup-copy")) {
            if ($row === null || !$row.length || !$row.is("[data-type=backup]") || ($row.filter("[data-type=backup]").length > 1) || !$row.hasClass('backup-type-archive')) {
                $but.disableButton();
                return true;
            } else {
                var selections_obj = tab.TreeTable.getSelectionObject();
                var instance = $row.attr("instance-id");
                if (instance === undefined) {
                    instance = $row.attr("data-id");
                }

                $but.removeClass("disabled")
                    .attr("href", "app/recover/backup-copy/recover-backup-copy.html")
                    .on("click", function (event) {
                        event.preventDefault();
                        $analytics && $analytics.eventTrack('Import Copy', {  category: 'Backup Copy', label: 'Full' });
                        PHD.wizard($but.attr("href"), "import-backups", {
                            title: gettext("Import Backup Copies"),
                            helpArticle: PHD.App.getHelpLink("filerecovery", true),
                            dialogClass: "dialog-recover-files dialog-modal-form",
                            open: function (event, ui) {
                                selections = selections_obj;
                                instance_id = instance;
                            }
                        })
                    });
            }
            return true;
        }

        if ($but.is(".btn-pin-backup")) {
            if ($row === null || !$row.length || !$row.is("[data-type=backup]")) {
                $but.disableButton();
                return true;
            } else {
                $row = $row.filter("[data-type=backup]");

                if ($row.is(".pin") && !$row.allAre(".pin")) {
                    $but.disableButton();
                    return true;
                }
                qs = "";

                if ($row.is(".pin")) {
                    action = "unpin";
                    $but.html('<i class="icon-pin icon-large"></i> ' + gettext('Unpin'));
                } else if (!$row.is(".pin")) {
                    action = "pin";
                    $but.html('<i class="icon-pin icon-large"></i> ' + gettext('Pin'));
                }

                $but.removeClass("disabled")
                    .attr("href", "/api/backups/hold/")
                    .on("click", function (event) {
                        event.preventDefault();
                        var selections = tab.TreeTable.getSelectionObject();
                        selections.backups.forEach (function(backup_id, index) {
                            var url = $but.attr("href") + backup_id + "/?sid=" + selections.system_id;
                            var load = PHD.showLoadingIndicator($but);
                            PHD.Ajax.put(url, null, load)
                                .done(function (data) {
                                    //PHD.hideLoadingIndicator(load);
                                    tab.pinRows(backup_id, action);
                                    tab.updateButtons();
                                });
                        });
                    });
            }
            return true;
        }

        if ($but.is(".btn-delete-backup")) {
            if ($row !== null && $row.length && $row.is("[data-type=backup]")) {
                $but.removeClass("disabled")
                    .attr("href", "/api/backups/related/")
                    .on("click", function (event) {
                        event.preventDefault();
                        if (!$row.hasClass('backup-type-archive')) {
                            $analytics && $analytics.eventTrack('Delete Backup', {  category: 'Manage', label: 'Backups' });
                            var selections = tab.TreeTable.getSelectionObject();
                            var bid_string = selections.backups.toString();
                            var url = $but.attr("href") + bid_string + "/?sid=" + selections.system_id + '&original';
                            var load = PHD.showLoadingIndicator($but);
                            PHD.Ajax.get(url, load)
                                .done(function (data) {
                                    //PHD.hideLoadingIndicator(load);
                                    tab.confirmDelete(data);
                                });
                        } else {
                            var message = "Individual Backup Copies cannot be deleted." + "<br/>" + "<br/>" +
                                    "Go to the appliance configuration page for the target to purge all backup copies.";
                            PHD.throwNotice({"result": [{"message": message }]}, {});
                        }
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-flr-details")) {
            if ($row !== null && $row.length) {
                $but.removeClass("disabled")
                    .attr("href", "/api/jobs/active/flr/" + $row.attr("flr_id") + "/?sid=" + $row.attr('system-id'))
                    .on("click", function (event) {
                        event.preventDefault();
                        tab.showFLRDetails($row.attr("flr_id"), $row.attr('system-id'));
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-delete-flr")) {
            if ($row !== null && $row.length) {
                var flr_id = $row.attr("flr_id");
                $but.removeClass("disabled")
                    .attr("href", "/api/jobs/" + $row.attr("flr_id") + "/?sid=" + $row.attr('system-id'))
                    .on("click", function (event) {
                        event.preventDefault();
                        $analytics && $analytics.eventTrack('Delete FLR', {  category: 'Recover', label: 'FLR' });
                        var url = $but.attr("href"),
                            confirm = PHD.confirmDialog(
                                "<p>" + gettext("Are you sure you want to delete this FLR Object?") + "</p>",
                                { title: gettext("Confirm FLR Deletion") }
                            );

                        confirm.done(function () {
                            var load = PHD.showLoadingIndicator($row.find("[data-source=\"status\"]"));
                            $row.find("[data-source=\"status\"]").html("Deleting");
                            PHD.Ajax.delete($but.attr("href"), null, load)
                                .done(function (data) {
                                    //PHD.hideLoadingIndicator(load);
                                    tab.deleteFLR(flr_id);
                                });
                        });
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-instant-restore")) {
            if ($row !== null && $row.length && ($row.is(".hv-type-xen") || $row.hasClass('backup-type-archive'))) {
                $but.disableButton();
                return true;
            }
            var catalogView = $but.data("catalog-view") || BACKUP_CATALOG_VIEW_VM;
            var selections_obj = getSelectedBackups(tab);

            $but.removeClass("disabled")
                .attr("href", "app/joborders/recover/recover.html")
                .on("click", function (event) {
                    event.preventDefault();
                    $analytics && $analytics.eventTrack('Recover - Instant', {  category: 'Recover', label: 'Instant' });
                    PHD.wizard($but.attr("href"), "instant-recovery", {
                        title: gettext("Create Instant Recovery Job"),
                        helpArticle: PHD.App.getHelpLink("instantrestore", true),
                        dialogClass: "dialog-ir dialog-modal-form",
                        // lmc
                        open: function (event, ui) {
                            //$("#id_name").val("Instant Recovery Job");
                            // Recover Jobs are only on-demand and show as such in the active jobs list.
                            $("#id_name").val("On-Demand");
                            $("#id_name").prop("disabled", true);
                            $("#full_show_time").remove();
                            jobEdit = false;
                            isIRJob = true;
                            selections = selections_obj;
                            backupCatalogView = catalogView;
                        }
                    });
                });
            return true;
        }

        if ($but.is(".btn-ir-end")) {
            if ($row !== null && $row.length && !$row.is(".instant-state-ending")) {
                var selectedJob = rowData;
                var url_sid = (selectedJob === null) ? "/" : ("/?sid=" + (selectedJob.system_id !== undefined ? selectedJob.system_id : selectedJob.sid));
                $but.removeClass("disabled")
                    .attr("href", "/api/jobs/" + $row.data("dbid") + url_sid)
                    .on("click", function (event) {
                        event.preventDefault();
                        $analytics && $analytics.eventTrack('Delete IR', {  category: 'Recover', label: 'IR' });
                        var url = $but.attr("href"),
                            confirm = PHD.confirmDialog(
                                "<p>" + gettext("Are you sure you want to stop this IR session?") + "<br/><br/>" +
                                        gettext("The recovery object will be removed from the appliance, and if the VM was created ") +
                                        gettext("in audit mode, it will be deleted from the hypervisor.") + "</p>",
                                { title: gettext("Confirm IR Tear Down") }
                            );

                        confirm.done(function () {
                            var load = PHD.showLoadingIndicator($row.find("[data-source=\"status\"]"));
                            $row.find("[data-source=\"status\"]").html("Deleting");
                            PHD.Ajax.delete($but.attr("href"), null, load)
                                .done(function (data) {
                                    //PHD.hideLoadingIndicator(load);
                                    // upon deletion, hide details.
                                    tab.hideJobDetails();
                                    tab.update();
                                });
                        });
                    })
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-archive")) {
            /*if ($row !== null && $row.is(".job-type-backup") && tab.namespace === "jobsmanage") {
                qs = "?backup_joborder_id=" + $row.data("dbid");
            }
            $but.removeClass("disabled")
                .attr("href", "app/joborders/archive/archive.html")
                .on("click", function (event) {
                    event.preventDefault();
                    PHD.wizard($but.attr("href"), "archive", {
                        title: gettext("Create Archive Job"),
                        helpArticle: PHD.App.getHelpLink("archive", true),
                        dialogClass: "dialog-archive dialog-modal-form"
                    });
                });
            */
            $but.attr("href", "app/joborders/archive/archive.html")
                .on("click", function (event) {
                    event.preventDefault();
                    $analytics && $analytics.eventTrack('Create Backup Copy', {  category: 'Backup Copy', label: 'Job' });
                    PHD.wizard($but.attr("href"), "add-job", {
                        title: gettext("Create Backup Copy Job"),
                        dialogClass: "dialog-add-job dialog-modal-form",
                        helpArticle: PHD.App.getHelpLink("archive", true),
                        // lmc
                        open: function () {
                            jobEdit = false;
                            jobID = 0;
                            preSelected = $but.data('selections');
                            var dateFormat = $.datepicker.formatDate(DATE_FORMAT, new Date());
                            $("#id_start_date").val(dateFormat);
                            $("#id_name").val("Backup Copy Job");
                        },
                        beforeClose: function (event, ui) {
                            $(".backup-job-popover").remove();
                            $(".datepicker input:text").datepicker("destroy");
                            $(".ui-spinner-input").spinner("destroy");
                        }
                    });
                });
            return true;
        }

        if ($but.is(".btn-export-job-history") || $but.is(".btn-export-ir-history") || $but.is(".btn-export-sync-history")) {
            $but.attr("href");
            return true;
        }

        if ($but.is(".btn-export-csv")) {
            $but.attr("href", tab.getUpdateUrl(true));
            return true;
        }

        if ($but.is(".btn-save-log")) {
            if ($row !== null && $row.length) {
            }
            return true;
        }

        if ($but.is(".btn-show-filter")) {
            $but.on("click", function (event) {
                event.preventDefault();
                if (tab) {
                    tab.showFilter();
                }
            });
            return true;
        }

        if ($but.is(".btn-manage-retention")) {
            $but.on("click", function (event) {
                event.preventDefault();
                $(document).trigger("launchdialog", {"action": "retention"});
            });
        }

        if ($but.parent().is(".btn-group")) {
            $but
                .off("click.btngroup")
                .on("click.btngroup", function (event) {
                    event.preventDefault();
                    event.stopPropagation();
                    var $menu = $but.parent().toggleClass("open");

                    if ($menu.is(".open")) {
                        $('body').one("click.btngroupoff", function (event) {
                            event.preventDefault();
                            $menu.removeClass("open");
                        });
                    }
                })
                .next("ul").find("a")
                .off("click.btngroup")
                .on("click.btngroup", function () {
                    event.preventDefault();
                    $but.parent().removeClass("open");
                });
            return true;
        }

        $but.on("click", function (event) {
            if (!$but.is(".plain-link") && !$but.closest(".note").length)
                event.preventDefault();
        });

        return false;
    };

})(jQuery, window.PHD || (window.PHD = {}), window);
