(function($, PHD, window, undefined) {


    var defaultBCBuilerOptions = {
        view: BACKUP_CATALOG_VIEW_VM,
        ir: false,
        vms: {
            show: true,
            checkboxes: true
        },
        backups: {
            show: true,
            checkboxes: true
        },
        disks: {
            show: false,
            checkboxes: false
        },
        dates: {
            show: true,
            checkboxes: true
        },
        bds: {
            show: true,
            checkboxes: true
        },
        checked: {
            "vms": [],
            "backups": [],
            "disks": []
        },
        loader: null
    };

    var BackupCatalogBuilder = function (options) {
        var self = this,
            worker = new Worker('scripts/utils/backupcatalog-worker.js?' + Date.now()),
            promise = $.Deferred(),
            url = "/api/catalog/?view=", resp;

        this.options = $.extend(true, {}, defaultBCBuilerOptions, options);

        var view = this.options.view;
        switch (view) {
            case BACKUP_CATALOG_VIEW_DATE:
                url += "day";
                break;
            case BACKUP_CATALOG_VIEW_VM:
                url += "instance";
                break;
            case BACKUP_CATALOG_VIEW_BDS:
                url += "storage";
                break;
            case BACKUP_CATALOG_VIEW_APPLIANCE:
                url += "system";
                break;
        }

        if(this.options.view === BACKUP_CATALOG_VIEW_ROLLBACK) {
            url += "&backup=" + this.options.checked.backups.join("&backup=");
        } else if(this.options.ir === true) {
            url += "&ir=1";
        }
        // lmc - TODO - filtering by start date!
        var start_date = moment().subtract(1, 'y').format('MM/DD/YYYY');
        url += "&start_date=" + start_date;

        resp = PHD.Ajax.get(url, this.options.loader);

        worker.addEventListener('message', function(e) {
            promise.resolve(e.data);
        }, false);

        resp.done(function(data) {
            var catalog = data.catalog;
            if (catalog.constructor === Array) {
                //catalog = catalog[0];
            }
            var obj = {
                "options": self.options,
                "backups": catalog,
                "trans": {
                    "vf": gettext("Virtual Full"),
                    "full": gettext("Full"),
                    "inc": gettext("Incremental")
                }
            };
            worker.postMessage(obj);
        });

        return promise;
    };

    PHD.BackupCatalogBuilder = BackupCatalogBuilder;
    
    /*
    *   TreeTable defines the behavior for an HTML table with a linear
    *   structure but hierarchical relationships between rows
    */

    var TreeTable = function TreeTable($treeTable, options) {
        if(this instanceof TreeTable) {
            this.$table = $treeTable;
            this.options = options;
        } else {
            return new TreeTable($treeTable, options).init();
        }
    };

    TreeTable.prototype = {
        init: function() {
            var self = this;
            this.$table
                .on("afterfilter", function() {
                    self.zebra();
                })
            .find("tbody")
                .on("click", ".expand-ctrl, .item-name", function() {
                    self.toggleItem($(this));
                })

                .on("change", ".row-select", function() {
                    var $checked = $(this),
                        $node = $checked.closest("tr");

                    self.handleCheckClick($checked, $node);
                    $checked.trigger("treeselect", [$node, $checked.prop("checked"), self]);
                });

            return this.refresh();
        },

        refresh: function(options) {
            if(options !== void 0) {
                this.options = options;
            }
            return this.collapseAll().zebra();
        },

        collapseAll: function() {
            this.$table
                .find("tbody tr:not([data-depth=0])")
                .not(".open")
                .addClass("closed");
            return this;
        },

        expandLevel: function(level) {
            this.$table
                .find("tbody tr")
                .filter("[data-depth=" + level + "]")
                .removeClass("closed")
                .addClass("open visible");
            return this;
        },

        setOption: function(level, key, value) {
            this.options[level][key] = value;
            return this;
        },

        handleCheckClick: function($checkbox, $node) {
            var checked = $checkbox.prop("checked"),
                depth = parseInt($node.data("depth"), 10),
                options = this.options[depth],
                self = this;

            if(checked && this.options.unique === $node.data("type")) {
                this.deselectByType($node.data("type"));
            }

            // Select/Deselect the node that was checked
            $node.find(".row-select").prop("checked", checked);

            // If only one node on this level can be selected, deselect
            // any sibling rows
            if(checked && options.multiselect === false) {
                this.deselectSiblings($node, depth);
            }

            if(checked && options.expandonselect && options.childselect) {
                self.toggleItem($node, true);
            }

            if(options.childselect === "all") {
                this.setDescendants($checkbox);
            } else if(options.childselect === "first" && checked) {
                this.setFirstChild($checkbox);
            }

            //console.log(this.$table.find(":checked").length);

            /*
                Placeholder for if we decide to address ancestors in
            if(depth > 0) {
                console.log(this.getAncestors($node));
            }
            */

            return this;
        },

        setChildren: function($checkbox) {
            var $node = $checkbox.closest("tr"),
                checked = $checkbox.prop("checked");

            this.getChildren($node).find(".row-select").prop("checked", checked).trigger("change");
            return this;
        },

        setDescendants: function($checkbox) {
            var $node = $checkbox.closest("tr"),
                checked = $checkbox.prop("checked");

            this.getDescendants($node).find(".row-select").prop("checked", checked).trigger("change");
            return this;
        },

        setFirstChild: function($checkbox) {
            var $node = $checkbox.closest("tr"),
                checked = $checkbox.prop("checked");

            var $child = $node.next("tr").find(".row-select").prop("checked", checked).trigger("change");
            return this.setDescendants($child);
        },

        deselectAll: function(type) {
            this.$table.find(":checkbox").prop("checked", false);
            return this;
        },

        deselectByType: function(type) {
            var self = this;
            this.$table
                .find("[data-type=" + type + "] .row-select")
                .prop("checked", false)
                .each(function() {
                    self.setDescendants($(this));
                });
        },

        deselectSiblings: function($node, depth) {
            var self = this,
                $siblings = this.getSiblings($node);

            $siblings.find(".row-select").prop("checked", false);
            this.setDescendants($siblings.find(".row-select"));
            return this;
        },

        toggleItem: function($clicked, force) {
            var $node = $clicked.closest("tr"),
                type = $node.data("type"),
                id = $node.attr(type + "_id"),
                $descendants = $();

            // If they clicked the item name, set $clicked to the 
            // .expand-ctrl element
            if(!$clicked.is(".expand-ctrl")) {
                $clicked = $clicked.prevAll(".expand-ctrl");
            }

            if($clicked.hasClass("expanded") && force !== true || force === false) {
                //console.log("Collapsing item " + id);
                $clicked.removeClass("expanded");
                this.getChildren($node).removeClass("open").addClass("closed");
                this.getDescendants($node).removeClass("visible");
            } else {
                //console.log("Expanding item " + id);
                $descendants = this.getDescendants($node);
                if($descendants.length) {
                    $clicked.addClass("expanded");
                    this.getChildren($node).removeClass("closed").addClass("open");
                    $descendants.addClass("visible");
                }
            }

            return this.zebra();
        },

        getDescendants: function($node) {
            var depth = $node.data("depth"),
                parentDepth = depth ? depth - 1 : 0;

            return $node.nextUntil("tr[data-depth=" + depth + "], tr[data-depth=" + parentDepth + "]");
        },

        getChildren: function($node) {
            var depth = parseInt($node.data("depth"), 10);
            return this.getDescendants($node).filter("tr[data-depth=" + (depth + 1) + "]");
        },

        getLastChild: function($node) {
            return this.getChildren($node).last();
        },

        getSiblings: function($node) {
            if($node.data("depth") == 0)
                return $node.siblings("[data-depth=0]");

            return this.getChildren(this.getParent($node)).not($node);
        },

        getParent: function($node) {
            var depth = parseInt($node.data("depth"), 10),
                parentDepth = depth - 1;

            if(parentDepth < 0) return $();
            return $node.prevAll("[data-depth=" + parentDepth + "]").first();
        },

        getAncestors: function($node) {
            var $parent = this.getParent($node),
                $parents = $().add($parent),
                depth = $node.data("depth");

            for(;depth>0;depth--) {
                $parent = this.getParent($parent);
                $parents = $parents.add($parent);
            }
            return $parents;
        },

        getSelections: function() {
            return this.$table.find(":checked").serialize();
        },

        getSelectionsExtra: function() {
            var selections = [];
            var $ids = this.$table.find(":checked");
            $ids.each(function(index, value) {
                var obj = {};
                $node = $(this).closest('tr');
                var backup_id = $node.attr('backup_id');
                if (backup_id !== undefined) {
                    backup_id = parseInt(backup_id, 10);
                    obj.backup_id = backup_id;
                }
                var archive_id = $node.attr('archive_id');
                if (archive_id !== undefined) {
                    archive_id = parseInt(archive_id, 10);
                    obj.archive_id = archive_id;
                }
                var system_id = $node.attr('system-id');
                if (system_id !== undefined) {
                    system_id = parseInt(system_id, 10);
                    obj.system_id = system_id;
                }
                var instance_id = $node.attr('instance-id');
                if (instance_id !== undefined) {
                    instance_id = parseInt(instance_id, 10);
                    obj.instance_id = instance_id;
                }
                selections.push(obj);
            });
            return selections;
        },

        getSelectionObject: function() {
            var $ids = this.$table.find(":checked"),
                vms = $ids.filter("[name=vm]").map(function(){return $(this).val()}).get(),
                backups = $ids.filter("[name=backup]").map(function(){return parseInt($(this).val(), 10)}).get(),
                disks = $ids.filter("[name=disk]").map(function(){return parseInt($(this).val(), 10)}).get();
                //system_ids = $ids.attr('data-system-id').map(function(){return parseInt($(this).val(), 10)}).get();
            //console.log("get selection object, sid = " + JSON.stringify(system_ids));
            console.log("ids are " + $ids.val());
            var system_ids = [];
            $ids.each(function(index, value) {
                console.log( $(this).text() );
                $node = $(this).closest('tr');
                var system_id = $node.attr('system-id');
                if (system_id !== undefined) {
                    system_id = parseInt($node.attr('system-id'), 10);
                }
                system_ids.push(system_id);
            });
            var $node = $ids.closest('tr');
            var attr_id = $node.attr('system-id');
            var system_id = 1;
            if (attr_id !== undefined) {
                system_id = parseInt($node.attr('system-id'), 10);
            }
            console.log("system_id is " + system_id);
            console.log("system ids are " + JSON.stringify(system_ids));
            return {"vms": vms, "backups": backups, "disks": disks, "system_id": system_id, "system_ids": system_ids };
        },

        deleteNode: function($node) {
            var hasSiblings = (this.getSiblings($node).length > 0),
                hasChildren = (this.getDescendants($node).length > 0);

            if(hasChildren) {
                this.getDescendants($node).remove();
            }
            
            if(!hasSiblings && $node.data("depth") > 0) {
                this.deleteNode(this.getParent($node));
            }

            $node.remove();

            return this.zebra();
        },

        deleteNodeById: function(type, id) {
            return this.deleteNode(this.getNodeById(type, id));
        },

        getNodeById: function(type, id) {
            return this.$table.find("[data-type=" + type + "][" + type + "_id='" + id + "']");
        },

        getNodesByType: function(type) {
            return this.$table.find("[data-type=" + type + "]");
        },

        getVisibleRows: function() {
            return this.$table.find("tr:visible");
        },

        zebra: function() {
            this.getVisibleRows().each(function(index) {
                if(index % 2) {
                    $(this).addClass("row-odd").removeClass("row-even");
                } else {
                    $(this).addClass("row-even").removeClass("row-odd");
                }
            });
            return this;
        }
    }

    PHD.TreeTable = TreeTable;

    var BackupCatalog = function($table, options) {
        if(this instanceof BackupCatalog) {
            this.$table = $table;
            this.options = options;
            //this.view = options.view || BACKUP_CATALOG_VIEW_VM;
            this.view = options.view || BACKUP_CATALOG_VIEW_DATE;
            this.tree = null;
            this.childTypes = {
                "bds": "vms",
                "date": "instances",
                "vm": "backups",
                "compute_resource": "backups",
                "backup": "disks"
            };
        } else {
            return new BackupCatalog($table, options).init();
        }
    }

    BackupCatalog.prototype = Object.create(TreeTable.prototype);

    BackupCatalog.prototype.load = function(data) {
        var $tbody = this.$table.find("tbody").html(""),
            $rows = this.makeRows(data.tree);

        $rows.appendTo($tbody);
        this.tree = data.tree;
        this.environments = data.environments;
        this.hosts = data.hosts;
        this.types = data.types;
        this.$table.on("mousedown", ".item-name", function(event) {
            event.preventDefault();
            if(event.which === 2) console.log($(this).closest("tr").data("node"));
        });
        this.$table.trigger("backupcatalogload");
        return this;
    };

    BackupCatalog.prototype.loadChildren = function($node) {
        var node = $node.data("node"),
            type = node.type,
            childType = this.getChildType(type),
            $rows;

        if(type === 'disk' | !childType) return this;

        $rows = this.makeRows(node.children);
        if($rows.length) {
            $rows.insertAfter($node);
            this.$table.trigger("backupcatalogloadchildren");
        }
        return this;
    };

    BackupCatalog.prototype.makeRows = function(nodes) {
        var $rows = $(), self = this, $row;
        if (nodes != null && nodes.length) {
            nodes.forEach(function(node) {
                if(!node.rendered) {
                    $row = $(node.html).data("node", node);
                    if(node.filtered.length) $row.addClass(node.filtered.join(" ") + " filtered");
                    $rows = $rows.add($row);
                    node.rendered = true;
                } else {
                    $row = $();
                }
                if(node.expanded) $rows = $rows.add(self.makeRows(node.children));
            });
        }
        return $rows;
    };

    BackupCatalog.prototype.filter = function() {
        var $toFilter = $(),
            $toClear = $();
        this.$table
            .find("tbody tr").each(function() {
                if($(this).data("node").filtered.length) {
                    $toFilter = $toFilter.add($(this));
                } else {
                    $toClear = $toClear.add($(this));
                }
            });
        $toFilter.addClass("filtered").find(".row-select").prop("checked", false);
        $toClear.removeClass("filtered");
        this.$table.trigger("afterfilter");
    }

    BackupCatalog.prototype.getChildType = function(type) {
        if(type === 'disk' | !this.childTypes.hasOwnProperty(type)) return null;
        return this.childTypes[type];
    };

    BackupCatalog.prototype.toggleItem = function($clicked, force) {
        var $node = $clicked.closest("tr"),
            type = $node.data("type"),
            id = $node.attr(type + "_id");

        if(!$clicked.is(".expand-ctrl")) {
            $clicked = $node.find(".expand-ctrl");
        }

        if($clicked.hasClass("expanded") && force !== true || force === false) {
            $clicked.removeClass("expanded");
            this.getChildren($node).removeClass("open").addClass("closed");
            this.getDescendants($node).removeClass("visible");
        } else {
            $clicked.addClass("expanded");
            this.loadChildren($node);
            this.getChildren($node).removeClass("closed").addClass("open");
            this.getDescendants($node).addClass("visible");
        }

        return this.zebra();
    };

    BackupCatalog.prototype.selectAll = function() {
        this.$table.find("tbody :checkbox").filter(":visible").each(function() {
            $(this).prop("checked", true).trigger("change");
        });
        return this;
    };

    BackupCatalog.prototype.setDescendants = function($checkbox) {
        var $node = $checkbox.closest("tr"),
            checked = $checkbox.prop("checked");

        this.safeDescendants($node).find(".row-select").prop("checked", checked).trigger("change");
        return this;
    };

    BackupCatalog.prototype.setFirstChild = function($checkbox) {
        var $node = $checkbox.closest("tr"),
            checked = $checkbox.prop("checked");

        var $child = $node.next("tr").find(".row-select").prop("checked", checked).trigger("change");
        return this.setDescendants($child);
    };

    BackupCatalog.prototype.safeDescendants = function($node) {
        var $out = $();
        this.getDescendants($node).each(function() {
            if(!$(this).data("node").filtered.length) {
                $out = $out.add($(this));
            }
        });
        return $out;
    };

    BackupCatalog.prototype.expandLevel = function(level) {
        var self = this;
        this.$table.find("tbody tr")
            .filter("[data-depth=" + level + "]")
            .removeClass("closed")
            .addClass("open visible")
            .each(function() {
                self.loadChildren($(this));
            });

        return this;
    };

    BackupCatalog.prototype.pin = function(ids) {
        var self = this;
        ids.forEach(function(id) {
            self.getNodeById("backup", id)
                .addClass("pin")
                .data("node").pinned = true;
        });
        return this;
    };

    BackupCatalog.prototype.unpin = function(ids) {
        var self = this;
        ids.forEach(function(id) {
            self.getNodeById("backup", id)
                .removeClass("pin")
                .data("node").pinned = false;
        });
        return this;
    };

    BackupCatalog.prototype.setView = function(view) {
        this.options.view = this.view = view;
        return this;
    };

    BackupCatalog.prototype.getEnvironments = function() {
        return this.environments;
    };

    BackupCatalog.prototype.getHosts = function() {
        return this.hosts;
    };

    BackupCatalog.prototype.getTypes = function() {
        return this.types;
    };

    PHD.BackupCatalog = BackupCatalog;

})(jQuery, window.PHD || (window.PHD = {}), window);
